*DECK CFCGET
      SUBROUTINE CFCGET(TINFO ,DBNAME,IPRINT,NBPARA,DBPARA)
C
C---------------------------  CFCGET  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : CFCGET
C      USE      : READ CFC OPTIONS
C      MODIFIED : 2005/05/09
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C      TINFO    : Title of database
C      DBNAME   : Database name
C      IPRINT   : print level
C      NBPARA   : Number of parameters
C      DBPARA   : Database parameters
C                 DBPARA( 1) = Nominal power level (PWR)
C                 DBPARA( 2) = Nominal T cool (TCR)
C                 DBPARA( 3) = Nominal T mode (TMR)
C                 DBPARA( 4) = Nominal T fuel (TFR)
C                 DBPARA( 5) = Nominal Density cool
C                 DBPARA( 6) = Nominal Density mode
C                 DBPARA( 7) = Nominal purity mode (XIR)
C                 DBPARA( 8) = Perturbed T fuel 1 (TFU)
C                 DBPARA( 9) = Perturbed T cool 1 (TCU)
C                 DBPARA(10) = Perturbed P1 (PWUL)
C                 DBPARA(11) = Perturbed P2 (PWDL)
C                 DBPARA(12) = Perturbed P3 (PWU)
C                 DBPARA(13) = Perturbed P4 (PWD)
C                 DBPARA(14) = Perturbed P mode (XI)
C
C---------------------------   CFCGET  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NAMSBR='CFCGET')
C----
C  ROUTINE PARAMETERS
C----
      CHARACTER   TINFO*72,DBNAME*9
      INTEGER     IPRINT,NBPARA
      REAL        DBPARA(NBPARA)
C----
C  INPUT PARAMETERS
C----
      INTEGER     ITYPLU,INTLIR
      CHARACTER   CARLIR*12
      REAL        REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      INTEGER     IREGIO,IGROUP,JGROUP,IMATER,ITC,NMIXME,NTMICR,
     >            ISO
C----
C  Initialize parameters
C----
      IPRINT=1
      CALL XDRSET(DBPARA,NBPARA,0.0)
      DBPARA(1)=615.0
      DBPARA(2)=560.66
      DBPARA(3)=345.66
      DBPARA(4)=941.29
      DBPARA(5)=1.08288
      DBPARA(6)=0.81212
      DBPARA(7)=0.99911
      DBPARA(8)=1241.29
      DBPARA(9)=660.66
      DBPARA(10)=878.57143
      DBPARA(11)=307.5
      DBPARA(12)=307.5
      DBPARA(13)=100.0
      DBPARA(14)=0.985
      DBPARA(15)=375.66
      DBPARA(16)=541.29
      DBPARA(17)=300.66
      DBPARA(18)=295.66
C----
C  READ TITLE
C----
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER DATA EXPECTED.')
      IF(CARLIR .EQ. 'EDIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .EQ. 1) THEN
          IPRINT=INTLIR
        ELSE
          CALL XABORT(NAMSBR//': EDIT LEVEL EXPECTED.')
        ENDIF
        CALL REDGET (ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER DATA EXPECTED.')
      ENDIF
      IF(CARLIR .EQ. 'INFOR') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,TINFO,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED.')
      ELSE
        CALL XABORT(NAMSBR//': INFOR KEY WORD EXPECTED.')
      ENDIF
C-----
C  READ THE RECORD NAME FOR THE DATABASE
C-----
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER DATA EXPECTED.')
      IF(CARLIR .EQ. 'DNAME') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED.')
        DBNAME=CARLIR(1:9)
      ENDIF
C----
C  LOOP OVER PARAMETERS TO READ
C----
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER DATA EXPECTED.')
      IF(CARLIR .EQ. ';') THEN
        GO TO 105
      ELSE IF(CARLIR .EQ. 'PWR') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal power EXPECTED.')
        DBPARA(1)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Power UP EXPECTED.')
        DBPARA(10)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Power DOWN 1 EXPECTED.')
        DBPARA(11)=REALIR
        DBPARA(12)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Power DOWN 2 EXPECTED.')
        DBPARA(13)=REALIR
      ELSE IF(CARLIR .EQ. 'TCOOL') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal coolant temperature expected.')
        DBPARA(2)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed up coolant temperature expected.')
        DBPARA(9)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed down coolant temperature expected.')
        DBPARA(17)=REALIR
      ELSE IF(CARLIR .EQ. 'TMODE') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal moderator temperature expected.')
        DBPARA(3)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed up moderator temperature expected.')
        DBPARA(15)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed down moderator temperature expected.')
        DBPARA(18)=REALIR
      ELSE IF(CARLIR .EQ. 'TFUEL') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal fuel temperature expected.')
        DBPARA(4)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed up fuel temperature expected.')
        DBPARA(8)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed down fuel temperature expected.')
        DBPARA(16)=REALIR
      ELSE IF(CARLIR .EQ. 'RHOM') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal moderator density expected.')
        DBPARA(5)=REALIR
      ELSE IF(CARLIR .EQ. 'RHOC') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal coolant density expected.')
        DBPARA(6)=REALIR
      ELSE IF(CARLIR .EQ. 'XIR') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Nominal Xenon.')
        DBPARA(7)=REALIR
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': Perturbed Xenon.')
        DBPARA(14)=REALIR
      ENDIF
      GO TO 100
 105  CONTINUE
      RETURN
      END
