*DECK CROWH2
      SUBROUTINE CROWH2(Z,ZI,COTE,SIGT,R,IAPP,TRONC,PIS,PSS,PII)
C
C---------------------------  CROWH2  ---------------------------------
C
C   1-  PROGRAM STATISTICS:
C
C          FUNCTION   :  COMPUTE THE COLLISION PROBABILITIES OF AN
C                        HOMOGENEOUS 2D HEXAGONAL CROWN WITH DP1
C                        SURFACE ANGULAR EXPANSION ON THE HEXAGONAL
C                        BOUNDARY. THE TRACKS ARE COMPUTED BY SYBHTK
C          REVISED    :  18-06-1990
C          AUTHOR     :  M.OUISLOUMEN
C
C   2-  PROGRAM PARAMETERS:
C
C INPUT PARAMETERS.
C   IAPP    : TYPE OF ANGULAR FLUX EXPANSION ON THE INTERNAL SURFACE
C            = 2   => DP1 , = 1  => DP0                  L
C   COTE    : LENGTH OF ONE OF SIDES OF THE HEXAGONE     R
C   SIGT    : TOTAL CROSS SECTION.                       R
C   Z,ZI    : INTEGRATION MESH                           R
C   R       : RADIUS OF INTERNAL TUBE                    R
C
C INTRINSEC FUNCTIONS
C   FUNC3   : COMPUTE THE BICKLEY NAYLOR FUNCTION OF ORDER 3
C   FUNC4   : COMPUTE THE BICKLEY NAYLOR FUNCTION OF ORDER 4
C   FUNC5   : COMPUTE THE BICKLEY NAYLOR FUNCTION OF ORDER 5
C
C OUTPUT PARAMETERS.
C   PIS     : LEAKAGE PROBABILITY                        R(20)
C             PIS(I)  -> VOLUME TO SIDE I,(I=1,20)
C   PSS     : TRANSMISSION PROBABILITY                   R(20,20)
C             PSS(I,J) -> SIDE I TO SIDE J (I=1,20,J=1,20)
C   PII     : VOLUME TO VOLUME REDUCED PROBABILITY.
C
C
C                                           SIDE A,B,C
C                      SIDE 6,7,8           DIR A -> ISOTROPIC
C                     XXXXXXXX              DIR C -> TANGENT TO SURFACE
C                    X        X             DIR B -> NORMAL  TO SURFACE
C     SIDE 9,10,11  X          X SIDE 3,4,5
C                  X   SIDE 1,2 X
C                 X      O       X
C                  X            X
C     SIDE 12,13,14 X          X SIDE 18,19,20
C                    X        X
C                     XXXXXXXX
C                      SIDE 15,16,17
C
C---------------------------   CROWH2  --------------------------------
C
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (SQRT3=1.732050807568877,SQRT2=1.414213562373095,
     +            PI=3.141592653589793,PI4=PI*.25,PI12=PI/12.,
     +            PI8=PI*.125,PI16=3.*PI/16.,C1=2./3.,C2=1./6.)
C
      REAL      PIS(*),PSS(18+IAPP,*),Z(*),P(24),PVS(5)
      INTEGER   ZI(*),IROT1(20,20),IROT0(19,19),IPVS0(19),IPVS1(20)
      LOGICAL   LGP1,LG6,LG5,LG51,SERIE
C
C ASSUME THAT BICKLEY KI TABLES HAVE THE SAME TABULATION POINTS AND
C THE SAME TRUNCATION LIMIT.
C
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
C
      SAVE IROT1,IROT0,IPVS1,IPVS0
      DATA IROT1/0,0,21,22,0,21,22,0,21,22,0,21,22,0,21,22,0,21,22,0,0,
     +    0,23,24,0,23,24,0,23,24,0,23,24,0,23,24,0,23,24,0,17,18,0,0,
     +    0,1,2,-3,7,8,-9,13,14,0,7,8,9,1,2,3,19,20,0,0,0,2,4,5,8,10,11,
     +    14,15, 0,8,10,-11,2,4,-5,0,0,0,0,0,3,-5,6,9,-11,12,0,0,16,-9,
     +    11,12,-3,5,6,17,18,1,2,3,0,0,0,1,2,-3,7,8,-9,13,14,0,7,8,9,
     +    19,20,2,4,-5,0,0,0,2,4,5,8,10,11,14,15,0,8,10,-11,0,0,-3,5,6,
     +    0,0,0,3,-5,6,9,-11,12,0,0,16,-9,11,12,17,18,7,8,9,1,2,3,0,0,
     +    0,1,2,-3,7,8,-9,13,14,0,19,20,8,10,-11,2,4,-5,0,0,0,2,4,5,8,
     +    10,11,14,15,0,0,0,-9,11,12,-3,5,6,0,0,0,3,-5,6,9,-11,12,0,0,
     +    16,17,18,13,14,0,7,8,9,1,2,3,0,0,0,1,2,-3,7,8,-9,19,20,14,15,
     +    0,8,10,-11,2,4,-5,0,0,0,2,4,5,8,10,11,0,0,0,0,16,-9,11,12,-3,
     +    5,6,0,0,0,3,-5,6,9,-11,12,17,18,7,8,-9,13,14,0,7,8,9,1,2,3,0,
     +    0,0,1,2,-3,19,20,8,10,11,14,15,0,8,10,-11,2,4,-5,0,0,0,2,4,5,
     +    0,0,9,-11,12,0,0,16,-9,11,12,-3,5,6,0,0,0,3,-5,6,17,18,1,2,-3,
     +    7,8,-9,13,14,0,7,8,9,1,2,3,0,0,0,19,20,2,4,5,8,10,11,14,15,0,
     +    8,10,-11,2,4,-5,0,0,0,0,0,3,-5,6,9,-11,12,0,0,16,-9,11,12,-3,
     +    5,6,0,0,0/
      DATA IPVS1/1,2,3,4,5,3,4,5,3,4,5,3,4,5,3,4,5,3,4,5/
      DATA IPVS0/1,3,4,5,3,4,5,3,4,5,3,4,5,3,4,5,3,4,5/
      DATA IROT0/
     +     0,21,22, 0,21,22, 0,21,22, 0,21,22, 0,21,22, 0,21,22, 0,
     +    17, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9, 1, 2, 3,
     +    19, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11, 2, 4,-5,
     +     0, 0, 0, 0, 3,-5, 6, 9,-11,12, 0, 0,16,-9,11,12,-3, 5, 6,
     +    17, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9,
     +    19, 2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11,
     +     0,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12, 0, 0,16,-9,11,12,
     +    17, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0,
     +    19, 8,10,-11,2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0,
     +     0,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12, 0, 0,16,
     +    17,13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,
     +    19,14,15, 0, 8,10,-11,2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,
     +     0, 0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,
     +    17, 7, 8,-9,13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3,
     +    19, 8,10,11,14,15, 0, 8,10,-11,2, 4,-5, 0, 0, 0, 2, 4, 5,
     +     0, 9,-11,12,0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6,
     +    17, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0,
     +    19, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11,2, 4,-5, 0, 0, 0,
     +     0, 3,-5, 6, 9,-11,12,0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0/
C
      FUNC3(X,K)=BI3(K,1)+X*(BI3(K,2)+X*BI3(K,3))
      FUNC4(X,K)=BI4(K,1)+X*(BI4(K,2)+X*BI4(K,3))
      FUNC5(X,K)=BI5(K,1)+X*(BI5(K,2)+X*BI5(K,3))
C
C    INITIALISATION
C
      DO 10 I=1,24
      P(I)=0.
   10 CONTINUE
C
      LGP1=.TRUE.
      IF(IAPP.NE.2)LGP1=.FALSE.
      RI5=.5/R
      NA=ZI(1)
      NX=ZI(2)
      IZ0=2
      IZ=4
      VOLUME=4.*(1.5*COTE*COTE*SQRT3-PI*R*R)
      SERIE=SIGT*VOLUME/(6.*COTE).LE.TRONC
      IF(SERIE) THEN
         PVS(1)=0.
         PVS(2)=0.
         P60S=0.
         P60I=0.
         P50S=0.
         P50I=0.
         P40S=0.
         P40I=0.
         P70I=0.
         P61S=0.
         P51S=0.
         P42S=0.
         P712S=0.
      ENDIF
C
C      INTEGRATE OVER ANGELES
C
          DO 150 IA=1,NA
C
      LG6=.TRUE.
      LG5=.FALSE.
      LG51=.FALSE.
      IZ0=IZ0+3
      NR5=ZI(IZ0)
      IMIN=ZI(IZ0-1)
      IMAX=NR5+IMIN-1
      IMIN6=ZI(IZ0-2)
      NR6=3-IMIN6
      IZZ1=IZ
      IZ=IZ+NR5+NR6+IMIN
      IF(IMIN6.EQ.1) LG6=.FALSE.
      IF(IMAX.EQ.2)LG5=.TRUE.
      IF(IMIN.EQ.2)LG51=.TRUE.
C
C       INTEGRATE OVER DISTANCE X
C
         DO 120 IX=1,NX
C
      IZ=IZ+9
      W600=Z(IZ)
      W411=Z(IZ-1)
      W420=Z(IZ-2)
      W511=Z(IZ-3)
      W611=Z(IZ-4)
      W610=Z(IZ-5)
      W702=Z(IZ-6)
      W711=Z(IZ-7)
      W722=Z(IZ-8)
C
C     TRANSMISSION PAR LES DIFFERENTES FACES
C     SURFACE 12,13,14
C
      IZ1=IZZ1
      IZ=IZ+2
      IZZ=IZ
      RI=Z(IZ-1)
      IZ1=IZ1+1
      DELR=Z(IZ1)
      TAU=SIGT*RI
      WW6=W600*DELR
      WW61=W610*DELR
      IF(SERIE)THEN
C
C        SERIES EXANSION FOR BICKLEY-NAYLOR FUNCTIONS
C
         XXF3=PI4*RI*RI
         XF3=-RI+SIGT*XXF3
         P60S=P60S+WW6*XF3
         P60I=P60I+WW6*XXF3
         P(1)=P(1)+WW6*(PI4+SIGT*XF3)
         XF4=RI*(-PI4+TAU*(.5-PI12*TAU))
         P61S=P61S+WW61*XF4
         P(3)=P(3)+WW61*(C1+SIGT*XF4)
         PKI5=DELR*(PI16+TAU*(-C1+TAU*(PI8-C2*TAU)))
         P(6)=P(6)+W611*PKI5
         P(5)=P(5)+W600*PKI5
      ELSE
C
C         DIRECT KI CALCULATIONS
C
         IF(TAU.LT.XLIM3) THEN
            K=NINT(PAS3*TAU)
            P(1)=P(1)+WW6*FUNC3(TAU,K)
            P(3)=P(3)+WW61*FUNC4(TAU,K)
            PKI5=DELR*FUNC5(TAU,K)
            P(6)=P(6)+W611*PKI5
            P(5)=P(5)+W600*PKI5
         ENDIF
      ENDIF
C
      IF(LG6) GOTO 8
C
C           CONTRIBUTION TO SURFACE 1,2
C
      IZ1=IZ1+1
      DELR=Z(IZ1)
      IZ=IZ+1
      RI=Z(IZ)
      IZ=IZ+2
      RC=Z(IZ)
      TAU7C=SIGT*RC
      RD=Z(IZ-1)-RC
      TAU72=SIGT*RD
      WW6=DELR*W600
      COS6=RI*RI5
      IF(SERIE)THEN
C
C         SERIES EXPANSIONS FOR KI FUNCTIONS
C
         XXF3=PI4*(RC*RC+RD*RD)
         XF3=-RC-RD+SIGT*XXF3
         PVS(1)=PVS(1)+WW6*XF3
         P70I=P70I+WW6*XXF3
         P(17)=P(17)+WW6*(2.*PI4+SIGT*XF3)
         XF4=RC*(-PI4+TAU7C*(.5-PI12*TAU7C))
         XF42=RD*(-PI4+TAU72*(.5-PI12*TAU72))
         F4=C1+SIGT*XF4
         F42=C1+SIGT*XF42
         P(19)=P(19)+DELR*(W702*F4+W722*F42)
         P712S=P712S+DELR*(W702*XF4+W722*XF42)
         IF(LGP1) THEN
            WWW6=WW6*COS6
            P(18)=P(18)+WWW6*(F4+F42)
            PVS(2)=PVS(2)+WWW6*(XF4+XF42)
            F5=PI16+TAU7C*(-C1+TAU7C*(PI8-C2*TAU7C))
            F52=PI16+TAU72*(-C1+TAU72*(PI8-C2*TAU72))
            P(20)=P(20)+COS6*(W702*F5+W722*F52)*DELR
         ENDIF
      ELSE
C
C      DIRECT KI CALCULATIONS
C
         IF(TAU7C.LT.XLIM3) THEN
            K=NINT(PAS3*TAU7C)
            F4=DELR*FUNC4(TAU7C,K)
            P(17)=P(17)+WW6*FUNC3(TAU7C,K)
            P(19)=P(19)+W702*F4
            IF(LGP1) THEN
               P(18)=P(18)+W600*COS6*F4
               P(20)=P(20)+COS6*W702*DELR*FUNC5(TAU7C,K)
            ENDIF
         ENDIF
         IF(TAU72.LT.XLIM3) THEN
            K=NINT(PAS3*TAU72)
            F4=DELR*FUNC4(TAU72,K)
            P(17)=P(17)+WW6*FUNC3(TAU72,K)
            P(19)=P(19)+W722*F4
            IF(LGP1) THEN
               P(18)=P(18)+COS6*W600*F4
               P(20)=P(20)+COS6*W722*DELR*FUNC5(TAU72,K)
            ENDIF
         ENDIF
      ENDIF
C
C     SURFACES 9,10,11
C
    8 IF(LG5) THEN
         IZ=IZ+2
         R0=Z(IZ-1)
         IZ1=IZ1+1
         DELR=Z(IZ1)
         TAU=SIGT*R0
         WW6=W600*DELR
         WW61=W610*DELR
         IF(SERIE)THEN
C
C       SERIES EXPANSION FOR KI FUNCTIONS
C
            XXF3=PI4*R0*R0
            XF3=-R0+SIGT*XXF3
            P50S=P50S+WW6*XF3
            P50I=P50I+WW6*XXF3
            P(7)=P(7)+WW6*(PI4+SIGT*XF3)
            XF4=R0*(-PI4+TAU*(.5-PI12*TAU))
            P51S=P51S+WW61*XF4
            P(9)=P(9)+WW61*(C1+SIGT*XF4)
            PKI5=DELR*(PI16+TAU*(-C1+TAU*(PI8-C2*TAU)))
            P(12)=P(12)+W511*PKI5
            P(11)=P(11)+W600*PKI5
            IF(LG51) GOTO 11
         ELSE
C
C         DIRECT KI CALCULATIONS
C
            IF(TAU.LT.XLIM3) THEN
               K=NINT(PAS3*TAU)
               P(7)=P(7)+WW6*FUNC3(TAU,K)
               P(9)=P(9)+WW61*FUNC4(TAU,K)
               PKI5=DELR*FUNC5(TAU,K)
               P(12)=P(12)+W511*PKI5
               P(11)=P(11)+W600*PKI5
            ENDIF
            IF(LG51) GOTO 11
         ENDIF
      ENDIF
C
C         CONTRIBUTION TO SURFACE 1,2
C
      IZ=IZ+1
      RI=Z(IZ)
      IZ=IZ+2
      RC=Z(IZ)
      IZ1=IZ1+1
      DELR=Z(IZ1)
      TAU7C=SIGT*RC
      RD=Z(IZ-1)-RC
      TAU72=SIGT*RD
      WW6=DELR*W600
      COS6=RI*RI5
      IF(SERIE)THEN
C
C          SERIES EXPANSION FOR KI FUNCTIONS
C
         XXF3=PI4*(RC*RC+RD*RD)
         XF3=-RC-RD+SIGT*XXF3
         PVS(1)=PVS(1)+WW6*XF3
         P70I=P70I+WW6*XXF3
         P(17)=P(17)+WW6*(2.*PI4+SIGT*XF3)
         XF4=RC*(-PI4+TAU7C*(.5-PI12*TAU7C))
         XF42=RD*(-PI4+TAU72*(.5-PI12*TAU72))
         F4=C1+SIGT*XF4
         F42=C1+SIGT*XF42
         P(19)=P(19)+DELR*(W420*F4+W711*F42)
         P712S=P712S+DELR*(W420*XF4+W711*XF42)
         IF(LGP1) THEN
            P(18)=P(18)+WW6*COS6*(F4+F42)
            PVS(2)=PVS(2)+WW6*COS6*(XF4+XF42)
            F5=PI16+TAU7C*(-C1+TAU7C*(PI8-C2*TAU7C))
            F52=PI16+TAU72*(-C1+TAU72*(PI8-C2*TAU72))
            P(20)=P(20)+COS6*(W420*F5+W711*F52)*DELR
         ENDIF
         GOTO 17
      ELSE
C
C          DIRECT KI CALCULATIONS
C
         IF(TAU7C.LT.XLIM3) THEN
            K=NINT(PAS3*TAU7C)
            F4=DELR*FUNC4(TAU7C,K)
            P(17)=P(17)+WW6*FUNC3(TAU7C,K)
            P(19)=P(19)+W420*F4
            IF(LGP1) THEN
               P(18)=P(18)+W600*COS6*F4
               P(20)=P(20)+COS6*W420*DELR*FUNC5(TAU7C,K)
            ENDIF
         ENDIF
         IF(TAU72.LT.XLIM3) THEN
            K=NINT(PAS3*TAU72)
            F4=DELR*FUNC4(TAU72,K)
            P(17)=P(17)+WW6*FUNC3(TAU72,K)
            P(19)=P(19)+W711*F4
            IF(LGP1) THEN
               P(18)=P(18)+COS6*W600*F4
               P(20)=P(20)+COS6*W711*DELR*FUNC5(TAU72,K)
            ENDIF
         ENDIF
         GOTO 17
      ENDIF
C
C     SURFACE 6,7,8
C
   11 IZ=IZ+2
      R0=Z(IZ-1)
      IZ1=IZ1+1
      DELR=Z(IZ1)
      TAU=SIGT*R0
      WW6=W600*DELR
      WW4=W420*DELR
      IF(SERIE) THEN
C
C       SERIES EXPANSION FOR KI FUNCTIONS
C
         XXF3=PI4*R0*R0
         XF3=-R0+SIGT*XXF3
         P40S=P40S+WW6*XF3
         P40I=P40I+WW6*XXF3
         XF4=R0*(-PI4+TAU*(.5-PI12*TAU))
         P42S=P42S+WW4*XF4
         P(13)=(13)+WW6*(PI4+SIGT*XF3)
         P(14)=P(14)+WW4*(C1+SIGT*XF4)
         PKI5=DELR*(PI16+TAU*(-C1+TAU*(PI8-C2*TAU)))
         P(16)=P(16)+W411*PKI5
         P(15)=P(15)+(W600-W411)*PKI5
      ELSE
C
C       DIRECT KI CALCULATIONS
C
         IF(TAU.LT.XLIM3) THEN
            K=NINT(PAS3*TAU)
            P(13)=P(13)+WW6*FUNC3(TAU,K)
            P(14)=P(14)+WW4*FUNC4(TAU,K)
            PKI5=DELR*FUNC5(TAU,K)
            P(16)=P(16)+W411*PKI5
            P(15)=P(15)+(W600-W411)*PKI5
         ENDIF
      ENDIF
C
C       CONTRIBUTION TO SURFACE 1,2
C
   17 IZ=IZ+3
      RC=Z(IZ)
      RI=Z(IZ-2)
      IZ1=IZ1+1
      DELR=.5*Z(IZ1)
      TAU7C=RC*SIGT
      RD=Z(IZ-1)-RC
      TAU72=SIGT*RD
      WW6=DELR*W600
      W2=W420*DELR
      COS6=RI*RI5
      IF(SERIE) THEN
C
C        SERIES EXPANSION FOR KI FUNCTIONS
C
         XXF3=PI4*(RC*RC+RD*RD)
         XF3=-RC-RD+SIGT*XXF3
         PVS(1)=PVS(1)+WW6*XF3
         P70I=P70I+WW6*XXF3
         P(17)=P(17)+WW6*(2.*PI4+SIGT*XF3)
         XF4=-PI4*RC+RC*TAU7C*(.5-PI12*TAU7C)
         XF42=-PI4*RD+RD*TAU72*(.5-PI12*TAU72)
         F4=C1+SIGT*XF4
         F42=C1+SIGT*XF42
         P(19)=P(19)+W2*(F4+F42)
         P712S=P712S+W2*(XF4+XF42)
         IF(LGP1) THEN
            P(18)=P(18)+WW6*COS6*(F4+F42)
            PVS(2)=PVS(2)+WW6*COS6*(XF4+XF42)
            F5=PI16+TAU7C*(-C1+TAU7C*(PI8-C2*TAU7C))
            F52=PI16+TAU72*(-C1+TAU72*(PI8-C2*TAU72))
            P(20)=P(20)+COS6*W2*(F5+F52)
         ENDIF
      ELSE
C
C        DIRECT KI CALCULATIONS
C
         IF(TAU7C.LT.XLIM3) THEN
            K=NINT(PAS3*TAU7C)
            F4=FUNC4(TAU7C,K)
            P(17)=P(17)+WW6*FUNC3(TAU7C,K)
            P(19)=P(19)+W2*F4
            IF(LGP1) THEN
               P(18)=P(18)+WW6*F4*COS6
               P(20)=P(20)+COS6*W2*FUNC5(TAU7C,K)
            ENDIF
         ENDIF
         IF(TAU72.LT.XLIM3) THEN
            K=NINT(PAS3*TAU72)
            F4=FUNC4(TAU72,K)
            P(17)=P(17)+WW6*FUNC3(TAU72,K)
            P(19)=P(19)+W2*F4
            IF(LGP1) THEN
               P(18)=P(18)+WW6*F4*COS6
               P(20)=P(20)+COS6*W2*FUNC5(TAU72,K)
            ENDIF
         ENDIF
      ENDIF
  120 CONTINUE
  150 CONTINUE
C
C     PSS ORTHONORMALIZATION
C
      E1=Z(1)
      E2=Z(2)
      E3=Z(3)
      E4=Z(4)
      COEF=COTE/(2.*PI*R)
      AUX=1./(6.*COTE)
      P17=AUX*P(17)
      P(17)=E1*E1*P17
      P(19)=AUX*E1*E2*P(19)-E3*E1*P17
C
C    LES PROBABILITES PSS(I->7) SONT OBTENUES PAR SYMETRIE
C    ET LES PSS(7->I) PAR RELATION DE RECIPROCITE
C
      P(21)=COEF*P(17)
      P(22)=COEF*P(19)
      IF(LGP1) THEN
         P(18)=AUX*E1*E2*P(18)-E3*E1*P17
         P(20)=P(20)*AUX*E2*E2-E3*((P(19)+P(18))/E1+E3*P17)
         P(23)=COEF*P(18)
         P(24)=COEF*P(20)
         PVS(2)=AUX*E1*(PVS(2)*E2-E3*PVS(1))
      ENDIF
      IF(SERIE) THEN
         P70I=AUX*P70I*E1*E1
         P712S=COEF*AUX*E1*(P712S*E2-E3*PVS(1))
         PVS(1)=AUX*PVS(1)*E1*E1
      ENDIF
C
      AUX1=AUX
      AUX=1./COTE
      AAUX1=AUX1
      ZZAUX=AUX1*E1*E1
      P13=P(13)*AUX1
      P(1)=P(1)*ZZAUX
      P(7)=P(7)*ZZAUX
      P(13)=P13*E1*E1
      PP=SQRT3*ZZAUX
      IF(SERIE) THEN
         P60S=ZZAUX*P60S
         P60I=ZZAUX*P60I
         P50I=ZZAUX*P50I
         P50S=ZZAUX*P50S
         P40S=ZZAUX*P40S
         P40I=ZZAUX*P40I
         P61S=P61S*PP
         P51S=P51S*PP
         P42S=E1*(AAUX1*P42S*E2-E3*P40S)
      ENDIF
      AUX1=AUX1*.5
      P(3)=-0.5*P(3)*AUX1*E1*E4*SQRT3
      P(9)=-0.5*P(9)*AUX1*E1*E4
      P(14)=E1*(AAUX1*P(14)*E2-E3*P13)
      SQRT6=.5*SQRT3*AUX1*E4*E2
      P(5)=SQRT6*P(5)+E3*P(3)/E1
      P(11)=SQRT6*P(11)+E3*P(9)/E1
      ZZAUX=AUX1*E4*E4
      P(6)=-ZZAUX*P(6)
      P(12)=-ZZAUX*P(12)
      P(16)=2.*ZZAUX*P(16)
      P(15)=AAUX1*E2*E2*P(15)-E3*(2.*P(14)/E1+E3*P13)
C
C                  * STOCKAGE DES PSS *
C
C    OTHER PROBABILITIES CALULATION
C
      P(2)=(-SQRT3*P(3)-4.*P(1))/SQRT2
      P(4)=-4.5*P(6)+SQRT3*(-SQRT2*P(5)+8.*P(3))+8.*P(1)
      P(8)=(-3.*SQRT3*P(9)-4.*P(7))/SQRT2
      P(10)=-4.5*P(12)+SQRT3*(SQRT2*P(11)+8.*P(9))+8.*P(7)
C
C    COMPUTATION OF PIS AND PII
C
      CORX=1./VOLUME
      COR=COTE*CORX
      IF(.NOT.SERIE) THEN
         COR=COR/SIGT
         PIS10=COR*(1.-2.*(P(1)+P(7))-P(13)-P(17))
         PIS12=-COR*(2.*(P(2)+P(8))+P(14)+P(19))
         COR=2.*PI*R*CORX/SIGT
         PVS(1)=COR*(1.-6.*P(21))
         IF(LGP1) PVS(2)=-6.*COR*P(23)
         PII=(1.-6.*PIS10-PVS(1))/SIGT
      ELSE
         PIS10=COR*(-2.*(P60S+P50S)-P40S-PVS(1))
         PIS12=-COR*(SQRT2*SQRT3*(P61S+3.*P51S)-4.*SQRT2*(P60S+P50S)
     +         +P42S+P712S)
         PVS(1)=-COR*6.*PVS(1)
         IF(LGP1) PVS(2)=-6.*COR*PVS(2)
         PII=6.*COR*(2.*(P60I+P50I+P70I)+P40I)
      ENDIF
      PVS(3)=PIS10
      PVS(4)=PIS12
      PVS(5)=0.
C
C     TRANSMISSION MATRIX AND LEAKAGE MATRIX
C
      IF(LGP1)THEN
         DO 58 I=1,20
            PIS(I)=PVS(IPVS1(I))
         DO 58 J=1,20
            IB=IROT1(J,I)
            IF(IB.GT.0)THEN
               PSS(I,J)=P(IB)
            ELSEIF(IB.LT.0)THEN
               PSS(I,J)=-P(-IB)
            ELSE
               PSS(I,J)=0.
            ENDIF
   58    CONTINUE
      ELSE
         DO 60 I=1,19
            PIS(I)=PVS(IPVS0(I))
         DO 60 J=1,19
            IB=IROT0(J,I)
            IF(IB.GT.0)THEN
               PSS(I,J)=P(IB)
            ELSEIF(IB.LT.0)THEN
               PSS(I,J)=-P(-IB)
            ELSE
               PSS(I,J)=0.
            ENDIF
   60    CONTINUE
      ENDIF
      RETURN
      END
