*DECK CROWR1
      SUBROUTINE CROWR1 (IAPP,COORD,Z,IZ,SIGT,TRONC,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 2-D
* SQUARE OR RECTANGULAR CROWN WITH DP-0 SURFACE ANGULAR FLUX EXPANSION
* ON THE EXTERNAL BOUNDARY. THE TRACKS ARE COMPUTED BY SYBRTK.
*
* INPUT PARAMETERS:
*   IAPP    : TYPE OF ANGULAR FLUX EXPANSION ON THE INTERNAL SURFACE.
*             =1: DP-0   ;   =2: DP-1
*   COORD(1): RADIUS OF THE INTERNAL TUBE.
*   COORD(2): DIMENSIONS OF THE EXTERNAL X SIDES.
*   COORD(3): DIMENSIONS OF THE EXTERNAL Y SIDES.
*   Z,IZ    : TRACKING INFORMATION.
*   SIGT    : TOTAL MACROSCOPIC CROSS SECTION.
*   TRONC   : VOIDED BLOCK CRITERION.
*
* OUTPUT PARAMETERS:
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES. DIMENSION PVS(IAPP+4)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             I TO J, WHERE I=1 AND J=1 IS THE INTERNAL CYLINDER AND
*             THE OTHER SURFACES ARE THE EXTERNAL SIDES.
*             DIMENSION PSS(IAPP+4,IAPP+4)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 27/10/89 ---
*
      PARAMETER (MKI3=600,MKI4=600)
      PARAMETER (PI=3.141592654,ZI20=1.0,ZI30=0.785398164,
     1 ZI40=0.666666667)
      DIMENSION COORD(*),Z(*),IZ(*),PVS(*),PSS(IAPP+4,*),PVB(4),PBB(13),
     1 ISL1(5,5),ISM1(5),ISL2(6,6),ISM2(6)
      LOGICAL ICARE,SMALL
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      SAVE ISL1,ISM1,ISL2,ISM2
      DATA ISL1/13, 6, 6, 5, 5,
     1           8,13, 4, 1, 1,
     2           8, 4,13, 1, 1,
     3           7, 3, 3,13, 2,
     4           7, 3, 3, 2,13/
      DATA ISM1/ 3, 2, 2, 1, 1/
      DATA ISL2/13,13, 6, 6, 5, 5,
     1          13,13,10,10, 9, 9,
     2           8,12,13, 4, 1, 1,
     3           8,12, 4,13, 1, 1,
     4           7,11, 3, 3,13, 2,
     5           7,11, 3, 3, 2,13/
      DATA ISM2/ 3, 4, 2, 2, 1, 1/
*
      RAYRE=COORD(1)
      A=COORD(2)
      B=COORD(3)
      ICARE=IZ(1).EQ.2
      SMALL=SIGT.LE.(TRONC*(A+B)/(2.0*A*B))
      AOB=A/B
      AOR=A/(2.0*PI*RAYRE)
      BOR=B/(2.0*PI*RAYRE)
      DO 10 I=1,13
   10 PBB(I)=0.0
      DO 20 I=1,4
   20 PVB(I)=0.0
      L=6
      NA=IZ(6)
      DO 170 IA=1,NA
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 70
      L=L+2
      DO 60 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      IF (KMAX.EQ.1) THEN
*        THE TRACK DOES NOT PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         IF (POP.GE.XLIM3) GO TO 30
         IF (SMALL) THEN
            PVB(1)=PVB(1)+2.0*ZI20*Z(L+1)*Z1
            PVB(2)=PVB(2)+2.0*ZI20*Z(L+1)*Z1*AOB
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         PBB(1)=PBB(1)+WI3*Z1
   30    L=L+1
      ELSE IF (KMAX.EQ.2) THEN
*        THE TRACK PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         ZMU=0.5*Z(L+2)/RAYRE
         IF (POP.GE.XLIM3) GO TO 40
         IF (SMALL) THEN
            PVB(1)=PVB(1)+2.0*ZI20*Z(L+1)*Z1
            PVB(3)=PVB(3)+4.0*ZI20*Z(L+1)*Z1*AOR
            PVB(4)=PVB(4)+4.0*ZI30*Z(L+1)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         PBB(5)=PBB(5)+2.0*WI3*Z1
         PBB(9)=PBB(9)+2.0*WI4*Z1*ZMU
   40    POP=SIGT*Z(L+3)
         IF (POP.GE.XLIM3) GO TO 50
         IF (SMALL) THEN
            PVB(2)=PVB(2)+2.0*ZI20*Z(L+3)*Z1*AOB
            PVB(3)=PVB(3)+4.0*ZI20*Z(L+3)*Z1*AOR
            PVB(4)=PVB(4)+4.0*ZI30*Z(L+3)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         PBB(6)=PBB(6)+2.0*WI3*Z1*AOB
         PBB(10)=PBB(10)+2.0*WI4*Z1*AOB*ZMU
   50    L=L+3
      ENDIF
   60 CONTINUE
*
   70 L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 130
      L=L+1
      DO 120 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      IF (KMAX.EQ.1) THEN
*        THE TRACK DOES NOT PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         IF (POP.GE.XLIM3) GO TO 80
         IF (SMALL) PVB(1)=PVB(1)+2.0*ZI20*Z(L+1)*Z1
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         PBB(2)=PBB(2)+2.0*WI3*Z1
   80    L=L+1
      ELSE IF (KMAX.EQ.2) THEN
*        THE TRACK PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         ZMU=0.5*Z(L+2)/RAYRE
         IF (POP.GE.XLIM3) GO TO 90
         IF (SMALL) THEN
            PVB(1)=PVB(1)+ZI20*Z(L+1)*Z1
            PVB(3)=PVB(3)+2.0*ZI20*Z(L+1)*Z1*AOR
            PVB(4)=PVB(4)+2.0*ZI30*Z(L+1)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         PBB(5)=PBB(5)+WI3*Z1
         PBB(9)=PBB(9)+WI4*Z1*ZMU
   90    POP=SIGT*Z(L+3)
         IF (POP.GE.XLIM3) GO TO 100
         IF (SMALL) THEN
            PVB(1)=PVB(1)+ZI20*Z(L+3)*Z1
            PVB(3)=PVB(3)+2.0*ZI20*Z(L+3)*Z1*AOR
            PVB(4)=PVB(4)+2.0*ZI30*Z(L+3)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         PBB(5)=PBB(5)+WI3*Z1
         PBB(9)=PBB(9)+WI4*Z1*ZMU
  100    L=L+3
      ENDIF
  120 CONTINUE
*
  130 IF (ICARE) GO TO 170
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 170
      L=L+1
      DO 165 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      IF (KMAX.EQ.1) THEN
*        THE TRACK DOES NOT PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         IF (POP.GE.XLIM3) GO TO 140
         IF (SMALL) PVB(2)=PVB(2)+2.0*ZI20*Z(L+1)*Z1
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         PBB(4)=PBB(4)+2.0*WI3*Z1
  140    L=L+1
      ELSE IF (KMAX.EQ.2) THEN
*        THE TRACK PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         ZMU=0.5*Z(L+2)/RAYRE
         IF (POP.GE.XLIM3) GO TO 150
         IF (SMALL) THEN
            PVB(2)=PVB(2)+ZI20*Z(L+1)*Z1
            PVB(3)=PVB(3)+2.0*ZI20*Z(L+1)*Z1*BOR
            PVB(4)=PVB(4)+2.0*ZI30*Z(L+1)*Z1*BOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         PBB(6)=PBB(6)+WI3*Z1
         PBB(10)=PBB(10)+WI4*Z1*ZMU
  150    POP=SIGT*Z(L+3)
         IF (POP.GE.XLIM3) GO TO 160
         IF (SMALL) THEN
            PVB(2)=PVB(2)+ZI20*Z(L+3)*Z1
            PVB(3)=PVB(3)+2.0*ZI20*Z(L+3)*Z1*BOR
            PVB(4)=PVB(4)+2.0*ZI30*Z(L+3)*Z1*BOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         PBB(6)=PBB(6)+WI3*Z1
         PBB(10)=PBB(10)+WI4*Z1*ZMU
  160    L=L+3
      ENDIF
  165 CONTINUE
  170 CONTINUE
      IF (ICARE) THEN
         PBB(1)=PBB(1)+PBB(1)
         PBB(4)=PBB(2)
         PBB(5)=PBB(5)+PBB(6)
         PBB(6)=PBB(5)
         PBB(9)=PBB(9)+PBB(10)
         PBB(10)=PBB(9)
         PVB(1)=PVB(1)+PVB(2)
         PVB(2)=PVB(1)
         PVB(3)=PVB(3)+PVB(3)
         PVB(4)=PVB(4)+PVB(4)
      ENDIF
      PBB(3)=PBB(1)*AOB
      PBB(7)=PBB(5)*AOR
      PBB(8)=PBB(6)*BOR
      PBB(11)=PBB(9)*AOR
      PBB(12)=PBB(10)*BOR
*
      DO 190 I=9,12
  190 PBB(I)=Z(2)*(Z(3)*PBB(I)-Z(4)*PBB(I-4))
      DO 200 I=1,8
  200 PBB(I)=PBB(I)*Z(2)**2
      PVB(4)=Z(2)*(Z(3)*PVB(4)-Z(4)*PVB(3))
      DO 210 I=1,3
  210 PVB(I)=PVB(I)*Z(2)**2
      IF (.NOT.SMALL) THEN
         PVB(1)=1.0-2.0*PBB(1)-PBB(2)-PBB(5)
         PVB(2)=1.0-2.0*PBB(3)-PBB(4)-PBB(6)
         PVB(3)=1.0-2.0*PBB(7)-2.0*PBB(8)
         PVB(4)=-2.0*PBB(11)-2.0*PBB(12)
         DO 220 I=1,4
  220    PVB(I)=PVB(I)/SIGT
      ENDIF
      VOL=A*B-PI*RAYRE**2
      PVB(1)=0.25*PVB(1)*A/VOL
      PVB(2)=0.25*PVB(2)*B/VOL
      PVB(3)=0.25*PVB(3)*(2.0*PI*RAYRE)/VOL
      PVB(4)=0.25*PVB(4)*(2.0*PI*RAYRE)/VOL
      IF (IAPP.EQ.1) THEN
         DO 230 I=1,5
         PVS(I)=PVB(ISM1(I))
         DO 230 J=1,5
  230    PSS(I,J)=PBB(ISL1(I,J))
      ELSE IF (IAPP.EQ.2) THEN
         DO 240 I=1,6
         PVS(I)=PVB(ISM2(I))
         DO 240 J=1,6
  240    PSS(I,J)=PBB(ISL2(I,J))
      ENDIF
      IF (SIGT.GT.0.0) THEN
         PVV=(1.0-PVS(1)-2.0*PVS(IAPP+1)-2.0*PVS(IAPP+3))/SIGT
      ELSE
         PVV=1.0E10
      ENDIF
      RETURN
      END
