*DECK CROWR2
      SUBROUTINE CROWR2 (IAPP,COORD,Z,IZ,SIGT,TRONC,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 2-D
* SQUARE OR RECTANGULAR CROWN WITH DP-1 SURFACE ANGULAR FLUX EXPANSION
* ON THE EXTERNAL BOUNDARY. THE TRACKS ARE COMPUTED BY SYBRTK.
*
* INPUT PARAMETERS:
*   IAPP    : TYPE OF ANGULAR FLUX EXPANSION ON THE INTERNAL SURFACE.
*             =1: DP-0   ;   =2: DP-1
*   COORD(1): RADIUS OF THE INTERNAL TUBE.
*   COORD(2): DIMENSIONS OF THE EXTERNAL X SIDES.
*   COORD(3): DIMENSIONS OF THE EXTERNAL Y SIDES.
*   Z,IZ    : TRACKING INFORMATION.
*   SIGT    : TOTAL MACROSCOPIC CROSS SECTION.
*   TRONC   : VOIDED BLOCK CRITERION.
*
* OUTPUT PARAMETERS:
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES. DIMENSION PVS(IAPP+12)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             I TO J, WHERE I=1 AND J=1 IS THE INTERNAL CYLINDER AND
*             THE OTHER SURFACES ARE THE EXTERNAL SIDES.
*             DIMENSION PSS(IAPP+12,IAPP+12)
*
*-------------------------------------- AUTHOR: A. HEBERT ; 30/10/89 ---
*
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (PI=3.141592654,ZI20=1.0,ZI30=0.785398164,
     1 ZI40=0.666666667)
      DIMENSION COORD(*),Z(*),IZ(*),PVS(*),PSS(IAPP+12,*),PVB(8),
     1 PBB(44),ISL1(13,13),ISM1(13),ISL2(14,14),ISM2(14)
      LOGICAL ICARE,SMALL
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
      SAVE ISL1,ISM1,ISL2,ISM2
      DATA ISL1/ 0, 30, 38,  0, 30, 38,  0, 29, 37,  0, 29, 37,  0,
     1          32,  0,  0,  0,  4, 12,  0,  1,  9,-19,  1,  9, 19,
     2          40,  0,  0,  0,  8, 16,  0,  5, 13,-23,  5, 13, 23,
     3           0,  0,  0,  0,  0,  0, 28, 17, 21,-25,-17,-21,-25,
     4          32,  4, 12,  0,  0,  0,  0,  1,  9, 19,  1,  9,-19,
     5          40,  8, 16,  0,  0,  0,  0,  5, 13, 23,  5, 13,-23,
     6           0,  0,  0, 28,  0,  0,  0,-17,-21,-25, 17, 21,-25,
     7          31,  3, 11, 20,  3, 11,-20,  0,  0,  0,  2, 10,  0,
     8          39,  7, 15, 24,  7, 15,-24,  0,  0,  0,  6, 14,  0,
     9           0,-18,-22,-27, 18, 22,-27,  0,  0,  0,  0,  0, 26,
     1          31,  3, 11,-20,  3, 11, 20,  2, 10,  0,  0,  0,  0,
     2          39,  7, 15,-24,  7, 15, 24,  6, 14,  0,  0,  0,  0,
     3           0, 18, 22,-27,-18,-22,-27,  0,  0, 26,  0,  0,  0/
      DATA ISM1/7,4,5,6,4,5,6,1,2,3,1,2,3/
      DATA ISL2/ 0,  0, 30, 38,  0, 30, 38,  0, 29, 37,  0, 29, 37,  0,
     1           0,  0, 34, 42,  0, 34, 42,  0, 33, 41,  0, 33, 41,  0,
     2          32, 36,  0,  0,  0,  4, 12,  0,  1,  9,-19,  1,  9, 19,
     3          40, 44,  0,  0,  0,  8, 16,  0,  5, 13,-23,  5, 13, 23,
     4           0,  0,  0,  0,  0,  0,  0, 28, 17, 21,-25,-17,-21,-25,
     5          32, 36,  4, 12,  0,  0,  0,  0,  1,  9, 19,  1,  9,-19,
     6          40, 44,  8, 16,  0,  0,  0,  0,  5, 13, 23,  5, 13,-23,
     7           0,  0,  0,  0, 28,  0,  0,  0,-17,-21,-25, 17, 21,-25,
     8          31, 35,  3, 11, 20,  3, 11,-20,  0,  0,  0,  2, 10,  0,
     9          39, 43,  7, 15, 24,  7, 15,-24,  0,  0,  0,  6, 14,  0,
     1           0,  0,-18,-22,-27, 18, 22,-27,  0,  0,  0,  0,  0, 26,
     2          31, 35,  3, 11,-20,  3, 11, 20,  2, 10,  0,  0,  0,  0,
     3          39, 43,  7, 15,-24,  7, 15, 24,  6, 14,  0,  0,  0,  0,
     4           0,  0, 18, 22,-27,-18,-22,-27,  0,  0, 26,  0,  0,  0/
      DATA ISM2/7,8,4,5,6,4,5,6,1,2,3,1,2,3/
*
      RAYRE=COORD(1)
      A=COORD(2)
      B=COORD(3)
      ICARE=IZ(1).EQ.2
      SMALL=SIGT.LE.(TRONC*(A+B)/(2.0*A*B))
      AOB=A/B
      AOR=A/(2.0*PI*RAYRE)
      BOR=B/(2.0*PI*RAYRE)
      DO 10 I=1,44
   10 PBB(I)=0.0
      DO 20 I=1,8
   20 PVB(I)=0.0
      L=6
      NA=IZ(6)
      DO 170 IA=1,NA
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 70
      SI=Z(L+1)
      CO=Z(L+2)
      L=L+2
      DO 60 IX=1,NLX
      Z1=Z(L+1)
      Z2=Z1*CO
      Z3=Z1*SI
      Z4=Z3*CO
      Z5=Z3*SI
      KMAX=IZ(L+2)
      L=L+2
      IF (KMAX.EQ.1) THEN
*        THE TRACK DOES NOT PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         IF (POP.GE.XLIM3) GO TO 30
         IF (SMALL) THEN
            PVB(1)=PVB(1)+2.0*ZI20*Z(L+1)*Z1
            PVB(2)=PVB(2)+2.0*ZI30*Z(L+1)*Z3
            PVB(4)=PVB(4)+2.0*ZI20*Z(L+1)*Z1*AOB
            PVB(5)=PVB(5)+2.0*ZI30*Z(L+1)*Z2*AOB
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(1)=PBB(1)+WI3*Z1
         PBB(5)=PBB(5)+WI4*Z2
         PBB(9)=PBB(9)+WI4*Z3
         PBB(13)=PBB(13)+WI5*Z4
         PBB(21)=PBB(21)+WI5*Z5
         PBB(23)=PBB(23)+WI5*(Z1-Z5)
   30    L=L+1
      ELSE IF (KMAX.EQ.2) THEN
*        THE TRACK PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         ZMU=0.5*Z(L+2)/RAYRE
         IF (POP.GE.XLIM3) GO TO 40
         IF (SMALL) THEN
            PVB(1)=PVB(1)+2.0*ZI20*Z(L+1)*Z1
            PVB(2)=PVB(2)+2.0*ZI30*Z(L+1)*Z3
            PVB(7)=PVB(7)+4.0*ZI20*Z(L+1)*Z1*AOR
            PVB(8)=PVB(8)+4.0*ZI30*Z(L+1)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(29)=PBB(29)+2.0*WI3*Z1
         PBB(33)=PBB(33)+2.0*WI4*Z1*ZMU
         PBB(37)=PBB(37)+2.0*WI4*Z3
         PBB(41)=PBB(41)+2.0*WI5*Z3*ZMU
   40    POP=SIGT*Z(L+3)
         IF (POP.GE.XLIM3) GO TO 50
         IF (SMALL) THEN
            PVB(4)=PVB(4)+2.0*ZI20*Z(L+3)*Z1*AOB
            PVB(5)=PVB(5)+2.0*ZI30*Z(L+3)*Z2*AOB
            PVB(7)=PVB(7)+4.0*ZI20*Z(L+3)*Z1*AOR
            PVB(8)=PVB(8)+4.0*ZI30*Z(L+3)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(30)=PBB(30)+2.0*WI3*Z1*AOB
         PBB(34)=PBB(34)+2.0*WI4*Z1*AOB*ZMU
         PBB(38)=PBB(38)+2.0*WI4*Z2*AOB
         PBB(42)=PBB(42)+2.0*WI5*Z2*AOB*ZMU
   50    L=L+3
      ENDIF
   60 CONTINUE
*
   70 L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 130
      SI=Z(L+1)
      L=L+1
      DO 120 IX=1,NLX
      Z1=Z(L+1)
      Z2=Z1*SI
      Z3=Z2*SI
      KMAX=IZ(L+2)
      L=L+2
      IF (KMAX.EQ.1) THEN
*        THE TRACK DOES NOT PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         IF (POP.GE.XLIM3) GO TO 80
         IF (SMALL) THEN
            PVB(1)=PVB(1)+2.0*ZI20*Z(L+1)*Z1
            PVB(2)=PVB(2)+2.0*ZI30*Z(L+1)*Z2
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(2)=PBB(2)+2.0*WI3*Z1
         PBB(6)=PBB(6)+2.0*WI4*Z2
         PBB(14)=PBB(14)+2.0*WI5*Z3
         PBB(26)=PBB(26)+2.0*WI5*(Z1-Z3)
   80    L=L+1
      ELSE IF (KMAX.EQ.2) THEN
*        THE TRACK PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         ZMU=0.5*Z(L+2)/RAYRE
         IF (POP.GE.XLIM3) GO TO 90
         IF (SMALL) THEN
            PVB(1)=PVB(1)+ZI20*Z(L+1)*Z1
            PVB(2)=PVB(2)+ZI30*Z(L+1)*Z2
            PVB(7)=PVB(7)+2.0*ZI20*Z(L+1)*Z1*AOR
            PVB(8)=PVB(8)+2.0*ZI30*Z(L+1)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(29)=PBB(29)+WI3*Z1
         PBB(33)=PBB(33)+WI4*Z1*ZMU
         PBB(37)=PBB(37)+WI4*Z2
         PBB(41)=PBB(41)+WI5*Z2*ZMU
   90    POP=SIGT*Z(L+3)
         IF (POP.GE.XLIM3) GO TO 100
         IF (SMALL) THEN
            PVB(1)=PVB(1)+ZI20*Z(L+3)*Z1
            PVB(2)=PVB(2)+ZI30*Z(L+3)*Z2
            PVB(7)=PVB(7)+2.0*ZI20*Z(L+3)*Z1*AOR
            PVB(8)=PVB(8)+2.0*ZI30*Z(L+3)*Z1*AOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(29)=PBB(29)+WI3*Z1
         PBB(33)=PBB(33)+WI4*Z1*ZMU
         PBB(37)=PBB(37)+WI4*Z2
         PBB(41)=PBB(41)+WI5*Z2*ZMU
  100    L=L+3
      ENDIF
  120 CONTINUE
*
  130 IF (ICARE) GO TO 170
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 170
      SI=Z(L+1)
      L=L+1
      DO 165 IX=1,NLX
      Z1=Z(L+1)
      Z2=Z1*SI
      Z3=Z2*SI
      KMAX=IZ(L+2)
      L=L+2
      IF (KMAX.EQ.1) THEN
*        THE TRACK DOES NOT PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         IF (POP.GE.XLIM3) GO TO 140
         IF (SMALL) THEN
            PVB(4)=PVB(4)+2.0*ZI20*Z(L+1)*Z1
            PVB(5)=PVB(5)+2.0*ZI30*Z(L+1)*Z2
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(4)=PBB(4)+2.0*WI3*Z1
         PBB(8)=PBB(8)+2.0*WI4*Z2
         PBB(16)=PBB(16)+2.0*WI5*Z3
         PBB(28)=PBB(28)+2.0*WI5*(Z1-Z3)
  140    L=L+1
      ELSE IF (KMAX.EQ.2) THEN
*        THE TRACK PASS THROUGH THE CYLINDER.
         POP=SIGT*Z(L+1)
         ZMU=0.5*Z(L+2)/RAYRE
         IF (POP.GE.XLIM3) GO TO 150
         IF (SMALL) THEN
            PVB(4)=PVB(4)+ZI20*Z(L+1)*Z1
            PVB(5)=PVB(5)+ZI30*Z(L+1)*Z2
            PVB(7)=PVB(7)+2.0*ZI20*Z(L+1)*Z1*BOR
            PVB(8)=PVB(8)+2.0*ZI30*Z(L+1)*Z1*BOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(30)=PBB(30)+WI3*Z1
         PBB(34)=PBB(34)+WI4*Z1*ZMU
         PBB(38)=PBB(38)+WI4*Z2
         PBB(42)=PBB(42)+WI5*Z2*ZMU
  150    POP=SIGT*Z(L+3)
         IF (POP.GE.XLIM3) GO TO 160
         IF (SMALL) THEN
            PVB(4)=PVB(4)+ZI20*Z(L+3)*Z1
            PVB(5)=PVB(5)+ZI30*Z(L+3)*Z2
            PVB(7)=PVB(7)+2.0*ZI20*Z(L+3)*Z1*BOR
            PVB(8)=PVB(8)+2.0*ZI30*Z(L+3)*Z1*BOR*ZMU
         ENDIF
         K=NINT(POP*PAS3)
         WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
         WI4=BI4(K,1)+POP*(BI4(K,2)+POP*BI4(K,3))
         WI5=BI5(K,1)+POP*(BI5(K,2)+POP*BI5(K,3))
         PBB(30)=PBB(30)+WI3*Z1
         PBB(34)=PBB(34)+WI4*Z1*ZMU
         PBB(38)=PBB(38)+WI4*Z2
         PBB(42)=PBB(42)+WI5*Z2*ZMU
  160    L=L+3
      ENDIF
  165 CONTINUE
  170 CONTINUE
      IF (ICARE) THEN
         PBB(1)=PBB(1)+PBB(1)
         PBB(4)=PBB(2)
         PBB(5)=PBB(5)+PBB(9)
         PBB(8)=PBB(6)
         PBB(9)=PBB(5)
         PBB(13)=PBB(13)+PBB(13)
         PBB(16)=PBB(14)
         PBB(21)=PBB(21)+PBB(23)
         PBB(23)=PBB(21)
         PBB(28)=PBB(26)
         PBB(29)=PBB(29)+PBB(30)
         PBB(30)=PBB(29)
         PBB(33)=PBB(33)+PBB(34)
         PBB(34)=PBB(33)
         PBB(37)=PBB(37)+PBB(38)
         PBB(38)=PBB(37)
         PBB(41)=PBB(41)+PBB(42)
         PBB(42)=PBB(41)
         PVB(1)=PVB(1)+PVB(4)
         PVB(2)=PVB(2)+PVB(5)
         PVB(4)=PVB(1)
         PVB(5)=PVB(2)
         PVB(7)=PVB(7)+PVB(7)
         PVB(8)=PVB(8)+PVB(8)
      ENDIF
      PBB(10)=PBB(6)
      PBB(12)=PBB(8)
      PBB(17)=PBB(9)
      PBB(19)=PBB(5)
      PBB(25)=PBB(13)
      PBB(3)=PBB(1)*AOB
      PBB(7)=PBB(9)*AOB
      PBB(11)=PBB(5)*AOB
      PBB(15)=PBB(13)*AOB
      PBB(18)=PBB(19)*AOB
      PBB(20)=PBB(17)*AOB
      PBB(22)=PBB(23)*AOB
      PBB(24)=PBB(21)*AOB
      PBB(27)=PBB(25)*AOB
      PBB(31)=PBB(29)*AOR
      PBB(32)=PBB(30)*BOR
      PBB(35)=PBB(33)*AOR
      PBB(36)=PBB(34)*BOR
      PBB(39)=PBB(37)*AOR
      PBB(40)=PBB(38)*BOR
      PBB(43)=PBB(41)*AOR
      PBB(44)=PBB(42)*BOR
*
* ORTHONORMALIZATION.
      Z1=Z(2)
      Z2=Z(3)
      Z3=Z(4)
      Z4=Z(5)
      DO 190 I=1,4
      DEN0=PBB(I)
      DEN1=PBB(4+I)
      DEN2=PBB(8+I)
      PBB(I)=Z1*Z1*DEN0
      PBB(4+I)=Z1*(Z2*DEN1-Z3*DEN0)
      PBB(8+I)=Z1*(Z2*DEN2-Z3*DEN0)
      PBB(12+I)=Z2*Z2*PBB(12+I)-Z2*Z3*(DEN1+DEN2)+Z3*Z3*DEN0
  190 PBB(24+I)=Z4*Z4*PBB(24+I)
      DO 200 I=1,2
      DEN1=PBB(16+I)
      DEN2=PBB(18+I)
      PBB(16+I)=Z1*Z4*DEN1
      PBB(18+I)=Z1*Z4*DEN2
      PBB(20+I)=(Z2*PBB(20+I)-Z3*DEN1)*Z4
  200 PBB(22+I)=(Z2*PBB(22+I)-Z3*DEN2)*Z4
      DO 210 I=1,4
      DEN0=PBB(28+I)
      DEN1=PBB(32+I)
      DEN2=PBB(36+I)
      PBB(28+I)=Z1*Z1*DEN0
      PBB(32+I)=Z1*(Z2*DEN1-Z3*DEN0)
      PBB(36+I)=Z1*(Z2*DEN2-Z3*DEN0)
  210 PBB(40+I)=Z2*Z2*PBB(40+I)-Z2*Z3*(DEN1+DEN2)+Z3*Z3*DEN0
      PVB(2)=Z1*(Z2*PVB(2)-Z3*PVB(1))
      PVB(1)=Z1*Z1*PVB(1)
      PVB(5)=Z1*(Z2*PVB(5)-Z3*PVB(4))
      PVB(4)=Z1*Z1*PVB(4)
      PVB(8)=Z1*(Z2*PVB(8)-Z3*PVB(7))
      PVB(7)=Z1*Z1*PVB(7)
*
      IF (.NOT.SMALL) THEN
         PVB(1)=1.0-2.0*PBB(1)-PBB(2)-PBB(29)
         PVB(2)=-2.0*PBB(9)-PBB(10)-PBB(37)
         PVB(4)=1.0-2.0*PBB(3)-PBB(4)-PBB(30)
         PVB(5)=-2.0*PBB(11)-PBB(12)-PBB(38)
         PVB(7)=1.0-2.0*PBB(31)-2.0*PBB(32)
         PVB(8)=-2.0*PBB(35)-2.0*PBB(36)
         DO 230 I=1,8
  230    PVB(I)=PVB(I)/SIGT
      ENDIF
      VOL=A*B-PI*RAYRE**2
      PVB(1)=0.25*PVB(1)*A/VOL
      PVB(2)=0.25*PVB(2)*A/VOL
      PVB(4)=0.25*PVB(4)*B/VOL
      PVB(5)=0.25*PVB(5)*B/VOL
      PVB(7)=0.25*PVB(7)*(2.0*PI*RAYRE)/VOL
      PVB(8)=0.25*PVB(8)*(2.0*PI*RAYRE)/VOL
      IF (IAPP.EQ.1) THEN
         DO 240 I=1,13
         PVS(I)=PVB(ISM1(I))
         DO 240 J=1,13
         IB=ISL1(I,J)
         PSS(I,J)=0.0
         IF (IB.GT.0) THEN
            PSS(I,J)=PBB(IB)
         ELSE IF (IB.LT.0) THEN
            PSS(I,J)=-PBB(-IB)
         ENDIF
  240    CONTINUE
      ELSE IF (IAPP.EQ.2) THEN
         DO 250 I=1,14
         PVS(I)=PVB(ISM2(I))
         DO 250 J=1,14
         IB=ISL2(I,J)
         PSS(I,J)=0.0
         IF (IB.GT.0) THEN
            PSS(I,J)=PBB(IB)
         ELSE IF (IB.LT.0) THEN
            PSS(I,J)=-PBB(-IB)
         ENDIF
  250    CONTINUE
      ENDIF
      IF (SIGT.GT.0.0) THEN
         PVV=(1.0-PVS(1)-2.0*PVS(IAPP+1)-2.0*PVS(IAPP+7))/SIGT
      ELSE
         PVV=1.0E10
      ENDIF
      RETURN
      END
