*DECK FLU
      SUBROUTINE FLU(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*----------------------------FLU----------------------------------------
*   1- PROGRAMME STATISTICS:
*       NAME     : FLU
*       USE      : MULTIGROUP FLUX SOLUTION MODULE FOR DRAGON.
*       MODIFIED : CALL ROUTINES FLUGPI AND FLUDRV
*                  2000/01/20 (T. COURAU)
*       AUTHOR   : A. HEBERT  1994/10/26
*
*   2- ROUTINE PARAMETERS:
*      INPUT/OUTPUT PARAMETERS:
*      NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*      IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*           =4 SEQUENTIAL ASCII FILE.
*      JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*      KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
*           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*           KENTRY(NENTRY)
*   3- LINKED LIST / XSM FILE:  (ORDER IS ARBITRARY FOR 2,3,4,5)
*     HENTRY(1) : CREATE OR MODIFICATION TYPE(L_FLUX).
*     HENTRY(2) : READ-ONLY TYPE(L_PIJ).
*     HENTRY(3) : READ-ONLY TYPE(L_MACROLIB OR L_LIBRARY).
*     HENTRY(4) : READ-ONLY TYPE(L_TRACK).
*     HENTRY(5) : READ-ONLY OPTIONAL BINARY TRACKING FILE.
*
*----------------------------FLU----------------------------------------
*
      IMPLICIT     NONE
      INTEGER      IOUT,NLEAK,NSDIR,NSTATE,NBEPS,NCALCA
      CHARACTER    NAMSBR*6
      REAL         ROUMIN,RINMIN
      PARAMETER   (IOUT=6,NLEAK=5,NSDIR=6,NSTATE=40,NBEPS=5,
     >             NCALCA=3,ROUMIN=1.0E-5,RINMIN=5.0E-5,
     >             NAMSBR='FLU   ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      NENTRY
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IMAT,IVOL,IIDL,JFUNKN,JSUNKN
C----
C  LOCAL PARAMETERS
C----
      INTEGER     IKSYS,IKTRK,IKLIB,IKBTF,IPSYS,IPTRK,IPLIB,IPBTF,
     >            IPFLUX,ISTATE(NSTATE),KCHAR(18),ICOPAR(6),
     >            ICALCA(NCALCA),NREG,IPRINT,NGROUP,ILEAK,
     >            ITYPEC,ISADJ,NUNKNO,IFFLU,
     >            ITRANC,NIFISS,MXNL,NBMIX,IPHASE,ISCATR,
     >            ITPIJ,ITC,IEN,ILCMTY,ILCMLN,INITFL,
     >            NBFLUC,IFLUC,ISADJC,ITYCC
      CHARACTER   TEXT12*12,TITLE*72,CMODUL*12,HSIGN*12,
     >            CXDOOR*12,COPTIO*4,CERROR*131
      LOGICAL     LEAKSW
      REAL        B2(4),EPSPAR(NBEPS)
      DOUBLE PRECISION REFKEF
      INTEGER     IFSUR
C----
C  DATA
C----
      CHARACTER   HTYPE(6)*8,CLEAK(NLEAK)*4,CSDIR(NSDIR)*1,
     >            NAMGPA(2)*12,NAMGAS*12
      SAVE        HTYPE,CLEAK,CSDIR,NAMGPA
      DATA        HTYPE
     >           /'   K-INF','   K-EFF','   K-EFF',
     >            'BUCKLING',' LEAKAGE','  SOURCE'/
      DATA        CLEAK
     >           /'PNLR','PNL','SIGS','ALBS','HETE'/
      DATA        CSDIR
     >           /'X','Y','Z','R','G','P'/
      DATA        NAMGPA
     >           /'PSEUDOGENADJ','FLUXGENADJ  '/
      DATA        NAMGAS
     >           /'FIXG      '/
C----
C  PARAMETER VALIDATION.
C----
      IF(NENTRY .LT. 4) CALL XABORT(NAMSBR//
     >  ': AT LEAST FOUR DATA STRUCTURES REQUIRED')
      IF(IENTRY(1) .NE. 1 .AND. IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >  ': FIRST DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IPFLUX=KENTRY(1)
      IF(JENTRY(1) .EQ. 0) THEN
        HSIGN='L_FLUX'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPFLUX,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPFLUX,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN .NE. 'L_FLUX') THEN
          CALL XABORT(NAMSBR//
     >      ': SIGNATURE OF '//HENTRY(1)//' IS '//HSIGN//
     >      '. L_FLUX EXPECTED.')
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//
     >    ': FLUX STRUCTURE IN READ-ONLY MODE FORBIDDEN')
      ENDIF
C----
C  SCAN READ-ONLY MODE DATA STRUCTURE
C  FOR PIJ, TRACK AND LIB AND BINARY TRACKING FILE
C----
      IKSYS=0
      IKTRK=0
      IKLIB=0
      IKBTF=0
      IPSYS=0
      IPTRK=0
      IPLIB=0
      IPBTF=0
      DO 100 IEN=2,NENTRY
        TEXT12=HENTRY(IEN)
        IF(JENTRY(IEN) .NE. 2) CALL XABORT(NAMSBR//
     >    ': DATA STRUCTURE '//TEXT12//' NOT IN READ-ONLY MODE')
        IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF(HSIGN .EQ. 'L_PIJ' .AND. IKSYS .EQ. 0) THEN
            IPSYS=KENTRY(IEN)
            IKSYS=IEN
          ELSE IF(HSIGN .EQ. 'L_TRACK' .AND. IKTRK .EQ. 0) THEN
            IPTRK=KENTRY(IEN)
            IKTRK=IEN
          ELSE IF(HSIGN .EQ. 'L_LIBRARY' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=IEN
          ELSE IF(HSIGN .EQ. 'L_MACROLIB' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=-IEN
          ENDIF
        ELSE IF(IENTRY(IEN) .EQ. 3 .AND. IKBTF .EQ. 0) THEN
          IPBTF=KENTRY(IEN)
          IKBTF=IEN
        ENDIF
 100  CONTINUE
C----
C  TEST FOR MISSING STRUCTURE (STRUCTURE ABSOLUTELY REQUIRED)
C----
      IF(IKSYS .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NO DATA STRUCTURE WITH SIGNATURE L_PIJ FOUND.')
      IF(IKTRK .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NO DATA STRUCTURE WITH SIGNATURE L_TRACK FOUND.')
      IF(IKLIB .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NO DATA STRUCTURE WITH SIGNATURE '//
     >   'L_LIBRARY OR L_MACROLIB FOUND.')
C----
C  RECOVER IPTRK INFORMATION.
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      CALL LCMGET(IPTRK,'TRACK-TYPE  ',KCHAR)
      WRITE(CMODUL,'(3A4)') (KCHAR(ITC),ITC=1,3)
      NREG=ISTATE(1)
      NUNKNO=ISTATE(2)
      LEAKSW=ISTATE(3).EQ.0
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IIDL)
      CALL LCMGET(IPTRK,'MATCOD      ',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME      ',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX      ',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE       ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN
        CALL LCMGET(IPTRK,'TITLE',KCHAR)
        WRITE(TITLE,'(18A4)') (KCHAR(ITC),ITC=1,18)
      ELSE
        TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
C----
C  RECOVER IPSYS INFORMATION.
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPSYS,'STATE-VECTOR',ISTATE)
      IPHASE=ISTATE(1)
      ISCATR=ISTATE(2)
      NGROUP=ISTATE(7)
      NBMIX=ISTATE(9)
      IF(LEAKSW) THEN
        IF(ISTATE(4) .EQ. 0) LEAKSW=.FALSE.
      ENDIF
      IFSUR=ISTATE(16)
C----
C  RECOVER MACROLIB INFORMATION.
C----
      IF(IKLIB .GT. 0) CALL LCMSIX(IPLIB,'MACROLIB',1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      IF(NBMIX .GT. ISTATE(2)) CALL XABORT(NAMSBR//
     > ': NUMBER OF MIXTURE IN LIBRARY IS '//
     > 'SMALLER THAN NUMBER OF MIXTURE IN GEOMETRY.')
      NBMIX=ISTATE(2)
      MXNL=ISTATE(3)
      NIFISS=ISTATE(4)
      ITRANC=ISTATE(6)
C----
C  COMPUTE THE FLUX.
C----
      CXDOOR='TRAFIC'
      IF(IPHASE .EQ. -1) THEN
        CXDOOR=CMODUL
        ITPIJ=ISCATR+1
      ELSE
        ITPIJ=2*IPHASE+ISCATR+1
      ENDIF
      IF(NREG .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NREG = 0')
      CALL SETARA(RBASE,NUNKNO*NGROUP,JFUNKN)
      CALL SETARA(RBASE,NUNKNO*NGROUP,JSUNKN)
C----
C  READ FLUX STATE VECTOR IF REQUIRED AND TEST FOR COMPATIBILITY
C---- 
      CALL XDISET(ISTATE,NSTATE,0)
      IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPFLUX,'STATE-VECTOR',ISTATE)
        ITYPEC=ISTATE(1)
        ILEAK=ISTATE(2)
        ICOPAR(1)=ISTATE(3)
        ICOPAR(2)=ISTATE(4)
        ICOPAR(3)=ISTATE(5)
        ICOPAR(4)=ISTATE(6)
        ICOPAR(5)=ISTATE(7)
        ICOPAR(6)=ISTATE(11)
        ISADJ=ISTATE(8)
        CALL LCMGET(IPFLUX,'EPS-CONVERGE',EPSPAR)
C----
C  USE FLUX FROM IPFLUX FOR INITIALIZATION OPTION ONLY IF
C  CURRENT FLUX DIMENSIONS COMPATIBLE WITH
C  DIMENSIONS PROVIDED BY SYSTEM MATRIX
C  OTHERWISE USE DEFAULT INITIALIZATION PROCEDURE
C----
        IF(NGROUP .EQ. ISTATE(9) .AND.
     >     NUNKNO .EQ. ISTATE(10) ) THEN
          INITFL=2
        ELSE
          INITFL=0
        ENDIF
      ELSE
        ITYPEC=-99
        ILEAK=0
        IF(ITPIJ .GE. 3) ILEAK=5
        ICOPAR(1)=3
        ICOPAR(2)=3
        ICOPAR(3)=1
        ICOPAR(4)=0
        ICOPAR(5)=0
        ICOPAR(6)=0
        ISADJ=0
        EPSPAR(1)=RINMIN
        EPSPAR(2)=ROUMIN
        EPSPAR(3)=0.0
        EPSPAR(4)=10.0
        EPSPAR(5)=100.0
        INITFL=0
      ENDIF
      CALL FLUGET(IPFLUX,LEAKSW,NGROUP,NREG  ,NUNKNO,ITPIJ ,
     >            ITYPEC,ISADJ ,ICOPAR,EPSPAR,COPTIO,ILEAK ,
     >            IPRINT,INITFL,B2    ,REFKEF,
     >            IBASE(IIDL),RBASE(JFUNKN))
      IF( ILEAK  .GE. 5 .AND.
     >    ITYPEC .GE. 2 .AND.
     >    ITYPEC .LE. 4       ) THEN
        IF(ITRANC .EQ. 0) THEN
          IF(COPTIO .EQ. 'B0TR' .OR. COPTIO .EQ. 'LKRD') THEN
            CERROR=' '
            WRITE(CERROR,9000) NAMSBR,COPTIO
            CALL XABORT(CERROR)
          ENDIF
        ENDIF
      ENDIF
      IF( ITYPEC .GE. 2 .AND.
     >    ITYPEC .LE. 4       .AND.
     >    ILEAK  .GE. 5) THEN
        IF(NUNKNO .LT. 4*NREG) CALL XABORT(NAMSBR//
     >  ': SET ANIS = 2 IN EXCELT: WITH PIJK CALCULATION')
      ENDIF
C----
C  BY DEFAULT AN ADJOINT CALCULATION IMPLIES
C  A DIRECT CALCULATION AND A GENERALIZED ADJOINT
C  CALCULATION IMPLIES BOTH A DIRECT AND ADJOINT
C  CALCULATION
C  -> IFFLU =1
C  IF ONE INTENDS TO TREAT ALL THESE CALCULATIONS
C  INDIVIDUALLY UNCOMMENT LINE
C  -> IFFLU =2 IN ADJOINT CALCULATION
C  AND LINE
C  -> IFFLU =3 IN GENERALIZED ADJOINT CALCULATION
C----
      IFFLU=1
      IF(ISADJ .EQ. 0) THEN
C----
C  ONLY FLUX CALCULATION CONSIDERED
C----
        NBFLUC=1
        ICALCA(1)=0
      ELSE IF(ISADJ .LE. 2) THEN
C----
C  FLUX AND ADJOINT CALCULATION CONSIDERED
C----
C        IFFLU=2
        NBFLUC=2
        ICALCA(1)=0
        ICALCA(2)=ISADJ
      ELSE
C----
C  FLUX AND ADJOINT AND GENERALIZED CALCULATION CONSIDERED
C----
C        IFFLU=3
        NBFLUC=3
        ICALCA(1)=0
        ICALCA(2)=1+MOD(ISADJ+1,2)
        ICALCA(3)=ISADJ
      ENDIF
      DO 110 IFLUC=IFFLU,NBFLUC
        ISADJC=ICALCA(IFLUC)
        ITYCC=ITYPEC
        IF(ISADJC .GE. 3) THEN
          IF (ITYPEC .EQ. 6 ) CALL XABORT(NAMSBR//'GENERALIZED ADJOINT'
     >    //'COMPUTAION FOR TYPE S CASE NOT AVAILABLE')
        ELSE IF(ISADJC .GE. 1) THEN
          IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5) THEN
            ITYCC=2
          ENDIF
        ENDIF
        IF(IPRINT .GE. 1 .AND. ITYCC .GT. 0) THEN
          WRITE(IOUT,6010) HTYPE(ITYCC)
          IF(ISADJC .GT. 0 ) THEN
            WRITE(IOUT,6014)
          ENDIF
          IF(ISADJC .GE. 3) THEN
            WRITE(IOUT,6016)
          ELSE IF(ITYCC .EQ. 6) THEN
            WRITE(IOUT,6015)
          ELSE IF(ITYCC .GE. 4) THEN
            IF(ILEAK .LT. 5) THEN
              WRITE(IOUT,6011) COPTIO,CLEAK(ILEAK),'G SEARCH'
              WRITE(IOUT,6012) B2(4)
            ELSE
              WRITE(IOUT,6011) COPTIO,CLEAK(5),
     >           CSDIR(ILEAK-5)//' SEARCH'
              WRITE(IOUT,6013) B2(1),B2(2),B2(3)
            ENDIF
          ELSE IF(ITYCC .GE. 2) THEN
            WRITE(IOUT,6011) COPTIO,CLEAK(ILEAK),' IMPOSED'
            IF(ILEAK .LT. 5) THEN
              WRITE(IOUT,6012) B2(4)
            ELSE
              WRITE(IOUT,6013) B2(1),B2(2),B2(3)
            ENDIF
          ENDIF
        ENDIF
C----
C  CALL FLUX CALCULATION MODULE
C----
         CALL FLUDRV(IPFLUX,IPSYS ,IPLIB ,IPTRK ,IPBTF ,ITPIJ ,
     >               CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC,ISADJC,
     >               ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREG  ,
     >               NUNKNO,NIFISS,MXNL  ,INITFL,B2    ,REFKEF,
     >               TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >               RBASE(JFUNKN),RBASE(JSUNKN),IBASE(IMAT)  ,
     >               RBASE(IVOL)  ,IBASE(IIDL)  )
 110  CONTINUE
      CALL LCMPUT(IPFLUX,'EPS-CONVERGE',NBEPS,2,EPSPAR)
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=ITYPEC
      ISTATE(2)=ILEAK
      ISTATE(3)=ICOPAR(1)
      ISTATE(4)=ICOPAR(2)
      ISTATE(5)=ICOPAR(3)
      ISTATE(6)=ICOPAR(4)
      ISTATE(7)=ICOPAR(5)
      ISTATE(11)=ICOPAR(6)
      ISTATE(8)=ISADJ
      ISTATE(9)=NGROUP
      ISTATE(10)=NUNKNO
      ISTATE(12)=IFSUR
      CALL LCMPUT(IPFLUX,'STATE-VECTOR',NSTATE,1,ISTATE)
      IF(IPRINT .GE. 2) THEN
        WRITE(IOUT,6000) NAMSBR,IPRINT,10,(ISTATE(ITC),ITC=1,10)
      ENDIF
      CALL RLSARA(RBASE(JSUNKN))
      CALL RLSARA(RBASE(JFUNKN))
C----
C  RELEASE GENERAL TRACKING INFORMATION.
C----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      IF(IKLIB.GT.0) CALL LCMSIX(IPLIB,'MACROLIB',2)
      RETURN
C----
C  FORMATS:
C----
 6000 FORMAT(/1X,A6,' PRINT LEVEL =',I8/
     > ' FLUXUNK STATE-VECTOR  (',I2,') ELEMENTS'/1X,17('-')/
     > ' ITYPEC =',I8,' (TYPE OF FLUX CALCULATION)'/
     > ' ILEAK  =',I8,' (LEAKAGE OPTION)'/
     > ' IFREE  =',I8,' (NUMBER OF FREE ITERATIONS)'/
     > ' IACCE  =',I8,' (NUMBER OF ACCELERATED ITERATIONS)'/
     > ' IREBAL =',I8,' (REBALANCING OPTION)'/
     > ' MAXTH  =',I8,' (MAXIMUM NUMBER OF THERMAL ITERATIONS)'/
     > ' MAXOUT =',I8,' (MAXIMUM NUMBER OF OUTER ITERATIONS)'/
     > ' ISADJ  =',I8,' (ADJOINT CALCULATION OPTION)'/
     > ' NGROUP =',I8,' (NUMBER OF GROUPS)'/
     > ' NUNKNO =',I8,' (NUMBER OF UNKNOWNS)'/1X,17('-')/)
 6010 FORMAT(////1X,'SOLUTION TO TRANSPORT EQUATION ',/
     > ' CALCULATION TYPE            =',2X,A8)
 6011 FORMAT(
     > ' LEAKAGE TYPE                =',6X,A4/
     > ' LEAKAGE OPTION              =',6X,A4/
     > ' BUCKLING                    =',2X,A8)
 6012 FORMAT(
     > ' INITIAL TOTAL BUCKLING      =',E13.5)
 6013 FORMAT(
     > ' INITIAL BUCKLING - X        =',E13.5/
     > ' INITIAL BUCKLING - Y        =',E13.5/
     > ' INITIAL BUCKLING - Z        =',E13.5)
 6014 FORMAT(
     > ' PROBLEM TYPE                =   ADJOINT')
 6015 FORMAT(
     > ' SOURCE TYPE                 =  EXTERNAL')
 6016 FORMAT(
     > ' SOURCE TYPE                 =   GEN-ADJ')
C----
C  ERROR FORMAT
C----
 9000 FORMAT(A6,': ILLEGAL OPTION ',A4,
     >' FOR HETEROGENEOUS LEAKAGE WITHOUT TRANSPORT CORRECTION')
      END
