*DECK FLUBAL
      SUBROUTINE FLUBAL(IPSYS ,IPMACR,IPRINT,ITPIJ ,KDIR  ,NGROUP,
     >                  NBMIX ,NREGIO,NUNKNO,LEAKSW,ILEAK ,MATCOD,
     >                  VOLUME,KEYFLX,FUNKNO,SUNKNO,ISADJ ,IGDEB ,
     >                  ILASTG,DB2   ,SIGT0 ,SIGS0 ,XSCAT ,IJJ   ,
     >                  NJJ   ,IPOS  ,
     >                  REBAL ,PIS   )
C
C---------------------------  FLUBAL  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUBAL
C       USE      : FLUX REBALANCING FOR NON CONVERGED GROUPS WITH
C                  UP-SCATTERING
C       MODIFIED : DEAL WITH ADJOINT FLUX
C                  2000/02/01 (G. MARLEAU)
C                  2000/01/20 (G. MARLEAU T. COURAU)
C                  1991/03/15 (G. MARLEAU)
C                  1991/01/28 (A. HEBERT)
C                  1986/05/14 (G. MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPMACR  : POINTER TO THE MACROLIB              I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES:    I
C               = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      ITPIJ   : TYPE OF CP AVAILABLE                 I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      KDIR    : DIRECTION FOR PC                     I
C                = 0 FOR PIJ OR WIJ
C                = K FOR PIJK OR WIJK K=1,2,3 B1
C                = K+3 FOR PIJK OR WIJK K=4,5,6 B0
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      LEAKSW  : LEAKAGE SWITCH.                      L
C      ILEAK   : METHOD USED TO INCLUDE DB2 EFFECT    I
C                = 0 NO DB2 EFFECT
C                = 1 THE SCATTERING MODIFIED CP MATRIX
C                    IS MULTIPLIED BY A PNL FACTOR
C                = 2 THE REDUCED CP MATRIX IS MULTIPLIED BY
C                    PNL FACTOR
C                = 3 SIGS0-DB2 APPROXIMATION
C                = 4 ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR SEARCH
C                  X->6 Y->7 Z->8C R->9 ALL->10
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION         R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.     I(NREGIO)
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.        R(NUNKNO,NGROUP)
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION  I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      IGDEB   : FIRST GROUP TO PROCESS               I
C      ILASTG  : LAST GROUP TO PROCESS                I
C      DB2     : LEAKAGE PARAMETERS.                  R(NGROUP)
C                FOR ITPIJ=1,2 -> DB2(G)=D(G)*B**2
C                FOR ITPIJ=3,4 -> DB2(G)=LAMBDA(B/SIGMA(G))
C      SIGT0   : TOTAL CROSS SECTION VECTOR           R(0:NBMIX)
C                SIGT0(0)=0.0 IS FOR VOID
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      XSCAT   : SCATTERING VECTOR                    R(0:NBMIX*
C                XSCAT(0) = 0.0                          NGROUP)
C      IJJ     : LAST SCATTERING GROUP                I(0:NBMIX)
C                IJJ(0) = 0
C      NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C      REBAL   : REBALANCING MATRIX                   R(NGROUP,
C                                                     NGROUP+1)
C      PIS     : LEAKAGE MATRIX                       R(NREGIO)
C---------------------------   FLUBAL  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUBAL')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPSYS,IPMACR,IPRINT,ITPIJ,KDIR,NGROUP,NBMIX,
     >            NREGIO,NUNKNO,ILEAK,ISADJ,IGDEB,ILASTG
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO),
     >            IJJ(0:NBMIX),NJJ(0:NBMIX),IPOS(0:NBMIX)
      LOGICAL     LEAKSW
      REAL        VOLUME(NREGIO),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),DB2(NGROUP),
     >            SIGT0(0:NBMIX),SIGS0(0:NBMIX),XSCAT(0:NBMIX*NGROUP),
     >            REBAL(NGROUP,NGROUP+1),PIS(NREGIO)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CAN*3,CNSIGW*12
      INTEGER     LONS0,IDIR,KRSK,ISTART,ISTOP,NGREB,IGR,JGR,IOFF,
     >            ILCMLN,ILCMTY,IREG,IBM,IRLOC,IER,
     >            NGSCAT,IFSCAT,ISCATP,ILSCAT,IFTMP,ILTMP
      REAL        FACT,FACTD,FACTS,FACTDS
C----
C  FOR DIRECTIONS KDIR=0 USE ISOTROPIC SCATTERING
C  FOR DIRECTIONS KDIR=1,2,3 USE ANISOTROPIC SCATTERING B1
C  FOR DIRECTIONS KDIR=4,5,6 USE ANISOTROPIC SCATTERING B0
C  KRSK = LOCATION OF FIRST ELEMENT FOR VARIOUS DIRECTION
C----
      IF(ISADJ .EQ. 0) THEN
        CAN='D 0'
      ELSE
        CAN='A 0'
      ENDIF
      IF(KDIR .EQ. 0) THEN
        CNSIGW='DRAGON-S0XSC'
        LONS0=NBMIX+1
        IDIR=0
      ELSE IF(KDIR .GT. 3) THEN
        IDIR=KDIR-3
      ELSE
        CAN(3:3)='1'
        CNSIGW='SIGW 1'
        LONS0=NBMIX
        IDIR=KDIR
      ENDIF
      KRSK=IDIR*NREGIO
      ISTART=MIN(IGDEB,ILASTG)
      ISTOP= MAX(IGDEB,ILASTG)
C---- 
C  MODIFICATION FOR GROUPS WITH VANISHING FLUX
C  DO NO REBALANCE THESE GROUPS
C----
      IFTMP=ISTART
      ILTMP=ISTOP
      DO 200 IGR=ISTART,ISTOP
        DO 210 IREG=1,NREGIO
          IRLOC=KRSK+KEYFLX(IREG)
          IF(FUNKNO(IRLOC,IGR) .NE. 0.0) THEN
            GO TO 215
          ENDIF
 210    CONTINUE
        IF(IGR .EQ. IFTMP) THEN
          IFTMP=IFTMP+1
        ELSE
          ILTMP= IGR-1
          GO TO 205
        ENDIF
 215    CONTINUE
 200  CONTINUE
 205  CONTINUE
      IF(IFTMP .GT. ISTART .OR. ILTMP .LT. ISTOP) THEN
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,8000) NAMSBR,ISTART,ISTOP,IFTMP,ILTMP 
        ENDIF
        ISTART=IFTMP
        ISTOP=ILTMP
      ENDIF
      IF(ISTART .GT. ISTOP) THEN
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,8001) NAMSBR
        ENDIF
        RETURN
      ENDIF
C----
C  INITIALIZE REBALANCE MATRIX
C----
      NGREB=ISTOP-ISTART+1
      DO 100 IGR=1,NGREB
        DO 101 JGR=1,NGREB+1
          REBAL(IGR,JGR)=0.0
 101    CONTINUE
 100  CONTINUE
C----
C  CREATE REBALANCE MATRIX
C----
      DO 110 IGR=ISTART,ISTOP
        IOFF=IGR-ISTART+1
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMLEN(IPSYS,'DRAGON-TXSC',ILCMLN,ILCMTY)
        IF(ILCMLN .NE. NBMIX+1) THEN
          CALL LCMLIB(IPSYS)
          CALL XABORT(NAMSBR//
     >    ': READ ERROR ON LCM RECORD = DRAGON-TXSC')
        ELSE
          CALL LCMGET(IPSYS,'DRAGON-TXSC',SIGT0)
        ENDIF
        CALL LCMLEN(IPSYS,'DRAGON-PIS',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NREGIO) THEN
          CALL LCMGET(IPSYS,'DRAGON-PIS',PIS)
        ELSE
          CALL XDRSET(PIS,NREGIO,0.0)
        ENDIF
        IF(KDIR .LE. 3) THEN
C----
C  READ TOTAL+SCATT X-SECTIONS.
C----
          IF(CNSIGW .EQ. 'DRAGON-S0XSC') THEN
            CALL LCMLEN(IPSYS,CNSIGW,ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. LONS0) THEN
              CALL LCMGET(IPSYS,CNSIGW,SIGS0(0))
            ELSE
              CALL LCMLIB(IPSYS)
              CALL XABORT(NAMSBR//
     >        ': READ ERROR ON LCM RECORD = '//CNSIGW)
            ENDIF
          ELSE
            CALL LCMLEN(IPMACR,CNSIGW,ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. LONS0) THEN
              CALL LCMGET(IPMACR,CNSIGW,SIGS0(1))
            ELSE
              CALL LCMLIB(IPMACR)
              CALL XABORT(NAMSBR//
     >        ': READ ERROR ON LCM RECORD = '//CNSIGW)
            ENDIF
            SIGS0(0)=0.0
          ENDIF
          CALL LCMLEN(IPMACR,'NJJ'//CAN,ILCMLN,ILCMTY)
          IF(ILCMLN .NE. NBMIX) THEN
            CALL LCMLIB(IPMACR)
            CALL XABORT(NAMSBR//
     >      ': READ ERROR ON LCM RECORD = NJJ'//CAN)
          ELSE
            CALL LCMGET(IPMACR,'NJJ'//CAN,NJJ(1))
            CALL LCMGET(IPMACR,'IJJ'//CAN,IJJ(1))
            CALL LCMGET(IPMACR,'IPO'//CAN,IPOS(1))
            CALL LCMGET(IPMACR,'SCA'//CAN,XSCAT(1))
          ENDIF
        ENDIF
C----
C  FIND FACT FOR ALL CASES
C  ITPIJ=1,2,3,4 AND IDIR=0  -> FACT=1.0
C  ITPIJ=3,4 AND IDIR=1,2,3  -> FACT=LAMBDA (LAMBDA=DB2(IGR))
C----
        IF(ILEAK .GE. 5 .AND. IDIR .GT. 0) THEN
          FACT=DB2(IGR)
          IF(ITPIJ .EQ. 4) THEN
            FACTD=FACT
          ELSE
            FACTD=1.0
          ENDIF
        ELSE
          FACT=1.0
          FACTD=1.0
        ENDIF
C----
C  SUM OVER REGIONS
C----
        DO 120 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IRLOC=KRSK+KEYFLX(IREG)
          FACTS=(1-PIS(IREG))*FACT
          FACTDS=(1-PIS(IREG))*FACTD
C----
C  FIXE + FISSION NEUTRON SOURCES
C----
          REBAL(IOFF,NGREB+1)=REBAL(IOFF,NGREB+1)
     >      +FACTS*SUNKNO(IRLOC,IGR)*VOLUME(IREG)
C----
C  INCLUDE SCATTERING SOURCES FROM CONVERGED
C  FLUX IN REBALANCE SOURCE
C----
          IF(KDIR .LE. 3) THEN
            NGSCAT=NJJ(IBM)
            IFSCAT=IJJ(IBM)-NGSCAT+1
            ISCATP=IPOS(IBM)-1+NGSCAT
            ILSCAT=IJJ(IBM)
            DO 121 JGR=IFSCAT,ISTART-1
              REBAL(IOFF,NGREB+1)=REBAL(IOFF,NGREB+1)+FACTS
     >          *FUNKNO(IRLOC,JGR)*XSCAT(ISCATP)*VOLUME(IREG)
              ISCATP=ISCATP-1
 121        CONTINUE
            ISCATP=IPOS(IBM)+ILSCAT-ISTOP-1
            DO 122 JGR=ISTOP+1,ILSCAT
              REBAL(IOFF,NGREB+1)=REBAL(IOFF,NGREB+1)+FACTS
     >          *FUNKNO(IRLOC,JGR)*XSCAT(ISCATP)*VOLUME(IREG)
              ISCATP=ISCATP-1
 122        CONTINUE
C----
C  INCLUDE SCATTERING SOURCES FROM NON CONVERGED
C  FLUX IN REBALANCE MATRIX
C----
            IF(IFSCAT .LT. ISTART) THEN
              NGSCAT=NGSCAT+IFSCAT-ISTART
              IFSCAT=ISTART
            ENDIF
            IF(ILSCAT .GT. ISTOP) THEN
              ILSCAT=ISTOP
            ENDIF
            ISCATP=IPOS(IBM)-1+NGSCAT
            DO 123 JGR=IFSCAT,ILSCAT
              IF(JGR .EQ. IGR) THEN
                REBAL(IOFF,IOFF)=REBAL(IOFF,IOFF)+FUNKNO(IRLOC,IGR)
     >            *(SIGT0(IBM)-FACTDS*SIGS0(IBM))*VOLUME(IREG)
              ELSE
                REBAL(IOFF,JGR-ISTART+1)=REBAL(IOFF,JGR-ISTART+1)
     >          -FACTS*FUNKNO(IRLOC,JGR)*XSCAT(ISCATP)*VOLUME(IREG)
              ENDIF
              ISCATP=ISCATP-1
 123        CONTINUE
          ELSE
            REBAL(IOFF,IOFF)=REBAL(IOFF,IOFF)+FUNKNO(IRLOC,IGR)
     >          *SIGT0(IBM)*VOLUME(IREG)
          ENDIF
 120    CONTINUE
C----
C  FOR ALL REGIONS ADD CONTRIBUTION DUE TO DB2 TERM
C  IF REQUIRED (ITPIJ < 3)
C----
        IF(ILEAK .LT. 5 .AND. ILEAK .GT. 0) THEN
          DO 124 IREG=1,NREGIO
            IRLOC=KRSK+KEYFLX(IREG)
            REBAL(IOFF,IOFF)=REBAL(IOFF,IOFF)
     >         +FUNKNO(IRLOC,IGR)*DB2(IGR)*VOLUME(IREG)
 124      CONTINUE
        ENDIF
        CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 110  CONTINUE
C----
C  SOLVE REBALANCE EQUATIONS
C----
      CALL ALSB(NGREB,1,REBAL,IER,NGROUP)
      IF(IER .NE. 0) THEN
C----
C  REBALANCING MATRIX IS SINGULAR
C  DO NOT REBALANCE
C---- 
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,8002) NAMSBR
        ENDIF
        RETURN
      ENDIF
C----
C  REBALANCE FLUXES
C----
      DO 130 IGR=ISTART,ISTOP
        IOFF=IGR-ISTART+1
        DO 131 IREG=1,NREGIO
          IRLOC=KRSK+KEYFLX(IREG)
          FUNKNO(IRLOC,IGR)=FUNKNO(IRLOC,IGR)*REBAL(IOFF,NGREB+1)
 131    CONTINUE
 130  CONTINUE
      RETURN
C----
C  FORMATS
C----
 8000 FORMAT(1X,A6,': WARNING -> FLUX IN SOME GROUPS VANISHES '/
     >       1X,'INITIAL REBALANCING GROUPS : FROM ',I5,' TO ',I5, 
     >       5X,'FINAL REBALANCING GROUPS   : FROM ',I5,' TO ',I5) 
 8001 FORMAT(1X,A6,': WARNING -> FLUX ALL GROUPS VANISHES '/
     >       1X,'REBALANCING TURNED OFF') 
 8002 FORMAT(1X,A6,': WARNING -> SINGULAR REBALANCING MATRIX '/
     >       1X,'REBALANCING TURNED OFF') 
      END
