*DECK FLUDB2
      SUBROUTINE FLUDB2(FMODUL,IPSYS ,IPTRK ,IPBTF ,IPRINT,NBMIX ,
     >                  NREGIO,NUNKNO,MATCOD,VOLUME,KEYFLX,ILEAK ,
     >                  FUNKNO,SUNKNO,ISADJ ,TITLE ,
     >                  DB2   ,SIGT0 ,SIGS0 ,PNL   )
C
C---------------------------  FLUDB2  ---------------------------------
C
C   1-  PROGRAMME STATISTICS:
C
C          NAME       :  FLUDB2
C          FUNCTION   :  FIND A PARAMETER (ALP1) TO MATCH THE INPUT DB2
C                        VALUE AND FIND THE CORRESPONDING FLUX.
C          VERSION    :  1.0
C          MODIFIED   :  98/02/18 (G.M. AND R.R)
C                        CORRECT PNL FOR SKIP CASE
C          DATE       :  88/10/05
C          AUTHOR     :  A.HEBERT
C
C   2-  INPUT AND OUTPUT PARAMETERS:
C
C      FMODUL  : FLUX SOLUTION MODULE.                EXT ROUTINE
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPTRK   : POINTER TO THE TRACKING              I
C      IPBTF   : POINTER TO THE BINARY TRACKING FILE  I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES     I
C      NBMIX   : NUMBER OF MIXTURES CONSIDERED        I
C      NREGIO  : NUMBER OF REGIONS CONSIDERED         I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM      I
C      MATCOD  : MATERIAL CODE IN REGION              I(NREGIO)
C      VOLUME  : VOLUME OF REGION                     R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM      I(NREGIO)
C      ILEAK   : METHOD USED TO INCLUDE DB2 EFFECT    I
C                = 0 NO DB2 EFFECT
C                = 1 THE SCATTERING MODIFIED CP MATRIX
C                    IS MULTIPLIED BY A PNL FACTOR
C                = 2 THE REDUCED CP MATRIX IS MULTIPLIED BY
C                    PNL FACTOR
C                = 3 SIGS0-DB2 APPROXIMATION
C                = 4 ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR SEARCH
C                  X->6 Y->7 Z->8C R->9 ALL->10
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR            R(NUNKNO)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN         R(NUNKNO)
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION  I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      TITLE   : TITLE DE L'EXECUTION                 C*72
C      DB2     : LEAKAGE PARAMETER                    R
C                DB2(G)=D(G)*B**2
C      SIGT0   : TOTAL CROSS SECTION VECTOR           R(0:NBMIX)
C                SIGT0(0)=0.0 IS FOR VOID
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      PNL     : NON LEAKAGE PROBABILITY              R
C
C--------------------------    FLUDB2    ------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUDB2')
C----
C  ROUTINE PARAMETERS
C----
      EXTERNAL    FMODUL
      INTEGER     IPSYS,IPTRK,IPBTF,IPRINT,NBMIX,
     >            NREGIO,NUNKNO,ILEAK,ISADJ
      CHARACTER   TITLE*72
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO)
      REAL        VOLUME(NREGIO),FUNKNO(NUNKNO),
     >            SUNKNO(NUNKNO),DB2,SIGT0(0:NBMIX),
     >            SIGS0(0:NBMIX),PNL
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ISUNKN,JF1,JF2,IWORK
C----
C  LOCAL PARAMETERS
C----
      INTEGER     ILCMLN,ILCMTY,I,IND,IR,IDIR
      DOUBLE PRECISION ZNUM,ZDEN,ALP1,ALPM1
C----
C  READ THE TRANSPORT CORRECTED CROSS SECTIONS.
C----
      IDIR=0
      IF(ILEAK .EQ. 1 .OR. ILEAK .EQ. 2 .OR. ILEAK .EQ. 4 ) THEN
        CALL LCMLEN(IPSYS,'DRAGON-TXSC',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NBMIX+1) THEN
          CALL LCMGET(IPSYS,'DRAGON-TXSC',SIGT0)
        ELSE
          CALL LCMLIB(IPSYS)
          CALL XABORT(NAMSBR//
     >    ': READ ERROR ON LCM RECORD= DRAGON-TXSC')
        ENDIF
        CALL LCMLEN(IPSYS,'DRAGON-S0XSC',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NBMIX+1) THEN
          CALL LCMGET(IPSYS,'DRAGON-S0XSC',SIGS0)
        ELSE
          CALL LCMLIB(IPSYS)
          CALL XABORT(NAMSBR//
     >    ': READ ERROR ON LCM RECORD= DRAGON-S0XSC')
        ENDIF
      ENDIF
C----
C  INCLUDE DB2 TERM IN SUNKNO
C----
      CALL SETARA(RBASE,NUNKNO,ISUNKN)
      DO 100 I=1,NUNKNO
        RBASE(ISUNKN+I-1)=SUNKNO(I)
 100  CONTINUE
      IF(ILEAK .EQ. 1 .OR. ILEAK .EQ. 2 ) THEN
        IF(DB2 .NE. 0.0) THEN
          IF(ISADJ .EQ. 0) THEN
            ZNUM=0.0D0
            ZDEN=0.0D0
            DO 110 I=1,NREGIO
              IND=KEYFLX(I)
              IR=MATCOD(I)
              ZNUM=ZNUM+DBLE(SIGT0(IR)*FUNKNO(IND)*VOLUME(I))
              ZDEN=ZDEN+DBLE(FUNKNO(IND)*VOLUME(I))
 110        CONTINUE
            ALP1=ZNUM/(ZNUM+DBLE(DB2)*ZDEN)
            PNL=REAL(ALP1)
          ELSE
            ALP1=DBLE(PNL)
          ENDIF
          IF(ILEAK .EQ. 2) THEN
            ALPM1=(1.0D0-ALP1)
          ELSE
            ALPM1=0.0D0
          ENDIF
          DO 120 I=1,NREGIO
            IND=KEYFLX(I)
            RBASE(ISUNKN+IND-1)=REAL(ALP1)*SUNKNO(IND)-
     >        REAL(ALPM1)*SIGS0(MATCOD(I))*FUNKNO(IND)
 120      CONTINUE
        ENDIF
        CALL FMODUL(IPSYS ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREGIO,
     >              NUNKNO,MATCOD,VOLUME,KEYFLX,
     >              FUNKNO,RBASE(ISUNKN),TITLE)
      ELSE IF(ILEAK.EQ.3) THEN
        DO 130 I=1,NREGIO
          IND=KEYFLX(I)
          RBASE(ISUNKN+IND-1)=SUNKNO(IND)-DB2*FUNKNO(IND)
 130    CONTINUE
        CALL FMODUL(IPSYS ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREGIO,
     >              NUNKNO,MATCOD,VOLUME,KEYFLX,
     >              FUNKNO,RBASE(ISUNKN),TITLE)
      ELSE IF(ILEAK.EQ.4) THEN
        CALL SETARA(RBASE,NREGIO,JF1)
        CALL SETARA(RBASE,NREGIO,JF2)
        CALL SETARA(RBASE,NREGIO**2,IWORK)
        CALL FLUALB(IPSYS,NREGIO,MATCOD,VOLUME,KEYFLX,FUNKNO,SUNKNO,
     >   SIGS0,SIGT0,RBASE(JF1),RBASE(JF2),RBASE(IWORK))
        CALL RLSARA(RBASE(IWORK))
        IF(IPRINT .GE. 5) THEN
          WRITE(IOUT,'(//33H N E U T R O N    S O U R C E S :)')
          WRITE(IOUT,'(1P,6(5X,E15.7))') (SUNKNO(KEYFLX(I)),I=1,NREGIO)
        ENDIF
        DO 160 I=1,NUNKNO
          FUNKNO(I)=0.0
 160    CONTINUE
        DO 170 I=1,NREGIO
          FUNKNO(KEYFLX(I))=RBASE(JF1+I-1)+DB2*RBASE(JF2+I-1)
 170    CONTINUE
        IF(IPRINT .GE. 5) THEN
          WRITE(IOUT,'(//33H N E U T R O N    F L U X E S   :)')
          WRITE(IOUT,'(1P,6(5X,E15.7))') (FUNKNO(KEYFLX(I)),I=1,NREGIO)
        ENDIF
        CALL RLSARA(RBASE(JF2))
        CALL RLSARA(RBASE(JF1))
      ENDIF
      CALL RLSARA(RBASE(ISUNKN))
      RETURN
      END
