*DECK FLULBD
      SUBROUTINE FLULBD(IPRINT,NGROUP,B2,SIGTN,LAMBDA)
C
C-------------------------    FLULBD    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : FLULBD
C     USE      : CALCULATION OF LAMBDA
C     DATE     : 94-06-08
C     AUTHOR   : I. PETROVIC
C
C 2- PARAMETERS:
C  INPUT
C     IPRINT  : PRINT SELECTION FOR FLUX MODULES:            I
C     NGROUP  : NUMBER OF GROUPS                             I
C     SIGTN   : TOTAL HOMOGENIZED CROSS SECTION              R(NGROUP)
C     B2      : CRITICAL BUCKLING OF A HOMOGENIZED MEDIUM    R
C  OUTPUT
C     LAMBDA  : LAMBDA                                       R(NGROUP)
C               LAMBDA=3*(1-ARCTG(AAA)/AAA)/ARCTG(AAA)/AAA,    B2>0
C               LAMBDA=3*((LNM/AAA)-1)/LNM/AAA            ,    B2<0
C               AAA=B/SIGTN  , LNM=0.5*LN((SIGTN+B)/(SIGTN-B))
C               FOR ABS(B/SIGTN)<CUTOFF USE SERIES EXPANSION
C
C-------------------------    FLULBD    -------------------------------
C
      PARAMETER  (IUNOUT=6,CUTOFF=1.0E-2)
      INTEGER    IPRINT,NGROUP
      REAL       B2,SIGTN(NGROUP),LAMBDA(NGROUP)
      IF(B2.EQ.0.)THEN
        DO 11 IG=1,NGROUP
          LAMBDA(IG)=1.0
  11    CONTINUE
      ELSE IF(B2.GT.0.)THEN
        B=SQRT(B2)
        DO 10 IG=1,NGROUP
          BS=B/SIGTN(IG)
          BS2=BS*BS
          IF(BS2.LT.CUTOFF) THEN
             LAMBDA(IG)=1.0-BS2*(4.0/15.0-44.0*BS2/315.0)
          ELSE
            BBS=1./BS
            ATG=ATAN(BS)
            LAMBDA(IG)=3.*BBS*(1.-BBS*ATG)/ATG
          ENDIF
  10    CONTINUE
      ELSE
        B=SQRT(-B2)
        DO 20 IG=1,NGROUP
          BS=B/SIGTN(IG)
          BS2=BS*BS
          IF(BS2.LT.CUTOFF) THEN
            LAMBDA(IG)=1.0+BS2*(4.0/15.0+44.0*BS2/315.0)
          ELSE
            SB=1.0/BS
            BLN=0.5*ALOG((SIGTN(IG)+B)/(SIGTN(IG)-B))
            LAMBDA(IG)=3.*SB*(SB*BLN-1.)/BLN
          ENDIF
  20    CONTINUE
      ENDIF
      IF(IPRINT.GE.10) THEN
        WRITE(IUNOUT,6000) B2
        WRITE(IUNOUT,6001) (SIGTN(IGR),IGR=1,NGROUP)
        WRITE(IUNOUT,6002) (LAMBDA(IGR),IGR=1,NGROUP)
      ENDIF
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(1X,'FLULBD OUTPUT'/1X,'HOMOGENIZED B2 = ',1P,E15.7)
 6001 FORMAT(1X,'HOMOGENIZED TOTAL XS (PER GROUP)'/1P,5(5X,E15.7))
 6002 FORMAT(1X,'LAMBDA (PER GROUP)'/1P,5(5X,E15.7))
 6003 FORMAT(1X,'HOMOGENIZED TOTAL XS (PER GROUP)'/1P,6(5X,E15.7))
      END
