*DECK JPMG2
      SUBROUTINE JPMG2 (MAXPTS,MAXJ,IPGEOM,IR,MAT,VOL,ILK,ITYPE,IHEX,
     1 IAPP,IQUAD,IHALT,XX,YY,ZZ,IMPX,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,
     2 MIX,DVX,NGEN,IGEN,ISURF,CHORD,XXX,YYY,ZZZ,IORI)
*
*-----------------------------------------------------------------------
*
* RECOVERY AND ANALYSIS OF THE GEOMETRY FOR THE DESCRIPTION OF A 1-D,
* 2-D OR 3-D CARTESIAN, CYLINDRICAL OR SPHERICAL DOMAIN.
*
* GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMBLK, NMERGE OR
*            NGEN.
*   MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJAT OR IJAS
*            WHERE IJAT IS THE TOTAL NUMBER OF DISTINCT OUT-CURRENTS
*            AND IJAS IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL
*            THE BLOCKS (MERGED OR NOT).
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   IR     : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(MAXPTS)
*   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   ITYPE  : TYPE OF GEOMETRY.
*   IHEX   : TYPE OF HEXAGONAL GEOMETRY. =0 FOR NON-HEXAGONAL GEOMETRY.
*   IAPP   : TYPE OF ANGULAR FLUX EXPANSION BETWEEN THE BLOCKS.
*   IQUAD  : QUADRATURE PARAMETER.
*   IHALT  : =1 TO STOP AT THE END OF JPMG2.
*   XX,YY, : X, Y AND Z-THICKNESS OF THE GENERATING BLOCKS.
*   ZZ       DIMENSION XX(NGEN),YY(NGEN),ZZ(NGEN)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
* PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS (MERGED OR NOT).
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
* PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH BLOCK.
*            DIMENSION INUM(NMBLK)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
* PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*
* SCRATCH STORAGE:
*   XXX    : DIMENSION XXX(MAXPTS+1)
*   YYY    : DIMENSION YYY(MAXPTS+1)
*   ZZZ    : DIMENSION ZZZ(MAXPTS+1)
*   IORI   : DIMENSION IORI(MAXPTS)
*
*---------------------------------------- AUTHOR: A. HEBERT 02/04/87 ---
*
      PARAMETER (PI=3.141592654)
      LOGICAL ILK,P1(6),FILLED,LCYL
      CHARACTER HSMG*131
      DIMENSION MAT(*),VOL(*),XX(*),YY(*),ZZ(*),IFR(*),ALB(*),INUM(*),
     1 MIX(*),DVX(*),IGEN(*),ISURF(*),CHORD(*),XXX(*),YYY(*),ZZZ(*),
     2 IORI(*),NCODE(6),ZCODE(6),ISM(6,6)
      COMMON BASE(1)
      DATA ISM/1,2,3,4,5,6,3,4,1,2,5,6,5,6,3,4,1,2,5,6,1,2,3,4,
     1         1,2,5,6,3,4,3,4,5,6,1,2/
*
      CALL SETARA(BASE,MAXPTS,IISPLX)
      CALL SETARA(BASE,MAXPTS,IISPLY)
      CALL SETARA(BASE,MAXPTS,IISPLZ)
      CALL READ3D(MAXPTS,MAXPTS,MAXPTS,MAXPTS,IPGEOM,IHEX,IR,MAT,ILK,
     1 SIDE,XXX,YYY,ZZZ,IMPX,LX,LY,LZ,NMBLK,NCODE,ZCODE,BASE(IISPLX),
     2 BASE(IISPLY),BASE(IISPLZ))
      CALL RLSARA(BASE(IISPLZ))
      CALL RLSARA(BASE(IISPLY))
      CALL RLSARA(BASE(IISPLX))
      LCYL=(ITYPE.EQ.3).OR.(ITYPE.EQ.4).OR.(ITYPE.EQ.6)
      IF((ITYPE.EQ.7).AND.(IQUAD.EQ.5)) IQUAD=16
      IF(IMPX.GE.1) THEN
         IF(IAPP.EQ.1) THEN
            WRITE (6,'(/20H DP-0 APPROXIMATION.)')
         ELSE IF(IAPP.EQ.2) THEN
            WRITE (6,'(/20H DP-1 APPROXIMATION.)')
         ELSE IF(IAPP.EQ.3) THEN
            WRITE (6,'(/43H DP-1 APPROXIMATION WITH ISOTROPIC BOUNDARY,
     1      12H CONDITIONS.)')
         ENDIF
         WRITE (6,'(/23H QUADRATURE PARAMETER =,I3/)') IQUAD
      ENDIF
      DO 50 I=1,6
      IF(NCODE(I).GT.5) CALL XABORT('JPMG2: WRONG BOUNDARY CONDITION.')
   50 CONTINUE
*
      NMERGE=0
      DO 60 IBLK=1,NMBLK
      IF(MAT(IBLK).GT.0) THEN
         NMERGE=NMERGE+1
         INUM(IBLK)=NMERGE
      ELSE
         INUM(IBLK)=0
      ENDIF
   60 CONTINUE
      NGEN=0
      IBLK=0
      IF(LCYL) THEN
         DO 80 K0=1,LZ
         DO 80 K2=1,LX
         IBLK=IBLK+1
         IF(MAT(IBLK).EQ.0) GO TO 80
         IKK=INUM(IBLK)
         A=XXX(K2)
         B=XXX(K2+1)
         C=ZZZ(K0+1)-ZZZ(K0)
         DO 70 JBLK=1,IBLK-1
         JKG=IGEN(INUM(JBLK))
         IF(MAT(IBLK).EQ.MAT(JBLK)) THEN
            P1(1)=(A.EQ.XX(JKG)).AND.(B.EQ.YY(JKG)).AND.(C.EQ.ZZ(JKG))
            IF(P1(1)) THEN
               IGEN(IKK)=JKG
               GO TO 80
            ENDIF
         ENDIF
   70    CONTINUE
         NGEN=NGEN+1
         IGEN(IKK)=NGEN
         XX(NGEN)=A
         YY(NGEN)=B
         ZZ(NGEN)=C
   80    CONTINUE
      ELSE IF(IHEX.EQ.0) THEN
         DO 120 K0=1,LZ
         DO 120 K1=1,LY
         LXM=1
         LXP=LX
         IF((NCODE(2).EQ.3).AND.(NCODE(3).EQ.3)) LXP=K1
         IF((NCODE(1).EQ.3).AND.(NCODE(4).EQ.3)) LXM=K1
         DO 120 K2=LXM,LXP
         IBLK=IBLK+1
         IF(MAT(IBLK).EQ.0) GO TO 120
         IKK=INUM(IBLK)
         A=XXX(K2+1)-XXX(K2)
         B=YYY(K1+1)-YYY(K1)
         C=ZZZ(K0+1)-ZZZ(K0)
         IORI(IBLK)=1
         DO 90 JBLK=1,IBLK-1
         JKG=IGEN(INUM(JBLK))
         IF(MAT(IBLK).EQ.MAT(JBLK)) THEN
            P1(1)=(A.EQ.XX(JKG)).AND.(B.EQ.YY(JKG)).AND.(C.EQ.ZZ(JKG))
            P1(2)=(B.EQ.XX(JKG)).AND.(A.EQ.YY(JKG)).AND.(C.EQ.ZZ(JKG))
            P1(3)=(C.EQ.XX(JKG)).AND.(B.EQ.YY(JKG)).AND.(A.EQ.ZZ(JKG))
            P1(4)=(B.EQ.XX(JKG)).AND.(C.EQ.YY(JKG)).AND.(A.EQ.ZZ(JKG))
            P1(5)=(A.EQ.XX(JKG)).AND.(C.EQ.YY(JKG)).AND.(B.EQ.ZZ(JKG))
            P1(6)=(C.EQ.XX(JKG)).AND.(A.EQ.YY(JKG)).AND.(B.EQ.ZZ(JKG))
            DO 85 I=1,6
            IF(P1(I)) THEN
               IGEN(IKK)=JKG
               IORI(IBLK)=I
               GO TO 120
            ENDIF
   85       CONTINUE
         ENDIF
   90    CONTINUE
         NGEN=NGEN+1
         IGEN(IKK)=NGEN
         XX(NGEN)=A
         YY(NGEN)=B
         ZZ(NGEN)=C
  120    CONTINUE
      ELSE
         CALL XABORT('JPMG2: GEOMETRY OPTION NOT IMPLEMENTED.')
      ENDIF
*
* COMPUTE VECTORS ISURF AND CHORD.
      IOF=0
      DO 140 IKG=1,NGEN
      A=XX(IKG)
      B=YY(IKG)
      C=ZZ(IKG)
      FILLED=(A.EQ.0.0)
      IF((IAPP.EQ.1).AND.(ITYPE.EQ.2)) THEN
         ISURF(IKG)=2
         CHORD(IOF+1)=4.0*A
         CHORD(IOF+2)=CHORD(IOF+1)
      ELSE IF((IAPP.EQ.1).AND.(ITYPE.EQ.5)) THEN
         ISURF(IKG)=4
         CHORD(IOF+1)=4.0*A
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=4.0*B
         CHORD(IOF+4)=CHORD(IOF+3)
      ELSE IF((IAPP.EQ.1).AND.(ITYPE.EQ.7)) THEN
         ISURF(IKG)=6
         CHORD(IOF+1)=4.0*A
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=4.0*B
         CHORD(IOF+4)=CHORD(IOF+3)
         CHORD(IOF+5)=4.0*C
         CHORD(IOF+6)=CHORD(IOF+5)
      ELSE IF((IAPP.EQ.2).AND.(ITYPE.EQ.2)) THEN
         ISURF(IKG)=4
         CHORD(IOF+1)=4.0*A
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=CHORD(IOF+1)
         CHORD(IOF+4)=CHORD(IOF+1)
      ELSE IF((IAPP.EQ.2).AND.(ITYPE.EQ.5)) THEN
         ISURF(IKG)=12
         DO 130 I=1,6
         CHORD(IOF+I)=4.0*A
  130    CHORD(IOF+I+6)=4.0*B
*
      ELSE IF((FILLED.AND.(IAPP.EQ.1).AND.(ITYPE.EQ.3)).OR.
     1 (FILLED.AND.(IAPP.EQ.3).AND.(ITYPE.EQ.3).AND.(NGEN.EQ.1))) THEN
         ISURF(IKG)=1
         CHORD(IOF+1)=2.0*B
      ELSE IF(FILLED.AND.(IAPP.GE.2).AND.(ITYPE.EQ.3)) THEN
         ISURF(IKG)=2
         CHORD(IOF+1)=2.0*B
         CHORD(IOF+2)=CHORD(IOF+1)
      ELSE IF(.NOT.FILLED.AND.(IAPP.EQ.1).AND.(ITYPE.EQ.3)) THEN
         ISURF(IKG)=2
         CHORD(IOF+1)=2.0*(B-A)*(B+A)/A
         CHORD(IOF+2)=2.0*(B-A)*(B+A)/B
      ELSE IF(.NOT.FILLED.AND.(IAPP.EQ.3).AND.(ITYPE.EQ.3).AND.
     1 (IKG.EQ.NGEN)) THEN
         ISURF(IKG)=3
         CHORD(IOF+1)=2.0*(B-A)*(B+A)/A
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=2.0*(B-A)*(B+A)/B
      ELSE IF(.NOT.FILLED.AND.(IAPP.GE.2).AND.(ITYPE.EQ.3)) THEN
         ISURF(IKG)=4
         CHORD(IOF+1)=2.0*(B-A)*(B+A)/A
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=2.0*(B-A)*(B+A)/B
         CHORD(IOF+4)=CHORD(IOF+3)
*
      ELSE IF((FILLED.AND.(IAPP.EQ.1).AND.(ITYPE.EQ.4)).OR.
     1 (FILLED.AND.(IAPP.EQ.3).AND.(ITYPE.EQ.3).AND.(NGEN.EQ.1))) THEN
         ISURF(IKG)=1
         CHORD(IOF+1)=4.0*B/3.0
      ELSE IF(FILLED.AND.(IAPP.GE.2).AND.(ITYPE.EQ.4)) THEN
         ISURF(IKG)=2
         CHORD(IOF+1)=4.0*B/3.0
         CHORD(IOF+2)=CHORD(IOF+1)
      ELSE IF(.NOT.FILLED.AND.(IAPP.EQ.1).AND.(ITYPE.EQ.4)) THEN
         ISURF(IKG)=2
         CHORD(IOF+1)=4.0*(B-A)*(B*B+A*B+A*A)/(3.0*A*A)
         CHORD(IOF+2)=4.0*(B-A)*(B*B+A*B+A*A)/(3.0*B*B)
      ELSE IF(.NOT.FILLED.AND.(IAPP.EQ.3).AND.(ITYPE.EQ.4).AND.
     1 (IKG.EQ.NGEN)) THEN
         ISURF(IKG)=3
         CHORD(IOF+1)=4.0*(B-A)*(B*B+A*B+A*A)/(3.0*A*A)
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=4.0*(B-A)*(B*B+A*B+A*A)/(3.0*B*B)
      ELSE IF(.NOT.FILLED.AND.(IAPP.GE.2).AND.(ITYPE.EQ.4)) THEN
         ISURF(IKG)=4
         CHORD(IOF+1)=4.0*(B-A)*(B*B+A*B+A*A)/(3.0*A*A)
         CHORD(IOF+2)=CHORD(IOF+1)
         CHORD(IOF+3)=4.0*(B-A)*(B*B+A*B+A*A)/(3.0*B*B)
         CHORD(IOF+4)=CHORD(IOF+3)
*
      ELSE IF(FILLED.AND.(IAPP.EQ.1).AND.(ITYPE.EQ.6)) THEN
         ISURF(IKG)=3
         CHORD(IOF+1)=2.0*B
         CHORD(IOF+2)=4.0*C
         CHORD(IOF+3)=CHORD(IOF+2)
      ELSE IF(.NOT.FILLED.AND.(IAPP.EQ.1).AND.(ITYPE.EQ.6)) THEN
         ISURF(IKG)=4
         CHORD(IOF+1)=2.0*(B-A)*(B+A)/A
         CHORD(IOF+2)=2.0*(B-A)*(B+A)/B
         CHORD(IOF+3)=4.0*C
         CHORD(IOF+4)=CHORD(IOF+3)
      ELSE
         CALL XABORT('JPMG2: INVALID TYPE OF ELEMENTARY BLOCK.')
      ENDIF
  140 IOF=IOF+ISURF(IKG)
*
* RECOMPUTE MAT AND CKECK IF MAXJ IS OK.
      IJAS=0
      DO 150 IBLK=1,NMBLK
      IF(INUM(IBLK).GT.0) THEN
         IJAS=MAX(IJAS,ISURF(IGEN(INUM(IBLK))))
         MAT(INUM(IBLK))=MAT(IBLK)
      ENDIF
  150 CONTINUE
      IJAS=IJAS*NMBLK
      IF(IJAS.GT.MAXJ) GO TO 610
*
* SURFACE ORDERING ALGORITHM.
      IF(LCYL) THEN
         CALL SETARA(BASE,4*NMERGE,IJF1)
         CALL SETARA(BASE,4*NMERGE,IJF2)
         CALL NUMERC(ITYPE,IAPP,NCODE,ZCODE,LX,LZ,VOL,IMPX,NMBLK,IFR,
     1   ALB,NMERGE,INUM,NGEN,IGEN,XX,YY,ZZ,ISURF,BASE(IJF1),BASE(IJF2))
         CALL RLSARA(BASE(IJF2))
         CALL RLSARA(BASE(IJF1))
         IJAR=0
         DO 160 IKK=1,NMERGE
         IKG=IGEN(IKK)
         IF(ITYPE.EQ.4) THEN
            VOL(IKK)=4.0*PI*(YY(IKG)-XX(IKG))*(XX(IKG)*XX(IKG)+XX(IKG)*
     1      YY(IKG)+YY(IKG)*YY(IKG))/3.0
         ELSE
            VOL(IKK)=PI*(YY(IKG)-XX(IKG))*(YY(IKG)+XX(IKG))*VOL(IKK)
            IF(ITYPE.EQ.6) VOL(IKK)=VOL(IKK)*ZZ(IKG)
         ENDIF
  160    IJAR=IJAR+ISURF(IKG)
         IJAS=IJAR
         DO 170 NSURF=1,IJAR
         MIX(NSURF)=NSURF
  170    DVX(NSURF)=1.0
      ELSE
         CALL SETARA(BASE,6*NMBLK,IJF2)
         CALL SETARA(BASE,NMBLK,IGG3)
         NCOUR=2
         IF(ITYPE.EQ.5) NCOUR=4
         IF(ITYPE.EQ.7) NCOUR=6
         CALL NUMER3(NCOUR,IAPP+2,NCODE,ZCODE,LX,LY,LZ,IORI,ISM,VOL,
     1   IMPX,NMBLK,IFR,ALB,NMERGE,INUM,MIX,DVX,IGEN,XX,YY,ZZ,
     2   BASE(IJF2),BASE(IGG3))
         CALL RLSARA(BASE(IGG3))
         CALL RLSARA(BASE(IJF2))
         IJAS=NCOUR*NMBLK
         IJAR=NCOUR*NMERGE
         DO 180 IKK=1,NMERGE
         IKG=IGEN(IKK)
         VOL(IKK)=XX(IKG)*VOL(IKK)
         IF(ITYPE.GE.5) VOL(IKK)=VOL(IKK)*YY(IKG)
         IF(ITYPE.EQ.7) VOL(IKK)=VOL(IKK)*ZZ(IKG)
  180    CONTINUE
      ENDIF
*
      IF(IMPX.GE.1) WRITE (6,620) NMBLK,NMERGE,NGEN,MAXPTS
      IJAT=0
      DO 190 NSURF=1,IJAR
  190 IJAT=MAX0(IJAT,MIX(NSURF))
      IF(IMPX.GE.1) WRITE (6,630) IJAT,IJAR,IJAS,MAXJ
      IF(IJAS.GT.MAXJ) GO TO 610
      IF(IHALT.EQ.1) CALL XABORT('JPMG2: STOP REQUESTED BY USER.')
      RETURN
*
  610 WRITE(HSMG,'(41HJPMG2: MAXJ SHOULD BE GREATER OR EQUAL TO,I5,
     1 1H.)') IJAS
      CALL XABORT(HSMG)
*
  620 FORMAT (/39H TOTAL NUMBER OF BLOCKS IN THE DOMAIN =,I5/
     1         32H TOTAL NUMBER OF MERGED BLOCKS =,I5/
     2         36H TOTAL NUMBER OF GENERATING BLOCKS =,I5,5X,
     3         26HAVAILABLE STORAGE MAXPTS =,I5/)
  630 FORMAT (/40H TOTAL NUMBER OF DISTINCT OUT-CURRENTS =,I5/
     1 61H TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE MERGED BLOCKS =,
     2 I5/57H TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL THE BLOCKS =,
     3 I5,5X,24HAVAILABLE STORAGE MAXJ =,I5/)
      END
