*DECK JPMGR2
      SUBROUTINE JPMGR2(NGEN2,NG,NSMAX,MICRO,IQUAD,IAPP,NS,IGI,RS,FRACT,
     1 ISURF2,CHORD2,ISURF,SIGT,MASK,PII,PIS,PSS,SIG2,RRRR,PIS2,PSS2,XX,
     2 YY,ZZ,JSURF)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE BLOCK COLLISION PROBABILITIES FOR THE DOUBLE
* HETEROGENEITY OPTION OF JPM. PART 2.
*
* INPUT PARAMETERS:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS IN THE OVERALL GEOMETRY.
*   NGEN2  : NUMBER OF GENERATING BLOCKS IN THE MACRO GEOMETRY.
*   NG     : NUMBER OF DIFFERENT KIND OF MICRO STRUCTURES. A KIND
*            OF MICRO STRUCTURE IS CHARACTERIZED BY THE RADIUS OF
*            ITS MICRO BLOCKS. ALL THE MICRO BLOCKS OF THE SAME
*            KIND SHOULD OWN THE SAME NUCLEAR PROPERTIES IN A GIVEN
*            MACRO BLOCK.
*   NSMAX  : MAXIMUM NUMBER OF TUBES OR SHELLS IN A MICRO BLOCK.
*   MICRO  : TYPE OF MICRO BLOCKS: =3: CYLINDER ; =4: SPHERE.
*   IQUAD  : QUADRATURE PARAMETER FOR THE CP CALCULATION OF THE
*            MICRO BLOCKS.
*   IAPP   : TYPE OF ANGULAR FLUX EXPANSION BETWEEN THE MICRO BLOCKS.
*   NS     : NUMBER OF MICRO BLOCKS IN EACH KIND OF MICRO STRUCTURE.
*            DIMENSION NS(NG)
*   IGI    : TYPE OF MIXTURE IN EACH GENERATING BLOCK OF THE MACRO
*            GEOMETRY. DIMENSION IGI(NGEN2)
*   RS     : RADIUS OF THE MICRO BLOCKS. DIMENSION RS(NSMAX+1,NG)
*   FRACT  : VOLUMIC FRACTION OF THE MICRO BLOCKS.
*            DIMENSION FRACT(NG,IR2) WHERE IR2 IS THE NUMBER
*            OF ORDINARY AND COMPOSITE MIXTURES.
*   ISURF2 : NUMBER OF SURFACES SURROUNDING EACH GENERATING BLOCK OF
*            THE MACRO GEOMETRY. DIMENSION ISURF2(NGEN2)
*   CHORD2 : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE OF THE
*            MACRO GEOMETRY. DIMENSION CHORD2(NPIS2) WHERE
*            NPIS2=SUM(ISURF2(I))
*   ISURF  : NUMBER OF SURFACES SURROUNDING EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   SIGT   : MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NGEN2)
*   MASK   : COLLISION PROBABILITIES ARE COMPUTED ONLY IF MASK=.TRUE.
*            DIMENSION MASK(NGEN2)
*   SIG2   : MODIFIED MACROSCOPIC CROSS SECTIONS. DIMENSION SIG2(NGEN2)
*   RRRR   : VOLUMIC SELF-SHIELDED FACTOR FOR JPMGR2.
*            DIMENSION RRRR(NGEN2)
*   PII    : VOLUME TO VOLUME COLLISION PROBABILITIES IN THE MACRO
*            GEOMETRY. DIMENSION PII(NGEN)
*   PIS2   : VOLUME TO SURFACE COLLISION PROBABILITIES IN THE MACRO
*            GEOMETRY. DIMENSION PIS2(SUM(ISURF2(I)))
*   PSS2   : SURFACE TO SURFACE COLLISION PROBABILITIES IN THE MACRO
*            GEOMETRY. DIMENSION PSS2(SUM(ISURF2(I)**2))
*
* OUTPUT PARAMETERS:
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
*
* SCRATCH STORAGE:
*   XX     : DIMENSION XX(MAXS)
*   YY     : DIMENSION YY(MAXS)
*   ZZ     : DIMENSION ZZ(MAXS)
*   JSURF  : DIMENSION JSURF(MAXS)
*
*----------------------------------------------- AUTHOR: A. HEBERT -----
*
      LOGICAL MASK(*)
      DOUBLE PRECISION RSLN
      DIMENSION NS(*),IGI(*),RS(NSMAX+1,*),FRACT(NG,*),ISURF2(*),
     1 CHORD2(*),ISURF(*),SIGT(*),SIG2(*),RRRR(*),PII(*),PIS(*),PSS(*),
     2 PIS2(*),PSS2(*),XX(*),YY(*),ZZ(*),JSURF(*)
*
      NPIS=0
      NPSS=0
      NPIS2=0
      NPSS2=0
      DO 10 IKG=1,NGEN2
      NCOUR=ISURF(IKG)
      NCOUR2=ISURF2(IKG)
      NPIS=NPIS+NCOUR
      NPSS=NPSS+NCOUR*NCOUR
      NPIS2=NPIS2+NCOUR2
   10 NPSS2=NPSS2+NCOUR2*NCOUR2
      DO 40 IKG=NGEN2,1,-1
      NCOUR=ISURF(IKG)
      NCOUR2=ISURF2(IKG)
      IF (MASK(IKG)) THEN
         DO 20 I=0,NCOUR2-1
   20    PIS(NPIS+NCOUR2-NCOUR-I)=PIS2(NPIS2-I)
         DO 30 I=0,NCOUR2*NCOUR2-1
   30    PSS(NPSS+(NCOUR2-NCOUR)*(1+NCOUR+I/NCOUR2)-I)=PSS2(NPSS2-I)
      ENDIF
      NPIS=NPIS-NCOUR
      NPIS2=NPIS2-NCOUR2
      NPSS=NPSS-NCOUR*NCOUR
   40 NPSS2=NPSS2-NCOUR2*NCOUR2
*
      NPIS=0
      NPSS=0
      NPIS2=0
      DO 110 IKG=1,NGEN2
      NCOUR=ISURF(IKG)
      NCOUR2=ISURF2(IKG)
      MIL=-IGI(IKG)
      IF ((MIL.GT.0).AND.MASK(IKG)) THEN
         DILF=1.0
         DO 50 J=1,NG
   50    DILF=DILF-FRACT(J,MIL)
         SIGB=RRRR(IKG)/SIG2(IKG)
         RSLN=(1.0D0-RRRR(IKG))*SIGB
         PII(IKG)=DILF*(RSLN+RRRR(IKG)*RRRR(IKG)*PII(IKG))
         DO 60 I=1,NCOUR2
   60    PIS(NPIS+I)=PIS(NPIS+I)*RRRR(IKG)
         J0=0
         DO 100 J=1,NG
         IF (FRACT(J,MIL).GT.0.00001) THEN
            J0=J0+1
            IOF=NPSS+(NCOUR2+J0-1)*NCOUR
            ZZZ=4.0*DILF*RS(NS(J)+1,J)/(REAL(MICRO-1)*FRACT(J,MIL))
            PIS(NPIS+NCOUR2+J0)=PII(IKG)/ZZZ
            DO 80 I=1,NCOUR2
            PSS(NPSS+(I-1)*NCOUR+NCOUR2+J0)=PIS(NPIS+I)
   80       PSS(IOF+I)=PIS(NPIS+I)*DILF*CHORD2(NPIS2+I)/ZZZ
            L0=0
            DO 90 L=1,NG
            IF (FRACT(L,MIL).GT.0.00001) THEN
               L0=L0+1
               PSS(IOF+NCOUR2+L0)=PIS(NPIS+NCOUR2+J0)
            ENDIF
   90       CONTINUE
         ENDIF
  100    CONTINUE
      ENDIF
      NPIS=NPIS+NCOUR
      NPSS=NPSS+NCOUR*NCOUR
  110 NPIS2=NPIS2+NCOUR2
*
      IND2=0
      DO 120 I=1,NGEN2
      IF (IGI(I).LT.0) IND2=MAX(IND2,-IGI(I))
  120 CONTINUE
      IND1=IND2
      DO 130 I=1,NGEN2
      IF (IGI(I).LT.0) IND1=MIN(IND1,-IGI(I))
  130 CONTINUE
      IKG=NGEN2
      DO 200 IND=IND1,IND2
      DO 200 J=1,NG
      IF (FRACT(J,IND).GT.0.00001) THEN
         DO 140 K=1,NS(J)
         JSURF(K)=2
         IF (IAPP.GE.2) JSURF(K)=4
         XX(K)=RS(K,J)
         YY(K)=RS(K+1,J)
  140    ZZ(K)=1.0
         JSURF(1)=1
         IBPP=1
         IF ((IAPP.GE.2).AND.(NS(J).GT.1)) THEN
            IBPP=3
            JSURF(1)=2
            JSURF(NS(J))=3
         ENDIF
         CALL JPM2(MICRO,IBPP,IQUAD,XX,YY,ZZ,NS(J),JSURF,SIGT(IKG+1),
     1   MASK(IKG+1),PII(IKG+1),PIS(NPIS+1),PSS(NPSS+1))
         DO 150 K=1,NS(J)
         IKG=IKG+1
         NCOUR=ISURF(IKG)
         NPIS=NPIS+NCOUR
  150    NPSS=NPSS+NCOUR*NCOUR
      ENDIF
  200 CONTINUE
      RETURN
      END
