*DECK JPMRES
      SUBROUTINE JPMRES (NMBLK,IFR,ALB,NMERGE,IJAT,INUM,MIX,DVX,MU1,
     1 IMA,WSSB,NGEN,IGEN,ISURF,CHORD,SIGS,PII,PIS,FUNKNO,SUNKNO,IGAT,
     2 IGAU,GAR)
*
*-----------------------------------------------------------------------
*
* FAST RESIDUAL CALCULATION FOR JPM.
*
* INPUT PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
* INPUT PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH
*            BLOCK. DIMENSION INUM(NMBLK)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS BELONGING TO THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*   MU1    : POSITION OF EACH DIAGONAL ELEMENT IN MATRIX (I-WSS*A).
*   IMA    : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN MATRIX
*            (I-WSS*A). DIMENSION MU1(IJAT),IMA(IJAT)
*   WSSB   : MATRIX (I-WSS*A) BEFORE FACTORIZATION.
*            DIMENSION WSSB(IMA(IJAT))
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
* INPUT PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*   SIGS   : WITHIN-GROUP SCATTERING CROSS SECTIONS OF THE GENERATING
*            BLOCKS. DIMENSION SIGS(NGEN)
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*
*   FUNKNO : INPUT FLUXES AND OUTCOMING CURRENTS.
*            DIMENSION FUNKNO(NMERGE+IJAT)
*   SUNKNO : INPUT SOURCES. DIMENSION SUNKNO(NMERGE)
*
* OUTPUT PARAMETER:
*   SUNKNO : RESIDUAL. DIMENSION SUNKNO(NMERGE+IJAT)
*
* SCRATCH STORAGE:
*   IGAT   : DIMENSION IGAT(NMERGE)
*   IGAU   : DIMENSION IGAU(NGEN)
*   GAR    : DIMENSION GAR(IJAT)
*
*-------------------------------------------- AUTHOR: A. HEBERT 26/09/89
*
      DIMENSION IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),MU1(*),IMA(*),
     1 IGEN(*),ISURF(*),CHORD(*),SIGS(*),PII(*),PIS(*),FUNKNO(*),
     2 SUNKNO(*),WSSB(*),IGAT(*),IGAU(*),GAR(*)
*
      IJAR=0
      DO 10 I=1,NMERGE
      IGAT(I)=IJAR
   10 IJAR=IJAR+ISURF(IGEN(I))
      NPIS=0
      DO 20 I=1,NGEN
      IGAU(I)=NPIS
   20 NPIS=NPIS+ISURF(I)
*
* COMPUTE THE SECOND PART OF THE RESIDUAL VECTOR.
      CALL ALLUM(IJAT,WSSB,FUNKNO(NMERGE+1),SUNKNO(NMERGE+1),MU1,IMA)
      DO 40 IKK=1,NMERGE
      IKG=IGEN(IKK)
      IT=IGAT(IKK)
      DO 40 IP=1,ISURF(IKG)
      ICC=MIX(IT+IP)
      PBJ=DVX(IT+IP)*PIS(IGAU(IKG)+IP)*CHORD(IGAU(IKG)+IP)
     1 /(1.0-SIGS(IKG)*PII(IKG))
   40 SUNKNO(NMERGE+ICC)=SUNKNO(NMERGE+ICC)-PBJ*SUNKNO(IKK)
*
* COMPUTE THE INCOMING CURRENT.
      DO 50 I=1,IJAT
   50 GAR(I)=0.0
      IS=0
      DO 70 ICEL=1,NMBLK
      IKK=INUM(ICEL)
      IKG=IGEN(IKK)
      NCOUR=ISURF(IKG)
      IT=IGAT(IKK)
      DO 60 IP=1,NCOUR
      J1=MIX(IT+IP)
   60 GAR(J1)=GAR(J1)+ALB(IS+IP)*DVX(IT+IP)*FUNKNO(NMERGE+IFR(IS+IP))
   70 IS=IS+NCOUR
*
* COMPUTE THE FIRST PART OF THE RESIDUAL VECTOR.
      DO 80 IKK=1,NMERGE
      IKG=IGEN(IKK)
      FACT=1.0/(1.0-SIGS(IKG)*PII(IKG))
      SUNKNO(IKK)=FUNKNO(IKK)-PII(IKG)*FACT*SUNKNO(IKK)
      IT=IGAT(IKK)
      DO 80 IP=1,ISURF(IKG)
   80 SUNKNO(IKK)=SUNKNO(IKK)-PIS(IGAU(IKG)+IP)*FACT*GAR(MIX(IT+IP))
      RETURN
      END
