*DECK LIBMIX
      SUBROUTINE LIBMIX(IPLIB,NBMIX,NGROUP,NBISO,ISONAM,MIX,DEN,MASK,
     1 MASKL,ITSTMP,TMPDAY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transformation of the isotope ordered microscopic cross sections to
* group ordered macroscopic cross sections (part 1).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature).
* NBMIX   number of material mixtures.
* NGROUP  number of energy groups.
* NBISO   number of isotopes present in the calculation domain.
* ISONAM  alias name of isotopes.
* MIX     mixture number of each isotope (can be zero).
* DEN     density of each isotope.
* MASK    mixture mask (=.true. if a mixture is to be made).
* MASKL   group mask (=.true. if an energy group is to be treated).
* ITSTMP  type of cross section perturbation (=0: perturbation
*         forbidden; =1: perturbation not used even if present;
*         =2: perturbation used if present).
* TMPDAY  time stamp in day/burnup/irradiation.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NBMIX,NGROUP,NBISO,ISONAM(3,NBISO),MIX(NBISO),ITSTMP
      REAL DEN(NBISO),TMPDAY(3)
      LOGICAL MASK(NBMIX),MASKL(NGROUP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      LOGICAL LSAME
      INTEGER ISTATE(NSTATE),KCHAR(3),IBASE(1)
      CHARACTER TEXT12*12
      REAL OLDTIM(3)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  RECOVER SOME LIBRARY PARAMETERS.
*----
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NL=ISTATE(4)
      ITRANC=ISTATE(5)
      NED=ISTATE(13)
      NDEL=ISTATE(19)
      CALL SETARA(BASE,2*NED,IJNED)
      CALL LCMGET(IPLIB,'ADDXSNAME',BASE(IJNED))
*----
*  LOOK FOR OLD LIBRARY DATA
*----
      CALL LCMLEN(IPLIB,'MACROLIB',LENGTH,ITYLCM)
      IF(LENGTH.EQ.-1) THEN
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
        IF(TEXT12.NE.'L_MACROLIB') THEN
          CALL XABORT('LIBMIX: INVALID SIGNATURE ON THE MACROLIB.')
        ENDIF
        CALL LCMLEN(IPLIB,'TIMESTAMP',LENGTH,ITYLCM)
        IF((LENGTH.GT.0).AND.(LENGTH.LE.3)) THEN
          CALL LCMGET(IPLIB,'TIMESTAMP',OLDTIM)
          IF(ITSTMP.EQ.0) THEN
            TMPDAY(1)=OLDTIM(1)
            TMPDAY(2)=OLDTIM(2)
            TMPDAY(3)=OLDTIM(3)
          ENDIF
        ENDIF
        CALL LCMSIX(IPLIB,' ',2)
      ENDIF
*----
*  COMPUTE THE MAXIMUM NUMBER OF FISSIONABLE ISOTOPES IN A MIXTURE.
*----
      DO 10 I=1,NBISO
      IF(MIX(I).GT.NBMIX) CALL XABORT('LIBMIX: NBMIX OVERFLOW.')
   10 CONTINUE
      CALL SETARA (BASE,NBMIX*NBISO,IINDFI)
      CALL LIBNFI (IPLIB,NGROUP,NBISO,NBMIX,NDEL,ISONAM,MIX,MAXNFI,
     1 LSAME,BASE(IINDFI))
      CALL RLSARA (BASE(IINDFI))
*----
*  RECOVER 'FGWITHUPSCAT' FOR DRAGON VERSION3.
*----
      CALL SETARA(IBASE,NL*NBMIX,IFGUPS)
      CALL XDISET(IBASE(IFGUPS),NL*NBMIX,NGROUP+1)
      CALL LCMLEN(IPLIB,'FGWITHUPSCAT',LLFG,ITFG)
      IF(LLFG.EQ.0) THEN
        CALL XDISET(IBASE(IFGUPS),NL*NBMIX,NGROUP+1)
      ELSE
        CALL LCMGET(IPLIB,'FGWITHUPSCAT',IBASE(IFGUPS))
      ENDIF
*----
*  BUILD THE MACROSCOPIC CROSS SECTIONS.
*----
      NBLK=MAX(12,NGROUP)
      CALL SETARA (BASE,NGROUP+1,IGA1)
      CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
      CALL SETARA (BASE,NDEL*MAXNFI,IGA3)
      CALL SETARA (BASE,NBMIX*(NGROUP+1),IGAR)
      CALL SETARA (BASE,NBMIX*NGROUP*NBLK,IGAF)
      CALL SETARA (BASE,NGROUP*NBMIX,ISCAT)
      CALL SETARA (BASE,NBMIX*NGROUP,INJJM)
      CALL SETARA (BASE,NBMIX*NGROUP,IIJJM)
      CALL SETARA (BASE,NBMIX,IIPOS)
      CALL SETARA (BASE,NBMIX*NGROUP*NL,ICHECK)
      CALL SETARA (BASE,NBMIX*MAXNFI,IINDFI)
      CALL SETARA (BASE,2*MAXNFI,INAMFI)
      CALL SETARA (BASE,NBMIX*MAXNFI*NGROUP*(NDEL+1),IZNUS)
      CALL SETARA (BASE,NBMIX*MAXNFI*NGROUP*(NDEL+1),IZCHI)
      CALL SETARA (BASE,NGROUP,IIJJ)
      CALL SETARA (BASE,NGROUP,INJJ)
      CALL SETARA (BASE,NBISO,ILMADE)
      CALL LIBDEN (IPLIB,NBLK,NGROUP,NBISO,NBMIX,NL,NDEL,ISONAM,MIX,DEN,
     1 MASK,MASKL,NED,BASE(IJNED),ITRANC,MAXNFI,LSAME,IBASE(IFGUPS),
     2 ITSTMP,TMPDAY,BASE(IGA1),BASE(IGA2),BASE(IGA3),BASE(IGAR),
     3 BASE(IGAF),BASE(ISCAT),BASE(INJJM),BASE(IIJJM),BASE(IIPOS),
     4 BASE(ICHECK),BASE(IINDFI),BASE(IZNUS),BASE(IZCHI),BASE(IIJJ),
     5 BASE(INJJ),BASE(ILMADE),BASE(INAMFI))
      CALL RLSARA (BASE(ILMADE))
      CALL RLSARA (BASE(INJJ))
      CALL RLSARA (BASE(IIJJ))
      CALL RLSARA (BASE(IZCHI))
      CALL RLSARA (BASE(IZNUS))
      CALL RLSARA (BASE(INAMFI))
      CALL RLSARA (BASE(IINDFI))
      CALL RLSARA (BASE(ICHECK))
      CALL RLSARA (BASE(IIPOS))
      CALL RLSARA (BASE(IIJJM))
      CALL RLSARA (BASE(INJJM))
      CALL RLSARA (BASE(ISCAT))
      CALL RLSARA (BASE(IGAF))
      CALL RLSARA (BASE(IGAR))
      CALL RLSARA (BASE(IGA3))
      CALL RLSARA (BASE(IGA2))
      CALL RLSARA (BASE(IGA1))
      CALL RLSARA (IBASE(IFGUPS))
      CALL RLSARA (BASE(IJNED))
      RETURN
      END
