*DECK MACXSR
      SUBROUTINE MACXSR(IPRINT,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >                  LOLDXS,LNEWXS,CARLIR,LADD  ,LUPD  ,
     >                  XSTOTL,XSTRAN,XSFISS,XSFIXE,XSSCAT,ISCATA,
     >                  XEFISS,LINIXS,
     >                  NALBP ,NALBR ,ALBP )
C
C---------------------------  MACXSR  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C
C       NAME     : MACXSR
C       USE      : READ CROSS SECTIONS INPUT FILE
C       MODIFIED : 2000-01-20 (T. COURAU)
C                  DEAL WITH ADJOINT AND GENERALIZED ADJOINT
C                  FIXED SOURCES: 'FIXE' , 'FIXA' , 'FIXG'
C                  91-01-24
C       AUTHOR   : G.MARLEAU
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C       IPRINT   : PRINT LEVEL                        I
C       NBMIX    : NUMBER OF MIXTURES                 I
C       NGROUP   : NUMBER OF ENERGY GROUPS            I
C       NANISO   : MAXIMUM LEGENDRE ORDER             I
C                  = 1 ISOTROPIC COLLISION
C                  = 2 LINEARLY ANISOTROPIC COLLISION
C       NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C       MAXFIS   : MAX(1,NIFISS)                      I
C       LOLDXS   : LOGICAL FOR X-S ALREADY PRESENT    L(NBXS)
C                  ON THE MACROLIB
C       NALBP    : Number of physical albedos         I
C       NALBR    : Physical albedos processing        I
C    INPUT/OUTPUT
C       LNEWXS   : LOGICAL FOR X-S MODIFIED           L(NBXS)
C       CARLIR   : LAST READ STRING FROM INPUT        C*12
C       LADD     : ADD CURRENT XS TO OLD ONE          L
C       LUPD     : UPDATE CURRENT XS                  L
C       XSTOTL   : TOTAL CROSS SECTION OF MIXTURE     R(NBMIX,NGROUP)
C       XSTRAN   : TRANSPORT CORRECTION XS OF MIXTURE R(NBMIX,NGROUP)
C       XSFISS   : FISSION CROSS SECTION OF MIXTURE   R(NBMIX,
C                                                   MAXFIS,NGROUP,3)
C       XSFIXE   : FIXE SOURCES                       R(NBMIX,
C                                                      NGROUP,3)
C       XSSCAT   : SCATT X-SECTION OF MIXTURE/GROUP   R(NBMIX,NANISO,
C                                                       NGROUP,NGROUP)
C       ISCATA   : CHECK FOR SCATTERING ANISOTROPY    I(NANISO)
C       XEFISS   : AVERAGE ENERGY PER FISSION         R(NBMIX,MAXFIS)
C       LINIXS   : LOGICAL FOR X-S READ PER MIXTURE   L(NBXS,NBMIX)
C       ALBP     : Physical albedos                   R(0:NALBP,NGROUP)
C
C---------------------------   MACXSR  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NBXS
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NBXS=10,NAMSBR='MACXSR')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPRINT,NBMIX,NGROUP,NANISO,
     >            NIFISS,MAXFIS
      CHARACTER   CARLIR*12
      INTEGER     ISCATA(NANISO)
      LOGICAL     LOLDXS(NBXS),LNEWXS(NBXS),LADD,LUPD,
     >            LINIXS(NBXS,NBMIX)
      REAL        XSTOTL(NBMIX,NGROUP),XSTRAN(NBMIX,NGROUP),
     >            XSFISS(NBMIX,MAXFIS,NGROUP,3),
     >            XSFIXE(NBMIX,NGROUP,3),
     >            XSSCAT(NGROUP,NBMIX,NANISO,NGROUP),
     >            XEFISS(NBMIX,MAXFIS)
      INTEGER     NALBP,NALBR
      REAL        ALBP(0:NALBP,NGROUP)
C----
C  INPUT
C----
      INTEGER     ITYPLU,INTLIR
      REAL        REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CARXST(NBXS)*6
      INTEGER     MATNUM,IMIX,ITXS,IGROUP,JGROUP,IFIS,JANS,ING,IFG
      INTEGER     IALB
C----
C  INITIALIZATION
C----
      MATNUM=0
      DO 200 IMIX=1,NBMIX
        DO 210 ITXS=1,NBXS
          LINIXS(ITXS,IMIX)=.TRUE.
 210    CONTINUE
 200  CONTINUE
C----
C  START READING KEYWORDS
C----
 1000 CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 1010 CONTINUE
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': READ ERROR - CROSS SECTION OR SOURCE TYPE EXPECTED')
      IF(CARLIR(1:4) .EQ. 'TOTA') THEN
C----
C  TOTAL XS
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(1,MATNUM)) THEN
          LINIXS(1,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': TOTAL XS FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(1)=.TRUE.
        DO 100 IGROUP=1,NGROUP
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL VARIABLE EXPECTED: TOTAL XS')
          IF(LADD) REALIR=REALIR+XSTOTL(MATNUM,IGROUP)
          XSTOTL(MATNUM,IGROUP)=REALIR
 100    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'TRAN') THEN
C----
C  TRANSPORT CORRECTION XS
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(6,MATNUM)) THEN
          LINIXS(6,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': TRAN FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(6)=.TRUE.
        DO 110 IGROUP=1,NGROUP
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL VARIABLE EXPECTED: TRAN XS')
          IF(LADD) REALIR=REALIR+XSTRAN(MATNUM,IGROUP)
          XSTRAN(MATNUM,IGROUP)=REALIR
 110    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'NUSI') THEN
C----
C  NUSIGF XS
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(NIFISS .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NIFISS EXPECTED GREATER THAN ZERO')
        IF(LINIXS(2,MATNUM)) THEN
          LINIXS(2,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': NUSIGF FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(2)=.TRUE.
        DO 120 IFIS=1,NIFISS
          DO 121 IGROUP=1,NGROUP
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >      ': READ ERROR - REAL VARIABLE EXPECTED: NUSIGF')
            IF(LADD) REALIR=REALIR+XSFISS(MATNUM,IFIS,IGROUP,1)
            XSFISS(MATNUM,IFIS,IGROUP,1)=REALIR
 121      CONTINUE
 120    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'NFTO') THEN
C----
C  NUSIGF XS
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(NIFISS .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NIFISS EXPECTED GREATER THAN ZERO')
        IF(LINIXS(9,MATNUM)) THEN
          LINIXS(9,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': NFTOT FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(9)=.TRUE.
        DO 122 IFIS=1,NIFISS
          DO 123 IGROUP=1,NGROUP
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >      ': READ ERROR - REAL VARIABLE EXPECTED: NFTOT')
            IF(LADD) REALIR=REALIR+XSFISS(MATNUM,IFIS,IGROUP,2)
            XSFISS(MATNUM,IFIS,IGROUP,2)=REALIR
 123      CONTINUE
 122    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'FIXE') THEN
C----
C  FIXED DIRECT SOURCES
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(3,MATNUM)) THEN
          LINIXS(3,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': FIXE FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(3)=.TRUE.
        DO 130 IGROUP=1,NGROUP
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL VARIABLE EXPECTED: FIXE')
          IF(LADD) REALIR=REALIR+XSFIXE(MATNUM,IGROUP,1)
          XSFIXE(MATNUM,IGROUP,1)=REALIR
 130    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'FIXA') THEN
C----
C  FIXED ADJOINT SOURCES
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >    ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(7,MATNUM)) THEN
          LINIXS(7,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': FIXA FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(7)=.TRUE.
        DO 140 IGROUP=1,NGROUP
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL VARIABLE EXPECTED: FIXA')
          IF(LADD) REALIR=REALIR+XSFIXE(MATNUM,IGROUP,2)
          XSFIXE(MATNUM,IGROUP,2)=REALIR
 140   CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'FIXG') THEN
C----
C  FIXED GENERALIZED ADJOINT SOURCES
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(8,MATNUM)) THEN
          LINIXS(8,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': FIXG FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(8)=.TRUE.
        DO 150 IGROUP=1,NGROUP
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL VARIABLE EXPECTED: FIXG')
          IF(LADD) REALIR=REALIR+XSFIXE(MATNUM,IGROUP,3)
          XSFIXE(MATNUM,IGROUP,3)=REALIR
 150    CONTINUE
      ELSE IF(CARLIR(1:3) .EQ. 'CHI') THEN
C----
C  FISSION SPECTRUM
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(NIFISS .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NIFISS EXPECTED GREATER THAN ZERO')
        IF(LINIXS(4,MATNUM)) THEN
          LINIXS(4,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': CHI FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(4)=.TRUE.
        DO 160 IFIS=1,NIFISS
          DO 161 IGROUP=1,NGROUP
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >      ': READ ERROR - REAL VARIABLE EXPECTED: CHI ')
            IF(LADD) REALIR=REALIR+XSFISS(MATNUM,IFIS,IGROUP,3)
            XSFISS(MATNUM,IFIS,IGROUP,3)=REALIR
 161      CONTINUE
 160    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'SCAT') THEN
C----
C  SCATTERING XS:
C  XSSCAT(JGROUP,MATNUM,JANS,IGROUP) WHERE IGROUP IS
C                 THE SECONDARY GROUP.
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(5,MATNUM)) THEN
          LINIXS(5,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': SCATT XS FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(5)=.TRUE.
        DO 170 JANS=1,NANISO
          ISCATA(JANS)=2
          DO 171 IGROUP=1,NGROUP
C----
C  READ NUMBER OF GROUPS AND FIRST GROUP
C----
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >      ': READ ERROR - NUMBER OF SCAT GROUP EXPECTED:  NGSCAT ')
            ING=INTLIR
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >      ': READ ERROR - FIRST SCAT GROUP NUMBER EXPECTED: NFSCAT ')
            IFG=INTLIR
            ING=IFG-ING
C----
C  READ SCATTERING XS
C----
            DO 172 JGROUP=NGROUP,1,-1
              IF(JGROUP .GT. IFG .OR. JGROUP .LE. ING) THEN
                IF (.NOT.LADD) XSSCAT(JGROUP,MATNUM,JANS,IGROUP)=0.0
              ELSE
                CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
                IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >          ': READ ERROR - REAL VARIABLE EXPECTED: SCAT')
                IF(LADD)
     >          REALIR=REALIR+XSSCAT(JGROUP,MATNUM,JANS,IGROUP)
                XSSCAT(JGROUP,MATNUM,JANS,IGROUP)=REALIR
              ENDIF
 172        CONTINUE
 171      CONTINUE
 170    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'EFIS') THEN
C----
C  ENERGY PER FISSION
C----
        IF(MATNUM .LE. 0) CALL XABORT(NAMSBR//
     >  ': NO MIXTURE NUMBER PROVIDED')
        IF(LINIXS(10,MATNUM)) THEN
          LINIXS(10,MATNUM)=.FALSE.
        ELSE
          CALL XABORT(NAMSBR//
     >    ': ENERGY PER FISSION FOR THIS MATERIAL ALREADY READ')
        ENDIF
        LNEWXS(10)=.TRUE.
        DO 190 IFIS=1,NIFISS
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL VARIABLE EXPECTED: EFISS')
          IF(LADD) REALIR=REALIR+XEFISS(MATNUM,IFIS)
          XEFISS(MATNUM,IFIS)=REALIR
 190    CONTINUE
      ELSE
        IF(.NOT. LUPD .AND. MATNUM .GT. 0) THEN
C----
C  RESET XS THAT WERE NOT READ FOR THIS MATERIAL TO 0.0
C----
          IF(LINIXS(1,MATNUM) .AND. LOLDXS(1)) THEN
            LNEWXS(1)=.TRUE.
            DO 300 IGROUP=1,NGROUP
              XSTOTL(MATNUM,IGROUP)=0.0
 300        CONTINUE
          ENDIF
          IF(LINIXS(2,MATNUM) .AND. LOLDXS(2)) THEN
            LNEWXS(2)=.TRUE.
            DO 310 IGROUP=1,NGROUP
              DO 311 IFIS=1,NIFISS
                XSFISS(MATNUM,IFIS,IGROUP,1)=0.0
 311          CONTINUE
 310        CONTINUE
          ENDIF
          IF(LINIXS(3,MATNUM) .AND. LOLDXS(3)) THEN
            LNEWXS(3)=.TRUE.
            DO 320 IGROUP=1,NGROUP
              XSFIXE(MATNUM,IGROUP,1)=0.0
 320        CONTINUE
          ENDIF
          IF(LINIXS(4,MATNUM) .AND. LOLDXS(4)) THEN
            LNEWXS(4)=.TRUE.
            DO 330 IGROUP=1,NGROUP
              DO 331 IFIS=1,NIFISS
                XSFISS(MATNUM,IFIS,IGROUP,3)=0.0
 331          CONTINUE
 330        CONTINUE
          ENDIF
          IF(LINIXS(5,MATNUM) .AND. LOLDXS(5)) THEN
            LNEWXS(5)=.TRUE.
            DO 340 JANS=1,NANISO
              IF(ISCATA(JANS) .GE. 1) THEN
                ISCATA(JANS)=2
                DO 341 IGROUP=1,NGROUP
                  DO 342 JGROUP=1,NGROUP
                    XSSCAT(JGROUP,MATNUM,JANS,IGROUP)=0.0
 342              CONTINUE
 341            CONTINUE
              ENDIF
 340        CONTINUE
          ENDIF
          IF(LINIXS(6,MATNUM) .AND. LOLDXS(6)) THEN
            LNEWXS(6)=.TRUE.
            DO 350 IGROUP=1,NGROUP
              XSTRAN(MATNUM,IGROUP)=0.0
 350        CONTINUE
          ENDIF
          IF(LINIXS(7,MATNUM) .AND. LOLDXS(7)) THEN
            LNEWXS(7)=.TRUE.
            DO 360 IGROUP=1,NGROUP
              XSFIXE(MATNUM,IGROUP,2)=0.0
 360        CONTINUE
          ENDIF
          IF(LINIXS(8,MATNUM) .AND. LOLDXS(8)) THEN
            LNEWXS(8)=.TRUE.
            DO 370 IGROUP=1,NGROUP
              XSFIXE(MATNUM,IGROUP,3)=0.0
 370        CONTINUE
          ENDIF
          IF(LINIXS(9,MATNUM) .AND. LOLDXS(9)) THEN
            LNEWXS(9)=.TRUE.
            DO 380 IGROUP=1,NGROUP
              DO 381 IFIS=1,NIFISS
                XSFISS(MATNUM,IFIS,IGROUP,2)=0.0
 381          CONTINUE
 380        CONTINUE
          ENDIF
        ENDIF
C----
C  READ MIXTURE INDEX
C----
        IF(CARLIR(1:3) .EQ. 'MIX') THEN
          MATNUM=MATNUM+1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(INTLIR .GT. NBMIX)
     >    CALL XABORT(NAMSBR//
     >    ': READ ERROR - MIXTURE NUMBER TOO LARGE')
          IF(ITYPLU .EQ. 1) THEN
            MATNUM=INTLIR
          ELSE
            GO TO 1010
          ENDIF
        ELSE
C----
C  ALL MIXTURE READ RETURN
C----
          IF(IPRINT .GE. 1) THEN
C----
C  FIND MATERIAL FOR WHICH XS ARE READ AND PRINT TYPE OF XS READ
C----
            WRITE(IOUT,6000) NANISO
            DO 220 IMIX=1,NBMIX
              DO 230 ITXS=1,NBXS
                IF(.NOT. LINIXS(ITXS,IMIX)) THEN
                  CARXST(ITXS)='READ  '
                ELSE IF(LOLDXS(ITXS)) THEN
                  CARXST(ITXS)='OLD   '
                ELSE
                  CARXST(ITXS)='      '
                ENDIF
 230          CONTINUE
              WRITE(IOUT,6001) IMIX,(CARXST(ITXS),ITXS=1,8)
 220        CONTINUE
          ENDIF
          GO TO 2000
        ENDIF
      ENDIF
C----
C  RETURN TO READ NEXT KEYWORD
C----
      GO TO 1000
 2000 CONTINUE
      IF(CARLIR .EQ. 'ALBPG') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >    ': READ ERROR - Physical albedo number missing')
        IF(INTLIR .GT. NALBP) CALL XABORT(NAMSBR//
     >    ': READ ERROR - Invalid physical albedo number')
        IALB=INTLIR
        NALBR=1
        DO IGROUP=1,NGROUP
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >    ': READ ERROR - REAL physical albedo expected')
            ALBP(IALB,IGROUP)=REALIR
        ENDDO
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': READ ERROR - Keyword expected')
        GO TO 2000
      ENDIF
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(' CROSS SECTION TYPE',5X,'TOTAL ',5X,'NUSIGF',5X,'FIXE  ',
     > 5X,'CHI   ',5X,'SCAT ',I3,3X,'TRANC',5X,'FIXA  ',5X,'FIXG  ')
 6001 FORMAT(' MATERIAL ',I5,4X,8(5X,A6))
      END
