*DECK MCIDRV
      SUBROUTINE MCIDRV(
     1           IPFLUX, IPTRK , IPMACR, RB2   , IFTRAK, NGRP  ,
     2           NMAT  , NIFIS , NANIS , NL    , NBN   , NUN   , 
     3           FLUXW , FLXMPI, NINTMX, AIEPS , NEXTMX, AEEPS ,
     4           IFRITR, IACITR, ITYPEC, ILEAK , NREGIO, MATCOD,
     5           VOL   , XSOLD , XSSIGT, XSSIGW, XSCHI , XSNUF ,
     6           XSTK  , NJJ   , IJJ   , IPOS  , XSCAT , VSCAT ,
     7           ALBEDO, LEAKSW, ILK   , ICAC  , ITRANC, ITRB  ,
     8           ITCRMX, ATCRMX, LJACC , LITLM , LINIF ,
     9           LETAB , MGTRK , MAXSCR,  IPRT  )
C
C---------------------------  MCIDRV  ---------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C     NAME     :  MCIDRV
C     USE      :  DRIVER FOR CHARACTERISTICS SOLVER
C     AUTHOR   :  Guang Jun WU 
C
C 2- ROUTINE PARAMETERS:
C   INPUT
C     IPFLUX   : POINTER TO THE FLUX                     I
C     IPTRK    : POINTER TO THE TRACKING                 I
C     IPMACR   : POINTER TO THE MACROLIB                 I
C     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
C     NGRP     : NUMBER OF GROUPS TREATED                I
C     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
C     NREGIO   : NUMBER OF REGIONS CONSIDERED            I
C     MATCOD   : MIXTURE CODE IN REGION                  I(NREGIO)
C     VOL      : VOLUME OF REGION                        R(NREGIO)
C     LEAKSW   : LEAKAGE SWITCH                          L
C     ITRANC   : TYPE OF TRANSPORT CORRECTIONS           I
C     TITRE    : EXECUTION TITLE                         C*72
C     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
C   WORK VECTOR
C     XSSIGT   : GROUP TOTAL CROSS SECTION               R(NMAT+1)
C     XSSIGW   : WITHIN GROUP SCATTERING                 R(NMAT+1)
C
C 3- INTERNAL PARAMETERS:
C     IUNOUT   : OUTPUT UNIT = 6                         I
C     ILCMUP   : GO UP LCM INDEX = 1                     I
C     ILCMDN   : GO DOWN LCM INDEX = 2                   I
C
C---------------------------   MCIDRV  --------------------------------
C
      IMPLICIT  NONE
      INTEGER   IUNOUT, ILCMUP, ILCMDN, NDIMO, NANIS, NIFIS, NUN,
     >          IPMACR, IREGIO, ITYLCM, IR, IMAT, IMIX, ITYPEC,
     >          IFIS, JGR, IFTRAK, NCOMNT, NTRK, ISPEC
      REAL      EPSR, ZERO, ONE, AIEPS, AEEPS, RKEFF, RB2, ATCRMX
      PARAMETER (IUNOUT=6,ILCMUP=1,ILCMDN=2)
      PARAMETER (NDIMO=3, EPSR=1.E-6)
      PARAMETER (ZERO=0., ONE=1.)
      LOGICAL     LEAKSW,ILK, LJACC, LITLM, LINIF, LETAB, L2LNG
      LOGICAL     LMBD, LPVM
      CHARACTER   TITRE*72,CGRPNM*12
      INTEGER     IPRT,NGRP,NMAT,NREGIO,MATCOD(*),ITCRMX,MGTRK,
     >            ITRANC,ITRB,IGR,ILCMLN,NJJ(*),IJJ(*),IPOS(*)
      REAL        VOL(*),XSSIGT(0:NMAT,NGRP),XSSIGW(0:NMAT,NGRP),
     >            XSOLD(0:NMAT,NGRP),
     >            XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),VSCAT(*),ALBEDO(6),
     >            XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >            XSTK(NMAT,NIFIS)
      REAL        FLUXW(NUN,NGRP), FLXMPI(NUN,NGRP), ASCRP
CD      DOUBLE PRECISION   FLUXW(NUN,NGRP)
      INTEGER     KDROPN,KDRCLS
      INTEGER     VOLSUR, MATALB, PHIANG, SIGANG, 
     >            SANINV, FLUX  , NUMERO, LENGHT, FLUXO ,
     >            EXP2  , FLM   , FLP   , ICYM  ,
     >            ICYP  , INVVOL, SOURCE, SOURCO, ISGNR ,
     >            KEYANI, MASQ  , Q     , REBAL , DA14  ,
     >            FLUC  , XCTOT , XCNUF , XCCHI , XCABS , XCS0  ,
     >            XCTR  , XCCAT , DIFF  , DB2,    DAST  , DBST  ,
     >            DCST  , YTDB2 , YSDB2 , IBETA , CUR   ,
     >            DENSTY, SURF  , VOID  , CMASQ ,
     >            FLUX1 , FACN  , 
     >            FCSUR , FACF  , DSURF , 
     >            FL    , NUM2  , LEN2  , FR    , FS, FD    
      INTEGER     TTSEG, MBDSEG, IS
      INTEGER     INTEGER2, INTEGER4, REAL4, REAL8
      INTEGER     IANI, IXSS, IC, NDIM, NREG, NSOUT, NALBG, NCOR,NANGL,
     >            MXSEG, NUNKNO, NPHI, JUN, NSCRP, IL, NR, NS, NRNL,NL, 
     >            MATMAX,INDSUR, NMOD,INDS, IFILE, IPFLUX, ICAC, NINTMX, 
     >            NEXTMX, ILEAK,NBN,IFRITR,IACITR,MAXSCR, IFTRK2,IPTRK
      PARAMETER ( INTEGER2=2, INTEGER4=3, REAL4=4, REAL8=6 )
C
      CHARACTER*2 CAN(0:9)
      CHARACTER   CINI*4, COMENT*80
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      DOUBLE PRECISION D(1)
      EQUIVALENCE (A(1),I(1)),(A(1),D(1))
      DATA CAN /' 0',' 1',' 2',' 3',' 4',' 5',' 6',' 7',' 8',' 9'/
      DATA CINI/'$TRK'/
      DOUBLE PRECISION temps_debut, temps_fin
C
      LMBD = .FALSE.
C
      IF(IPRT.GE.1) THEN
        WRITE(IUNOUT,6200) 
        IF(ITRANC.EQ.-1) THEN
          WRITE(IUNOUT,6100)
        ELSE IF(ITRANC.GE.1) THEN
          WRITE(IUNOUT,6101) ITRANC
        ENDIF
      ENDIF
      DO 115 IREGIO=1,NREGIO
  115 CONTINUE      
      IF(IPRT.GE.2) THEN
        WRITE(IUNOUT,6000)
        WRITE(IUNOUT,6001) (IREGIO,VOL(IREGIO),MATCOD(IREGIO),
     >                      IREGIO=1,NREGIO)
      ENDIF
C      STOP
      DO 150 IGR=1,NGRP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMACR,'TOTAL',ILCMLN,ITYLCM)
        IF(ILCMLN .GT.0) THEN
          CALL LCMGET(IPMACR,'TOTAL',XSSIGT(1,IGR))
          XSSIGT(0,IGR)=ZERO
          DO 111 IR= 0, NMAT
 111      XSOLD(IR,IGR)=XSSIGT(IR,IGR)
        ELSE
          CALL LCMLIB(IPMACR)
          CALL XABORT('MCIDRV: READ ERROR ON LCM RECORD= TOTAL')
        ENDIF
        CALL LCMLEN(IPMACR,'SIGW 0',ILCMLN,ITYLCM)
        IF(ILCMLN.GT.0) THEN
          CALL LCMGET(IPMACR,'SIGW 0',XSSIGW(1,IGR))
          XSSIGW(0,IGR)=ZERO
        ELSE
          DO 110 IR=0,NMAT
 110      XSSIGW(IR,IGR)=ZERO
        ENDIF
        IF(ITRANC.NE.0) THEN
*         TRANSPORT CORRECTION.
          CALL LCMLEN(IPMACR,'TRANC',ILCMLN,ITYLCM)
          IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPMACR,'TRANC',VSCAT)
            DO 120 IMAT=1,NMAT
              XSSIGT(IMAT,IGR)=XSSIGT(IMAT,IGR)-VSCAT(IMAT)
              XSSIGW(IMAT,IGR)=XSSIGW(IMAT,IGR)-VSCAT(IMAT)
 120        CONTINUE
          ENDIF
        ENDIF
        IF(IPRT.GE.3) THEN
          WRITE(IUNOUT,6002)  IGR
          WRITE(IUNOUT,6003) (IMIX,XSSIGT(IMIX,IGR),
     >                             XSSIGW(IMIX,IGR),IMIX=1,NMAT)
        ENDIF
        IF( ITYPEC.NE.0 )THEN
           CALL LCMGET(IPMACR,'CHI',XSTK)
           DO 132 IFIS= 1, NIFIS
              XSCHI(0,IFIS,IGR)= ZERO
              DO 131 IMAT= 1, NMAT
                 XSCHI(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  131         CONTINUE
  132      CONTINUE
           CALL LCMGET(IPMACR,'NUSIGF',XSTK)
           DO 134 IFIS= 1, NIFIS
              XSNUF(0,IFIS,IGR)= ZERO
              DO 133 IMAT= 1, NMAT
                 XSNUF(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  133         CONTINUE
  134      CONTINUE
           IF( IPRT.GT.3 )THEN
              WRITE(IUNOUT,6004) (IMIX,XSNUF(IMIX,1,IGR),
     >                                 XSCHI(IMIX,1,IGR),IMIX=1,NMAT)
           ENDIF
        ELSE
           IF( NIFIS.NE.1 )THEN
              CALL XABORT('MCIDRV: FOR FIXED SOURCE PROBLEM, NIFIS = 1')
           ENDIF
           XSNUF(0,1,IGR)= ZERO
           CALL LCMGET(IPMACR,'FIXE',XSNUF(1,1,IGR))
        ENDIF
        DO 50 IANI=0,NANIS
          CALL LCMLEN(IPMACR,'NJJD'//CAN(IANI),ILCMLN,ITYLCM)
          DO 39 IMAT= 0, NMAT
            DO 38 JGR= 1, NGRP
              XSCAT(JGR,IMAT,IANI,IGR)= ZERO
   38       CONTINUE
   39     CONTINUE
          IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPMACR,'NJJD'//CAN(IANI),NJJ)
            CALL LCMGET(IPMACR,'IJJD'//CAN(IANI),IJJ)
            CALL LCMGET(IPMACR,'IPOD'//CAN(IANI),IPOS)
            CALL LCMGET(IPMACR,'SCAD'//CAN(IANI),VSCAT)
            DO 45 IMAT= 1, NMAT
              DO 40 JGR= IJJ(IMAT)-NJJ(IMAT)+1, IJJ(IMAT)
                IXSS= IPOS(IMAT)+IJJ(IMAT)-JGR
                XSCAT(JGR,IMAT,IANI,IGR)= VSCAT(IXSS)
   40         CONTINUE
   45       CONTINUE
          ENDIF
   50   CONTINUE
        CALL LCMSIX(IPMACR,' ',ILCMDN)
 150  CONTINUE
C
C     READ HEADERS OF TRACKING FILE
      REWIND IFTRAK
      READ(IFTRAK) CINI,NCOMNT,NTRK
      IF( CINI.NE.'$TRK' .AND. CINI.NE.'$MBD') GO TO 998
      IF( CINI.EQ.'$MBD' ) LMBD=.TRUE.
      IF( NCOMNT.LT.0      ) GO TO 998
      IF( NTRK  .EQ.0      ) GO TO 998
      DO 30 IC= 1,NCOMNT
         READ(IFTRAK) COMENT
   30 CONTINUE
      IF( LMBD )THEN
         READ(IFTRAK) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL,MXSEG,
     >                TTSEG,MBDSEG
      ELSE
         READ(IFTRAK) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL,MXSEG
      ENDIF
      IF( NREG.NE.NREGIO )THEN
         CALL XABORT('MCIDRV: INVALID NUMBER OF REGIONS')
      ENDIF
      IF( NDIM.NE.NDIMO )THEN
         CALL XABORT('MCIDRV: NON 3D TRACKING FILE')
      ENDIF
      IF( ISPEC.NE.0 )THEN
         CALL XABORT('MCIDRV: NON ISOTROPIC TRACKING FILE')
      ENDIF
      NUNKNO= NREG+NSOUT+1
      IF(NUNKNO.NE.NUN) THEN
C      IF(NREG.NE.NUN .or. NSOUT.NE.NUNS) THEN
         CALL XABORT('MCIDRV: MOTCLE SURF NEEDED IN EXCELT')
      ENDIF 
      NPHI  = NANGL
      CALL SETARA( A, NUNKNO, VOLSUR )
      CALL SETARA( I, NUNKNO, MATALB )
      CALL SETARA( A, NPHI*3, PHIANG )
      CALL SETARA( A, NPHI  , DENSTY )
      READ(IFTRAK) (A(VOLSUR+JUN),JUN=0,NUNKNO-1)
      READ(IFTRAK) (I(MATALB+JUN),JUN=0,NUNKNO-1)
      READ(IFTRAK) ( NSCRP,JUN=1,NALBG)
      READ(IFTRAK) ( ASCRP,JUN=1,NALBG)
      READ(IFTRAK) ( A(PHIANG+3*IL),A(PHIANG+3*IL+1),IL=0,NANGL-1)
      READ(IFTRAK) ( A(DENSTY+IL-1),IL=1,  NANGL)
      WRITE(6,'(1H1 ///)')
      NR = NREG
      NS = NSOUT
      NRNL=   (NL+1)**2
C
C     CHECK COMPATIBLITY OF MATCOD
      MATMAX= 0
      DO 20 IS=-NS,NR
         IF( IS.LT.0 )THEN
            IF( I(MATALB+NSOUT+IS).LT.0 )THEN
Cwu               ILK = ILK.AND.ALBEDO(-I(MATALB+NSOUT+IS)).EQ.ONE
            ELSE
               CALL XABORT('MCIDRV: IS THIS A PROBLEM?')
            ENDIF
         ELSEIF( IS.GT.0 )THEN
            MATMAX= MAX(MATMAX,I(MATALB+NSOUT+IS))
            IF( I(MATALB+NSOUT+IS).NE.MATCOD(IS) )THEN
               WRITE(6,*) 'MATCOD HAS BEEN CHANGED ',
     >                    I(MATALB+NSOUT+IS),'->',MATCOD(IS)
               I(MATALB+NSOUT+IS)= MATCOD(IS) 
            ENDIF
            IF( ABS(VOL(IS)-A(VOLSUR+NSOUT+IS))/VOL(IS).GT.EPSR )THEN
               CALL XABORT('MCIDRV: INCOMPATIBLE VOLS')
            ENDIF
         ENDIF
   20 CONTINUE
      IF( ILK )THEN
         WRITE(6,*)' *** NO LEAKAGE IN THE CELL: REBALANCING *ON * ***'
         IF( LEAKSW )THEN
            WRITE(6,*) ' *** INCOMPATIBILITY ON LEAKAGE SWITCH ***'
            CALL XABORT('MCIDRV: ERROR ON LEAKAGE SWITCH')
         ENDIF
      ELSE
         WRITE(6,*)' ***    LEAKAGE IN THE CELL: REBALANCING *OFF* ***'
         IF( .NOT.LEAKSW )THEN
            WRITE(6,*) ' *** INCOMPATIBILITY ON LEAKAGE SWITCH ***'
         ENDIF
      ENDIF
      IF( MATMAX.GT.NMAT )THEN
         CALL XABORT('MCIDRV: NOT ENOUGH MATERIALS ARE DEFINED')
      ENDIF
C      STOP
      CALL SETARA( A, NGRP*(NR+7), SIGANG )
      CALL SETARA( A, NGRP*(NR+7), SANINV )
*
      CALL SETARA( A, 6*NGRP*(NR+NS+1), FLUX  )
      CALL SETARA( A, 2*NGRP*NR,        FLUX1 )
      CALL SETARA( A, 2*NUNKNO*NGRP, FLUXO )
      IF( .NOT.LMBD ) MBDSEG=MXSEG
      CALL SETARA( I, MXSEG , NUMERO )
      CALL SETARA( A, MBDSEG, LENGHT )
      CALL SETARA( A, 2*MBDSEG*NGRP , EXP2 )
      CALL SETARA( A, MXSEG , FLM )
      CALL SETARA( A, MXSEG , FLP )
      CALL SETARA( A, MXSEG , ICYM )
      CALL SETARA( A, MXSEG , ICYP )
      CALL SETARA( A, (NSOUT+NR+1), INVVOL )
      CALL SETARA( A,   2*NGRP*NR , SOURCE )
      CALL SETARA( A,   2*NGRP*NR , SOURCO )
      CALL SETARA( I,      8*NRNL , ISGNR  )
      CALL SETARA( I,        NRNL , KEYANI )
      CALL SETARA( I,        NGRP , MASQ   )
      CALL SETARA( I,        NGRP , CMASQ  )
      CALL SETARA( I,       MXSEG , VOID   )
      CALL SETARA( I,       NSOUT , INDSUR )
*
      CALL MOCIK3(NDIM,NL,I(ISGNR),I(KEYANI))
      IF(NDIM.EQ.2) THEN
          NMOD = 4
      ELSE
          NMOD = 1
      ENDIF
      CALL SETARA( A, NREG*NMOD*NGRP , Q )
      CALL SETARA( A, NGRP*(NGRP+1) , REBAL )
      CALL SETARA( A, (NMAT+1)*NIFIS, DA14)
      CALL SETARA( A, NGRP, FLUC)
      CALL SETARA( A, NGRP, XCTOT)
      CALL SETARA( A, NGRP, XCTR)
      CALL SETARA( A, NGRP, XCS0)
      CALL SETARA( A, NGRP, XCNUF)
      CALL SETARA( A, NGRP, XCCHI)
      CALL SETARA( A, NGRP, XCABS)
      CALL SETARA( A, NGRP*NGRP*(NANIS+1) , XCCAT)
      CALL SETARA( A, NGRP, DIFF)
      CALL SETARA( A, NGRP, DB2)
      CALL SETARA( A, 2*NGRP*(NGRP+1), DAST)
      CALL SETARA( A, 2*NGRP*NGRP, DBST)
      CALL SETARA( A, 2*NGRP, DCST)
      CALL SETARA( A, (NMAT+1)*NGRP, YTDB2)
      CALL SETARA( A, (NMAT+1)*NGRP*(NANIS+1), YSDB2)
      CALL SETARA( A, 2*NGRP, IBETA)
      CALL SETARA( A, 2*NSOUT*NGRP, CUR)
      CALL SETARA( A, NSOUT, SURF)
      CALL SETARA( A, 2*NREG*NGRP,    FACN   )
      CALL SETARA( A, 2*NSOUT*NGRP,   FCSUR  )
      CALL SETARA( A, 2*NREG*NGRP,    FACF   )
      CALL SETARA( I, MXSEG , NUM2 )
      CALL SETARA( A, MXSEG , LEN2 )
C     TMT au maximum
      CALL SETARA( A, 2*NREG*NGRP,    FR   )
      CALL SETARA( A, 2*NSOUT*NGRP,   FS   )
      CALL SETARA( I, NSOUT,          INDS )
      CALL SETARA( A, 2*NSOUT,      DSURF  )
CD
      CALL SETARA( A, 2*NSOUT,      FD  )
C
*WU cpu_test
      write(*,*) "Total Regions: ",NREG
      write(*,*) "Total Surfaces: ",NSOUT
      Write(*,*) "Number of groups: ",NGRP
      IFILE = KDROPN('DUMMYSQ1',0,2,0,0)
      IF( IFILE.LE.0 ) GO TO 997
      IFTRK2 = IFILE
      REWIND IFTRK2
      WRITE(IFTRK2) 'GJWU',0,0
      CALL MCI2FC( IFTRAK, A(FACN), A(FACF), A(EXP2), A(DSURF),
     1             A(SURF),        A(FCSUR),         I(INDSUR), 
     >                     I(NUMERO),       A(LENGHT),        MATCOD  ,
     2             XSSIGT, XSSIGW, VOL    , NGRP   , NREG   , NSOUT   ,
     3             MXSEG , MBDSEG, NMAT  , NTRK   , LETAB , 
     4             L2LNG , I(INDS), IPRT )
      CALL RLSARA( A(DSURF ) )
      CALL RLSARA( I( INDS  ) )
      CALL RLSARA( A( FS    ) )
      CALL RLSARA( A( FR    ) )
CD
      CALL RLSARA( A( FD    ) )
C
      IF( LINIF ) THEN
         CALL MCIGFL(IPFLUX,ITYPEC,NGRP,NUNKNO,NREG,FLUXW,A(SURF))
      ENDIF
C
      IF( LMBD )THEN
         CALL SETARA( A, NGRP*MBDSEG, FL    )
      ELSE
         CALL SETARA( A, NGRP*2,      FL    )
      ENDIF
*
      DO 2001 IS=0,NSOUT-1
         A(INVVOL+IS)=ONE/A(VOLSUR+IS)
 2001 CONTINUE
C
      CALL MCI2DR ( IPFLUX,IFTRAK,
     7              NGRP,NREG,NSOUT,NANIS,NMAT,MXSEG,NTRK,
     3              NPHI, NMOD, I(KEYANI), 
     5              MATCOD,I(MATALB),VOL,A(PHIANG),A(DENSTY),
     3              A(LENGHT),I(NUMERO),A(EXP2),
     5              A(FLM),A(FLP),A(ICYM),A(ICYP),I(VOID),
     1              A(INVVOL), 
     3              A(SIGANG),A(SANINV),ALBEDO,
     4              XSOLD,XSSIGT,XSSIGW,XSCAT,
     6              NIFIS,XSNUF,XSCHI,A(YTDB2),A(YSDB2),ITRB,
     6              A(SOURCE),A(Q),A(FLUX),A(FLUXO),A(FLUX1),FLUXW,
     6              FLXMPI,I(MASQ),I(CMASQ),LEAKSW,ILK,ICAC,
     4              NINTMX,AIEPS,NEXTMX,AEEPS,
     7              IPRT,ITYPEC,ILEAK,NL,NBN,RKEFF,RB2,
     4              A(SOURCO),A(REBAL),IFRITR,IACITR,
     2              A(DA14),A(FLUC),
     3              A(XCTOT),A(XCTR),A(XCS0),
     4              A(XCCAT),A(XCNUF),A(XCCHI),A(XCABS),
     6              A(DIFF),A(DB2),A(IBETA),A(DAST),A(DBST),A(DCST),
     4              A(CUR),A(SURF),ITCRMX,ATCRMX,
     3              A(FACN), A(FCSUR),A(FACF),
     2              A(FL),I(INDSUR), 
     5              LJACC, LETAB, L2LNG, MAXSCR, LITLM )
*
C     
      CALL MCI2WR ( IPFLUX,ITYPEC,RKEFF,RB2,
     >              NSOUT,NREG,NGRP,A(FLUXO),FLUXW,A(DIFF))
*
      IFTRK2= IFILE
      IFTRK2= KDRCLS(IFTRK2,2)
      IF( IFTRK2.LT.0 ) GO TO 999
      CALL RLSARA( A(FL    ))
      CALL RLSARA( A(LEN2 ) )
      CALL RLSARA( I(NUM2 ) )
      CALL RLSARA( A(FACF  ))
      CALL RLSARA( A(FCSUR ))
      CALL RLSARA( A(FACN  ))
*
      CALL RLSARA( A(SURF   ) )
      CALL RLSARA( A(CUR    ) )
      CALL RLSARA( A(IBETA  ) )
      CALL RLSARA( A(YSDB2  ) )
      CALL RLSARA( A(YTDB2  ) )
      CALL RLSARA( A(DCST   ) )
      CALL RLSARA( A(DBST   ) )
      CALL RLSARA( A(DAST   ) )
      CALL RLSARA( A(DB2    ) )
      CALL RLSARA( A(DIFF   ) )
      CALL RLSARA( A(XCCAT  ) )
      CALL RLSARA( A(XCABS  ) )
      CALL RLSARA( A(XCCHI  ) )
      CALL RLSARA( A(XCNUF  ) )
      CALL RLSARA( A(XCS0   ) )
      CALL RLSARA( A(XCTR   ) )
      CALL RLSARA( A(XCTOT  ) )
      CALL RLSARA( A(FLUC   ) )
      CALL RLSARA( A(DA14   ) )
      CALL RLSARA( A(REBAL  ) )
*
      CALL RLSARA( A(Q      ) )
      CALL RLSARA( I(INDSUR ) )
      CALL RLSARA( I(VOID   ) )
      CALL RLSARA( I(CMASQ  ) )
      CALL RLSARA( I(MASQ   ) )
      CALL RLSARA( I(KEYANI ) )
      CALL RLSARA( I(ISGNR  ) )
      CALL RLSARA( A(SOURCO ) )
      CALL RLSARA( A(SOURCE ) )
      CALL RLSARA( A(INVVOL ) )
      CALL RLSARA( A(ICYP   ) )
      CALL RLSARA( A(ICYM   ) )
      CALL RLSARA( A(FLP    ) )
      CALL RLSARA( A(FLM    ) )
      CALL RLSARA( A(EXP2   ) )
      CALL RLSARA( A(LENGHT ) )
      CALL RLSARA( I(NUMERO ) )
      CALL RLSARA( A(FLUXO  ) )
      CALL RLSARA( A(FLUX1  ) )
      CALL RLSARA( A(FLUX   ) )
      CALL RLSARA( A(SANINV ) )
      CALL RLSARA( A(SIGANG ) )
      CALL RLSARA( A(DENSTY) )
      CALL RLSARA( A(PHIANG) )
      CALL RLSARA( I(MATALB) )
      CALL RLSARA( A(VOLSUR) )
C
      RETURN
C
C------ FORMATS
C
 5090 FORMAT(31HMCIDRV: INVALID VALUE OF NMAT (,I5,2H).)
 6000 FORMAT(//30X,' EDITION REGION/VOL/MIXTURE '//
     >3(5X,'REGION',5X,'VOL  ',5X,'MIXTURE')/)
 6001 FORMAT(1P,3(5X,I4,4X,E12.5,4X,I4,4X))
 6002 FORMAT(//30X,' G R O U P : ',I5//
     >30X,' TOTAL MACROSCOPIC CROSS SECTIONS PER MIXTURE '/)
 6003 FORMAT(3(1X,'MIXTURE',4X,'TOTAL',11X,'SIGW',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6004 FORMAT(3(1X,'MIXTURE',4X,'NUSIGF',11X,'CHI ',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6100 FORMAT(//' USE APOLLO TYPE TRANSPORT CORRECTED CROSS-SECTIONS')
 6101 FORMAT(//' USE WIMS TYPE TRANSPORT CORRECTED CROSS-SECTIONS (IT',
     >'RANC=',I4,' )')
 6200 FORMAT(//' COMPUTATION WITH THE CHARACTERISTIC METHOD       ')
 998  WRITE(6,'(30H ECHO = FIRST RECORD DESTROYED      )')
      CALL XABORT('MCIDRV: PROBLEM WITH THE TRACKING FILE')
 997  WRITE(IUNOUT,'(31H ECHO = UNABLE TO OPEN  FILE FT,I4)') IFILE
      CALL XABORT('XELDRV: OPEN  FAILED')
 999  WRITE(IUNOUT,'(31H ECHO = UNABLE TO CLOSE FILE FT,I4)') IFILE
      CALL XABORT('XELDRV: CLOSE FAILED')
C
      END
