************************************************************************
      SUBROUTINE MCU2FC ( FACN  , FACF  , EXP2  , DSURF ,
     +                  SURF  , FCSUR , INDSUR, NUMERO, LENGHT, MATCOD,
     +                  XSTRC , XSDIA , VOL  , NG   , NR , NS ,
     +                  MXSEG , NMAT  , LETAB , L2LNG , INDS  ,
     +                  IPRT, NANGLE, DENUSR, ANGLES, DENSTY,
     +                  NTOTCL, NEXTGE, MAXR, REMESH, LINMAX, RCUTOF,
     +                  NSUR, NVOL, INDEL, MINDIM, ISYMM,
     +                  MAXDIM, ICOORD, INCR,ICUR,TRKBEG,CONV,TRKDIR,
     +                  SWVOID,  NORE, NRMV, VOLTRK, KEYMRG,MAXPTS)
************************************************************************
*                                                                      *
*      NAME: MCU2FC                                                    *
*      COMPONENT: MCU                                                  *
*      AUTHOR: M. DAHMANI                                              *
*                                                                      *
*      SUBROUTINE: THIS ROUTINE COMPUTE THE PJJ                        *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPRT   / INTERMEDIATE PRINTING LEVEL FOR OUTPUT.      /I./INT        *
* NANGLE / # OF ANGLES USED IN THE TRACKING PROCESS.    /I./INT        *
* DENUSR / DENSITY OF TRACKS IN THE PLANE PERPENDICULAR /I./REL        *
*        /      TO THE TRACKING ANGLES.                 /  /           *
* ISYMM  / FLAG FOR SYMETRY.  (1/0 for on/off)          /I./I          *
*          ISYMM = 2 -> reflection plane normal to X axis
*          ISYMM = 4 -> reflection plane normal to Y axis
*          ISYMM = 8  -> reflection plane normal to X and Y axis  
*          ISYMM =16 -> reflection plane normal to Z axis
*          ISYMM =18 -> reflection plane normal to X and Z axis
*          ISYMM =20 -> reflection plane normal to Y and Z axis
*          ISYMM =24 -> reflection plane normal to X, Y and Z axis  
* ANGLES / 3D ANGLE VALUES.                             /I./R(3*NSOLAN)*
* DENSTY / DENSITY OF TRACKS ANGLE BY ANGLE.            /I./REL(NSOLAN)*
* NTOTCL / # OF CYLINDRES OF A TYPE + 3.                /I./INT        *
* NEXTGE / FOR TUBEZ, NEXTGE=1                          /I./INT        *
* MAXR   / MAX NUMBER OF REAL MESH VALUES IN "REMESH".  /I./INT        *
* REMESH / REAL MESH VALUES (RECT/CYL).                 /I./REL(MAXR  )*
* LINMAX / MAX. # OF TRACK SEGMENTS IN A SINGLE TRACK.  /I./INT        *
* RCUTOF / CUTOF FOR CORNER TRACKING( 0.25 SUGGESTED )  /I./REL        *
* NSUR   / # OF SURFACES.                               /I./INT        *
* NVOL   / # OF ZONES.                                  /I./INT        *
* INDEL  / #ING OF SURFACES & ZONES.                    /I./INT(4*NVS) *
* MINDIM / MIN INDEX VALUES FOR ALL AXES (RECT/CYL).    /I./INT(NTOTCL)*
* MAXDIM / MAX INDEX VALUES FOR ALL AXES (RECT/CYL).    /I./INT(NTOTCL)*
* ICOORD / PRINCIPAL AXES DIRECTION (X/Y/Z) FOR MESHES. /I./INT(NTOTCL)*
*   ICUR / CURRENT ZONAL LOCATION FOR A TRACK SEGMENT.  /../INT(NTOTCL)*
*   INCR / INCREMENT DIRECTION FOR NEXT TRACK SEGMENT.  /../INT(NTOTCL)*
* TRKBEG / POSITION WHERE A TRACK BEGINS.               /../REL(NTOTCL)*
*   CONV / SEGMENTS OF TRACKS.                          /../REL(NTOTCL)*
* TRKDIR / DIRECTION OF A TRACK IN ALL AXES.            /../REL(NTOTCL)*
* LENGHT / RELATIVE LENGHT OF EACH SEGMENT IN A TRACK.  /../REL(LINMAX)*
* NUMERO / MATERIAL IDENTIFICATION OF EACH TRACK SEGMENT/../INT(LINMAX)*
* NORE   / TRACK NORMALIZATION (0:YES 1:NO)             /I./I          *
* NRMV   / VOLUME FACTORS ONLY (0:NO,1:YES).            /I./INT        *
* VOLTRK / VOLUME FACTORS                               /.O/DBL(...   )*
* NSOLAN / # OF ANGLES EFFECTIVELY CONSIDERED           /I./INT        *
*--------+--------------- R O U T I N E S --------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* XELEQN / TO SELECT 'EQN' ANGLES.                                     *
* XELLSR / TO FIND BEGINNING/ENDING SURFACES OF A TRACK.               *
* XELLIN / TO COMPUTE TRACKS LENGTHS OF A TRACK.                       *
************************************************************************
C
      IMPLICIT      NONE
C
      INTEGER       IPRT,NANGLE,NTOTCL,NEXTGE,MAXR,LINMAX,
     >              NSUR,NVOL,NGRP,NORE,NRMV,NSOUT,NREG
      REAL          DENUSR
C
      REAL          TRKBEG(NTOTCL), TRKDIR(NTOTCL), CONV(NTOTCL),
     >              REMESH(MAXR), LENGHT(LINMAX), DENSTY(*), RCUTOF,
     >              TRKEND(3), TRKORI(3), TRKOR2(3), PROJC2(3),
     >              ANGEQN(3,3), ANGLE2(3), ANGLE3(3), BARY(3),
     >              TRKCUT(3,2), TCUTOF(3,4), TORIC(3), ANGLES(3,*),
     >              SEGLEN(LINMAX), ZMAX
CMD      REAL          STAYIN(NG,LINMAX)
      DOUBLE PRECISION VOLTRK(NSUR:NVOL,0:*) 
      INTEGER       NSOLAN
      INTEGER       MINDIM(NTOTCL), MAXDIM(NTOTCL), ICUR(NTOTCL),
     >              ICOORD(NTOTCL), INCR(NTOTCL), NUMERO(LINMAX),
     >              INDEL(4,*), NSBEG(4), NSEND(4), NSCUT(2),
     >              NRSEG(LINMAX), KEYMRG(NSUR:NVOL),
     >              NCSEG, NOLDS, NNEWS
      INTEGER       ISYMM
      INTEGER       IQUART(4)
      LOGICAL       LANGLE, SWVOID
      EQUIVALENCE ( ANGEQN(1,2), ANGLE2 ), ( ANGEQN(1,3), ANGLE3 )
      CHARACTER     TEDATA*13
      INTEGER       NPAN, IOUT
      DOUBLE PRECISION DZERO
      PARAMETER   ( NPAN=3, IOUT=6, DZERO=0.0D0 )
C
      INTEGER       NDIM, NSOUTM, I, J, NPOINT, NPO2, NCUTOF, LINM2,
     >              NOTRAK, IVS, IANG, IVS1, IVS2, IGRP, ISB,
     >              NANGLS, IPAN, NESTIM, JANG, IX, IY, IREF1, I2, I3,
     >              NSOLMX, NTTRK, IANG0, NDEBS, N, IZZ, IANGL, K, K3,
     >              NCROS, LINE, JC, MAXPTS
      REAL          ANORM2, A, B, DENLIN, RCIRC, R2CIRC, DP, FCUTOF,
     >              SURTOT, VOLTOT, WEIGHT, DEPART, X, Y, ANN, ODDNXT,
     >              TOTLEN, TOTXXX, DDENST
CMD
      INTEGER   NG, NR, NS, MXSEG,  NMAT, 
     >          MATCOD(NR), INDSUR(-NS:-1),
     >          INDS(-NS:-1),ttseg,ttline
      REAL
     >          SURF(-NS:-1),
     >          SURFTM(-NS:-1),
     >          VOL(MAXPTS),
     >          XSTRC(0:NMAT,NG),XSDIA(0:NMAT,NG)
      DOUBLE PRECISION
     >          FACN(NG,NR),
     >          FACF(NG,NR),
     >          FCSUR(NG,-NS:-1),
CD     >          FR(NG,NR),
CD     >          FS(NG,-NS:-1),
     >          DSURF(-NS:-1),
     >          EXP2(NG)
      LOGICAL   LETAB, L2LNG
CD
      INTEGER     INTEGER2, INTEGER4, REAL4, REAL8
      PARAMETER ( INTEGER2=2, INTEGER4=3, REAL4=4, REAL8=6 )
C
C     VARIABLES TEMPORAIRES
      INTEGER   IR, IG, IL, IT, NOIL, NOIL0, NOIL9, IMAT
C      INTEGER   IANG, LINE
      REAL      ONE, ZERO,  TAU, XTMP,XTRTMP
      DOUBLE PRECISION DONE, DHALF, DEXTMP
      LOGICAL LPRNT
      PARAMETER ( DONE=1.0D0,  DHALF=0.5D0,
     >            ONE=1., ZERO=0. )
C     Table de (ONE-EXP)!!!!
      REAL E0, E1, PAS1, DX1, XLIM1
      INTEGER          MEX1, LAU
      PARAMETER      ( MEX1=7936 )
      COMMON /EXP1/ E0(0:MEX1),E1(0:MEX1),PAS1,DX1,XLIM1
      ANORM2( A, B ) = A*A + B*B
C
      L2LNG = .FALSE.
      LPRNT = IPRT.GE.5
      LPRNT = .TRUE.
C
C--------------------------------------------------------------------------
C
      NDIM= 3
      NSOUT = NS
      NSOUTM= -NSOUT
      NREG = NR
      NGRP = NG
C
C     ONE WEIGHT FOR ALL TRACKS
C     DENLIN= # OF TRACKS / CM
      DENLIN= SQRT(DENUSR)
C
C     COMPUTE THE CIRCUMSCRIBED RADIUS AND
C             THE  COORDINATES FOR THE TRUE CENTER OF THE CELL
      R2CIRC= 0.0
      DO 10 I = 1, 3
         BARY(I)= 0.5 * (REMESH(MAXDIM(I)) + REMESH(MINDIM(I)))
         IF( NEXTGE.EQ.1 )THEN
            CALL XABORT('XELTI3: TUBEZ NOT SUPPORTED')
         ELSE
            R2CIRC =  R2CIRC
     >             + (REMESH(MAXDIM(I)) - REMESH(MINDIM(I)))**2
         ENDIF
   10 CONTINUE
      ZMAX=MAX(ABS(REMESH(MAXDIM(3))),ABS(REMESH(MINDIM(I))))
      R2CIRC= 0.25 * R2CIRC
      RCIRC = SQRT(R2CIRC)
C
C     NPOINT= # OF TRACKS ALONG ONE PERPENDICULAR AXIS
      NPOINT= INT( 2. * RCIRC * DENLIN )
***** BEWARE ***** BEWARE ***** BEWARE ***** BEWARE ***** BEWARE *****
***** CHANGE THIS "NPOINT" PARAMETER HAS TREMENDOUS EFFECTS ON TRACKING
***** BEWARE ***** BEWARE ***** BEWARE ***** BEWARE ***** BEWARE *****
C
C     OTHER POSSIBLE CHOICES (EXPLORED WITHOUT SUCCESS) ARE ==>
C1-)  NPOINT=  INT( 2. * RCIRC * DENLIN ) + 1
C2-)  NPOINT= NINT( 2. * RCIRC * DENLIN )
C3-)  NPOINT= NINT( 2. * RCIRC * DENLIN ) + 1
C
C     KEEP "NPOINT" ODD & CORRECT DENSITY
      NPO2  = NPOINT / 2
      NPOINT= 2 * NPO2 + 1
      DP    = 2. * RCIRC / NPOINT
      DENLIN= 1. / DP
      DENUSR= DENLIN**2
      ODDNXT= (2*NPO2+3) / (2.*RCIRC)
      IF( RCUTOF.EQ.0.0 )THEN
         NCUTOF= 1
      ELSE
         NCUTOF= 4
      ENDIF
      LINM2= LINMAX-2*NCUTOF
      FCUTOF= 1.0/REAL(NCUTOF)
      NOTRAK= 0
      SURTOT= 0.0
      VOLTOT= 0.0
      IQUART(1)=1
      IQUART(2)=1
      IQUART(3)=1
      IQUART(4)=1
      IF(NEXTGE .EQ. 1) THEN
        IQUART(2)=0
        IQUART(3)=0
        IQUART(4)=0
        NSOLAN= (NANGLE * (NANGLE + 2)) / 2
      ELSE
        IF( ISYMM .EQ. 8 .OR. ISYMM .EQ. 24 )THEN
          NSOLAN= (NANGLE * (NANGLE + 2)) / 8
          IQUART(2)=0
          IQUART(3)=0
          IQUART(4)=0
        ELSE IF( ISYMM .EQ. 4 .OR. ISYMM .EQ. 20 )THEN
          NSOLAN= (NANGLE * (NANGLE + 2)) / 4
          IQUART(2)=0
          IQUART(4)=0
        ELSE IF( ISYMM .EQ. 2 .OR. ISYMM .EQ. 18 )THEN
          NSOLAN= (NANGLE * (NANGLE + 2)) / 4
          IQUART(3)=0
          IQUART(4)=0
        ELSE
          NSOLAN= (NANGLE * (NANGLE + 2)) / 2
        ENDIF
      ENDIF
      DDENST= 1.0/(NPAN*DENUSR)
      NANGLS=  (NANGLE * (NANGLE + 2)) / 2
      CALL XELEQN( 3, 0, ANGEQN )
      IANG= 0
      DO 15 JANG= 1, NANGLS
      DO 16 IPAN= 1, NPAN
         CALL    XELEQN( 3, NANGLE, ANGEQN )
   16 CONTINUE
        IF(IQUART(MOD(JANG-1,4)+1).NE.1 ) GO TO 15
        IANG= IANG+1
        DENSTY(IANG)= REAL(2*NSOLAN)
        ANGLES(1,IANG)= ANGEQN(1,1)
        ANGLES(2,IANG)= ANGEQN(2,1)
        ANGLES(3,IANG)= ANGEQN(3,1)
   15 CONTINUE
C
C     TO REINITIATE THE EQN ANGLES
      CALL XELEQN( 3, 0, ANGEQN )
      IF( NEXTGE.EQ.1 )THEN
         DDENST= 12.0*DDENST
      ENDIF
      WEIGHT= 0.5*DDENST/REAL(NSOLAN)
      IF( IPRT.GT.1 )THEN
C
C        PREPARE & PRINT THE ESTIMATED NUMBER OF TRACKS
         NESTIM= 0
         DEPART= - (NPO2+1) * DP
         X     = DEPART
         DO 20 IX = 1, NPOINT
            X     = X + DP
            Y     = DEPART
            DO 20 IY = 1, NPOINT
               Y = Y + DP
               IF( ANORM2( X, Y ) .LE. R2CIRC ) NESTIM= NESTIM + 1
   20    CONTINUE
         WRITE(IOUT,'(1H )')
         WRITE(IOUT,'( 8H0ECHO =  ,I8,20H TRACKS/AXIS/ANGLE      )')
     >                          NPOINT
      WRITE(IOUT,'( 8H ECHO =  ,I8,25H TRACKS/CIRCLE/ANGLE          )')
     >                          NESTIM
         NESTIM= NESTIM * NPAN * NSOLAN
      WRITE(IOUT,'(30H ECHO = NEXT ODD    DENSITY >  ,F15.7,4H/CM2)')
     >                                                ODDNXT**2
      WRITE(IOUT,'( 8H ECHO = ,28H ESTIMATED NUMBER OF TRACKS=  ,I8 )')
     >                                                           NESTIM
C
C        PREPARE PRINTING WITH VARIABLE FORMAT
         WRITE(IOUT,'(1H )')
         WRITE(IOUT,'( 8H0ECHO = ,I3,27H SOLID ANGLES TO BE TRACKED )')
     >                   NANGLS
         NSOLMX= MIN(9, NANGLS/10)
         IREF1 = 0
         WRITE(IOUT,'( 1H0,10(I1,9X))') (IREF1, IZZ=0,NSOLMX)
         WRITE(IOUT,'( 1H ,10(I1,9X))') (MOD(IZZ,10), IZZ=0,NSOLMX)
         WRITE(IOUT,'( 2H 0)')
         TEDATA= '(1H+,TXXX,I1)'
      ENDIF
C
C------------------INITIALIZE NORMALIZED FACTORS
C
      IF( NRMV.EQ.1 )THEN
         MXSEG = 0
         DO 12 IVS= NSUR, NVOL
           DO 11 IANG= 0, NSOLAN
              VOLTRK(IVS,IANG)= DZERO
   11      CONTINUE
   12    CONTINUE
      ELSE
         CALL XDISET(INDS(-NS),NS,0)
         CALL XDISET(INDSUR(-NS),NS,0)
      
          DO  3 IR=1,NR
          DO  3 IG=1,NG
             FACN(IG,IR)= DZERO
             FACF(IG,IR)= DZERO
    3     CONTINUE
          DO  5 IR=-NS,-1
             DO  1 IG=1,NG
               FCSUR(IG,IR)= DZERO
    1        CONTINUE
             DSURF(IR)= DZERO
    5     CONTINUE
      ENDIF
C
      ttseg=0
      ttline=0
C---------------------------------------------------------------------
C------------------------------TRACKING-------------------------------
C---------------------------------------------------------------------
      IANG  = 0
      IANG0 = 0
      NTTRK = 0
      DO 290 IANGL= 1, NANGLS
         IANG=IANG+1
         LANGLE= .FALSE.
         IF(IQUART(MOD(IANG-1,4)+1).NE.1)THEN
C----
C  Do not track this angle because of the problem symmetry
C----
           LANGLE= .FALSE.
         ELSE
C----
C  Track this angle
C----
           IANG0= IANG0+1
           LANGLE=.TRUE.
         ENDIF
         IF( IPRT.GT.1) THEN
           IF( MOD(IANGL,100) .EQ. 0 )THEN
             IREF1=IREF1+1
             NDEBS= NSOLMX+1
             NSOLMX=MIN(NDEBS+9, NANGLS/10)
             WRITE(IOUT,'( 1H0,10(I1,9X))')(IREF1,IZZ=NDEBS,NSOLMX)
             WRITE(IOUT,'( 1H ,10(I1,9X))')
     >            (MOD(IZZ,10),IZZ=NDEBS,NSOLMX)
             WRITE(IOUT,'( 2H 0)')
           ELSE
             IF( IPRT.GT.10000.AND.MOD(IANGL,100).NE.0 )THEN
               WRITE(IOUT,'( 1H ,10(I1,9X))')(IREF1,IZZ=NDEBS,NSOLMX)
               WRITE(IOUT,'( 1H ,10(I1,9X))')
     >               (MOD(IZZ,10),IZZ=NDEBS,NSOLMX)
               WRITE(IOUT,'( 2H  )')
             ENDIF
             WRITE(TEDATA(7:9),'(I3.3)') MOD(IANGL,100) + 2
             WRITE(IOUT,TEDATA) MOD(IANGL,10)
           ENDIF
         ENDIF
C
C     NPAN AXES DESCRIPTION (X=0.0, Y=0.0 & Z=0.0)
      DO 250 IPAN= 1, NPAN
         CALL    XELEQN( 3, NANGLE, ANGEQN )
         IF(.NOT.LANGLE) GO TO 250
         IF( NEXTGE.EQ.1 )THEN
            IF( IPAN.NE.2 ) GO TO 250
         ENDIF
         DO 30 I   = 1, 3
            N     = ICOORD(I)
            TRKDIR(N)= ANGEQN(N,1)
            INCR(I)= +1
            IF( TRKDIR(N) .LT. 0.0 ) INCR(I)= -1
C
C           MODIFY ANGLES TO TAKE INTO ACCOUNT DP
            ANGLE2(I)= DP * ANGLE2(I)
            ANGLE3(I)= DP * ANGLE3(I)
            IF( NCUTOF.NE.1 )THEN
               TCUTOF(I,1)= RCUTOF*( ANGLE2(I)+ ANGLE3(I) )
               TCUTOF(I,2)= RCUTOF*( ANGLE2(I)- ANGLE3(I) )
               TCUTOF(I,3)= -TCUTOF(I,2)
               TCUTOF(I,4)= -TCUTOF(I,1)
            ENDIF
C
C           DETERMINE THE ORIGIN OF ALL TRACKS
            TRKOR2(I)= BARY(I) - (NPO2+1)*(ANGLE2(I)+ANGLE3(I))
   30    CONTINUE
         DO 45 I   = 1, 3
            PROJC2(I)= 0.0
            DO 40 J   = 1, 3
               IF( I.EQ.J ) GO TO 40
               PROJC2(I)= PROJC2(I) + TRKDIR(J) * TRKDIR(J)
   40       CONTINUE
   45    CONTINUE
C
C        SCAN ALL TRACKS IN THE PERPENDICULAR PLANE
         DO 180 I2  = 1, NPOINT
            DO 50 J   = 1, 3
               TRKOR2(J)= TRKOR2(J) + ANGLE2(J)
               TRKORI(J)= TRKOR2(J)
   50       CONTINUE
            DO 170 I3  = 1, NPOINT
               ANN   = 0.0
               DO 60 J    = 1, 3
                  TRKORI(J)= TRKORI(J) + ANGLE3(J)
                  ANN= ANN + (TRKORI(J)-BARY(J))**2
   60          CONTINUE
C
C              ELIMINATE TRACKS OUTSIDE CIRCUMSCRIBED CIRCLE
               IF( ANN.GT.R2CIRC ) GO TO 170
*
*              WRITE(IOUT,7002) I2,I3,(TRKORI(JJ),JJ=1,3)
*7002          FORMAT(' ORIGINE MESH:',I10,5X,I10,5X,3(F11.5))
C
C              WHICH EXTERNAL SURFACES DO THIS TRACK CROSS ?
               NTTRK=NTTRK+1
               CALL XELLSR(  NDIM, NTOTCL, NSUR, MAXR, REMESH,
     >                      INDEL, MINDIM, MAXDIM, ICOORD, ICUR, INCR,
     >                      TRKORI, TRKDIR, TRKCUT, NSCUT, NCROS,
     >                      TOTLEN)
C
C              WHEN NOT SURFACES ARE CROSSED, ELIMINATE THE TRACK
               IF(NCROS.LT.2) GO TO 170
               DO 70 K= 1, NDIM
                  TRKBEG(K)= TRKCUT(K,1)
                  TRKEND(K)= TRKCUT(K,2)
   70          CONTINUE
               DO 75 K= 1, 4
                  NSBEG(K)= NSCUT(1)
                  NSEND(K)= NSCUT(2)
   75          CONTINUE
               IF( NCUTOF.NE.1 )THEN
                  DO 77 K= 1, 4
                     DO 76 K3= 1, 3
                        TORIC(K3)= TRKORI(K3)+TCUTOF(K3,K)
   76                CONTINUE
                     CALL XELLSR( NDIM, NTOTCL, NSUR, MAXR, REMESH,
     >                       INDEL, MINDIM, MAXDIM, ICOORD, ICUR, INCR,
     >                           TORIC, TRKDIR, TRKCUT, NSCUT, NCROS,
     >                           TOTXXX)
                     IF(NSCUT(1).NE.0) NSBEG(K)= NSCUT(1)
                     IF(NSCUT(2).NE.0) NSEND(K)= NSCUT(2)
   77             CONTINUE
               ENDIF
               CALL XELLIN(   NDIM, NTOTCL, MAXR, REMESH,
     >                NSUR,   NVOL,  INDEL, MINDIM, MAXDIM,
     >              ICOORD,   ICUR,   INCR, TRKBEG, TRKEND, TRKDIR,
     >              PROJC2, TOTLEN,   CONV,  LINM2,
     >              LENGHT(NCUTOF+1), NUMERO(NCUTOF+1), LINE)
               NOTRAK= NOTRAK+1
C
               DO 78 ISB=1,NCUTOF
                  NUMERO(ISB)= NSBEG(ISB)
                  LENGHT(ISB)= FCUTOF
                  NUMERO(NCUTOF+LINE+ISB)= NSEND(ISB)
                  LENGHT(NCUTOF+LINE+ISB)= FCUTOF
   78          CONTINUE
               LINE= LINE+2*NCUTOF
               IF( IPRT.GT.10000)THEN
                  WRITE(IOUT,6001) NOTRAK,
     >                         NCUTOF,(TRKBEG(JC),JC=1,3),
     >                         NCUTOF,(TRKEND(JC),JC=1,3),
     >                                (TRKDIR(JC),JC=1,3)
                  WRITE(IOUT,6002) (LENGHT(I),NUMERO(I),I=1,LINE)
               ENDIF
               IF( NRMV.EQ.1 )THEN
                  MXSEG= MAX(MXSEG,LINE)
                  DO 301 I= 1, LINE
                     VOLTRK(NUMERO(I),IANG0)= VOLTRK(NUMERO(I),IANG0)
     >                                    + DBLE( WEIGHT * LENGHT(I))
  301             CONTINUE
               ELSE
                  IF( NORE.EQ.0 )THEN
                     DO 302 I= 1, LINE
                        IF( NUMERO(I).GT.0 )THEN
                           LENGHT(I) = LENGHT(I) *
     >                                 SNGL(VOLTRK(NUMERO(I),IANG0))
                        ENDIF
  302                CONTINUE
                  ENDIF
C
                  DO 303 I= 1, LINE
                     NUMERO(I)= KEYMRG(NUMERO(I))
  303             CONTINUE
C
C                 START MODIFICATIONS 98/06 (G.M.)
C                 COMPRESS TRACKING FILE FOR
C                          SUCCESSIVE IDENTICAL REGIONS
                  NOLDS=NUMERO(1)
                  NCSEG=1
                  DO 304 IL = 2, LINE
                     NNEWS=NUMERO(IL)
                     IF( NNEWS.LT.0 .OR. NNEWS.NE.NOLDS )THEN
                        NOLDS=NNEWS
                        NCSEG=NCSEG+1
                        NUMERO(NCSEG)=NUMERO(IL)
                        LENGHT(NCSEG)=LENGHT(IL)
                     ELSEIF( NNEWS.EQ.NOLDS )THEN
                        LENGHT(NCSEG)=LENGHT(NCSEG)+LENGHT(IL)
                     ENDIF
  304             CONTINUE
                  LINE=NCSEG
C---------------------------------------------------------------------------
C---------------------------- COMPUTE Pjj-----------------------------------
C---------------------------------------------------------------------------
C     Calcul les surfaces
                  NOIL0= NUMERO(1)
                  INDSUR(NOIL0)= NUMERO(2)
                  DSURF(NOIL0) = DSURF(NOIL0) +DBLE(WEIGHT)
CD
                  NOIL9= NUMERO(LINE)
                  INDSUR(NOIL9)= NUMERO(LINE-1)
                  DSURF(NOIL9) = DSURF(NOIL9) + DBLE(WEIGHT)
C       Calcul les probabilites
C step1: When IL=2
C---------------------
                  IL=2
                  NOIL= NUMERO(2)
                  IMAT= MATCOD(NOIL)
                  IF( LETAB ) THEN
                    DO 101 IG= 1,NG
                      TAU =  XSTRC(IMAT,IG)*LENGHT(2)
                      LAU = INT( TAU*PAS1 )
                      IF( LAU.GT.MEX1 ) THEN
                         LAU = MEX1
                         L2LNG = .TRUE.
                      ENDIF
                      EXP2(IG) = (E0(LAU)+E1(LAU)*TAU)*WEIGHT
                      FACN(IG,NOIL)= FACN(IG,NOIL) + EXP2(IG)
                      FCSUR(IG,NOIL0)= FCSUR(IG,NOIL0) + EXP2(IG)
 101                CONTINUE
                  ELSE
                    DO 102 IG= 1,NG
                      TAU =  XSTRC(IMAT,IG)*LENGHT(2)
                      EXP2(IG)= WEIGHT - EXP(-TAU)*WEIGHT
                      FACN(IG,NOIL)= FACN(IG,NOIL) + EXP2(IG)
                      FCSUR(IG,NOIL0)= FCSUR(IG,NOIL0) + EXP2(IG)
 102                CONTINUE
                 ENDIF
c     STEP2: WHEN IL IS NOT AT THE BORD
                 IF( LETAB ) THEN
                     DO 103 IL= 3, LINE-1
                       NOIL = NUMERO(IL)
                       IMAT = MATCOD(NOIL)
                       DO 104 IG= 1,NG
                         TAU =  XSTRC(IMAT,IG)*LENGHT(IL)
                         LAU = INT( TAU*PAS1 )
                         IF( LAU.GT.MEX1 ) THEN
                            LAU = MEX1
                            L2LNG = .TRUE.
                         ENDIF
                         EXP2(IG)=( E0(LAU)+E1(LAU)*TAU )*WEIGHT
                         FACN(IG,NOIL)= FACN(IG,NOIL) + EXP2(IG)
 104                   CONTINUE
 103                 CONTINUE
                  ELSE
                     DO 105 IL= 3, LINE-1
                       NOIL = NUMERO(IL)
                       IMAT = MATCOD(NOIL)
                       DO 106 IG= 1,NG
                         TAU =  XSTRC(IMAT,IG)*LENGHT(IL)
                         EXP2(IG)= WEIGHT - EXP(-TAU) * WEIGHT
                         FACN(IG,NOIL)= FACN(IG,NOIL) + EXP2(IG)
 106                   CONTINUE
 105                CONTINUE
                 ENDIF
C     STEP3: WHEN IL= LINE-1
                 DO 109 IG= 1,NG
                   FCSUR(IG,NOIL9)= FCSUR(IG,NOIL9) + EXP2(IG)
 109             CONTINUE
          ENDIF
C
C------------END PJJ------------------------------------------------------
C
  170       CONTINUE
  180    CONTINUE
  250 CONTINUE
  290 CONTINUE
CC
      IF( NRMV.NE.1 )THEN      
         DO  4 IR=1,NR
         DO  4 IG=1,NG
           FACN(IG,IR)= 2.D0*FACN(IG,IR)
           FACF(IG,IR)= FACN(IG,IR)
    4    CONTINUE
C
         DO  7 IR=-NS,-1
           DEXTMP= DSURF(IR)
CD        SURF(IR)= REAL(DEXTMP)
          SURFTM(IR)= REAL(DSURF(IR))
          NOIL= INDSUR(IR)
          IMAT= MATCOD(INDSUR(IR))
          DO  6 IG=1,NG
            XTRTMP=XSTRC(IMAT,IG)
            FACN(IG,NOIL)= FACN(IG,NOIL)
     >                 - FCSUR(IG,IR)*FCSUR(IG,IR)
     >                   /DEXTMP
            FCSUR(IG,IR)= FCSUR(IG,IR)/XSTRC(IMAT,IG)
    6     CONTINUE
    7   CONTINUE
C
        DO  IR=-NS,-1
           SURF(IR)=SURFTM(IR)
        ENDDO
        DO  2 IR=1,NR
        DO  2 IG=1,NG
           FACN(IG,IR)= DONE - FACN(IG,IR)/DBLE(VOL(IR))
     >                       /DBLE( XSTRC(MATCOD(IR),IG) )
           FACN(IG,IR)= FACN(IG,IR)/XSTRC(MATCOD(IR),IG)
    2   CONTINUE
C
      ENDIF
C
      IF( IPRT.GT.1 )THEN
         WRITE(IOUT,'(1H )')
         WRITE(IOUT,'(27H0ECHO = TRACKING PROPERTIES )')
         WRITE(IOUT,'( 8H0ECHO = ,I3,20H ANGLES AND DENSITY:,
     >               F9.3,4H/CM2)')
     >                         NANGLE, DENUSR
         WRITE(IOUT,'( 8H0ECHO = ,I10,3H / ,I10,
     >                             27H TRACKS NOT STORED ON TAPE /)')
     >                         NOTRAK,NTTRK
      ENDIF
C
      RETURN
C
 6001 FORMAT(' #',I8,1P,' B',I1,'(',2(E10.2,','),E10.2,')',
     >                  ' E',I1,'(',2(E10.2,','),E10.2,')',
     >                  ' D(',2(E10.2,','),E10.2,')' )
 6002 FORMAT(1P,5(1X,E15.7,1X,I6))
      END
