************************************************************************
      SUBROUTINE MCUDRV(IPFLUX, IPTRK, IPGEOM, IPMACR,  
     >     MAXPTS ,NORE, TITLE, CUTOFX , NGRP, MAXMIX, XSSIGT ,
     >     XSSIGW, MATCOD,  VOLUME, KEYFLX, RB2   , 
     >     NIFIS , NANIS , NL, NUN  , NBN   , 
     >     NINTMX, AIEPS , NEXTMX, AEEPS,IFRITR,IACITR,ITYPEC,
     >     ILEAK , XSOLD , XSCHI , XSNUF , XSCAT, VSCAT, XSTK ,
     >     LEAKSW, ILK   , ICAC  , ITRANC, ITRB  ,
     >     ITCRMX, ATCRMX, LJACC , LITLM , LINIF ,
     >                        LETAB , MGTRK , MAXSCR,   IPRT  )
************************************************************************
*                                                                      *
*           NAME: MCUDRV                                               *
*      COMPONENT: MCU                                                  *
*         AUTHOR: M. DAHMANI                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE IS USED TO: SET UP DRIVER FOR CARTESIAN *
*                 2D/3D ASSEMBLIES                                     *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPTRK  / POINTER TO THE EXCELL TRACKING (L_EXCELL)    /IO/INT        *
* IPGEOM / POINTER TO THE GEOMETRY (L_GEOM)             /I./INT        *
* IPRT   / TRACKING PRINT LEVEL                         /I./I          *
* MAXPTS / # OF ZONES ACCORDING TO USER                 /I./INT        *
* NORE   / TRACK NORMALIZATION (0:YES 1:NO)             /I./I          *
* TITLE  / TITLE FOR THIS CASE                          /I./C*72       *
* CUTOFX / MFP CUTOFF FOR SPECULAR INTEGRATION          /I./REL        *
* NGRP   / NUMBER OF GROUPS                             /I./I          *
* MAXMIX / # OF MIXTURES IN THE MACROLIB                /I./INT        *
* XSSIGT / TOTAL XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRP) *
* XSSIGW / SCATT XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRP) *
* MATCOD / MATERIAL #S FOR ZONES IN THE SUPERCELL       /.O/INT(MAXPTS)*
* VOLUME / VOLUMES OF ZONES IN THE SUPERCELL            /.O/REL(MAXPTS)*
* KEYFLX / ZONE KEY FOR THE UNKNOWN VECTORS (FLUXES...) /.O/REL(MAXPTS)*
*--------+---------------- R O U T I N E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* MCUTRK / TO TREAT A CARTESIAN ASSEMBLY OF CELL(GEOMETRY/TRACKING).   *
************************************************************************
      IMPLICIT        NONE
      INTEGER         NSTATE
      PARAMETER     ( NSTATE=40 )
      INTEGER         IPTRK, IPGEOM, NSUR, NVOL
      INTEGER         MAXPTS,IPRT,NREG,NUNKNO,
     >                NORE,ISTATE(NSTATE),
     >                ITYPE,MAXMIX,NGRP,I,NTRK,IR,NSOUT,
     >                NDIM,ISPEC,NANGL,MXSEG,IOUT,NMAT,
     >                ITGEO,IUSED(6),ICODE(6),ISYMM
      INTEGER         MATCOD(MAXPTS),KEYFLX(MAXPTS)
CMD      INTEGER         MATCOD(NREG),KEYFLX(NREG)
      LOGICAL         LEAKSW, LCM
      REAL            VOLUME(MAXPTS),CUTOFX,ALBEDO(6),EXTKOP(NSTATE),
     >                ZERO, ONE,XSSIGT(0:MAXMIX,NGRP),
     >                XSSIGW(0:MAXMIX,NGRP),PCORN,DENS
      CHARACTER       GEONAM*12,TITLE*72,CORIEN(6)*4,CUSED(0:1)*6,
     >                TEXT12*12,NAMT*12
C
      PARAMETER     ( IOUT=6, ZERO=0.0, ONE=1.0 )
CMD
      INTEGER   IUNOUT, ILCMUP, ILCMDN, NDIMO, NANIS, NIFIS, NUN,
     >          IPMACR, IPFLUX, ITYPEC,
     >          IFRITR, IACITR, ILEAK, ICAC
      REAL      EPSR, RB2 
      PARAMETER (IUNOUT=6,ILCMUP=1,ILCMDN=2)
      PARAMETER (NDIMO=3, EPSR=1.E-6)
      LOGICAL     ILK, LJACC, LITLM, LINIF, LETAB
      CHARACTER   TITRE*72
      INTEGER     ITCRMX,MGTRK,
     >            ITRANC,ITRB
      INTEGER     NL, NBN, NINTMX, NEXTMX
      REAL        AEEPS, AIEPS, ATCRMX, MAXSCR
      REAL        XSOLD(0:MAXMIX,NGRP),
     >            XSCAT(NGRP,0:MAXMIX,0:NANIS,NGRP),VSCAT(*),
     >            XSNUF(0:MAXMIX,NIFIS,NGRP),XSCHI(0:MAXMIX,NIFIS,NGRP),
     >            XSTK(MAXMIX,NIFIS)
CMD
      DATA          ( CORIEN(IR),IR=1,6)
     >              / ' X- ',' X+ ',' Y- ',' Y+ ',' Z- ',' Z+ '/
      DATA           ( CUSED(IR),IR=0,1 ) / 'UNUSED','  USED' /
C
      NMAT=MAXMIX
C
      IF( IPRT.GT.0 ) WRITE(IOUT,'(1X,A72//)') TITLE
      IF( IPGEOM.EQ.0 )THEN
         CALL XABORT('MCUDRV: USER MUST PROVIDE A GEOMETRY')
      ELSE
         NAMT=' '
         CALL LCMNXT(IPGEOM,NAMT,GEONAM,TEXT12,LCM)
      ENDIF
      NTRK  = 0
C
      IF( IPRT.GT.0 )THEN
         WRITE(IOUT,'(27H ECHO = >>> GEOMETRY NAME: ,A8)') GEONAM
         IF(     NORE.EQ.-2 )THEN
            WRITE(IOUT,'(30H ECHO = >>>                   )')
         ELSEIF( NORE.EQ.-1 )THEN
            WRITE(IOUT,'(30H ECHO = >>> NORMALIZED-GLOBAL )')
         ELSEIF( NORE.EQ. 0 )THEN
            WRITE(IOUT,'(30H ECHO = >>> NORMALIZED-LOCAL  )')
         ELSEIF( NORE.EQ.+1 )THEN
            WRITE(IOUT,'(30H ECHO = >>> NOT NORMALIZED    )')
         ENDIF
      ENDIF
C
C        THERE IS A GEOMETRY, GO TO EXCELL MODULES TO ANALYZE IT
         CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
         ITYPE= ISTATE(1)
         IF( ITYPE.EQ.3.OR.ITYPE.EQ.6 )THEN
            ITGEO= 1
         ELSEIF( ITYPE.EQ. 8 .OR. ITYPE.EQ. 9 .OR.
     >           ITYPE.EQ.24 .OR. ITYPE.EQ.25 )THEN
            ITGEO= 2
         ELSEIF( ITYPE.EQ. 5 .OR. ITYPE.EQ. 7 .OR. ITYPE.EQ.20 .OR.
     >           ITYPE.EQ.21 .OR. ITYPE.EQ.22 .OR. ITYPE.EQ.23 )THEN
            ITGEO= 3
         ELSE
            ITGEO= 0
         ENDIF
         IF( ITGEO.EQ.3 )THEN
C
C--------------------DRIVER FOR CARTESIAN 2D/3D ASSEMBLIES----------
C
            CALL MCUTRK(  IPGEOM,  IPTRK, GEONAM, 
     +                    IPRT,   NDIM,  ISPEC, 
     +                    NANGL,  ISYMM,   DENS,  PCORN,  MXSEG,
     +                    ALBEDO,  ICODE,   NORE,
     +                    NGRP, MAXMIX, XSSIGT, XSSIGW,
     +                    TITLE, MAXPTS, MATCOD, VOLUME, KEYFLX,
     +                    LEAKSW,  IUSED , 
     +                    IPFLUX, IPMACR,RB2, NIFIS,NANIS,NL,NUN, 
     +                    NBN,NINTMX, AIEPS , NEXTMX, AEEPS ,
     +                    IFRITR, IACITR, ITYPEC, ILEAK , 
     +                    XSOLD , XSCHI , XSNUF , XSCAT , VSCAT ,
     +                    XSTK  , ILK   , ICAC  , ITRANC, ITRB  ,
     +                    TITRE , ITCRMX, ATCRMX, LJACC , LITLM ,
     +                    LINIF , LETAB, MGTRK , MAXSCR, NSUR,NVOL)
         ELSE
            CALL XABORT('MCUDRV: INVALID TYPE OF GEOMETRY')
         ENDIF
C
C     STOCK INFORMATION (OUTPUT TO DRAGON DRIVER)
      NREG = NVOL
      NSOUT = -NSUR 
      NUNKNO= NREG+NSOUT+1
C
      IF( IPRT.GT.0 )THEN
         IF( IPRT.GT.1 )THEN
            WRITE(IOUT,'(1H )')
            WRITE(IOUT,'(8H  SIDE  ,2X,6(7X,A4))')
     >                                  (CORIEN(IR),IR=1,6)
            WRITE(IOUT,'(8H  GEOM #,2X,6(7X,I4.0))')
     >                                  (MAX(0,-ICODE(IR)),IR=1,6)
            WRITE(IOUT,'(8H  PHYS #,2X,6(7X,I4.0))')
     >                                  (MAX(0,ICODE(IR)),IR=1,6)
            WRITE(IOUT,'(8H  ALBEDO,2X,1P,6E11.4)')
     >                                  (ALBEDO(IR),IR=1,6)
            WRITE(IOUT,'(8H        ,2X,6(5X,A6))')
     >                                  (CUSED(IUSED(IR)),IR=1,6)
            WRITE(IOUT,'(1H )')
         ENDIF
C         WRITE(IOUT,'(50H ECHO = >>>   CHARACTERISTICS CALCULATION  )')
         IF( CUTOFX.EQ.ZERO )THEN
            WRITE(IOUT,'(27H ECHO = >>>    NO CUT-OFF     )')
         ELSE
            WRITE(IOUT,'(27H ECHO = >>> MFP.  CUT-OFF: ,1P,E10.4 )')
     >      CUTOFX
         ENDIF
         WRITE(IOUT,'(28H ECHO = >>> NB. OF REGIONS: ,I5)') NREG
      ENDIF
C
C     SAVE GENERAL TRACKING INFORMATION
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NREG
      ISTATE(2)=NUNKNO
      IF( LEAKSW )THEN
         ISTATE(3)=0
      ELSE
         ISTATE(3)=1
      ENDIF
      IR=0
      DO 100 I=1,NREG
         IR=MAX(IR,MATCOD(I))
  100 CONTINUE
      ISTATE(4)=IR
      ISTATE(5)=NSOUT
      ISTATE(6)=1
      ISTATE(7)=1
      ISTATE(8)=NORE
      ISTATE(9)=0
      ISTATE(10)=0
      ISTATE(11)=NANGL
      ISTATE(12)=ISYMM
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,ISTATE)
C
C     SAVE THE SPECIFIC TRACKING INFORMATION
      CALL LCMPUT(IPTRK,'MATCOD',MAXPTS,1,MATCOD)
      CALL LCMPUT(IPTRK,'VOLUME',MAXPTS,2,VOLUME)
      CALL LCMPUT(IPTRK,'KEYFLX',MAXPTS,1,KEYFLX)
      CALL LCMPUT(IPTRK,'ALBEDO',6,2,ALBEDO)
      CALL XDRSET(EXTKOP,NSTATE,0.0)
      EXTKOP(1)=CUTOFX
      EXTKOP(2)=DENS
      EXTKOP(3)=PCORN
      CALL LCMPUT(IPTRK,'EXCELTRACKOP',3,2,EXTKOP)
      IF( IPRT.GE.1 )THEN
        WRITE(IOUT,6000) IPRT,12,(ISTATE(IR),IR=1,12)
        WRITE(IOUT,6001)(EXTKOP(IR),IR=1,3)
      ENDIF
      RETURN
C
 6000 FORMAT(' MCU PRINT LEVEL =',I8/
     > ' TRACK STATE-VECTOR  (',I2,') ELEMENTS'/
     > ' -----------------'/
     > ' NREG   =',I8,' (NUMBER OF REGIONS)'/
     > ' NUNKNO =',I8,' (NUMBER OF UNKNOWNS IN SYSTEM)'/
     > ' ILEAK  =',I8,' (LEAKAGE FLAG)'/
     > ' MAXMIX =',I8,' (NUMBER OF MIXTURES)'/
     > ' NSOUT  =',I8,' (NUMBER OF OUTER SURFACES)'/
     > ' NANIS  =',I8,' (FLUX ANISOTROPY ORDER)'/
     > ' ITROP  =',I8,' (GEOMETRY TYPE)'/
     > ' NORE   =',I8,' (TRACK NORMALIZATION OPTION)'/
     > ' KTOPT  =',I8,' (TYPE OF TRACKING)'/
     > ' KSEPC  =',I8,' (TYPE OF BOUNDARY CONDITION)'/
     > ' NANGL  =',I8,' (NUMBER OF TRACKING ANGLES)'/
     > ' ISYMM  =',I8,' (TRACKING SYMMETRY FACTOR)'/
     > ' -----------------'/ )
 6001 FORMAT(
     > '   TRACK OPTIONS '/
     > ' CUTOFX =',F20.8,' (CUTOFF FOR TRACK LENGTH)'/
     > ' DENS   =',F20.8,' (TRACK DENSITY)'/
     > ' PCORN  =',F20.8,' (CORNER DUPLICATION DISTANCE)'/
     > ' -----------------'/ )
      END
