*DECK MOC2BL
      SUBROUTINE MOC2BL( NG, IG0, NA, PHI, XCSOU, XCTRC, XCDIA, XCCAT,
     >                   DB2, REBAL, FLUN, IPRT , JGDEB, LDIRECT )
*---------------------------  MOC2BL  ---------------------------------
*
*   1- PROGRAMME STATISTICS:
*       NAME     :  MOC2BL
*       USE      :  REBALANCE SPEEDING-UP
*       AUTHOR   :  R. ROY (01/01/98)
*       MODIFIED :  MONCHAI A. (01/01/05)
*
*   2- ROUTINE PARAMETERS:
*    INPUT
*       DB2      : D x B^2  COEFFICIENT               D(NG)
*       IG0      : STARTING GROUP INDEX               I
*       JGDEB    : ENDING GROUP INDEX                 I
*       NG       : NUMBER OF ENERGY GROUPS            I
*       PHI      : DOMAIN-INTEGRATED GROUP FLUXES     D(NG)
*       XCDIA    : HOMOGENIZED WITHIN-GROUP           D(NG)
*                  SCATTERING CROSS SECTION
*       XCCAT    : HOMOGENIZED GROUP                  D(NG,0:NA,NG)
*                  SCATTERING CROSS SECTION
*       XCSOU    : DOMAIN-INTEGRATED GROUP TOTAL      D(NG)
*                  FISSION + SCATTERING SOURCES
*       XCTRC    : HOMOGENIZED GROUP TRANSPORT-       D(NG)
*                  CORRECTED TOTAL CROSS SECTION
*
*    WORKING
*       REBAL    : REBALANCE MATRIX FOR SOLUTION OF   D(NG,NG+1)
*                  LINEAR EQUATION
*
*    OUTPUT
*       FLUN     : GROUP REBALANCE FACTOR             D(NG)
*
*
*---------------------------   MOC2BL  --------------------------------
*
      IMPLICIT  NONE
      INTEGER   NG, NA, IPRT, IG0, IG, JG, JGDEB
      INTEGER   NGUNC, NSOUR, IGUNC, JGUNC, IER,IOUT
      DOUBLE PRECISION PHI(NG),XCSOU(NG),XCTRC(NG),XCDIA(NG),
     >          XCCAT(NG,0:NA,NG),DB2(NG),
     >          FLUN(NG),REBAL(NG,NG+1),DZERO
      LOGICAL   LDIRECT
      PARAMETER (IOUT=6,DZERO=0.0D0 )
      NGUNC = NG - IG0 + 1  - JGDEB
      NSOUR= NGUNC + 1
      IGUNC= 0
      DO  3 IG= IG0,(NG-JGDEB)
         IGUNC= IGUNC+1
         DO  1 JG=1,NSOUR
            REBAL(IGUNC,JG)= DZERO 
    1    CONTINUE
         REBAL(IGUNC,IGUNC)= REBAL(IGUNC,IGUNC)+XCTRC(IG)*PHI(IG)
         IF(LDIRECT) THEN
           REBAL(IGUNC,IGUNC)= REBAL(IGUNC,IGUNC)+DB2(IG)*PHI(IG)
         ELSE
CCC        REBAL(IGUNC,IGUNC)= REBAL(IGUNC,IGUNC)+DB2(IG)*PHI(IG) 
         ENDIF
         REBAL(IGUNC,NSOUR)= REBAL(IGUNC,NSOUR)+XCSOU(IG)
         JGUNC= 0
         DO  7 JG= 1, NG
          IF( JG.GE.IG0 .AND.  JG .LE. (NG-JGDEB) )THEN
            JGUNC= JGUNC+1
            IF(IG.EQ.JG)THEN
             REBAL(IGUNC,JGUNC)= REBAL(IGUNC,JGUNC)-XCDIA(JG)*PHI(JG)
            ELSE
             IF(LDIRECT) THEN
              REBAL(IGUNC,JGUNC)= REBAL(IGUNC,JGUNC)-
     >                            XCCAT(JG,0,IG)*PHI(JG)
             ELSE
              REBAL(IGUNC,JGUNC)= REBAL(IGUNC,JGUNC)-
     >                            XCCAT(IG,0,JG)*PHI(JG)
             ENDIF
            ENDIF
          ELSE
            IF(IG.EQ.JG)THEN
              REBAL(IGUNC,NSOUR)= REBAL(IGUNC,NSOUR)+XCDIA(JG)*PHI(JG)
            ELSE
              IF(LDIRECT) THEN
               REBAL(IGUNC,NSOUR)= REBAL(IGUNC,NSOUR)+
     >                             XCCAT(JG,0,IG)*PHI(JG)
              ELSE
               REBAL(IGUNC,NSOUR)= REBAL(IGUNC,NSOUR)+
     >                             XCCAT(IG,0,JG)*PHI(JG)
              ENDIF
            ENDIF
          ENDIF
    7    CONTINUE
    3 CONTINUE
      IF( IPRT.GT.10000 )THEN
         DO 32 IGUNC= 1, NGUNC
           WRITE(IOUT,1000) (REBAL(IGUNC,JGUNC),JGUNC=1,NGUNC+1)
   32    CONTINUE
      ENDIF
      CALL MOCSOL(NGUNC,1,REBAL,IER,NG)
      IF( IER.NE.0 ) CALL XABORT(' MOC2BL: SINGULAR REBALANCING MATRIX')
      IGUNC= 0
      DO 13 IG= IG0, (NG-JGDEB)
         IGUNC= IGUNC+1
         FLUN(IG)= REBAL(IGUNC,NSOUR)
         IF( IPRT.GT.1000 )THEN
           WRITE(IOUT,7000) IG,REBAL(IGUNC,NSOUR)
         ENDIF
   13 CONTINUE
      IF(IPRT.GT.5)THEN
         WRITE(IOUT,1040)(REBAL(IGUNC,NSOUR),IGUNC=1,NGUNC)
      ENDIF
      RETURN
 1040 FORMAT (7H REBA  ,2H: ,1P,10E12.5/(9X,10E12.5))
 1000 FORMAT(1X,1P,10(E15.6,2X))
 7000 FORMAT(1X,'REBAL(',I10,')=',1P,E20.7)
      END
