*DECK MOC2DR
      SUBROUTINE MOC2DR(IPFLUX,IPTRK,IFTRAK,IPMACR,NGRP,NREG,NSOUT,
     >              NANIS,NMAT,NGSS,MXSEG, NTRK, NPHI, NFUNL, NMOD,
     >              KEYANI,MATALB,VOL,VOLINV,WGSS,XGSS,YGSS,PHIANG,
     >              SEGLEN,NRSEG,EXPT,EXP2,EXPTAJ,EXP2AJ,CYCLE,FLM,
     >              FLP,FLMADJ,FLPADJ,CYM,CYP,CYMAJ,CYPAJ,ALPHA,DFLUX,
     >              DWEIG,FMEAN,SIGANG,NALB,ALBEDO,XSTOT,XSTRC,
     >              XTINV,XSDIA,XSCAT,NIFIS,XSNUF,XSCHI,ITRB,
     >              FLUX,FLUXO,DIFFB1,ISGNR,RHARM,LEAKSW,LREBAL,
     >              NINTMX,AIEPSF,NEXTMX,AEEPSF,IPRT,ITYPEC,ILEAK, 
     >              NL,NBN,RKEFF,RB2,TRHAR,REBAL,CPTL,CPTA,
     >              PHI,FLUN,PNL,XCSOU,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,
     >              XCCHI,XCABS,DIFF,DB2,BETA,LEXIN,NOVOID,REFKEF,
     >              GMXUPS,LMODE ,ADJFLX,ADJFLO,ADJSOU,ADJFXS,
     >              XCSOUA,ISADJ,RAJKEF,CDIRFL,CADJFL,LFLD,LDAJCB,
     >              IPTRK2,NTRK2,NANGL2,IFTRK2,LCACA,NGSS2,
     >              XMTOT,XMTRC,XMCAT,XMDIA,ANGPHI,ANGADJ   ) 
*  
*---------------------------   MOC2DR  --------------------------------
* 1- PROGRAMME STATISTICS:
*     NAME     :  MOC2DR
*     USE      :  DIRECT/ADJOINT FLUX CALCULATION MODULE
*                 FOR CASE (ITYPEC = 1 OR 6) AND (ISADJ = 1, 2 OR 3)
*     AUTHOR   :  ROY R.
*     MODIFIED :  MONCHAI A. (22/03/07)
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     ADJFLX   : ADJOINT FLUX ARRAY                      D(NFUNL,NREG,
*                                                          NGRP,8)
*     ADJFLO   : GLOBAL ADJOINT FLUX ARRAY               R(NUN,NFUNL,
*                FOR MODULE DATA EXCHANGE                  NGRP)
*     ADJSOU   : POINTER FOR EXTERNAL ADJOINT SOURCE     R(NREG,NGRP)
*     ANGPHI   : ANGULAR FLUX                            D(NREG,-NPHI:
*                                                        NPHI,NGSS,NGRP)
*     ANGADJ   : ANGULAR ADJOINT FLUX                    D(NREG,-NPHI:
*                                                        NPHI,NGSS,NGRP)
*     CDIRFL   : FLUX DIRECTORY NAME                     C*12
*     CADJFL   : ADJOINT FLUX DIRECTORY NAME             C(2)*12
*                FOR STANDARD AND GENERALIZED ADJOINT
*                PROBLEM
*     CSTDAJ   : STANDARD ADJOINT FLUX DIRECTORY NAME    C*12
*     CSTDDR   : STANDARD DIRECT FLUX DIRECTORY NAME     C*12
*     FLUX     : FLUX ARRAY                              D(NFUNL,NREG,
*                                                          NGRP,8)
*     FLUXO    : GLOBAL DIRECT FLUX ARRAY                R(NUN,NFUNL,
*                FOR MODULE DATA EXCHANGE                  NGRP)
*     GMXUPS   : MAX.ENERGY GROUP CONTAINGING            I
*                UPSCATTERING XS(EX. GROUP 27)
*     IACITR   : NUMBER OF ACCELERATED ITERATIONS        I
*     IPFLUX   : POINTER TO THE FLUX                     I
*     IPTRK    : POINTER TO THE TRACKING                 I
*     IPMACR   : POINTER TO THE MACROLIB                 I
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     ITRANC   : THE TYPE OF TRANSPORT CORRECTION        I
*     ITYPEC   : TYPE OF FLUX CALCULATION                I
*        ITYPEC= 0 TO SKIP THE FLUX CALCULATION; 
*        ITYPEC= 1 TO COMPUTE THE DIRECT PROBLEM 
*                  SOLUTION WITH FISSION SOURCES 
*                  AND KEFF CONVERGENCE
*        ITYPEC= 2 TO COMPUTE THE DIRECT PROBLEM 
*                  SOLUTION WITH FISSION SOURCES,
*                  KEFF CONVERGENCE AND IMPOSED 
*                  LEAKAGE
*        ITYPEC= 3 TO COMPUTE THE DIRECT PROBLEM 
*                  SOLUTION WITH FISSION SOURCES,
*                  KEFF CONVERGENCE AND IMPOSED 
*                  BUCKLING
*        ITYPEC= 4 TO COMPUTE THE DIRECT PROBLEM 
*                  SOLUTION WITH FISSION SOURCES,
*                  IMPOSED KEFF AND CONVERGENCE 
*                  ON BUCKLING
*        ITYPEC= 5 DIRECT PROBLEM WITH LEAKAGE 
*                  SOURCES AND CONVERGENCE ON 
*                  BUCKLING
*        ITYPEC= 6 DIRECT PROBLEM FIXED SOURCES.
*     ISADJ  : FLAG FOR TYPE OF ADJOINT CALCULATION      I
*        ISADJ = 1 ADJOINT FLUX CALCULATION WITHOUT-- 
*                  EXTERNAL SOURCE FOR ADJOINT KEFF PROBLEM
*        ISADJ = 2 ADJOINT FLUX CALCULATION WITH--
*                  EXTERNAL SOURCE FOR ADJOINT FIXED SOURCE
*                  PROBLEM
*        ISADJ = 3 GENERALIZED ADJOINT FLUX CALCULATION WITH--
*                  EXTERNAL GENERALIZED ADJOINT SOURCE 
*     IGDEB    : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     ILEAK    : THE LEAKAGE MODEL                       I
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     LSPLIT   : CONTROL FLAG FOR ITERATION WITH GROUP   L
*                SPLITTING
*     LCWDR1   : CONTROL FLAG FOR DIRECT MODE            L
*     LCWAJ2   : CONTROL FLAG FOR ADJOINT MODE           L
*     LFLD     : FLAG TO TURN ON DIRECT FLUX CALCULATION L
*     LDAJCB   : FLAG TO TURN ON DIRECT AND ADJOINT FLUX L
*                CALCULATIONS
*     LEAKSW   : LEAKAGE SWITCH                          L
*     LMODE    : CONTROL FLAG FOR CALCULATION MODE       L(20)
*     MATCOD   : MATERIAL CODE IN REGIONS.               I(NREG)
*     NANIS    : NUMBER OF ANISOTROPY IN                 I
*                  SCATTERING CROSS SECTIONS
*     NBN      : NUMBER OF LEGENDRE ORDER FOR LEAKAGE    I 
*     NEXTMX   : MAXIMUM NUMBER OF OUTER ITERATIONS      I
*     NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX       I
*     NINTMX   : MAXIMUM NUMBER OF INNER ITERATIONS      I
*     NGRP     : NUMBER OF GROUPS TREATED                I
*     NIFIS    : NUMBER OF FISSILE ISOTOPES              I
*     NL       : NUMBER OF LEGENDRE ORDER FOR THE FLUX   I
*     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
*     NREG     : NUMBER OF REGIONS CONSIDERED            I
*     REFKEF   : KEFF VALUE FOR B2 SEARCH                D
*     RKEFF    : REFERENCE KEFF VALUE FOR EIGENVALUE     R 
*                PROBLEM 
*     TITRE    : EXECUTION TITLE                         C*72
*     VOL      : VOLUME OF REGION                        R(NREG)
*     VOLINV   : INVERSE OF VOLUMES                      R(NREG)
*
*   WORK VECTOR
*     ADJFXS   : ADJFIX SOURCE BY MIXTURE/NGRP           R(0:NMAT,NGRP)
*     XSTOT    : GROUP TOTAL CROSS SECTION               R(NMAT+1)
*     XSTRC    : GROUP TRANSPORT-CORRECTED TOTAL         R(NMAT+1)
*                CROSS SECTION       
*     XSDIA    : WITHIN GROUP SCATTERING                 R(NMAT+1)
*                CROSS SECTION
*     XSCAT    : GROUP SCATTERING CROSS SECTION          R(NMAT+1)
*     XSNUF    : FISSION NEUTRON PRODUCTION              R(0:NMAT,NIFIS,
*                CROSS SECTION                             NGRP)
*     XSCHI    : FISSION NEUTRON DISTRIBUTION            R(0:NMAT,NIFIS,
*                                                          NGRP)
*     XMTOT    : REFERENCE TOTAL CROSS SECTION           R(NMAT+1)
*     XMTRC    : REFERENCE TRANSPORT-CORRECTED TOTAL     R(NMAT+1)
*                CROSS SECTION       
*     XMDIA    : REFERENCE WITHIN GROUP SCATTERING       R(NMAT+1)
*                CROSS SECTION
*     XMCAT    : REFERENCE SCATTERING CROSS SECTION      R(NMAT+1)
*     LACCOU   : CONTROL FLAG FOR OUTER ACCELERATION     L
*     LACCIN   : CONTROL FLAG FOR INNER ACCELERATION     L
*
* 3- INTERNAL PARAMETERS:
*     IOUT     : OUTPUT UNIT = 6                         I
*     ILCMUP   : GO UP LCM INDEX = 1                     I
*     ILCMDN   : GO DOWN LCM INDEX = 2                   I
* 
*---------------------------   MOC2DR  --------------------------------
*
      IMPLICIT NONE
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      INTEGER NGRP,NREG,NANIS,NMAT,NGSS,MXSEG,NTRK,NSOUT,MXGAUS,
     >        IPRT,ITYPEC,IFTRAK,NPHI,NFUNL,NMOD,IPTRK,IPTRK2,
     >        NL,NBN,IPFLUX,IPMACR,ITRB,NALB,
     >        CPTL,CPTA,CTOT,CPTM,MAXITR,IOUT
      PARAMETER(IOUT=6,MXGAUS=64)
      INTEGER MATALB(-NSOUT:NREG),NRSEG(MXSEG),KEYANI(NFUNL),
     >        ISGNR(NFUNL,NMOD)
      INTEGER IR,IG,JG,IA,IT,JT,IS,ITERF,IGDEB,IGDEBO,JGDEBO,JGCOUN,
     >        IGMIN,JGMIN,IGCOUN,IGADJ,IGDUM,IDUM,
     >        NEXTMX,NINTMX,NINTMO,IEXTF,ILEAK,NIFIS,
     >        IGDAJ,JGDAJ,IGDAJO,JGDAJO,JGCOAJ
      INTEGER IPIJ,IQSOU,ICP,JM,ISADJ,ISADJO,ILCMLN,ILCMTY,ILCMUP,
     >        ILCMDN,NINGR,INGR,AZIWGT,IPTRCL,NANGCL,IANG,IE,IFFIL,
     >        JGDEB, GMXUPS,ILOOP,NSTART,NSTOP,NSTEP,NMODE,NGSSCL,
     >        ILPCN0,ILPCN1,ILPCN2,ILPCN3,AFLXDU,IPVEC,IXVEC,IYVEC,
     >        IZVEC,ISVEC,NTRK2,NANGL2,IFTRK2,LCACA,NGSS2,NTRACK
      REAL    SEGLEN(MXSEG),ALBEDO(NALB),RKEFF,RB2,EXPT(NGSS,MXSEG),
     >        EXP2(NGSS,MXSEG),EXPTAJ(NGSS,MXSEG),EXP2AJ(NGSS,MXSEG),
     >        CYCLE(NGSS),SIGANG(NGSS,-NALB:NMAT,NGRP),WPOLAR(MXGAUS),
     >        XSTOT(0:NMAT,NGRP),XSTRC(0:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >        XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >        WGSS(MXGAUS),XGSS(MXGAUS),YGSS(MXGAUS),PHIANG(NPHI),
     >        XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >        XMTOT(0:NMAT,NGRP),XMTRC(0:NMAT,NGRP),
     >        XMCAT(NGRP,0:NMAT,0:NANIS,NGRP),XMDIA(0:NMAT,NGRP) 
      REAL    VOL(NREG),VOLINV(NREG),ALPHA(NGSS),DFLUX(NGSS),
     >        FLUXO(NREG,NFUNL,NGRP),DIFFB1(NGRP),RAJKEF,FMEAN(NGSS),
     >        RHARM(NGSS,*),TRHAR(NFUNL,NGSS,*),DWEIG(NGSS),
     >        EBMX,MU,MUAJ,AIEPSF,AEEPSF,ONE,ZERO,
     >        BETASC
      DOUBLE PRECISION EEXT,EEMX,EEXTAJ,EINN,FINN,GINN,EIMX,EIMXO,
     >                 EINNAJ,FINNAJ,GINNAJ
      DOUBLE PRECISION PHI(NGRP),XCSOU(NGRP),XCTOT(NGRP),XCABS(NGRP),
     >                 XCCAT(NGRP,0:NANIS,NGRP),DIFF(NGRP),DB2(NGRP),
     >                 PNL(NGRP),AKEEP(8),XCSOUA(NGRP),AKEEAJ(8),
     >                 XCNUF(NGRP),XCCHI(NGRP),XCTRC(NGRP),XCDIA(NGRP)
      DOUBLE PRECISION AFLNOR,FISOUR, AKEFF,AKEFFO,OLDBIL,OLDBAJ,
     >                 B2VAL, B2VALO,BFLNOR, AKFRB,OOL,OOLPAJ,
     >                 ADJKEO,REFKEF,ADJKEF,AJFNOR
      DOUBLE PRECISION REBAL(NGRP,NGRP+1),BETA(NGRP),FLUN(NGRP),
     >                 FLM(NGSS,MXSEG),FLUX(NFUNL,NREG,NGRP,8),
     >                 FLP(NGSS,MXSEG),CYM(NGSS,MXSEG),CYP(NGSS,MXSEG),
     >                 CYMAJ(NGSS,MXSEG),CYPAJ(NGSS,MXSEG),
     >                 PRODFS,FLMADJ(NGSS,MXSEG),FLPADJ(NGSS,MXSEG)
      DOUBLE PRECISION ANGPHI(NREG,-NPHI:NPHI,NGSS,NGRP),
     >                 ANGADJ(NREG,-NPHI:NPHI,NGSS,NGRP),
     >                 ADJFLX(NFUNL,NREG,NGRP,8)
      DOUBLE PRECISION DDUM, AKEFF1,ANZFAC,DONE, DZERO, DTHIRD
      REAL       ADJFLO(NREG,NFUNL,NGRP),ADJSOU(NREG,NGRP),
     >           ADJFXS(0:NMAT,NGRP)
      LOGICAL    LAMH,LPRODF,LANZFA,LFLD,LFLDO,LFANG,LMODE(20),
     >           LSPLIT,LDAJCB,LDAJCO,OLDAJC,OLDRFL,OOLDRF,OOLDAJ,
     >           LCWDR1,LCWAJ2,LXSCAT,LEXTSO,LEAKSW,LREBAL,LREBAO,
     >           NOVOID,LEXACT,LEXIN,LDRTMP,LAJTMP,LOWTRK,LBETAS,
     >           LPJC,LPJCO
      CHARACTER*12 CDIRFL,CADJFL(2),CADJFO,CSTDAJ,CSTDDR,CADJCP,CCPDIR,
     >             DIRPIS,CAZIMU,CPOLAR,CSAV,CANGDR,CANGAJ
      CHARACTER*8  NAMFLX
      CHARACTER*5  MESSIN, MESSOU
      PARAMETER (ZERO=0.00000000,ONE=1.00000000,DZERO=0.00000000D+0,
     >           DONE=1.00000000D+0,DTHIRD=DONE/3.00000000D0,
     >           ILCMUP=1,ILCMDN=2 )
      IF (.NOT.LFLD .AND. .NOT.LDAJCB) RETURN
      IF (LDAJCB .AND. (ITYPEC.NE.1 .AND. ITYPEC.NE.6) ) THEN 
       CALL XABORT('MOC2DR:INVALID ITYPEC FOR '//
     >             'COMBINED DIRECT & ADJOINT FLUX CALCULATION')
      ENDIF
      AZIWGT = 0
      CALL SETARA( A, NPHI, AZIWGT )
      AFLXDU = 0
      CALL SETARA( A,2*(2*NPHI)*NREG*NGSS, AFLXDU )
      CAZIMU = 'AZIMUTHALWGT'
      CPOLAR = 'POLAR WEIGHT'
      CANGDR = 'ANGULAR DIR'
      CANGAJ = 'ANGULAR ADJ'
      CCPDIR = 'P_JI DIR    '
      DIRPIS = 'PIS DIR     '
      B2VAL = DZERO
      LCWDR1 = .TRUE.
      LCWAJ2 = .FALSE.
      IGDEB  = 1
      JGDEB  = 0
      ILPCN0 = 0
      ILPCN1 = 0
      ILPCN2 = 0
      ILPCN3 = 0
      EEXT   = DZERO
      EEXTAJ = DZERO
      EINN   = DZERO
      EINNAJ = DZERO
      ILOOP  = 0
      JGDEB  = 0
      LEXACT = .TRUE.
      ISADJO = ISADJ
      OLDBIL= DZERO
      OLDBAJ= DZERO
      LREBAO = LREBAL
      AKEFF  = DONE
      AKEFFO = AKEFF
      AFLNOR = DONE
      ADJKEF = DONE
      ADJKEO = ADJKEF
      AJFNOR = DONE
      LDAJCO = LDAJCB
      OLDAJC = LDAJCB
      OOLDAJ = LDAJCB
      LFLDO = LFLD
      OLDRFL = LFLD
      OOLDRF = LFLD
      CSTDDR = 'FLUXDIRECT  '
      CSTDAJ = 'FLUXADJOINT '
      CADJCP = 'PSEUDOADJ   '
      LSPLIT = LMODE(1)  
      LPJC  = LMODE(2)   
      LPJCO = LPJC
      BETASC = 0.000000000
      LBETAS = LMODE(3)
      IF (LPJC) THEN
        LSPLIT = .FALSE.
        LBETAS = .FALSE.
      ENDIF
      IF (IFTRK2 .GT. 0) THEN
        LOWTRK = .TRUE. 
      ELSE
        LOWTRK = .FALSE.
      ENDIF
      IF (LPJC)  LOWTRK = .FALSE.
       IF(LSPLIT) THEN
        NMODE = 1 
       ELSE
        NMODE = 0 
       ENDIF
      LCWDR1 = .TRUE.
      LCWAJ2 = .FALSE.
      LFANG = LMODE(6)
      IF (LPJC .AND. LFANG) CALL XABORT('MOC2DR: ANGULAR FLUX/ADJOINT 
     > CALCULATIONS IN PIJ MODE  N/A')
      CALL MOCCPY((NMAT+1)*NGRP ,XSTOT  ,XMTOT )
      CALL MOCCPY((NMAT+1)*NGRP ,XSTRC  ,XMTRC )
      CALL MOCCPY((NMAT+1)*NGRP ,XSDIA  ,XMDIA )
      CALL MOCCPY((NMAT+1)*NGRP*NGRP*(NANIS+1) ,XSCAT ,XMCAT )
      CALL SETARA(A       , 2* NGRP*NREG  , IPVEC )
      CALL XDRSET(A(IPVEC), 2* NGRP*NREG  , 0.0)
      CALL SETARA(A       , 2* NGRP*NREG  , IXVEC )
      CALL XDRSET(A(IXVEC), 2* NGRP*NREG  , 0.0)
      CALL SETARA(A       , 2* NGRP*NREG  , IYVEC )
      CALL XDRSET(A(IYVEC), 2* NGRP*NREG  , 0.0)
      CALL SETARA(A       , 2* NGRP*NREG  , IZVEC )
      CALL XDRSET(A(IZVEC), 2* NGRP*NREG  , 0.0)
      CALL SETARA(A       , 2* NGRP*NREG  , ISVEC )
      CALL XDRSET(A(ISVEC), 2* NGRP*NREG  , 0.0)
       LANZFA = .FALSE.
       LAMH   = .FALSE.
       LPRODF = .TRUE. 
       JM=0
       IF (LAMH) JM=JM+1
       IF (LPRODF) JM=JM+1
       IF (LANZFA) JM=JM+1
       IF(JM.GT.1.OR.JM.EQ.0)CALL XABORT('MOC2DR:INVALID NORMLZ SELECT')
      LEXACT= LEXIN
      OOL    = DZERO
      OOLPAJ = DZERO
      EBMX= 1.E-6
      AKEEP(1)=DONE
      AKEEP(2)=DONE
      AKEEP(3)=DONE
      AKEEP(4)=DONE
      AKEEP(5)=DONE
      AKEEP(6)=DONE
      AKEEP(7)=DONE
      AKEEAJ(1)=DONE
      AKEEAJ(2)=DONE
      AKEEAJ(3)=DONE
      AKEEAJ(4)=DONE
      AKEEAJ(5)=DONE
      AKEEAJ(6)=DONE
      AKEEAJ(7)=DONE
      IF( IPRT.GT.1 )THEN
         WRITE(IOUT,999)
      ENDIF
      IF( NANIS.GT.0 )THEN
         DO 31 JG=1,NGRP
         DO 21 IA=1,NANIS
         DO 11 IR=1,NMAT
         DO  1 IG=1,NGRP
            XSCAT(IG,IR,IA,JG)= REAL(2*IA+1)*XSCAT(IG,IR,IA,JG)
    1    CONTINUE
   11    CONTINUE
   21    CONTINUE
   31    CONTINUE
      ENDIF
      DO 1003 IG=1,NGRP
         DB2(IG)= DZERO
         PNL(IG)= DONE
 1003 CONTINUE
      DO 30 JG=1,NGRP
      DO 20 IG=1,NGSS
         DO 13 IR=-NALB,-1
            SIGANG(IG,IR,JG)= ONE-ALBEDO(-IR)
   13    CONTINUE
         SIGANG(IG,0,JG) = ZERO
         DO 10 IR=1,NMAT
            SIGANG(IG,IR,JG) = XSTRC(IR,JG) * YGSS(IG)
   10     CONTINUE
   20 CONTINUE
   30 CONTINUE 
      IF( LFLD) THEN
       AKEFFO= DZERO
       CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
       DO 22 IG= 1, NGRP
         XCSOU(IG)= DZERO
         WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
         CALL LCMGET(IPFLUX,NAMFLX,FLUXO(1,1,IG))
         DO 12 IR= 1, NREG
            FLUX(1,IR,IG,2) = FLUXO(IR,1,IG)
            IF( ITYPEC .EQ. 6 )THEN
                FLUX(1,IR,IG,4) = XSNUF(MATALB(IR),1,IG)
                XCSOU(IG)= XCSOU(IG) + FLUX(1,IR,IG,4)*VOL(IR)
            ELSE
                FLUX(1,IR,IG,4) = ZERO
            ENDIF
            IF( NL.GT.0 )THEN
               DO 2 IA= 2, NFUNL
                  FLUX(IA,IR,IG,2) = ZERO
                  FLUX(IA,IR,IG,4) = ZERO
    2          CONTINUE
            ENDIF
   12    CONTINUE
   22  CONTINUE
       CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN) 
       IF( ITYPEC .EQ. 6 )THEN
         AKEFFO= DONE
         AKEFF=  DONE
         AFLNOR= DONE
       ELSE
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,FLUX(1,1,1,2),XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,
     >               XSCHI,PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,
     >               XCABS,IPRT,LCWDR1,IPFLUX,DIRPIS,LEAKSW,.FALSE.,
     >               A(IXVEC),FLUX(1,1,1,4),XCSOU)
         CALL MOC2MH(1,NBN,NGRP,NANIS,
     >               PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >               DIFF,BETA,REBAL,FLUN,REFKEF,AKFRB,B2VAL,OOL,EBMX,
     >               1,PRODFS,LEAKSW )

         DO 2200 IG= 1, NGRP
            DO 2210 IR= 1, NREG
               DO 2220 IS= 1, NFUNL

                  IF (PRODFS .GT. 0.000D0) THEN
                   FLUX(IS,IR,IG,2)= FLUX(IS,IR,IG,2)/PRODFS
                  ENDIF

 2220          CONTINUE
 2210       CONTINUE
 2200    CONTINUE
         CALL MOC2KF(NGRP,LEAKSW,OLDBIL,
     >               PHI,XCABS,XCNUF,XCCHI,DB2,
     >               AKEFF,AFLNOR,IPRT)
         AKEFFO= AKEFF
         AKEEP(2)= AKEFF
       ENDIF
      ENDIF
      B2VAL=  DZERO
      B2VALO= DZERO
      CTOT = CPTA+CPTL
      IF( CPTA.EQ.0 )THEN
         CPTM=CTOT+1
      ELSE
         CPTM= CPTA
      ENDIF
      MAXITR= 0
      EEMX  = DBLE(AEEPSF)
      EIMX  = DBLE(AIEPSF)
      EIMXO = EIMX
      NINTMO= NINTMX
      MESSOU= '     '
      IF( IPRT.GT.0 )THEN
         WRITE(IOUT,2065) 0,'FLX:',ONE,EEMX,AKEFFO,B2VALO
      ENDIF
      IF (LDAJCB) THEN 
        CALL LCMLEN(IPFLUX,CADJFL(1),ILCMLN,ILCMTY)
       CADJFO = CADJFL(1)
       IF (ILCMLN .NE. -1) THEN
        IF (CADJFL(1) .EQ. CSTDAJ) THEN
          CALL LCMLEN(IPFLUX,CADJCP,ILCMLN,ILCMTY)
          IF (ILCMLN.EQ. -1) THEN
            CADJFO = CADJCP
          ELSE
            CALL XABORT('MOC2DR: '//CADJFL(1)//' AND '//CADJCP
     >       //' DIRECTORIES DO NOT EXIST')  
          ENDIF
        ELSE
          CALL XABORT('MOC2DR:'//CADJFL(1)//' DIRECTORY DOES NOT EXIST')             
        ENDIF
       ENDIF
        CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
          CALL MOCAJS(IPFLUX,IPMACR,ISADJ ,NMAT ,NSOUT,NGRP,NREG,
     >                ADJSOU,ADJFXS,MATALB,LMODE,VOL)
        CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN) 
        CALL LCMSIX(IPFLUX,CADJFO,ILCMUP)
       DO 211 IG= 1, NGRP
         XCSOUA(IG)= DZERO 
         WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
         CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
         IF(ILCMLN.EQ.NREG) THEN
          CALL LCMGET(IPFLUX,NAMFLX,ADJFLO(1,1,IG))
         ELSE
          CALL XABORT('MOC2DR:INVALID DATA LENGTH OF ADJFLUX IN IPFLUX')
         ENDIF
         DO 111 IR= 1, NREG
            ADJFLX(1,IR,IG,2) = ADJFLO(IR,1,IG)
            IF (ITYPEC.EQ.6) THEN 
                 IF (ISADJ.EQ.2) THEN
                   ADJFLX(1,IR,IG,4) = ADJSOU(IR,IG) 
                   XCSOUA(IG)= XCSOUA(IG) + ADJFLX(1,IR,IG,4)*VOL(IR)
                 ELSE
                   CALL XABORT('MOC2DR: INVALID ISADJ FOR ITYPEC=6')
                 ENDIF
            ELSE
                 ADJFLX(1,IR,IG,4) = DZERO
            ENDIF
  111    CONTINUE   
  211  CONTINUE  
       CALL LCMSIX(IPFLUX,CADJFO,ILCMDN)
       IF( ITYPEC .EQ. 6 )THEN
         ADJKEO = DONE
         ADJKEF = DONE
         AJFNOR = DONE    
         AKEEAJ(1)= ADJKEF
         AKEEAJ(2)= ADJKEF
         AKEEAJ(3)= ADJKEF
         AKEEAJ(4)= ADJKEF
         AKEEAJ(5)= ADJKEF
         AKEEAJ(6)= ADJKEF   
         AKEEAJ(7)= ADJKEF
         AKEEAJ(8)= ADJKEF
       ELSE 
         IF ( (ITYPEC.EQ.1).AND.(ISADJ.EQ.1) ) THEN
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,ADJFLX(1,1,1,2),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWAJ2,
     >               IPFLUX,DIRPIS,LEAKSW,.FALSE.,A(IXVEC),
     >               ADJFLX(1,1,1,4),XCSOUA)
          PRODFS=0.0D0
            CALL MOCAKF( NGRP, NANIS, PHI, DB2,ADJKEF,AJFNOR,LEAKSW,
     >                   XCTOT,XCDIA,XCCAT,XCNUF,XCCHI, IPRT,PRODFS,
     >                   OLDBAJ)
          IF (LAMH) THEN
           CALL MOCAMH(1,NBN,NGRP,NANIS,           
     >             PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >             DIFF,BETA,REBAL,FLUN,AKFRB,B2VAL,OOLPAJ,EBMX,1,
     >             AKEFF1)
           ADJKEF=AKEFF1
           AJFNOR=DONE/ADJKEF
          ENDIF
           DO 100 IG= 1, NGRP
              DO 110 IR= 1, NREG
                DO 120 IS= 1, NFUNL
            IF (LAMH)  ADJFLX(IS,IR,IG,2)= ADJFLX(IS,IR,IG,2)
     >                                  * FLUN(IG)
            IF(LPRODF) ADJFLX(IS,IR,IG,2)=ADJFLX(IS,IR,IG,2)/PRODFS
120             CONTINUE
110           CONTINUE 
100        CONTINUE
         ENDIF
         ADJKEO   = ADJKEF
         AKEEAJ(1)= ADJKEF
         AKEEAJ(2)= ADJKEF
         AKEEAJ(3)= ADJKEF
         AKEEAJ(4)= ADJKEF
         AKEEAJ(5)= ADJKEF
         AKEEAJ(6)= ADJKEF   
         AKEEAJ(7)= ADJKEF
         AKEEAJ(8)= ADJKEF
         AJFNOR = DONE/ADJKEF
       ENDIF
      ENDIF
       CALL MOCRWD(IFTRAK)
       IF (IFTRK2.GT.0)  CALL MOCRWD( IFTRK2 )
       IF (LEAKSW) THEN 
          IGDEB = 1
          JGDEB = 0
          LOWTRK = .FALSE.
            CALL MOCPIS(IPFLUX,DIRPIS,
     >                  NREG  ,NGRP  ,NL,NMAT,NANIS ,NSOUT ,NIFIS,NGSS,
     >                  NALB  ,MXGAUS,IFTRK2 ,IFTRAK,NTRK2 ,NTRK,MXSEG,
     >                  MATALB,NRSEG ,SIGANG ,XSDIA ,XSCAT ,XTINV,XSTRC,
     >                  SEGLEN,ALBEDO,EXPT   ,EXP2  ,WGSS ,YGSS,NGSS2,
     >                  FLM   ,FLP   ,CYM    ,CYP   ,LOWTRK,IPRT,
     >                  VOL   ,VOLINV,MAXITR ,IGDEB ,JGDEB ,
     >                  A(IXVEC) ,A(IYVEC) ,A(IZVEC) ,A(ISVEC) )
       ENDIF
      CALL MOCRWD( IFTRAK ) 
      IF (IFTRK2.GT.0)  CALL MOCRWD( IFTRK2 )
       IF (LPJC) THEN
          IF (LOWTRK) 
     >     CALL XABORT('MOC2DR:LOWTRK ARE NOT APPLICABLE FOR CP EVAL.')
          IPIJ  = 0
          IQSOU = 0
          CALL SETARA(A,NREG*NREG,ICP)
          CALL SETARA(A,2*NREG*NREG,IPIJ)
          CALL SETARA(A,2*NREG*NREG,IQSOU)
          IF (IPIJ .EQ. 0 .OR. IQSOU .EQ. 0)  THEN
            CALL XABORT('MOC2DR: INSUFFICIENT MEM FOR PIJ OR QSOU ')
          ENDIF
            CALL MOCCP2(IPFLUX,CCPDIR,NREG,NGRP,NL,NMAT,NANIS,NSOUT,
     >                  NIFIS,NGSS,NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,NTRK,
     >                  MXSEG,MATALB,NRSEG,SIGANG,XSDIA,XSCAT,XTINV,
     >                  XSTRC,SEGLEN,ALBEDO,EXPT,EXP2,WGSS,YGSS,NGSS2,
     >                  FLM,FLP,CYM,CYP,LOWTRK,IPRT,VOL,VOLINV,MAXITR,
     >                  IGDEB,JGDEB,A(IXVEC),A(IYVEC),A(IZVEC),A(ISVEC),
     >                  A(IPIJ))
            CALL RLSARA( A( IPIJ  ) )
            CALL RLSARA( A( IQSOU ) )
       ENDIF
      CALL MOCRWD( IFTRAK ) 
      IF (IFTRK2.GT.0)  CALL MOCRWD(IFTRK2)
      DO 210 IT=1,NEXTMX
         MESSIN='     '
         IGDEB = 1
         JGDEB = 0
         IGDAJ = 1
         JGDAJ = 0
         IF (LFLD) THEN 
          IF( ITYPEC .LT. 6 )THEN
            DO 92 IG= 1, NGRP
               DO 91 IR= 1,NREG
                  FLUX(1,IR,IG,4)= ZERO
   91          CONTINUE
   92       CONTINUE
            DO 96 IR= 1,NREG
               DO 95 IS= 1,NIFIS
                  FISOUR= DZERO
                  DO 93 IG= 1, NGRP
                     FISOUR= FISOUR+
     >                       XSNUF(MATALB(IR),IS,IG)*FLUX(1,IR,IG,2)
   93             CONTINUE
                  FISOUR= FISOUR*AFLNOR
                  DO 94 IG= 1,NGRP
                     FLUX(1,IR,IG,4)= FLUX(1,IR,IG,4)+
     >                                XSCHI(MATALB(IR),IS,IG)*FISOUR
   94             CONTINUE
   95          CONTINUE
   96       CONTINUE
            DO 98 IG= 1, NGRP
               XCSOU(IG)= DZERO
               DO 97 IR= 1,NREG
                  XCSOU(IG)= XCSOU(IG)+FLUX(1,IR,IG,4)*VOL(IR)
   97          CONTINUE
   98       CONTINUE
          ENDIF
          DO 783 IG= 1, NGRP
            DO 782 IR= 1, NREG
               DO 781 IS= 1, NFUNL
                  FLUX(IS,IR,IG,6)= FLUX(IS,IR,IG,2)
                  FLUX(IS,IR,IG,7)= FLUX(IS,IR,IG,2)
  781          CONTINUE
  782       CONTINUE
  783     CONTINUE
         ENDIF
         IF (LDAJCB) THEN 
          IF (ITYPEC.LT.6) THEN
            DO 921 IG= 1, NGRP
               DO 911 IR= 1,NREG
                  ADJFLX(1,IR,IG,4)= DZERO
                  IF(ISADJ.EQ.2 .OR. ISADJ.EQ.3) THEN 
                   ADJFLX(1,IR,IG,4)= ADJSOU(IR,IG)
                  ENDIF
  911          CONTINUE
  921       CONTINUE
            DO 961 IR= 1,NREG
               DO 951 IS= 1,NIFIS
                  FISOUR= DZERO
                  DO 931 IG= 1, NGRP
                     FISOUR= FISOUR+
     >                       XSCHI(MATALB(IR),IS,IG)*ADJFLX(1,IR,IG,2)
  931             CONTINUE
                  FISOUR= FISOUR*AJFNOR
                  DO 941 IG= 1,NGRP
                     ADJFLX(1,IR,IG,4)= ADJFLX(1,IR,IG,4)+
     >                              XSNUF(MATALB(IR),IS,IG)*FISOUR
  941             CONTINUE
  951          CONTINUE
  961       CONTINUE
            DO 981 IG= 1, NGRP
               XCSOUA(IG)= DZERO
               DO 971 IR= 1,NREG
                  XCSOUA(IG)= XCSOUA(IG)+ADJFLX(1,IR,IG,4)*VOL(IR)
  971          CONTINUE
  981       CONTINUE
          ENDIF
          DO 7831 IG= 1, NGRP
            DO 7821 IR= 1, NREG
               DO 7811 IS= 1, NFUNL
                  ADJFLX(IS,IR,IG,6)= ADJFLX(IS,IR,IG,2)
                  ADJFLX(IS,IR,IG,7)= ADJFLX(IS,IR,IG,2)
 7811          CONTINUE
 7821       CONTINUE
 7831     CONTINUE
         ENDIF
11111    CONTINUE
         IF (LSPLIT) THEN
           IF (ILOOP.EQ.0.AND.NGRP.GE.GMXUPS.AND.(GMXUPS.GT.1))THEN
               ILOOP = 1 
               IGDEB = 1
               IGDAJ = 1
               JGDEB = NGRP - ( GMXUPS-1 ) 
               JGDAJ = NGRP - ( GMXUPS-1 )  
           ELSEIF (ILOOP.EQ.1.AND.NGRP.GE.GMXUPS.AND.(GMXUPS.GT.1))THEN
               ILOOP = 2
               IGDEB = GMXUPS
               IGDAJ = GMXUPS
               JGDEB = 0
               JGDAJ = 0
           ELSEIF (ILOOP.EQ.2.AND.NGRP.GE.GMXUPS.AND.(GMXUPS.GT.1))THEN
               ILOOP = 3 
               IGDEB = 1
               IGDAJ = 1
               JGDEB = NGRP - ( GMXUPS-1 ) 
               JGDAJ = NGRP - ( GMXUPS-1 )  
           ENDIF
         ENDIF
        IF (LBETAS) THEN
          BETASC = 0.10000000000     
        ELSE 
          BETASC = 0.00000000000           
        ENDIF 
        IF (LBETAS .OR. LOWTRK) THEN
          EIMX   = 1.0D-02 
        ELSE
          EIMX =    EIMXO 
        ENDIF 
            CALL MOCSIG(NGSS,NGSS2,NALB,NMAT,NGRP,NANIS,NSOUT,MXGAUS,
     >                  ALBEDO,XSTOT,XSTRC,XSCAT,XSDIA,XTINV,XGSS,YGSS,
     >                  WGSS,XMTOT,XMTRC,XMCAT,XMDIA,LCACA,
     >                  SIGANG,BETASC,IFTRK2,LOWTRK)
          EINN   = DZERO
          EINNAJ = DZERO
        NINTMX = NINTMO
        DO 500 JT= 1, NINTMX
            IGMIN = MIN(IGDEB,IGDAJ)
            JGMIN = MIN(JGDEB,JGDAJ)
            IF(.NOT. LFLD) THEN
              IGMIN = IGDAJ
              JGMIN = JGDAJ
            ENDIF
            IF(.NOT. LDAJCB) THEN
              IGMIN = IGDEB
              JGMIN = JGDEB
            ENDIF
            IF(ILOOP.EQ.1) THEN
                 NSTEP  =  +1
            ELSEIF(ILOOP.EQ.2 .OR. ILOOP.EQ.0) THEN
                 NSTEP  =  +1  
            ELSEIF(ILOOP.EQ.3 ) THEN
                 NSTEP  =  -1
            ENDIF
            IF (ILOOP.EQ.1) THEN
              LFLD = OOLDRF
              LDAJCB = .FALSE.
            ELSEIF (ILOOP.EQ.2) THEN
              LFLD = OOLDRF
              LDAJCB = OOLDAJ
            ELSEIF (ILOOP.EQ.3) THEN
              LFLD = .FALSE.
              LDAJCB = OOLDAJ
            ENDIF
            IF(NSTEP .EQ. +1) THEN
              NSTART =  IGMIN
              NSTOP  =  (NGRP-JGMIN)
            ELSE
              NSTART =  (NGRP-JGMIN)
              NSTOP  =  IGMIN
            ENDIF
          IF( ITYPEC .EQ. 4 .AND. ILEAK .EQ. 2 )THEN
             DO 5 IG=NSTART, NSTOP , NSTEP 
                PNL(IG)= XCTRC(IG)/(XCTRC(IG)+DB2(IG))
    5        CONTINUE
          ENDIF
          IGCOUN = 1
          DO 45 IG= NSTART, NSTOP , NSTEP 
            IF (IG .LE. ( GMXUPS-1 ) ) THEN
                 NINGR  =   NINTMX                
            ELSE
                 NINGR  =   1 
            ENDIF
           DO 451 INGR= 1, NINGR
             IGADJ = IG
             IF (NSTEP .EQ. 1) THEN
               IGCOUN = IGCOUN +1
             ELSEIF (NSTEP .EQ. -1) THEN
               IGCOUN = IGCOUN -1 
             ENDIF
            IF(LPJC) THEN
               LXSCAT = .TRUE. 
               LEXTSO = .TRUE.
               IF(LFLD.AND.IG.GE.IGDEB.AND. IG.LE.(NGRP-JGDEB)) THEN 
                 CALL MOCCWP(IPFLUX,CCPDIR,NREG,NGRP,NMAT ,NANIS ,NSOUT,
     >                  MATALB,XSDIA ,XSCAT,LXSCAT,LEXTSO,
     >                  IG ,NGRP-IG , LCWDR1 ,FLUX(1,1,1,7),
     >                  FLUX(1,1,1,4),FLUX(1,1,1,7),A(ISVEC),A(ICP))
               ENDIF
               IF (LDAJCB.AND.IGADJ.GE.IGDAJ.AND. IGADJ.LE.(NGRP-JGDAJ))
     >           THEN 
                 CALL MOCCWP(IPFLUX,CCPDIR,NREG,NGRP,NMAT ,NANIS ,NSOUT,
     >                  MATALB,XSDIA ,XSCAT,LXSCAT,LEXTSO,
     >                  IGADJ ,NGRP-IGADJ , LCWAJ2 ,ADJFLX(1,1,1,7),
     >                  ADJFLX(1,1,1,4),ADJFLX(1,1,1,7),A(ISVEC),A(ICP))
               ENDIF
            ELSE
             DO 65 IR= 1,NREG
               LDRTMP = .FALSE.
               IF(LFLD.AND.IG.GE.IGDEB.AND. IG.LE.(NGRP-JGDEB)) THEN 
                FLUX(1,IR,IG,8)= FLUX(1,IR,IG,4)
                DO 35 JG= 1,NGRP
                   IF( IG.EQ.JG )THEN
                      FLUX(1,IR,IG,8)= FLUX(1,IR,IG,8)+
     >                XSDIA(MATALB(IR),IG)*FLUX(1,IR,JG,7)

                      IF( ITYPEC .EQ. 4 .AND. ILEAK .EQ. 3 )THEN
                         FLUX(1,IR,IG,8)= FLUX(1,IR,IG,8)-
     >                   DB2(IG)*FLUX(1,IR,JG,7)
                      ENDIF
                   ELSE
                      FLUX(1,IR,IG,8)= FLUX(1,IR,IG,8)+
     >                XSCAT(JG,MATALB(IR),0,IG)*FLUX(1,IR,JG,7)
                   ENDIF
   35           CONTINUE
                FLUX(1,IR,IG,8)= FLUX(1,IR,IG,8)
     >                         * XTINV(MATALB(IR),IG) * PNL(IG)
                IF( NL.GT.0 )THEN
C                  ADDING ANISOTROPIC SCATTERING, IF NEEDED
                   DO 55 IA= 2,NFUNL
                     FLUX(IA,IR,IG,8)= ZERO
                     DO 54 JG= 1,NGRP
                        FLUX(IA,IR,IG,8)= FLUX(IA,IR,IG,8)+
     >                            XSCAT(JG,MATALB(IR),KEYANI(IA),IG)*
     >                            FLUX(IA,IR,JG,7)
   54                CONTINUE
                     FLUX(IA,IR,IG,8)= FLUX(IA,IR,IG,8)
     >                                * XTINV(MATALB(IR),IG)
   55              CONTINUE
                ENDIF
                LDRTMP = .TRUE.
               ENDIF
               LAJTMP = .FALSE.
               IF (LDAJCB.AND.IGADJ.GE.IGDAJ.AND. IGADJ.LE.(NGRP-JGDAJ))
     >         THEN 
                ADJFLX(1,IR,IGADJ,8)= ADJFLX(1,IR,IGADJ,4)
                DO 351 JG= 1,NGRP
                   IF( IGADJ.EQ.JG )THEN
                      ADJFLX(1,IR,IGADJ,8)= ADJFLX(1,IR,IGADJ,8)+
     >                XSDIA(MATALB(IR),JG)*ADJFLX(1,IR,JG,7)
                   ELSE
                      ADJFLX(1,IR,IGADJ,8)= ADJFLX(1,IR,IGADJ,8)+
     >                XSCAT(IGADJ,MATALB(IR),0,JG)*ADJFLX(1,IR,JG,7)
                   ENDIF
  351           CONTINUE
                ADJFLX(1,IR,IGADJ,8)= ADJFLX(1,IR,IGADJ,8)
     >                         * XTINV(MATALB(IR),IGADJ) * PNL(IGADJ)  
                LAJTMP = .TRUE.
               ENDIF
   65        CONTINUE
             IF (LOWTRK .AND. IFTRK2.GT.0 ) THEN
                  IPTRCL  = IPTRK2
                  IFFIL   = IFTRK2
                  NTRACK  = NTRK2
                  NGSSCL  = NGSS2
                  NANGCL  = NANGL2
             ELSE
                  IPTRCL  = IPTRK
                  IFFIL   = IFTRAK
                  NTRACK  = NTRK
                  NGSSCL  = NGSS
                  NANGCL  = NPHI
             ENDIF
             CALL LCMGET(IPFLUX , CPOLAR, WPOLAR )
             CALL LCMGET(IPFLUX , CAZIMU, A(AZIWGT) )
             IF( NL.EQ.0 )THEN
                  CALL MOC3I0(LMODE,NPHI,IFFIL, NGRP, NMAT, NREG, NGSS, 
     >                   NSOUT,
     >                   MXSEG, NTRACK, MATALB, WGSS, YGSS,SEGLEN,NRSEG,
     >                   EXPT, EXP2, FLM, FLP, CYM, CYP, NALB, SIGANG,
     >                   FLMADJ,FLPADJ,FLUX(1,1,1,8), FLUX(1,1,1,7), IG,
     >                   LDRTMP,LAJTMP,ADJFLX(1,1,1,8),ADJFLX(1,1,1,7),
     >                   NGSSCL,ANGPHI,ANGADJ )
             ELSE
               CALL XABORT('MOC2DR: ANISOTROPIC OPTION *N/A* YET')
             ENDIF
             IF (LFLD.AND.IG.GE.IGDEB.AND. IG.LE.(NGRP-JGDEB)) THEN 
              DO 164 IR= 1,NREG
              DO 165 IA= 1,NFUNL
                  FLUX(IA,IR,IG,7) =
     >            (FLUX(IA,IR,IG,8) / REAL(2*KEYANI(IA)+1)
     >          + FLUX(IA,IR,IG,7) * XTINV(MATALB(IR),IG) * VOLINV(IR))

               IF (LFANG) THEN 
               DO 166 IANG= -NANGCL,NANGCL
                DO 167 IE= 1,NGSSCL
                 IF (ABS(IANG).GT. 0 ) THEN
                  ANGPHI(IR,IANG,IE,IG) =
     >              FLUX(IA,IR,IG,8) / REAL(2*KEYANI(IA)+1) +
     >              ANGPHI(IR,IANG,IE,IG)/YGSS(IE)/A(AZIWGT+ABS(IANG)-1)
     >              * XTINV(MATALB(IR),IG)*VOLINV(IR)  
                  IF(YGSS(IE).LE.ZERO.OR. A(AZIWGT+ABS(IANG)-1).LE.ZERO)
     >             THEN
                   CALL XABORT('MOC2DR: YGSS OR AZIMUTHAL WGT < 0.0')
                  ENDIF
                 ENDIF
  167           CONTINUE
  166          CONTINUE
               ENDIF
  165         CONTINUE
  164         CONTINUE
              IF (LFANG) THEN 
               DO 16412 IR= 1,NREG
               DO 16512 IA= 1,NFUNL
                FLUX(IA,IR,IG,7) =  DZERO 
                 DO 16612 IANG= -NANGCL,NANGCL
                  DO 16712 IE= 1,NGSSCL
                  IF (ABS(IANG).GT. 0) THEN
                  FLUX(IA,IR,IG,7)=FLUX(IA,IR,IG,7)+WPOLAR(IE)*
     >            A(AZIWGT+ABS(IANG)-1)*ANGPHI(IR,IANG,IE,IG) 
                  ENDIF
16712             CONTINUE
16612            CONTINUE
16512          CONTINUE
16412          CONTINUE
              ENDIF
             ENDIF
             IF (LDAJCB.AND.IGADJ.GE.IGDAJ.AND. IGADJ.LE.(NGRP-JGDAJ))
     >          THEN 
              DO 1641 IR= 1,NREG
               DO 1651 IA= 1,NFUNL
               ADJFLX(IA,IR,IGADJ,7) =
     >         (ADJFLX(IA,IR,IGADJ,8)/REAL(2*KEYANI(IA)+1)+ 
     >         ADJFLX(IA,IR,IGADJ,7)*XTINV(MATALB(IR),IGADJ)*VOLINV(IR))
               IF (LFANG) THEN 
                DO 1661 IANG= -NANGCL,NANGCL
                 DO 1671 IE= 1,NGSSCL
                 IF (ABS(IANG).GT. 0 ) THEN
                  ANGADJ(IR,IANG,IE,IGADJ) = 
     >             ADJFLX(IA,IR,IGADJ,8) / REAL(2*KEYANI(IA)+1)+
     >             ANGADJ(IR,IANG,IE,IGADJ)/YGSS(IE)
     >             /A(AZIWGT+ABS(IANG)-1)
     >             * XTINV(MATALB(IR),IGADJ)*VOLINV(IR)  
                  IF(YGSS(IE).LE.ZERO.OR.A(AZIWGT+ABS(IANG)-1).LE.ZERO)
     >             THEN
                  CALL XABORT('MOC2DR: YGSS OR AZIMUTHAL WGT < 0.0')
                  ENDIF
                 ENDIF
 1671            CONTINUE
 1661           CONTINUE
                DO 1681 IANG= 1,NANGCL
                 DO 1691 IE= 1,NGSSCL
                 DDUM = ANGADJ(IR,IANG,IE,IGADJ)
                 ANGADJ(IR,IANG,IE,IGADJ) = ANGADJ(IR,-IANG,IE,IGADJ)
                 ANGADJ(IR,-IANG,IE,IGADJ) = DDUM 
 1691            CONTINUE
 1681           CONTINUE
               ENDIF
 1651          CONTINUE
 1641         CONTINUE
              IF (LFANG) THEN 
               DO 16411 IR= 1,NREG
               DO 16511 IA= 1,NFUNL
                ADJFLX(IA,IR,IGADJ,7) =  DZERO 
                DO 16611 IANG= -NANGCL,NANGCL
                DO 16711 IE= 1,NGSSCL
                IF(ABS(IANG).GT.0)THEN
                ADJFLX(IA,IR,IGADJ,7)=ADJFLX(IA,IR,IGADJ,7)+WPOLAR(IE)*
     >          A(AZIWGT+ABS(IANG)-1)*ANGADJ(IR,IANG,IE,IGADJ) 
                ENDIF
16711           CONTINUE
16611           CONTINUE
16511          CONTINUE
16411          CONTINUE
              ENDIF
             ENDIF
            ENDIF
            CALL MOCRWD( IFTRAK )
            IF (IFTRK2.GT.0.AND.LOWTRK)  CALL MOCRWD( IFTRK2 )
            IF(LDRTMP .OR. LAJTMP)  MAXITR= MAXITR+1
            IF(NINGR.GT.1 .AND. LREBAL.AND.LDAJCB )THEN                                                     
             CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,
     >                   MATALB(1),VOL,ADJFLX(1,1,1,7),
     >                   XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,PHI,
     >                   XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >                   LCWAJ2,IPFLUX,DIRPIS,LEAKSW,.TRUE.,A(IXVEC),
     >                   ADJFLX(1,1,1,4),XCSOUA )
             CALL MOC2BL(NGRP,IGDAJ,NANIS,PHI,XCSOUA,XCTRC,XCDIA,XCCAT,
     >                   DB2,REBAL,FLUN,IPRT ,JGDAJ,LCWAJ2)
             DO 2301 IGDUM= IGDAJ, (NGRP-JGDAJ)
              DO 2311 IR= 1, NREG                                               
               DO 2321 IS= 1, NFUNL                                          
               ADJFLX(IS,IR,IGDUM,7)= ADJFLX(IS,IR,IGDUM,7)*FLUN(IGDUM)
               IF (FLUN(IGDUM).LT.DZERO) THEN
                WRITE(IOUT,'(20H ADJOINT REBAL FLUN(,I3,2H)=,E15.7, /)') 
     >                      IGDUM,FLUN(IGDUM)
               ENDIF
 2321          CONTINUE
 2311         CONTINUE
 2301        CONTINUE
            ENDIF
            IF(NINGR.GT.1 .AND. LFLD .AND. LREBAL )THEN 
             CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,
     >               MATALB(1),VOL,FLUX(1,1,1,7),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWDR1,
     >               IPFLUX,DIRPIS,LEAKSW, .TRUE. ,A(IXVEC),
     >               FLUX(1,1,1,4),XCSOU) 
             CALL MOC2BL(NGRP,IGDEB,NANIS,PHI,XCSOU,XCTRC,XCDIA,XCCAT,
     >               DB2,REBAL,FLUN,IPRT ,JGDEB,LCWDR1)
             DO 1301 IGDUM= IGDEB, (NGRP-JGDEB)
              DO 1311 IR= 1, NREG
               DO 1321 IS= 1, NFUNL
                  FLUX(IS,IR,IGDUM,7)= FLUX(IS,IR,IGDUM,7)* FLUN(IGDUM)
 1321          CONTINUE
 1311         CONTINUE
 1301        CONTINUE 
            ENDIF
            IF (NINGR.GT.1) THEN
              IF (LFLD ) THEN
               EINN = DZERO
               GINN = DZERO
               FINN = DZERO
               DO 2222 IR= 1, NREG
                DO 2212 IA= 1, NFUNL
                 GINN= MAX(GINN,ABS(FLUX(IA,IR,IG,6)-FLUX(IA,IR,IG,7)))
                 FINN= MAX(FINN,ABS(FLUX(IA,IR,IG,7)))
                 FLUX(IA,IR,IG,5)= FLUX(IA,IR,IG,6)
                 FLUX(IA,IR,IG,6)= FLUX(IA,IR,IG,7)
 2212           CONTINUE
 2222          CONTINUE
               IF (FINN .GT. DZERO) THEN
                GINN= GINN/FINN
               ELSE
                GINN= 1.0D+01
               ENDIF
               EINN=MAX(EINN,GINN)
              ENDIF
              IF (LDAJCB )  THEN
               EINNAJ = DZERO
               GINNAJ = DZERO
               FINNAJ = DZERO
               DO 2223 IR= 1, NREG
                DO 2213 IA= 1, NFUNL
                 GINNAJ= MAX(GINNAJ,
     >                 ABS(ADJFLX(IA,IR,IGADJ,6)-ADJFLX(IA,IR,IGADJ,7)))
                 FINNAJ= MAX(FINNAJ,ABS(ADJFLX(IA,IR,IGADJ,7)))
                 ADJFLX(IA,IR,IGADJ,5)= ADJFLX(IA,IR,IGADJ,6)
                 ADJFLX(IA,IR,IGADJ,6)= ADJFLX(IA,IR,IGADJ,7)
 2213           CONTINUE
 2223          CONTINUE
               IF ( FINNAJ .GT. ZERO) THEN
                 GINNAJ = GINNAJ/FINNAJ
               ELSE
                GINNAJ= 1.0D+01
               ENDIF
               EINNAJ=MAX(EINNAJ,GINNAJ)
              ENDIF
            ENDIF
            IF( EINN.LT.EIMX .AND. EINNAJ.LT.EIMX ) GO TO 45
 451       CONTINUE
 45       CONTINUE
         IF( LREBAL.AND.LDAJCB )THEN                                                     
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,ADJFLX(1,1,1,7),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWAJ2,
     >               IPFLUX,DIRPIS,LEAKSW, .TRUE. ,A(IXVEC),
     >               ADJFLX(1,1,1,4),XCSOUA)
         CALL MOC2BL(NGRP,IGDAJ,NANIS,PHI,XCSOUA,XCTRC,XCDIA,XCCAT,DB2,
     >               REBAL,FLUN,IPRT ,JGDAJ,LCWAJ2)
           DO 2300 IG= IGDAJ, (NGRP-JGDAJ)
            DO 2310 IR= 1, NREG                                               
               DO 2320 IS= 1, NFUNL                                          
               ADJFLX(IS,IR,IG,7)= ADJFLX(IS,IR,IG,7) * FLUN(IG)       
               IF (FLUN(IG).LT.DZERO) THEN
               WRITE(IOUT,6000) IG,FLUN(IG)
               ENDIF
 2320          CONTINUE
 2310       CONTINUE
 2300      CONTINUE
         ENDIF
         IF(LFLD.AND.LREBAL)THEN 
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,FLUX(1,1,1,7),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWDR1,
     >               IPFLUX,DIRPIS,LEAKSW, .TRUE. ,A(IXVEC),
     >               FLUX(1,1,1,4),XCSOU) 
         CALL MOC2BL(NGRP,IGDEB,NANIS,PHI,XCSOU,XCTRC,XCDIA,XCCAT,DB2,
     >               REBAL,FLUN,IPRT ,JGDEB,LCWDR1)
         DO 1300 IG= IGDEB, (NGRP-JGDEB)
            DO 1310 IR= 1, NREG
               DO 1320 IS= 1, NFUNL
                  FLUX(IS,IR,IG,7)= FLUX(IS,IR,IG,7)*FLUN(IG)
 1320          CONTINUE
 1310       CONTINUE
 1300    CONTINUE
         ENDIF
         IF( MOD(JT-1,CTOT).GE.CPTM )THEN
            IF (LFLD) THEN
            CALL MOC2AC( NGRP, NFUNL, NREG, FLUX(1,1,1,5), AKEEP(5),
     >                   IGDEB, MU  , JGDEB)
            ENDIF
            IF (LDAJCB) THEN
            CALL MOC2AC( NGRP, NFUNL, NREG, ADJFLX(1,1,1,5), AKEEAJ(5),
     >                   IGDAJ, MUAJ, JGDAJ )
            ENDIF
         ELSE
            MU= ONE
            MUAJ= ONE
         ENDIF
         IF (LFLD ) THEN
           EINN   = DZERO
           IGDEBO = IGDEB
           JGDEBO = JGDEB  
           JGCOUN = 0
          DO 220 IG= IGDEBO, (NGRP-JGDEBO)
           GINN= DZERO
           FINN= DZERO
           DO 222 IR= 1, NREG
             DO 221 IA= 1, NFUNL
              GINN= MAX(GINN,ABS(FLUX(IA,IR,IG,6)-FLUX(IA,IR,IG,7)))
              FINN= MAX(FINN,ABS(FLUX(IA,IR,IG,7)))
              FLUX(IA,IR,IG,5)= FLUX(IA,IR,IG,6)
              FLUX(IA,IR,IG,6)= FLUX(IA,IR,IG,7)
  221        CONTINUE
  222      CONTINUE
           IF (.NOT. LFLD ) THEN
            GINN= DZERO
            FINN= DONE
           ENDIF
           IF (FINN .GT. ZERO) THEN
               GINN= GINN/FINN
           ELSE
               GINN= 1.0D+01
           ENDIF
           IF( GINN.LT.EIMX .AND.IGDEB.EQ.IG ) THEN
             IGDEB= IGDEB+1
           ENDIF
           IF( GINN.LT.EIMX  ) THEN
             JGCOUN = JGCOUN + 1
           ELSE
             JGCOUN = 0
           ENDIF
           EINN=MAX(EINN,GINN)
  220     CONTINUE
          JGDEB = JGDEBO + JGCOUN
         ENDIF
         IF (LDAJCB )  THEN
           EINNAJ = DZERO
           IGDAJO = IGDAJ
           JGDAJO = JGDAJ
           JGCOAJ = 0
          DO 2201 IG= IGDAJO, (NGRP-JGDAJO)
           GINNAJ= DZERO
           FINNAJ= DZERO
           DO 2221 IR= 1, NREG
             DO 2211 IA= 1, NFUNL
              GINNAJ= MAX(GINNAJ,
     >                    ABS(ADJFLX(IA,IR,IG,6)-ADJFLX(IA,IR,IG,7)))
              FINNAJ= MAX(FINNAJ,ABS(ADJFLX(IA,IR,IG,7)))
              ADJFLX(IA,IR,IG,5)= ADJFLX(IA,IR,IG,6)
              ADJFLX(IA,IR,IG,6)= ADJFLX(IA,IR,IG,7)
 2211        CONTINUE
 2221      CONTINUE
           IF (FINNAJ.GT.DZERO) THEN
               GINNAJ = GINNAJ/FINNAJ
           ELSE
               GINNAJ= 1.0D+01
           ENDIF
           IF(GINNAJ.LT.EIMX .AND. IGDAJ.EQ.IG ) THEN
             IGDAJ= IGDAJ+1 
           ENDIF
           IF(GINNAJ.LT.EIMX  ) THEN
             JGCOAJ = JGCOAJ + 1
           ELSE
             JGCOAJ = 0
           ENDIF
           EINNAJ=MAX(EINNAJ,GINNAJ)
 2201     CONTINUE
          JGDAJ = JGDAJO + JGCOAJ
         ENDIF
        ITERF= JT
        IF (ILOOP.EQ.0) ILPCN0 = ILPCN0 + 1
        IF (ILOOP.EQ.1.AND.LFLD) ILPCN1 = ILPCN1 + 1
        IF (ILOOP.EQ.2.AND.(LFLD.OR.LDAJCB)) ILPCN2 = ILPCN2 + 1
        IF (ILOOP.EQ.3.AND.LDAJCB) ILPCN3 = ILPCN3 + 1
        IF( IPRT.GT.0 )THEN
          IF (LFLD) THEN
*           WRITE(IOUT,2061)
           WRITE(IOUT,2064) JT,'FLX:',EINN,EIMX,IGDEB,MU
          ENDIF
          IF (LDAJCB) THEN
*           WRITE(IOUT,2062)
           WRITE(IOUT,2064) JT,'ADJ:',EINNAJ,EIMX,IGDAJ,MU
          ENDIF
        ENDIF
        IF(IPRT .EQ. 99) WRITE(IOUT,'(1X,A6,2X,I4,A6,2X,I4)')
     >    'IGDEB=',IGDEB,'JGDEB=',JGDEB
        IF(IPRT .EQ. 99) WRITE(IOUT,'(1X,A6,2X,I4,A6,2X,I4)')
     >    'IGDAJ=',IGDAJ,'JGDAJ=',JGDAJ
        IF( EINN.LT.EIMX) THEN
          LFLD= .FALSE.
          OOLDRF= .FALSE.
        ENDIF
        IF( EINNAJ.LT.EIMX) THEN 
          LDAJCB= .FALSE.
          OOLDAJ= .FALSE.
        ENDIF
        IF( EINN.LT.EIMX .AND. EINNAJ.LT.EIMX ) GO TO 750
  500  CONTINUE
       MESSIN='*NOT*'
  750  CONTINUE
       EIMX =    EIMXO   
       IF (ILOOP .EQ. 1) THEN
       LFLD = OLDRFL
       OOLDRF = OLDRFL
       LDAJCB = OLDAJC
       OOLDAJ = OLDAJC
       GOTO 11111
       ENDIF
       IF (ILOOP .EQ. 2) THEN
       LFLD = OLDRFL
       OOLDRF = OLDRFL
       LDAJCB = OLDAJC
       OOLDAJ = OLDAJC
       GOTO 11111
       ENDIF
       IF (ILOOP .EQ. 3) THEN
         ILOOP = 0
       ENDIF
       IF( IPRT.GT.1 )THEN
          WRITE(IOUT,1060) 'INTERNAL',MESSIN,ITERF
       ENDIF
       LFLD = OLDRFL
       OOLDRF = OLDRFL
       LDAJCB = OLDAJC
       OOLDAJ = OLDAJC 
       IF(LDAJCB) THEN
        DO 763 IG= 1, NGRP
          DO 762 IR= 1, NREG
             DO 761 IS= 1, NFUNL
               ADJFLX(IS,IR,IG,3)= ADJFLX(IS,IR,IG,7)
  761        CONTINUE
  762     CONTINUE
  763   CONTINUE
       ENDIF
      IF (LDAJCB.AND.(ITYPEC.EQ.1).AND.(ISADJ.EQ.1) ) THEN
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,ADJFLX(1,1,1,3),
     >               XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,XSCHI,
     >               PHI,
     >               XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,IPRT,
     >               LCWAJ2,
     >               IPFLUX,DIRPIS,LEAKSW, .FALSE. ,A(IXVEC),
     >               ADJFLX(1,1,1,4),XCSOUA) 
            CALL MOCAKF( NGRP, NANIS, PHI, DB2,ADJKEF,AJFNOR,LEAKSW,
     >                   XCTOT,XCDIA,XCCAT,XCNUF,XCCHI, IPRT,PRODFS,
     >                   OLDBAJ)
        IF (LAMH) THEN
        CALL MOCAMH(ITYPEC,NBN,NGRP,NANIS,           
     >       PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >       DIFF,BETA,REBAL,FLUN,AKFRB,B2VAL,OOLPAJ,EBMX,1,AKEFF1)
        ADJKEF=AKEFF1
        AJFNOR=1.0D0/ADJKEF
        ENDIF 
         DO 12000 IG= 1, NGRP
          DO 12100 IR= 1, NREG                       
          DO 12200 IS= 1, NFUNL
          IF(LAMH) ADJFLX(IS,IR,IG,3)=ADJFLX(IS,IR,IG,3)*FLUN(IG)
          IF(LPRODF) ADJFLX(IS,IR,IG,3)=ADJFLX(IS,IR,IG,3)/PRODFS
12200     CONTINUE
12100     CONTINUE
12000    CONTINUE
        IF (LANZFA)  THEN 
         CALL MOCANZ(NGRP,NREG,NFUNL,ADJFLX(1,1,1,3),ANZFAC)
        ENDIF
        EEXTAJ = ABS((ADJKEF-ADJKEO)/ADJKEF)
      ENDIF
      IF(LFLD)THEN
       DO 7631 IG= 1, NGRP
          DO 7621 IR= 1, NREG
             DO 7611 IS= 1, NFUNL
                FLUX(IS,IR,IG,3)= FLUX(IS,IR,IG,7)
 7611        CONTINUE
 7621     CONTINUE
 7631  CONTINUE
       IF(ITYPEC.LT.6)THEN
         CALL MOC2HM(NGRP,NREG,NANIS,NMAT,NFUNL,NIFIS,KEYANI,MATALB(1),
     >               VOL,FLUX(1,1,1,3),XSTOT,XSTRC,XSDIA,XSCAT,XSNUF,
     >               XSCHI,PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,
     >               XCABS,IPRT,LCWDR1,IPFLUX,DIRPIS,LEAKSW,.FALSE. ,
     >               A(IXVEC),FLUX(1,1,1,4),XCSOU) 
         IF( IT.EQ.1 )THEN
C         INITIALIZE DIFFUSION COEFFICIENTS
             DO 1004 IG= 1, NGRP
                DIFF(IG)= DTHIRD/XCTRC(IG)
 1004        CONTINUE
         ENDIF
C      SOLVE BN EQUATION ON THE HOMOGENIZED CELL
         CALL MOC2MH(ITYPEC,NBN,NGRP,NANIS,
     >               PHI,XCTOT,XCTRC,XCDIA,XCCAT,XCNUF,XCCHI,XCABS,
     >               DIFF,BETA,REBAL,FLUN,REFKEF,AKFRB,B2VAL,OOL,EBMX,
     >               1,PRODFS,LEAKSW)
         DO 1200 IG= 1, NGRP
            DO 1210 IR= 1, NREG
               DO 1220 IS= 1, NFUNL
                  IF (PRODFS .GT. 0.000D0) THEN
                   FLUX(IS,IR,IG,3)= FLUX(IS,IR,IG,3)/PRODFS
                  ENDIF
 1220          CONTINUE
 1210       CONTINUE
 1200    CONTINUE
       ENDIF
       IF(ITYPEC.EQ.4)THEN
          DO 335 IG=1,NGRP
             DB2(IG)= DIFF(IG)*B2VAL
  335     CONTINUE
          IF( B2VAL.EQ.DZERO )THEN
             BFLNOR= DONE
          ELSE
             BFLNOR= DONE/ABS(B2VAL)
          ENDIF
          EEXT= ABS(B2VAL-B2VALO)*BFLNOR
          B2VALO= B2VAL
       ENDIF
       IF( ITYPEC .LT. 6 )THEN
C      ** CHANGE 'FLUN' TO 'PHI'
C      ---ORIGINAL 
C           CALL MOC2KF(NGRP,LEAKSW,OLDBIL,
C     >                 FLUN,XCABS,XCNUF,XCCHI,DB2,
C     >                 AKEFF,AFLNOR,IPRT)
           CALL MOC2KF(NGRP,LEAKSW,OLDBIL,PHI,XCABS,XCNUF,XCCHI,DB2,
     >                 AKEFF,AFLNOR,IPRT)

           EEXT= ABS(AKEFF-AKEFFO)/AKEFF
       ELSE
           EEXT= DZERO
       ENDIF
       AKEEP(3)= AKEFF
      ENDIF
      AKEEAJ(3)= ADJKEF
      IF( MOD(IT-1,CTOT).GE.CPTM )THEN
          IF(LFLD) CALL MOC2AC( NGRP , NFUNL, NREG, FLUX(1,1,1,1), 
     >                            AKEEP(1),1, MU, 0 )
          IF (LDAJCB) CALL MOC2AC( NGRP, NFUNL, NREG, ADJFLX(1,1,1,1),
     >                             AKEEAJ(1),1, MUAJ , 0 )
      ELSE
          MU= ONE
          MUAJ= ONE
      ENDIF
      IEXTF= IT
       IF( ITYPEC .LT. 6 )THEN
           IF( IPRT.GT.0 )THEN
              WRITE(IOUT,2065) IT,'FLX:',EEXT  ,EEMX,AKEFF ,B2VAL
              IF (LDAJCB) THEN
                WRITE(IOUT,2065) IT,'ADJ:',EEXTAJ,EEMX,ADJKEF,B2VAL
              ENDIF
           ENDIF
           IF( EEXT.LT. 1.0D+2 )THEN
            EINN   = DZERO
            EINNAJ = DZERO
            DO 322 IG= 1, NGRP
             GINN = DZERO
             FINN = DZERO
             GINNAJ = DZERO
             FINNAJ = DZERO
             DO 321 IR= 1, NREG
              DO 320 IA= 1, NFUNL
                IF (LFLD) THEN
                  GINN= MAX(GINN,ABS(FLUX(IA,IR,IG,2)-FLUX(IA,IR,IG,3)))
                  FINN= MAX(FINN,ABS(FLUX(IA,IR,IG,3)))
                  FLUX(IA,IR,IG,1)= FLUX(IA,IR,IG,2)
                  FLUX(IA,IR,IG,2)= FLUX(IA,IR,IG,3)
                ENDIF
                IF (LDAJCB ) THEN
                GINNAJ= MAX(GINNAJ,
     >                      ABS(ADJFLX(IA,IR,IG,2)-ADJFLX(IA,IR,IG,3)))
                FINNAJ= MAX(FINNAJ,ABS(ADJFLX(IA,IR,IG,3)))
                ADJFLX(IA,IR,IG,1)= ADJFLX(IA,IR,IG,2)
                ADJFLX(IA,IR,IG,2)= ADJFLX(IA,IR,IG,3)
                ENDIF
  320         CONTINUE
  321        CONTINUE
             IF (.NOT. LFLD ) THEN
              GINN= DZERO
              FINN= DONE
             ENDIF
             IF (FINN .GT. DZERO) THEN
              GINN= GINN/FINN
             ELSE
              GINN= 1.0D+01
             ENDIF
             IF (.NOT. LDAJCB ) THEN
              GINNAJ= DZERO
              FINNAJ= DONE
             ENDIF
             IF (FINNAJ .GT. DZERO) THEN
                GINNAJ = GINNAJ/FINNAJ
             ELSE
                GINNAJ= 1.0D+01
             ENDIF
             EINN   = MAX(EINN  ,GINN)
             EINNAJ = MAX(EINNAJ,GINNAJ)
  322       CONTINUE
            IF( IPRT.GT.0 )THEN
             IF (LFLDO) THEN 
*              WRITE(IOUT,2061)
              WRITE(IOUT,2066) IT,'FLX:',EINN,EIMX,AFLNOR,MU
             ENDIF
             IF (LDAJCO) THEN
*              WRITE(IOUT,2062)
              WRITE(IOUT,2066) IT,'ADJ:',EINNAJ,EIMX,AJFNOR,MUAJ
             ENDIF
            ENDIF
           ELSE
            DO 325 IG= 1, NGRP
             DO 324 IR= 1, NREG
              DO 323 IA= 1, NFUNL
               IF (LFLD) THEN
                FLUX(IA,IR,IG,1)= FLUX(IA,IR,IG,2)
                FLUX(IA,IR,IG,2)= FLUX(IA,IR,IG,3)
               ENDIF
               IF (LDAJCB) THEN
                ADJFLX(IA,IR,IG,1)= ADJFLX(IA,IR,IG,2)
                ADJFLX(IA,IR,IG,2)= ADJFLX(IA,IR,IG,3)
               ENDIF
  323         CONTINUE
  324        CONTINUE
  325       CONTINUE
            IF( IPRT.GT.0 )THEN
             WRITE(IOUT,2067) IT,AFLNOR,MU
            ENDIF
           ENDIF
       ELSE
         DO 425 IG= 1, NGRP
          DO 424 IR= 1, NREG
           DO 423 IA= 1, NFUNL
            IF (LFLD) THEN
             FLUX(IA,IR,IG,1)= FLUX(IA,IR,IG,2)
             FLUX(IA,IR,IG,2)= FLUX(IA,IR,IG,3)
            ENDIF
            IF (LDAJCB) THEN
             ADJFLX(IA,IR,IG,1)= ADJFLX(IA,IR,IG,2)
             ADJFLX(IA,IR,IG,2)= ADJFLX(IA,IR,IG,3)
            ENDIF
  423      CONTINUE
  424     CONTINUE
  425    CONTINUE
       ENDIF
        AKEEP(1)= AKEEP(2)
        AKEEP(2)= AKEEP(3)
        AFLNOR= (AKEFF/AKEEP(3))*AFLNOR
        AKEEAJ(1)= AKEEAJ(2)
        AKEEAJ(2)= AKEEAJ(3)
        AJFNOR= (ADJKEF/AKEEAJ(3))*AJFNOR
        AKEFFO = AKEFF
        ADJKEO = ADJKEF
        IF (EEXT.LT. 1.0D-02 .AND. EINN.LT. 1.0D-02 .AND. LBETAS
     >      .AND. EEXTAJ.LT.1.0D-02 .AND. EINNAJ.LT.1.0D-02 )THEN
         LBETAS = .FALSE.
        ENDIF
        IF (EEXT.LT.1.0D-02 .AND. EINN.LT.1.0D-02 .AND. LOWTRK 
     >      .AND. EEXTAJ.LT.1.0D-02 .AND. EINNAJ.LT.1.0D-02 )THEN
         LOWTRK = .FALSE.
        ENDIF
        IF (EEXT.LT.EEMX.AND.EINN.LT.EIMX) THEN
         LFLD = .FALSE. 
         OLDRFL = .FALSE.
        ENDIF
        IF (EEXTAJ.LT.EEMX.AND.EINNAJ.LT.EIMX) THEN
         LDAJCB = .FALSE. 
         OLDAJC = .FALSE.
        ENDIF
        IF( EEXT.LT.EEMX .AND. EEXTAJ.LT.EEMX .AND. 
     >      EINN.LT.EIMX .AND. EINNAJ.LT.EIMX) THEN
           IF(LEXACT) GO TO 240
           IGDEB= 1
           JGDEB= 0
           IGDAJ= 1
           JGDAJ= 0
           LEXACT= .TRUE.
           IF( IPRT.GE. 1)THEN
              WRITE(IOUT,7000)
           ENDIF
        ENDIF
 210  CONTINUE
      IF( IPRT.GT.0 )THEN
         WRITE(IOUT,7001)
      ENDIF
      MESSOU= '*NOT*'
 240  CONTINUE
      LREBAL = LREBAO 
      LPJC   = LPJCO
      ISADJ  = ISADJO
      LFLD = LFLDO
      LDAJCB = LDAJCO
      RAJKEF = ADJKEF 
      RKEFF  = AKEFF
      RB2    = B2VAL
       IF (LDAJCB) THEN
       CALL MOCFIM(NREG  ,NGRP  ,NL,NMAT,NANIS ,NIFIS,NFUNL,
     >             ADJKEF,ISADJ ,ITYPEC ,NSOUT ,MATALB,XSDIA,
     >             XSCAT ,ADJSOU,XSNUF  ,XSCHI ,ADJFLX)
       DO 776 IG= 1, NGRP
         DO 775 IR= 1, NREG
            DO 774 IS= 1, NFUNL
                ADJFLO(IR,IS,IG)= ADJFLX(IS,IR,IG,4)
  774       CONTINUE
  775    CONTINUE
  776  CONTINUE
       CALL LCMSIX(IPFLUX,CADJFL(2),ILCMUP)
       CALL MOCSAJ(IPFLUX,NGRP,NREG,ADJFLO)
       CALL LCMPUT(IPFLUX,'ADJ K-EFF   ',1,2,RAJKEF)
       CALL LCMSIX(IPFLUX,CADJFL(2),ILCMDN)
       ENDIF
      IF( IPRT.GT.0 )THEN
         WRITE(IOUT,1060) 'EXTERNAL',MESSOU,IEXTF
         IF (LFLDO .AND. .NOT.LDAJCO) THEN
          WRITE(IOUT,1061) MAXITR,AKEFF,B2VAL,EEXT
         ENDIF
         IF (LDAJCO .AND. .NOT.LFLDO) THEN
          WRITE(IOUT,1062) MAXITR,ADJKEF,B2VAL,EEXTAJ
         ENDIF
         IF (LDAJCO .AND. LFLDO) THEN
          WRITE(IOUT,1063) MAXITR,AKEFF,ADJKEF
         ENDIF
         WRITE(IOUT,2070) ILPCN0
         WRITE(IOUT,2071) ILPCN1
         WRITE(IOUT,2072) ILPCN2
         WRITE(IOUT,2073) ILPCN3
      ENDIF
      IF( ITYPEC .EQ. 4 )THEN
         DO 326 IG= 1, NGRP
            DIFFB1(IG)= DIFF(IG)
  326    CONTINUE
      ENDIF
      IF( IPRT.GE.2 )THEN
         WRITE(IOUT,1030) (IR,IR=1,NREG)
         DO 161 IG=1,NGRP
            WRITE(IOUT,1090) IG
            WRITE(IOUT,1040) (FLUX(1,IR,IG,3)*AFLNOR,IR=1,NREG)
            DO 261 IA= 2, NFUNL
               WRITE(IOUT,1045) IA,
     >         (FLUX(IA,IR,IG,3)*AFLNOR,IR=1,NREG)
  261       CONTINUE
  161    CONTINUE
      ENDIF
      DO 773 IG= 1, NGRP
         DO 772 IR= 1, NREG
            DO 771 IS= 1, NFUNL
                FLUXO(IR,IS,IG)= FLUX(IS,IR,IG,3)
  771       CONTINUE
  772    CONTINUE
  773 CONTINUE
      DO 873 IG= 1, NGRP
         DO 872 IR= 1, NREG
            DO 871 IS= 1, NFUNL
                ADJFLO(IR,IS,IG)= ADJFLX(IS,IR,IG,3)
  871       CONTINUE
  872    CONTINUE
  873 CONTINUE
      CALL RLSARA(A(IPVEC))
      CALL RLSARA(A(IXVEC))
      CALL RLSARA(A(IYVEC))
      CALL RLSARA(A(IZVEC))
      CALL RLSARA(A(ISVEC))
      IF (LPJC)   CALL RLSARA(A(ICP))
      IF (LFLD) THEN
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        CALL MOC2WR(IPFLUX,ITYPEC,RKEFF,RB2,
     >              NFUNL,NREG,NGRP,FLUXO,DIFFB1)
        IF (LFANG) THEN 
          CALL MOCSAC(IPFLUX,IPRT,CANGDR,NGRP,NPHI,NGSS,NREG,
     >              ANGPHI,A(AFLXDU))
        ENDIF
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      ENDIF
      IF ( LDAJCB ) THEN
        CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
        CALL MOCSAJ(IPFLUX,NGRP,NREG,ADJFLO)
        CALL LCMPUT(IPFLUX,'ADJ K-EFF   ',1,2,RAJKEF)
        IF (LFANG) THEN 
          CALL MOCSAC(IPFLUX,IPRT,CANGAJ,NGRP,NPHI,NGSS,NREG,
     >                ANGADJ,A(AFLXDU))
        ENDIF
        CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN)
      ENDIF
      CALL RLSARA( A( AZIWGT )  )
      CALL RLSARA( A (AFLXDU )  )
      RETURN
  999 FORMAT (/28H SOLVING TRANSPORT EQUATION   //)
 1010 FORMAT (/28H PROPERTIES OF THE GEOMETRY:/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1020 FORMAT (1H ,A6,2H: ,1P,10E12.5/(9X,10E12.5))
 1025 FORMAT (1H ,A2,1H(,I2,3H): ,1P,10E12.5/(9X,10E12.5))
 1030 FORMAT (/28H FLUXES AVERAGED           :/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1035 FORMAT (/28H CURRENT AVERAGED          :/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1040 FORMAT (7H FLUX  ,2H: ,1P,10E12.5/(9X,10E12.5))
 1045 FORMAT (5H CUR ,I2,2H: ,1P,10E12.5/(9X,10E12.5))
 1055 FORMAT (8H SOURCE ,2H: ,1P,10E12.5/(9X,10E12.5))
 1060 FORMAT ( 1X,A8,
     1            13H CONVERGENCE ,A5,14H REACHED AFTER ,I6,
     2            12H ITERATIONS.  )
 1061 FORMAT (/1X,18H TRACKING CALLED= ,I6,6H TIMES ,
     1            13H FINAL KEFF= ,1P,D15.7,5H, B2=,E15.6,
     1            11H PRECISION=,E9.2)

 1062 FORMAT (/1X,'TRACKING CALLED= ',I6,1X,'TIMES',1X,
     1            'FINAL ADJOINT KEFF= ',1P,D15.7,', B2=',E15.6,1X,
     1            'PRECISION= ',E9.2)
 1063 FORMAT (/1X,'TRACKING CALLED= ',I6,1X,' TIMES',1X,
     1            'FINAL DIRECT KEFF= ',1P,D15.7,',',4X,
     1            'FINAL ADJOINT KEFF= ',1P,D15.7  )
 1067 FORMAT ( 10X,14H  GROUP CONV= ,100I1)
 1070 FORMAT (/21H FINAL VALUE OF KEFF=  ,1P,E15.6 )
 1080 FORMAT (11H ITERATION ,I4)
 1090 FORMAT (/14H ENERGY GROUP ,I6)
 1100 FORMAT (15H FLUX AVERAGED ,2H: ,1P,10E12.5/(9X,10E12.5))
* 2061 FORMAT (     1X,23HDIRECT FLUX CALCULATION )
* 2062 FORMAT (     1X,24HADJOINT FLUX CALCULATION )
 2064 FORMAT (     10X,'IN(',I3,') ',A4,
     1             ' PRC=',1P,E9.2,
     2             ' TAR=',   E9.2,
     3             ' IGDEB=', I13,
     4             ' ACCE=',0P,F12.5)
 2065 FORMAT (     ' OUT(',I3,') EIG',A4,
     1             ' PRC=',1P,E9.2,
     2             ' TAR=',   E9.2,
     3             ' KEF=',D15.7,
     4             ' BUCK=',E12.5)
 2066 FORMAT (     ' OUT(',I3,') EXT',A4,
     1             ' PRC=',1P,E9.2,
     2             ' TAR=',   E9.2,
     3             ' FNOR=',   E13.6,
     4             ' ACCE=',0P,F12.5)
 2067 FORMAT (     5H OUT(,I3,6H) FLX:,
     1             28X,
     3             6H FNOR=,1P,E13.6,
     4             6H ACCE=,0P,F12.5)
 2070 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H NON-GROUPSPLITING =     ,I6)
 2071 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H GROUPSPLITTING SET #1 = ,I6)
 2072 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H GROUPSPLITTING SET #2 = ,I6)
 2073 FORMAT (34H TOTAL NO. OF INNER ITERATIONS FOR,
     1        25H GROUPSPLITTING SET #3 = ,I6)
 6000 FORMAT(' ADJOINT REBAL FLUN(',I3,')=',E15.7/)
 7000 FORMAT(1X,'<<< FROM NOW ON, USING EXACT EXP(-X) CALLS >>>')
 7001 FORMAT(1X,3('*** MOC2DR: CONVERGENCE NOT REACHED ***'/))
      END
