*DECK MOC3A0
      SUBROUTINE MOC3A0(IFTRAK, NG, MT, NR, NGSS, NS ,MXS  ,NT  ,
     >                  MATALB, WG, YG, SEGLEN, NRSEG,EXPT ,EXP2,
     >                  FLM   ,FLP, CYM, CYP, NA, SIGANG,SR,FLUX, 
     >                  IG0 ,JGDEB,NE)
*  
*---------------------------   MOC3A0  --------------------------------
* 1- PROGRAMME STATISTICS:
*     NAME     :  MOC3A0
*     USE      :  NON-COMBINING SPATIAL ITERATION LOOP 
*     AUTHOR   :  ROY R. FROM MOC3I0.f
*     MODIFIED :  MONCHAI A. (22/03/07)
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     FLUX     : FLUX ARRAY                              D(NF,NR,NG)
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     IG0      : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     MATALB   : MATERIAL CODE IN REGIONS.               I(-NS:NR)
*     MT       : NUMBER OF MIXTURES CONSIDERED           I
*     MXE      : FIXED MAXIMUM NUMBER OF SEGMENTS ON     I
*                EACH TRACK (FIXED = 64) (MXE >= MXS)
*     MXS      : MAXIMUM NUMBER OF SEGMENTS ON EACH      I
*                TRACK (VARIABLE ACC. TO TRACKING LINE)
*     NA       : NUMBER OF SURFACE ALBEDO                I 
*     NF       : NUMBER OF LEGENDRE ORDER FOR FLUX       I
*     NG       : NUMBER OF GROUPS TREATED                I
*     NGSS     : MAXIMUM NUMBER OF POLAR QUADRATURE      I
*     NE       : NUMBER OF POLAR QUADRATURE              I
*                (LOW OR HIGH ORDER TRACKING LINE)
*     NR       : NUMBER OF REGIONS CONSIDERED            I
*     NRSEG    : KEY INDEX FOR REGIONAL NUMBER SEGMENT   I(MXS)
*     NS       : NUMBER OF SURFACES CONSIDERED           I
*     SEGLEN   : LENGTH OF EACH SEGMENT                  R(MXS)  
*     SR       : REGIONAL TOTAL SOURCES                  D(NF,NR,NG)
*     TITRE    : EXECUTION TITLE                         C*72
*
*   WORK VECTOR
*     FLM      : DUMMY VARIABLE                          D(NGSS,MXS)
*     FLP      : DUMMY VARIABLE                          D(NGSS,MXS) 
*     CYM      : DUMMY VARIABLE                          D(NGSS,MXS)
*     CYP      : DUMMY VARIABLE                          D(NGSS,MXS)
*---------------------------   MOC3A0  --------------------------------
      IMPLICIT  NONE
      INTEGER   NF,MXE
      PARAMETER (NF=1,MXE=64)
      INTEGER   IFTRAK, NG, MT, NR, NS , NE   , MXS, NT, NA,
     >          IG    , IR, IE, IT, IG0, JGDEB,NGSS,
     >          NOIL, MTIL, LINE, IANG, IL, JL, NUMOLD
      INTEGER   NRSEG(MXS), MATALB(-NS:NR)
      DOUBLE PRECISION   SR(NF,NR,NG), FLUX(NF,NR,NG)
      REAL      SIGANG(NGSS,-NA:MT,NG),
     >          EXPT(NGSS,MXS), EXP2(NGSS,MXS), SEGLEN(MXS)
      REAL      DWEIG(MXE), WG(MXE), YG(MXE)
      REAL      WEIGHT,ONE, ZERO
      DOUBLE PRECISION FLM(NGSS,MXS),FLP(NGSS,MXS),
     >                 CYM(NGSS,MXS),CYP(NGSS,MXS)
      DOUBLE PRECISION FL0(MXE), CY0(MXE), FL1(MXE), CY1(MXE)
      DOUBLE PRECISION REAL2,DUMMY1,DUMMY2,TAU,CUTEXP,DONE,DZERO,HALF,
     >                 SIXT
      INTEGER   IOUT
      PARAMETER ( IOUT=6,ONE=1.0E0,ZERO=0.0E0,DONE=1.0D0,DZERO=0.0D0, 
     >            CUTEXP=2.0D-02,HALF=0.5D0)
      SIXT = HALF/3.0D0
      DO 1000 IG= IG0, (NG-JGDEB)
         DO 1100 IR=1,NR
            FLUX (NF,IR,IG)= DZERO
 1100    CONTINUE
 1000 CONTINUE
      DO 4000 IT= 1, NT
           READ(IFTRAK)  IANG,LINE,WEIGHT,
     >       (NRSEG(IL),IL= 1,LINE),(SEGLEN(IL),IL= 1,LINE)
           IF (LINE .GT. MXS) CALL XABORT('MOC3A0: LINE>MXS ') 
           DO 4100 IE=1,NE
              DWEIG(IE)= WEIGHT*WG(IE)
               IF (DWEIG(IE) .GT. ZERO) THEN
                CALL XABORT('MOC3A0: DWEIG(IE) IS POSITIVE') 
               ENDIF
 4100      CONTINUE
           DO 4200 IG= IG0, (NG-JGDEB)
              DO 4210 IE=1,NE
                 FL0(IE)= DZERO
                 FL1(IE)= DZERO
                 CY0(IE)= DONE
                 CY1(IE)= DONE
 4210         CONTINUE
              NUMOLD= NRSEG(1)
              DO 4220 IL= 1, LINE
                 NOIL  = NRSEG(IL)
                 MTIL  = MATALB(NOIL)
                 IF( MTIL.LT.0 )THEN
                    IF( NUMOLD.NE.NOIL )THEN
                       DO 4221 IE=1,NE
                          FLM (IE,IL)= DZERO
                          FLP (IE,IL)= DZERO
                          EXP2(IE,IL)= SIGANG(IE,MTIL,IG)
                          EXPT(IE,IL)= ONE
 4221                  CONTINUE
                    ELSE
                       DO 4229 IE=1,NE
                          FLM (IE,IL)= DZERO
                          FLP (IE,IL)= DZERO
                          EXP2(IE,IL)= ZERO
                          EXPT(IE,IL)= ZERO
 4229                  CONTINUE
                    ENDIF
                 ELSE
                    DO 4222 IE=1,NE
                       TAU = SIGANG(IE,MTIL,IG)*SEGLEN(IL)
                       IF( TAU.EQ.DZERO )THEN
                          EXP2(IE,IL)= SEGLEN(IL)*YG(IE)
                          EXPT(IE,IL)= ZERO
                       ELSEIF (SIGANG(IE,MTIL,IG).LT.CUTEXP) THEN
CCCC                          EXP2(IE,IL)= TAU - (HALF-SIXT*TAU )* TAU**2
                          EXP2(IE,IL)= TAU - (HALF-SIXT*TAU )* TAU**2
                          EXPT(IE,IL)= ONE
                       ELSE
                          EXP2(IE,IL)= ONE-EXP(-TAU)
                          EXPT(IE,IL)= ONE
                       ENDIF
                       FLM (IE,IL)= DWEIG(IE)*SR(NF,NOIL,IG)
                       FLP (IE,IL)= DWEIG(IE)*SR(NF,NOIL,IG)
 4222               CONTINUE
                 ENDIF
                 NUMOLD= NOIL
 4220         CONTINUE 
              DO 4226 IL= 1, LINE
                 JL= LINE + 1 - IL
                 DO 4225 IE=1,NE
C  ORIGINAL                 
CCCC                    FLM(IE,IL) = (FLM(IE,IL) + FL0(IE)) * EXP2(IE,IL)
CCCC                    CYM(IE,IL) =               CY0(IE)  * EXP2(IE,IL)
CCCC                    FLP(IE,JL) = (FLP(IE,JL) + FL1(IE)) * EXP2(IE,JL)
CCCC                    CYP(IE,JL) =               CY1(IE)  * EXP2(IE,JL)
CCCC                    FL0(IE)    = FL0(IE) - FLM(IE,IL)*EXPT(IE,IL)
CCCC                    CY0(IE)    = CY0(IE) - CYM(IE,IL)*EXPT(IE,IL)
CCCC                    FL1(IE)    = FL1(IE) - FLP(IE,JL)*EXPT(IE,JL)
CCCC                    CY1(IE)    = CY1(IE) - CYP(IE,JL)*EXPT(IE,JL)
                    DUMMY1     = (FLM(IE,IL) + FL0(IE)) * EXP2(IE,IL)
                    DUMMY2     = (FLP(IE,JL) + FL1(IE)) * EXP2(IE,JL)
                    FLM(IE,IL) = (FL0(IE)) * EXP2(IE,IL)
                    CYM(IE,IL) =               CY0(IE)  * EXP2(IE,IL)
                    FLP(IE,JL) = (FL1(IE)) * EXP2(IE,JL)
                    CYP(IE,JL) =               CY1(IE)  * EXP2(IE,JL)
                    FL0(IE)    = FL0(IE) - DUMMY1*EXPT(IE,IL)
                    CY0(IE)    = CY0(IE) - CYM(IE,IL)*EXPT(IE,IL)
                    FL1(IE)    = FL1(IE) - DUMMY2*EXPT(IE,JL)
                    CY1(IE)    = CY1(IE) - CYP(IE,JL)*EXPT(IE,JL)
 4225            CONTINUE
 4226         CONTINUE
              DO 4230 IE=1,NE
                 FL0(IE)=  FL0(IE)/(DONE-CY0(IE))
                 FL1(IE)=  FL1(IE)/(DONE-CY1(IE))
 4230         CONTINUE
              DO 4240 IL= 1, LINE
                 NOIL  = NRSEG(IL)
                 MTIL  = MATALB(NOIL)
                 IF( NOIL.GT.0 )THEN
                    DO 4241 IE=1,NE
                    TAU = SIGANG(IE,MTIL,IG)*SEGLEN(IL)
                    IF( TAU.EQ.DZERO )THEN
                     DUMMY2= -DWEIG(IE)*SR(NF,NOIL,IG)*0.5*EXP2(IE,IL)*
     >                        EXP2(IE,IL)
                    ELSE
                     DUMMY2=-DWEIG(IE)*SR(NF,NOIL,IG)*(TAU-EXP2(IE,IL))
                    ENDIF
                    FLUX(NF,NOIL,IG)= FLUX(NF,NOIL,IG)+(FLM(IE,IL)+
     >              DUMMY2+FL0(IE)*CYM(IE,IL))+(FLP(IE,IL)+DUMMY2+
     >              FL1(IE)*CYP(IE,IL))
 4241               CONTINUE 
                 ENDIF 
 4240         CONTINUE
 4200       CONTINUE
 4000 CONTINUE
      RETURN
      END
