*DECK MOCDRV
      SUBROUTINE MOCDRV(IPFLUX,IPTRK,IPMACR,IFTRAK,NGRP,NMAT,NIFIS,
     > NANIS,NL,NBN,NUN,FLUXO,DIFFB1,
     > NINTMX,AIEPS,NEXTMX,AEEPS,IFRITR,IACITR,ITYPEC,ILEAK,
     > NREGIO,MATCOD,VOL,VOLINV,
     > XSOLD,XSSIGT,XTINV,XSSIGW,XSCHI,XSNUF,XSTK,
     > NJJ,IJJ,IPOS,XSCAT,VSCAT,NALB,ALBEDO,
     > LEAKSW,ILK,ITRANC,ITRB,TITRE,LCACA,NGSS,LEXIN,REFKEF,
     > IPRT,CDIRFL,
     > ADJFLO,CADJFL,ISADJ,LDRFLX,LDAJCB,RKEFF,LMODE,
     > IPTRK2,IFTRK2,NGSS2,FLUX) 
*  
* ---------------------------  MOCDRV  ---------------------------------
* 
*  1- PROGRAMME STATISTICS:
* 
*      NAME     :  MOCDRV
*      USE      :  DRIVER FOR CHARACTERISTICS SOLVER
*      AUTHOR   :  R. ROY (22/01/97)
*      MODIFIED :  MONCHAI (01/01/05)
*
*  2- ROUTINE PARAMETERS:
*    INPUT
*      FLUX     : FLUX ARRAY                                   D(NFUNL,
*                                                                NREG,
*                                                                NGRP,8)
*      ADJFLX   : ADJOINT FLUX ARRAY                           D(NFUNL,
*                                                                NREG,
*                                                                NGRP,8)
*      FLUXO    : GLOBAL DIRECT FLUX ARRAY                     R(NUN,
*                                                                NFUNL,
*                 FOR MODULE DATA EXCHANGE                       NGRP)
*      IPFLUX   : POINTER TO THE FLUX                          I
*      IPTRK    : POINTER TO THE TRACKING                      I
*      IPMACR   : POINTER TO THE MACROLIB                      I
*      IFTRAK   : FILE UNIT NUMBER FOR TRACKS.                 I
*      NGRP     : NUMBER OF GROUPS TREATED                     I
*      NANIS    : NUMBER OF ANISOTROPY IN                      I
*                  SCATTERING CROSS SECTIONS
*      NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX            I
*      NIFIS    : NUMBER OF FISSILE ISOTOPES                   I
*      MATCOD   : MATERIAL CODE IN REGIONS.                    I(NREGIO)
*      NMAT     : NUMBER OF MIXTURES CONSIDERED                I
*      NREGIO   : NUMBER OF REGIONS CONSIDERED                 I
*      NUNKNO   : INTEGER SCALAR VARIABLE CONTAINING           I
*                 THE NUMBER OF UNKNOWNS (NUMBER OF 
*                 REGIONS AND NUMBER OF CURRENTS). 
*      MATCOD   : MIXTURE CODE IN REGION                       I(NREGIO)
*      VOL      : VOLUME OF REGION                             R(NREGIO)
*      VOLINV   : INVERSE OF VOLUMES                           R(NREGIO)
*      LEAKSW   : LEAKAGE SWITCH                               L
*      ITRANC   : TYPE OF TRANSPORT CORRECTIONS                I
*      TITRE    : EXECUTION TITLE                              C*72
*      REFKEF   : KEFF VALUE FOR B2 SEARCH                     D
*      IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER       I
*      CDIRFL   : FLUX DIRECTORY NAME                          C*12
*      WORD     : EQUAL TO 2:- TO CONVERT VARIABLE TO
*                 DOUBLE PRECISION                             I
*      ITYPEC   : TYPE OF FLUX CALCULATION                     I
*         ITYPEC= 0 TO SKIP THE FLUX CALCULATION; 
*         ITYPEC= 1 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES 
*                   AND KEFF CONVERGENCE
*         ITYPEC= 2 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES,
*                   KEFF CONVERGENCE AND IMPOSED 
*                   LEAKAGE
*         ITYPEC= 3 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES,
*                   KEFF CONVERGENCE AND IMPOSED 
*                   BUCKLING
*         ITYPEC= 4 TO COMPUTE THE DIRECT PROBLEM 
*                   SOLUTION WITH FISSION SOURCES,
*                   IMPOSED KEFF AND CONVERGENCE 
*                   ON BUCKLING
*         ITYPEC= 5 DIRECT PROBLEM WITH LEAKAGE 
*                   SOURCES AND CONVERGENCE ON 
*                   BUCKLING
*         ITYPEC= 6 DIRECT PROBLEM FIXED SOURCES.
*     ISADJ  : FLAG FOR TYPE OF ADJOINT CALCULATION      I
*        ISADJ = 1 ADJOINT FLUX CALCULATION W/O 
*                  EXTERNAL SOURCE FOR ADJOINT KEFF 
*                  PROBLEM
*        ISADJ = 2 ADJOINT FLUX CALCULATION W/
*                  EXTERNAL SOURCE FOR ADJOINT FIXED 
*                  SOURCE PROBLEM
*        ISADJ = 3 GENERALIZED ADJOINT FLUX CALCULATION W/
*                  EXTERNAL GENERALIZED ADJOINT SOURCE 
*      ADJFLX   : POINTER INDEX FOR ADJOINT FLUX ARRAY         I
*      ADJSOU   : POINTER INDEX FOR EXTERNAL ADJOINT SOURCE    I
*      CADJFL   : STRING CONTAINING THE NAME OF ADJOINT FLUX   C(2)*8
*                 DIRECTORY IN IPFLUX DATA STRUCTURE
*        CADJFL(1) FOR SOURCE IMPORTANCE
*        CADJFL(2) FOR FLUX IMPORTANCE
*      ADJFLO   : ADJOINT FLUX ARRAY FOR MODULE DATA EXCHANGE  R(NUN,NGRP)
*      ADJFXS   : INDEX POINTER OF ADJFIX SOURCE               I
*      LDRFLX   : CONTROL FLAG FOR DIRECT FLUX CALCULATION     L
*                 [ .TRUE. : ENABLE DIRECT FLUX CAL. ]
*                 [ .FALSE.: DISABLE DIRECT FLUX CAL.]
*      GMXUPS   : MAX.ENERGY GROUP CONTAINING UPSCATTERING XS  I
*                 (EX. GROUP 27)
*      LSPLIT   : CONTROL FLAG FOR ITERATION WITH GROUP        L
*                 SPLITTING
*      LKGALB   : ALBEDO LEAKAGE SWITCH FOR CHECKING W/LEAKSW  L
*      XSCAT    : GROUP SCATTERING CROSS SECTION               R(NMAT+1)
*      XSCHI    : FISSION NEUTRON DISTRIBUTION                 R(0:NMAT,
*                                                                NIFIS,
*                                                                NGRP)
*      XSNUF    : DUMMY VARIABLES                              R(0:NMAT,
*                                                                NIFIS,
*                                                                NGRP)
*      XSSIGT   : GROUP TOTAL CROSS SECTION                    R(NMAT+1)
*      XSSIGW   : WITHIN GROUP SCATTERING                      R(NMAT+1)
*      XSTK     : DUMMY VARIABLES                              R(NMAT,
*                                                                NIFIS)
*
*  3- INTERNAL PARAMETERS:
*      IOUT     : OUTPUT UNIT = 6                              I
*      ILCMUP   : GO UP LCM INDEX = 1                          I
*      ILCMDN   : GO DOWN LCM INDEX = 2                        I
* 
* ---------------------------   MOCDRV  --------------------------------
*
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN,NDIMO,MXGAUS
      INTEGER     IPFLUX,IPTRK,IPMACR,IFTRAK,NGRP,NMAT,NIFIS,
     >            NANIS,NL,NBN,NUN,NINTMX,NEXTMX,IFRITR,IACITR,
     >            ITYPEC,ILEAK,NREGIO,ITRANC,ITRB,LCACA,NGSS,IPRT
      REAL        AIEPS,AEEPS
      REAL        ZERO,ONE,EPSR
      PARAMETER   (IOUT=6,ILCMUP=1,ILCMDN=2)
      PARAMETER   (NDIMO=2, MXGAUS=64, EPSR=1.E-6)
      PARAMETER   (ZERO=0., ONE=1.)
      LOGICAL     LKGALB,LEAKSW,ILK,LEXIN,NOVOID
      CHARACTER   TITRE*72,CDIRFL*12,CGRPNM*12
      INTEGER     IGR,IS,ILNLCM,NS,NR,NFUNL,NRNL,NSCRP,JUN,NALB,
     >            IL,NMOD,MATMAX,MATMIN,JGSS,NPHI,IANI,IFIS,JGR,
     >            NCOMNT,IXSS,ITYLCM,IREGIO,IR,IMAT,NTRK,NCOR,
     >            NALBG,NANGL,NUNKNO,NDIM,IC,ISPEC,NSOUT,NREG,MXSEG
      REAL        ASCRP,RKEFF,RB2,RAJKEF
      INTEGER     MATCOD(NREGIO),NJJ(NMAT),IJJ(NMAT),IPOS(NMAT)
      REAL        VOL(NREGIO),VOLINV(NREGIO),
     >            XSSIGT(0:NMAT,NGRP),XSSIGW(0:NMAT,NGRP),
     >            XSOLD(0:NMAT,NGRP),XTINV(0:NMAT,NGRP),
     >            XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),
     >            VSCAT(NMAT*NGRP*NGRP),ALBEDO(NALB),
     >            XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >            XSTK(NMAT,NIFIS),DIFFB1(NGRP)
      REAL        XGSS(MXGAUS),WGSS(MXGAUS),YGSS(MXGAUS)
      REAL        FLUXO(NUN,NGRP)
      REAL        ADJFLO(NUN,NGRP)
      CHARACTER   CADJFL(2)*12,CPOLAR*12,CAZIMU*12
      INTEGER     ADJFLX,ADJSOU,ISADJ, ADJFXS,IIM,WORD,IPTRK2,IFTRK2,
     >            NGSS2,NTRK2,MXSEG2,NANGL2,XMTOT,XMTRC,XMCAT,XMDIA
      LOGICAL     LDRFLX,LDAJCB,LMODE(20)
      INTEGER     GMXUPS, FGUPSC, IM, JM
      INTEGER     VOLSUR, MATALB, PHIANG, PHIDUM, SIGANG,AZIWGT,
     >            NUMERO, LENGHT, EXPTAJ, EXP2AJ,
     >            EXPT  , EXP2  , ICYCLE, FLM   , FLP   ,FLMADJ,FLPADJ,
     >            ICYM  , ICYP  , ICYMAJ, ICYPAJ, IALPHA,DFLUX ,DWEIG ,
     >            FMEAN , ISGNR , PNL   ,
     >            RHARM , KEYANI, TRHAR , XCDIA , DIFF  ,
     >            REBAL , FLUC  , XCSOU,  XCTOT , XCNUF , XCTRC , FLUN,
     >            XCCHI , XCABS , XCCAT , DB2   , IBETA , XCSOUA,
     >            ANGPHI, ANGADJ
      CHARACTER*2 CAN(0:9)
      CHARACTER   CINI*4, COMENT*80
      DOUBLE PRECISION REFKEF,FLUX(1,NREGIO,NGRP,8)
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      DATA CAN /' 0',' 1',' 2',' 3',' 4',' 5',' 6',' 7',' 8',' 9'/
      DATA CINI/'$TRK'/
       WORD = 2
       CPOLAR = 'POLAR WEIGHT'
       CAZIMU = 'AZIMUTHALWGT'
       CALL SETARA( I, NMAT*(NANIS+1),FGUPSC)
       CALL LCMGET(IPMACR,'FGWITHUPSCAT',I(FGUPSC)  )
       GMXUPS = 99999
       DO 1001 IM =    1, (NL+1)
         DO 1002 JM =  1, NMAT
          GMXUPS = MIN(GMXUPS,I(FGUPSC+(IM-1)*NMAT + JM-1 ))
1002     CONTINUE
1001   CONTINUE      
      IF(IPRT.GE.1) THEN
        WRITE(IOUT,6200)
        IF(ITRANC.GT.0) THEN
          WRITE(IOUT,6100)
        ENDIF
      ENDIF
      IF(IPRT.GE.10) THEN
        WRITE(IOUT,6000)
        WRITE(IOUT,6001) (IREGIO,VOL(IREGIO),MATCOD(IREGIO),
     >                    IREGIO=1,NREGIO)
      ENDIF
      DO 150 IGR=1,NGRP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMACR,'TOTAL',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(IPMACR,'TOTAL',XSSIGT(1,IGR))
          XSSIGT(0,IGR)=ZERO
          DO 111 IR= 0, NMAT
 111      XSOLD(IR,IGR)=XSSIGT(IR,IGR)
        ELSE
          CALL LCMLIB(IPMACR)
          CALL XABORT('MOCDRV: READ ERROR ON LCM RECORD= TOTAL')
        ENDIF
        CALL LCMLEN(IPMACR,'SIGW 0',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(IPMACR,'SIGW 0',XSSIGW(1,IGR))
          XSSIGW(0,IGR)=ZERO
        ELSE
          DO 110 IR=0,NMAT
 110      XSSIGW(IR,IGR)=ZERO
        ENDIF
        IF(ITRANC.NE.0) THEN
          CALL LCMLEN(IPMACR,'TRANC',ILNLCM,ITYLCM)
          IF(ILNLCM.GT.0) THEN
            CALL LCMGET(IPMACR,'TRANC',VSCAT)
            DO 120 IMAT=1,NMAT
              XSSIGT(IMAT,IGR)=XSSIGT(IMAT,IGR)-VSCAT(IMAT)
              XSSIGW(IMAT,IGR)=XSSIGW(IMAT,IGR)-VSCAT(IMAT)
 120        CONTINUE
          ENDIF
        ENDIF
        XTINV(0,IGR)= ONE
        DO 121 IMAT=1,NMAT
           IF( XSSIGT(IMAT,IGR).NE.ZERO )THEN
              XTINV(IMAT,IGR)= ONE/XSSIGT(IMAT,IGR)
           ELSE
              XTINV(IMAT,IGR)= ONE
           ENDIF
 121    CONTINUE
        IF(IPRT.GE.10) THEN
          WRITE(IOUT,6002)  IGR
          WRITE(IOUT,6003) (IMAT,XSSIGT(IMAT,IGR),
     >    XSSIGW(IMAT,IGR),IMAT=1,NMAT)
        ENDIF
        IF( ITYPEC .EQ. 6 ) THEN      
         XSNUF(0,1,IGR)= ZERO
         IF (LDRFLX) CALL LCMGET(IPMACR,'FIXE',XSNUF(1,1,IGR))
        ELSE
           CALL LCMGET(IPMACR,'CHI',XSTK)
           DO 132 IFIS= 1, NIFIS
              XSCHI(0,IFIS,IGR)= ZERO
              DO 131 IMAT= 1, NMAT
                 XSCHI(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  131         CONTINUE
  132      CONTINUE
           CALL LCMGET(IPMACR,'NUSIGF',XSTK)
           DO 134 IFIS= 1, NIFIS
              XSNUF(0,IFIS,IGR)= ZERO
              DO 133 IMAT= 1, NMAT
                 XSNUF(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  133         CONTINUE
  134      CONTINUE
           IF( IPRT.GE.10 )THEN
              WRITE(IOUT,6004) (IMAT,XSNUF(IMAT,1,IGR),
     >                                 XSCHI(IMAT,1,IGR),IMAT=1,NMAT)
           ENDIF
        ENDIF
        DO 50 IANI=0,NANIS
          CALL LCMLEN(IPMACR,'NJJD'//CAN(IANI),ILNLCM,ITYLCM)
          DO 39 IMAT= 0, NMAT
            DO 38 JGR= 1, NGRP
              XSCAT(JGR,IMAT,IANI,IGR)= ZERO
   38       CONTINUE
   39     CONTINUE
          IF(ILNLCM.GT.0) THEN
            CALL LCMGET(IPMACR,'NJJD'//CAN(IANI),NJJ)
            CALL LCMGET(IPMACR,'IJJD'//CAN(IANI),IJJ)
            CALL LCMGET(IPMACR,'IPOD'//CAN(IANI),IPOS)
            CALL LCMGET(IPMACR,'SCAD'//CAN(IANI),VSCAT)
            DO 45 IMAT= 1, NMAT
              DO 40 JGR= IJJ(IMAT)-NJJ(IMAT)+1, IJJ(IMAT)
                IXSS= IPOS(IMAT)+IJJ(IMAT)-JGR
                XSCAT(JGR,IMAT,IANI,IGR)= VSCAT(IXSS)
   40         CONTINUE
   45       CONTINUE
          ENDIF
          IF( IPRT.GT.10 )THEN
             DO 49 IMAT= 1, NMAT
                WRITE(IOUT,7000) IMAT,IANI,IGR,
     >                        (XSCAT(JGR,IMAT,IANI,IGR),JGR=1,NGRP)
   49        CONTINUE
          ENDIF
   50   CONTINUE
        CALL LCMSIX(IPMACR,' ',ILCMDN)
 150  CONTINUE
      NTRK2  = 0
      NANGL2 = 0
      MXSEG2 = 0
      IF (IFTRK2 .GT. 0) THEN
       REWIND IFTRK2
       READ(IFTRK2) CINI,NCOMNT,NTRK2
       IF( CINI.NE.'$TRK'.OR.NCOMNT.LT.0.OR.NTRK2.EQ.0) THEN
         WRITE(IOUT,9000) 'MOCDRV'
         CALL XABORT('MOCDRV: PROBLEM WITH THE TRACKING FILE')
       ENDIF
       DO 31 IC= 1,NCOMNT
          READ(IFTRK2) COMENT
   31  CONTINUE
       READ(IFTRK2) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL2,MXSEG2
       IF( NREG.NE.NREGIO )THEN
          CALL XABORT('MOCDRV:(IFTRK2)INVALID NUMBER OF REGIONS')
       ENDIF
       IF( NALB.NE.NALBG )THEN
          CALL XABORT('MOCDRV:(IFTRK2)INVALID NUMBER OF ALBEDOS')
       ENDIF
       IF( NDIM.NE.NDIMO )THEN
          CALL XABORT('MOCDRV:(IFTRK2)NON 2D TRACKING FILE')
       ENDIF
       IF( ISPEC.NE.1 )THEN
          CALL XABORT('MOCDRV:(IFTRK2)NON SPECULAR TRACKING FILE')
       ENDIF
       IF (NGSS.LT.NGSS2) THEN
          CALL XABORT('MOCDRV:  NGSS < NGSS2')
       ENDIF
      ENDIF
      REWIND IFTRAK
      READ(IFTRAK) CINI,NCOMNT,NTRK
      IF( CINI.NE.'$TRK'.OR.NCOMNT.LT.0.OR.NTRK.EQ.0) THEN
        WRITE(IOUT,9000) 'MOCDRV'
        CALL XABORT('MOCDRV: PROBLEM WITH THE TRACKING FILE')
      ENDIF
      DO 30 IC= 1,NCOMNT
         READ(IFTRAK) COMENT
   30 CONTINUE
      READ(IFTRAK) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL,MXSEG
      IF( NREG.NE.NREGIO )THEN
         CALL XABORT('MOCDRV: INVALID NUMBER OF REGIONS')
      ENDIF
      IF( NALB.NE.NALBG )THEN
         CALL XABORT('MOCDRV: INVALID NUMBER OF ALBEDOS')
      ENDIF
      IF( NDIM.NE.NDIMO )THEN
         CALL XABORT('MOCDRV: NON 2D TRACKING FILE')
      ENDIF
      IF( ISPEC.NE.1 )THEN
         CALL XABORT('MOCDRV: NON SPECULAR TRACKING FILE')
      ENDIF
      IF(MXSEG2.GT.0) MXSEG = MAX(MXSEG,MXSEG2)
      IF (IFTRK2.GT.0 .AND. NANGL2.GT.NANGL) THEN 
        CALL XABORT('MOCDRV: INVALID  NANGL2 > NANGL ')
      ENDIF
      NUNKNO= NREG+NSOUT+1
      NPHI  = NANGL
      CALL SETARA( A,2*(2*NANGL+1)*NGSS*NREG*NGRP, ANGPHI )
      CALL XDRSET(A(ANGPHI),2*(2*NANGL+1)*NGSS*NREG* NGRP, 0.0)
      CALL SETARA( A,2* (2*NANGL+1)*NGSS*NREG*NGRP, ANGADJ )
      CALL XDRSET(A(ANGADJ),2*(2*NANGL+1)*NGSS*NREG*NGRP, 0.0)
      IF (ANGPHI .EQ. 0) 
     > CALL XABORT(' MOCDRV: INVALID MEM ALLOC FOR ANGULAR FLUX ')
      IF (ANGADJ .EQ. 0)
     > CALL XABORT(' MOCDRV: INVALID MEM ALLOC FOR ANGULAR ADJOINTS')
      CALL SETARA(A,NANGL,AZIWGT )
      CALL XDRSET(A(AZIWGT),NANGL,0.0)
      CALL SETARA(A,NUNKNO,VOLSUR )
      CALL SETARA(I,NUNKNO,MATALB )
      CALL SETARA(A,NPHI,PHIANG )
      CALL SETARA(A,NPHI,PHIDUM )
      READ(IFTRAK) (A(VOLSUR+JUN),JUN=0,NUNKNO-1)
      READ(IFTRAK) (I(MATALB+JUN),JUN=0,NUNKNO-1)
      READ(IFTRAK) ( NSCRP,JUN=1,NALBG)
      READ(IFTRAK) ( ASCRP,JUN=1,NALBG)
      READ(IFTRAK) ( A(PHIANG+IL-1),A(PHIDUM+IL-1),IL=1,NANGL)
      READ(IFTRAK) ( A(AZIWGT+JUN-1),JUN=1,NANGL)
      CALL RLSARA( A(PHIDUM) )
      NR = NREG
      NS = NSOUT
      NRNL=   (NL+1)**2
      NFUNL= ((NL+1)*(NL+2))/2
      DO 19 IL = 1 , NANGL
         A(AZIWGT+IL-1) = 1.0E0 / A(AZIWGT+IL-1) 
19    CONTINUE
      CALL LCMPUT(IPFLUX, CAZIMU, NANGL, 2, A(AZIWGT) )
      IF( NFUNL*NREG.GT.NUN )THEN
         WRITE(IOUT,7001) NL,NFUNL,NREG,NUN
         CALL XABORT(' MOCDRV: INCOMPATIBLE DIMENSION FOR FLUX')
      ENDIF
      MATMIN= 0
      MATMAX= 0
      NOVOID= .TRUE.
      LKGALB= .FALSE.
      DO 20 IS=-NS,NR
         IF( IS.LT.0 )THEN
            IF( I(MATALB+NSOUT+IS).LT.0 )THEN
               MATMIN= MAX(MATMIN,I(MATALB+NSOUT+IS))
             LKGALB = LKGALB.OR.( ALBEDO(-I(MATALB+NSOUT+IS)).LT.ONE )
            ELSE
               CALL XABORT('MOCDRV: IS THIS A PROBLEM?')
            ENDIF
         ELSEIF( IS.GT.0 )THEN
            MATMAX= MAX(MATMAX,I(MATALB+NSOUT+IS))
            IF( I(MATALB+NSOUT+IS).NE.MATCOD(IS) )THEN
               WRITE(IOUT,7002) I(MATALB+NSOUT+IS),MATCOD(IS)
               I(MATALB+NSOUT+IS)= MATCOD(IS)
            ENDIF
            IF( ABS(VOL(IS)-A(VOLSUR+NSOUT+IS))*VOLINV(IS).GT.EPSR )THEN
               CALL XABORT('MOCDRV: INCOMPATIBLE VOLS')
            ENDIF
            DO 25 IGR=1,NGRP
               NOVOID= NOVOID.AND.(XSSIGT(MATCOD(IS),IGR).NE.ZERO)
   25       CONTINUE
         ELSE
            IF(  I(MATALB+NSOUT+IS).NE.0 )
     >         CALL XABORT('MOCDRV: MATALB(0).NE.0 ?')
         ENDIF
   20 CONTINUE
      IF( IPRT.GT.0 )THEN
         IF( NL.EQ.0 )THEN
            IF( NOVOID )THEN
               WRITE(IOUT,7010) 'MOCC  '
            ELSE
               WRITE(IOUT,7011) 'MOCC  '
            ENDIF
         ELSE
            IF( NOVOID )THEN
               WRITE(IOUT,7012) 'MOCC  '
            ELSE
               WRITE(IOUT,7013) 'MOCC  '
            ENDIF
         ENDIF
         IF (LEAKSW) THEN
            WRITE(IOUT,7023) 'MOCC  '
         ELSE
            WRITE(IOUT,7024) 'MOCC  '
         ENDIF
         IF( ILK )THEN
            WRITE(IOUT,7020) 'MOCC  '
         ELSE
            WRITE(IOUT,7022) 'MOCC  '
         ENDIF
      ENDIF
      IF( MATMIN.LT.-NALB )THEN
         CALL XABORT('MOCDRV: NOT ENOUGH ALBEDOS   ARE DEFINED')
      ENDIF
      IF( MATMAX.GT.NMAT )THEN
         CALL XABORT('MOCDRV: NOT ENOUGH MATERIALS ARE DEFINED')
      ENDIF
      IF( LCACA.EQ.0 )THEN
         CALL ALGPT ( NGSS, ZERO, ONE, XGSS, WGSS)
      ELSE
         CALL ALCACT( LCACA, NGSS, XGSS, WGSS)
      ENDIF
      IF( IPRT.GT.0 )THEN
        WRITE(IOUT,7030) 'MOCDRV',LCACA,NGSS
      ENDIF      
      CALL LCMPUT(IPFLUX,'POLAR MU    ', NGSS, 2, XGSS)
      CALL LCMPUT(IPFLUX, CPOLAR, NGSS, 2, WGSS)
      DO 109 JGSS= 1, NGSS
         YGSS(JGSS)= ONE/SQRT(ONE - XGSS(JGSS)*XGSS(JGSS))
         WGSS(JGSS)= -WGSS(JGSS)*SQRT(ONE - XGSS(JGSS)*XGSS(JGSS))
  109 CONTINUE
      WORD=2
      CALL SETARA( A,WORD* MXSEG*NGSS , FLM )
      CALL SETARA( A,WORD* MXSEG*NGSS , FLP )
      CALL SETARA( A,WORD* MXSEG*NGSS , FLMADJ )
      CALL SETARA( A,WORD* MXSEG*NGSS , FLPADJ )
      CALL SETARA( A,WORD* MXSEG*NGSS , ICYM )
      CALL SETARA( A,WORD* MXSEG*NGSS , ICYP )
      CALL SETARA( A,WORD* MXSEG*NGSS , ICYMAJ)
      CALL SETARA( A,WORD* MXSEG*NGSS , ICYPAJ)
      IF (NR.NE.NREGIO) CALL XABORT('MOCDRV: NGRP.NE.NREGIO ')
      CALL SETARA( A       , WORD* 8*NGRP*NR*NFUNL , ADJFLX )
      CALL XDRSET(A(ADJFLX), WORD* 8*NGRP*NR*NFUNL ,0.0)
      CALL SETARA( A       , (NMAT+1)*NGRP   , ADJFXS )
      CALL XDRSET(A(ADJFXS), (NMAT+1)*NGRP , 0.0)
      CALL SETARA( A       ,  NGRP*NR  , ADJSOU )
      CALL XDRSET(A(ADJSOU),  NGRP*NR  ,0.0)
      CALL SETARA( I, MXSEG      , NUMERO )
      CALL SETARA( A, MXSEG      , LENGHT )
      CALL SETARA( A, MXSEG*NGSS , EXPT )
      CALL SETARA( A, MXSEG*NGSS , EXP2 )
      CALL SETARA( A, MXSEG*NGSS , EXPTAJ)
      CALL SETARA( A, MXSEG*NGSS , EXP2AJ)
      CALL SETARA( A,       NGSS , ICYCLE )
      CALL SETARA( A,       NGSS , IALPHA )
      CALL SETARA( A,       NGSS , DFLUX )
      CALL SETARA( A,       NGSS , DWEIG )
      CALL SETARA( A,       NGSS , FMEAN )
      CALL SETARA( I,         8*NRNL , ISGNR )
      CALL SETARA( A,      NGSS*NRNL , RHARM )
      CALL SETARA( I,           NRNL , KEYANI )
      CALL MOCIK3(NDIM,NL,I(ISGNR),I(KEYANI))
      IF(NDIM.EQ.2) THEN
          NMOD = 4
      ENDIF
      CALL SETARA( A, NFUNL*NMOD*NGSS , TRHAR )
      CALL SETARA( A, 2*NGRP*(NGRP+1) , REBAL )
      CALL SETARA( A, 2*NGRP, FLUC)
      CALL SETARA( A, 2*NGRP, FLUN)
      CALL SETARA( A, 2*NGRP, PNL)
      CALL SETARA( A, 2*NGRP, XCSOU)
      CALL SETARA( A, 2*NGRP, XCSOUA)
      CALL SETARA( A, 2*NGRP, XCTOT)
      CALL SETARA( A, 2*NGRP, XCTRC)
      CALL SETARA( A, 2*NGRP, XCDIA)
      CALL SETARA( A, 2*NGRP, XCNUF)
      CALL SETARA( A, 2*NGRP, XCCHI)
      CALL SETARA( A, 2*NGRP, XCABS)
      CALL SETARA( A, 2*NGRP*NGRP*(NANIS+1) , XCCAT)
      CALL SETARA( A, 2*NGRP, DIFF)
      CALL SETARA( A, 2*NGRP, DB2)
      CALL SETARA( A, 2*NGRP, IBETA)
      CALL SETARA( A, NGSS*NGRP*(NMAT+NALB+1), SIGANG )
      CALL SETARA( A, (NMAT+1)*NGRP, XMTOT)
      CALL SETARA( A, (NMAT+1)*NGRP, XMTRC)
      CALL SETARA( A, (NMAT+1)*NGRP, XMDIA)
      CALL SETARA( A, (NMAT+1)*NGRP*NGRP*(NANIS+1), XMCAT  )
      CALL XDRSET( A(XMTOT), (NMAT+1)*NGRP    ,0.0)
      CALL XDRSET( A(XMTRC), (NMAT+1)*NGRP    ,0.0)
      CALL XDRSET( A(XMDIA), (NMAT+1)*NGRP    ,0.0)
      CALL XDRSET( A(XMCAT), (NMAT+1)*NGRP*NGRP*(NANIS+1) ,0.0)
      IF (LDRFLX.OR.(LDAJCB.AND. (ISADJ.EQ.1.OR.ISADJ.EQ.2))) THEN
          CALL MOC2DR(IPFLUX,IPTRK,IFTRAK,IPMACR,NGRP,NREG,NSOUT,
     >              NANIS,NMAT,NGSS,MXSEG,NTRK,NPHI,NFUNL,NMOD,
     >              I(KEYANI),I(MATALB),VOL,VOLINV,WGSS,XGSS,YGSS,
     >              A(PHIANG),A(LENGHT),I(NUMERO),A(EXPT),A(EXP2),
     >              A(EXPTAJ),A(EXP2AJ),A(ICYCLE),A(FLM),A(FLP),
     >              A(FLMADJ),A(FLPADJ),A(ICYM),A(ICYP),A(ICYMAJ),
     >              A(ICYPAJ),A(IALPHA),A(DFLUX),A(DWEIG),A(FMEAN),
     >              A(SIGANG),NALB,ALBEDO,XSOLD,XSSIGT,XTINV,XSSIGW,
     >              XSCAT,NIFIS,XSNUF,XSCHI,ITRB,FLUX,FLUXO,DIFFB1,
     >              I(ISGNR),A(RHARM),LEAKSW,ILK,NINTMX,AIEPS,NEXTMX,
     >              AEEPS,IPRT,ITYPEC,ILEAK,NL,NBN,RKEFF,RB2,A(TRHAR),
     >              A(REBAL),IFRITR,IACITR,A(FLUC),A(FLUN),A(PNL),
     >              A(XCSOU),A(XCTOT),A(XCTRC),A(XCDIA),A(XCCAT),
     >              A(XCNUF),A(XCCHI),A(XCABS),A(DIFF),A(DB2),A(IBETA),
     >              LEXIN,NOVOID,REFKEF,GMXUPS,LMODE,A(ADJFLX),ADJFLO,
     >              A(ADJSOU),A(ADJFXS),A(XCSOUA),ISADJ,RAJKEF,
     >              CDIRFL,CADJFL,LDRFLX,LDAJCB,
     >              IPTRK2,NTRK2,NANGL2,IFTRK2,LCACA,NGSS2,
     >              A(XMTOT),A(XMTRC),A(XMCAT),A(XMDIA),
     >              A(ANGPHI),A(ANGADJ)    ) 
       ENDIF
       IF (LDRFLX ) THEN
          CALL LCMPUT(IPFLUX,'K-EFFECTIVE',1,2,RKEFF)
       ENDIF
      IF((.NOT.LDAJCB.AND.(ISADJ.EQ.1.OR.ISADJ.EQ.2)).OR.ISADJ.GT.2)THEN
          CALL MOC2AJ(IPFLUX,IPTRK,IFTRAK, NGRP,NREG,NSOUT,NANIS,
     >              NMAT,NGSS,MXSEG,NTRK,NPHI,NFUNL,NMOD,I(KEYANI),
     >              I(MATALB),VOL,VOLINV,WGSS,XGSS,YGSS,A(PHIANG),
     >              A(LENGHT),I(NUMERO),A(EXPT),A(EXP2),A(ICYCLE),
     >              A(FLM),A(FLP),A(ICYM),A(ICYP),A(IALPHA),A(DFLUX),
     >              A(DWEIG),A(FMEAN),A(SIGANG),NALB,ALBEDO,
     >              XSOLD,XSSIGT,XTINV,XSSIGW,XSCAT,NIFIS,XSNUF,XSCHI,
     >              ITRB,FLUX,A(ADJFLX),ADJFLO,DIFFB1,I(ISGNR),A(RHARM),
     >              LEAKSW,ILK,NINTMX,AIEPS,NEXTMX,AEEPS,IPRT,ITYPEC,
     >              ILEAK,NL,NBN,RKEFF,RB2,A(TRHAR),A(REBAL),IFRITR,
     >              IACITR,A(FLUC),A(FLUN),A(PNL),A(XCSOU),A(XCTOT),
     >              A(XCTRC),A(XCDIA),A(XCCAT),A(XCNUF),A(XCCHI),
     >              A(XCABS),A(DIFF),A(DB2),A(IBETA),LEXIN,NOVOID,
     >              REFKEF,A(ADJSOU),ISADJ,IPMACR,A(ADJFXS),GMXUPS,
     >              LMODE,CADJFL,FLUXO,IPTRK2,NTRK2,NANGL2,IFTRK2,LCACA,
     >              NGSS2,A(XMTOT),A(XMTRC),A(XMCAT),A(XMDIA),
     >              A(ANGPHI),A(ANGADJ)    ) 
      ENDIF
        CALL MOCCOR(IPFLUX,IFTRAK,NGRP,NREG,NSOUT, NANIS,NMAT,NGSS,
     >              MXSEG , NTRK, NFUNL, I(KEYANI),NIFIS,NL,NBN,
     >              I(MATALB),  VOL,VOLINV,WGSS,XGSS,YGSS,
     >              A(LENGHT),I(NUMERO),A(EXPT),A(EXP2),
     >              A(FLM),A(FLP),A(ICYP),A(ICYM),A(SIGANG),NALB,ALBEDO,
     >              XSOLD,XSSIGT,XTINV,XSSIGW,XSCAT ,XSNUF,XSCHI,
     >              FLUX,A(ADJFLX),LEAKSW, IPRT,ILEAK,IPMACR,LMODE,
     >              CDIRFL,CADJFL,FLUXO,NTRK2,IFTRK2,NGSS2  )
      CALL RLSARA( A(SIGANG ) )
      CALL RLSARA( A(IBETA  ) )
      CALL RLSARA( A(DB2    ) )
      CALL RLSARA( A(DIFF   ) )
      CALL RLSARA( A(XCCAT  ) )
      CALL RLSARA( A(XCABS  ) )
      CALL RLSARA( A(XCCHI  ) )
      CALL RLSARA( A(XCNUF  ) )
      CALL RLSARA( A(XCDIA  ) )
      CALL RLSARA( A(XCTRC  ) )
      CALL RLSARA( A(XCTOT  ) )
      CALL RLSARA( A(XCSOU  ) )
      CALL RLSARA( A(XCSOUA ) )
      CALL RLSARA( A(PNL    ) )
      CALL RLSARA( A(FLUN   ) )
      CALL RLSARA( A(FLUC   ) )
      CALL RLSARA( A(REBAL  ) )
      CALL RLSARA( A(TRHAR  ) )
      CALL RLSARA( I(KEYANI ) )
      CALL RLSARA( A(RHARM  ) )
      CALL RLSARA( I(ISGNR  ) )
      CALL RLSARA( A(FMEAN  ) )
      CALL RLSARA( A(DWEIG  ) )
      CALL RLSARA( A(DFLUX  ) )
      CALL RLSARA( A(IALPHA ) )
      CALL RLSARA( A(ICYP   ) )
      CALL RLSARA( A(ICYM   ) )
      CALL RLSARA( A(ICYPAJ ) )
      CALL RLSARA( A(ICYMAJ ) )
      CALL RLSARA( A(FLP    ) )
      CALL RLSARA( A(FLM    ) )
      CALL RLSARA( A(FLPADJ ) )
      CALL RLSARA( A(FLMADJ ) )
      CALL RLSARA( A(ICYCLE ) )
      CALL RLSARA( A(EXP2   ) )
      CALL RLSARA( A(EXPT   ) )
      CALL RLSARA( A(EXP2AJ ) )
      CALL RLSARA( A(EXPTAJ ) )
      CALL RLSARA( A(LENGHT ) )
      CALL RLSARA( I(NUMERO ) )
      CALL RLSARA( I(FGUPSC ) )
      CALL RLSARA( A(ADJFLX ) )
      CALL RLSARA( A(ADJSOU ) )
      CALL RLSARA( A(ADJFXS ) )
      CALL RLSARA( A( XMTOT  ) )
      CALL RLSARA( A( XMTRC  ) )
      CALL RLSARA( A( XMCAT  ) )
      CALL RLSARA( A( XMDIA  ) )
      CALL RLSARA( A( ANGPHI  ) )
      CALL RLSARA( A( ANGADJ  ) )
      CALL RLSARA( A( AZIWGT )  )
      CALL RLSARA( A(PHIANG ) )
      CALL RLSARA( I(MATALB ) )
      CALL RLSARA( A(VOLSUR ) )
      RETURN
C     FORMATS
 6000 FORMAT(//30X,' EDITION REGION/VOL/MIXTURE '//
     >3(5X,'REGION',5X,'VOL  ',5X,'MIXTURE')/)
 6001 FORMAT(1P,3(5X,I4,4X,E12.5,4X,I4,4X))
 6002 FORMAT(//30X,' G R O U P : ',I5//
     >30X,' TOTAL MACROSCOPIC CROSS SECTIONS PER MIXTURE '/)
 6003 FORMAT(3(1X,'MIXTURE',4X,'TOTAL',11X,'SIGW',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6004 FORMAT(3(1X,'MIXTURE',4X,'NUSIGF',11X,'CHI ',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6100 FORMAT(//' USE TRANSPORT CORRECTED CROSS-SECTIONS')
 6200 FORMAT(//' COMPUTATION WITH THE CHARACTERISTIC METHOD       ')
 7000 FORMAT(1X,'MAT(',I10,') SCAT(',I10,') TO GROUP:',I10,1P/
     >       5E20.7)
 7001 FORMAT(1X,'MOCDRV: NL   =',I10/
     >       1X,'MOCDRV: NFUNL=',I10/
     >       1X,'MOCDRV: NREG =',I10/
     >       1X,'MOCDRV: NUN  =',I10)
 7002 FORMAT(1X,'MATCOD HAS BEEN CHANGED ',I10,'->',I10)
 7010 FORMAT(1X,A6,': ISOTROPIC OPTION *WITHOUT VOID*')
 7011 FORMAT(1X,A6,': ISOTROPIC OPTION *WITH    VOID*')
 7012 FORMAT(1X,A6,': ANISOTROPIC OPTION *WITHOUT VOID*')
 7013 FORMAT(1X,A6,': ANISOTROPIC OPTION *WITH    VOID*')
 7020 FORMAT(1X,A6,': REBALANCING MODE * ON *')
 7022 FORMAT(1X,A6,': REBALANCING MODE * OFF *')
 7023 FORMAT(1X,A6,': SYSTEM WITH LEAKAGE PRESENTED')
 7024 FORMAT(1X,A6,': SYSTEM WITH NO LEAKAGE')
 7030 FORMAT(1X,A6,': Polar quadrature '/
     >       1X,'LCACT =',I8/
     >       1X,'NMU   =',I8)
 9000 FORMAT(1X,A6,': ECHO = FIRST RECORD DESTROYED      ')
      END