*DECK MOCSAC
      SUBROUTINE MOCSAC(IPFLUX,IPRINT,CANGD,NGRP,NPHI,NGSS,NREG,
     >                  ANGFA,AFDS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Save angular flux and adjoints components.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* 2005/10/14: Validated with TSTVOL and documented in
*             Report IGE-260.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPFLUX  pointer to the flux data structure
* IPRINT  print option
* CANGD   name of adjoint directory where information is 
*         to be saved
* NGRP    number of groups
* NPHI    number of azimuthal directions
* NGSS    number of polar directions
* NREG    number of regions
* ANGFA   reference angular flux or adjoint
*Parameters: output
* AFDS    reformatted angular flux or adjoint
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPFLUX,IPRINT
      CHARACTER*12     CANGD
      INTEGER          NGRP,NPHI,NGSS,NREG
      DOUBLE PRECISION ANGFA(NREG,-NPHI:NPHI,NGSS,NGRP),
     >                 AFDS(NREG*NGSS*2*NPHI)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='MOCSAC')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
*----
*  Local variables
*----
      INTEGER          IG,IPHI,IPOL,IR,IPOS
      CHARACTER*12     CSAV
*----
*  Initialize default values for IPRINT
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
        WRITE(IOUT,6010) CANGD
      ENDIF
      CALL LCMSIX(IPFLUX,CANGD,ILCMUP)
      DO IG= 1,NGRP             
        CALL XDDSET(AFDS,2*NPHI*NREG*NGSS,0.0D0)          
        IPOS=0
        DO IPHI=-NPHI,NPHI
          DO IPOL=1,NGSS
            IF(ABS(IPHI) .GT. 0) THEN
              DO IR=1,NREG             
                IPOS=IPOS+1
                AFDS(IPOS)=ANGFA(IR,IPHI,IPOL,IG)
              ENDDO
            ENDIF
          ENDDO
        ENDDO
        WRITE(CSAV,'(A5,I3,4X)') 'AFLUX',IG
        CALL LCMPUT(IPFLUX,CSAV,2*NREG*NGSS*2*NPHI,4,AFDS)
        IF(IPRINT .GE. 10) THEN
          WRITE(IOUT,6011) CSAV
          WRITE(IOUT,6012) (AFDS(IPOS),IPOS=1,NREG*NGSS*2*NPHI)
        ENDIF
      ENDDO
      CALL LCMSIX(IPFLUX,CANGD,ILCMDN)
*----
*  Processing finished, return
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT('Directory ',A12)
 6011 FORMAT('Group     ',A12)
 6012 FORMAT(1P,5E20.10)
      END 
