*DECK MOCSCL
      SUBROUTINE MOCSCL(NANIS ,NREG  ,NGRP ,NFUNL ,NIFIS ,NMAT ,NSOUT,
     >                  ADJFLO,XSCHI ,XSNUF,ADJSOU,MATALB,XSCAT,XSDIA,
     >                  SCALE ,IPRT  ,RKEFF)
*
*---------------------------  MOCSCL  ---------------------------------
*
* 1- PROGRAMME STATISTICS:
*
*     NAME     :  MOCSCL
*     USE      :  DETERMINE A FACTOR FOR GENERALIZED
*                 ADJOINT FLUX BIASING
*     MODIFIED BY :  MONCHAI A. (24/02/05)   
*
*  2- ROUTINE PARAMETERS:
*    INPUT
*      ADJFLX   : GENERALIZED ADJOINT FLUX               D(NFUNL,
*                                                          NREG,
*                                                          NGRP,8)
*      ADJFLO   : STANDARD ADJOINT FLUX                  R(NUN,
*                                                          NFUNL,
*                                                          NGRP)
*      ADJSOU   : EXTERNAL GENERALIZED ADJOINT SOURCE    R(NREG,NGRP)
*      MATALB   : MATERIAL CODE IN REGIONS.              I(-NSOUT:
*                                                          NREG)
*      NGRP     : NUMBER OF GROUPS TREATED               I
*      NANIS    : NUMBER OF ANISOTROPY IN                I
*                 SCATTERING CROSS SECTIONS
*      NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX      I
*      NIFIS    : NUMBER OF FISSILE ISOTOPES             I
*      NMAT     : NUMBER OF MIXTURES CONSIDERED          I
*      NREG     : NUMBER OF REGIONS CONSIDERED           I
*      NSOUT    : NUMBER OF SURFACES CONSIDERED          I
*      RKEFF    : REFERENCE K-EFFECTIVE                  R
*      SCALE    : SCALEUP BIASING FACTOR                 R
*      XSDIA    : WITHIN GROUP SCATTERING                R(NMAT+1)
*                 CROSS SECTION
*      XSCAT    : GROUP SCATTERING CROSS SECTION         R(NMAT+1)
*      XSCHI    : FISSION NEUTRON DISTRIBUTION           R(0:NMAT,
*                                                          NIFIS,
*                                                          NGRP)
*      XSNUF    : GROUP FISSION NEUTRON PRODUCTION       R(0:NMAT,
*                 CROSS SECTION                            NIFIS,
*                                                          NGRP)
*   WORK VECTOR
*
*--------------------------   MOCSCL    ------------------------------
*
      IMPLICIT NONE
      INTEGER      NREG,NGRP,NFUNL ,NIFIS,NMAT,NSOUT,NANIS,
     >             IG,JG,IR,IS,IOUT,IPRT,MATALB(-NSOUT:NREG)
      REAL         ADJFLO(NREG,NFUNL,NGRP),ADJSOU(NREG,NGRP),
     >             XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >             XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XSDIA(0:NMAT,NGRP),
     >             SCALE,RKEFF 
      DOUBLE PRECISION  MAXALF,ALFA,FISOUR,SCASOU,DZERO,SUMFIS,FACTOR,
     >                  DONE,FISO,SCAS,ADJS
      IOUT = 6
      DZERO=0.00000000000D0
      DONE =1.00000000000D0
      MAXALF = -1.0D+20
      FACTOR = 1.0D+01
      DO 100 IR= 1,NREG
       DO 200 IG= 1,NGRP
          SCASOU = DZERO
          FISOUR = DZERO
          DO 95 IS= 1,NIFIS
             SUMFIS = DZERO
             DO 93 JG= 1, NGRP
              SUMFIS=SUMFIS+XSCHI(MATALB(IR),IS,JG)*ADJFLO(IR,1,JG)
 93          CONTINUE
             FISOUR=FISOUR+XSNUF(MATALB(IR),IS,IG)*SUMFIS
 95       CONTINUE
          DO 35 JG= 1,NGRP
           IF( IG.EQ.JG )THEN
            SCASOU= SCASOU+XSDIA(MATALB(IR),JG)*ADJFLO(IR,1,JG)
           ELSE
            SCASOU= SCASOU+XSCAT(IG,MATALB(IR),0,JG)*ADJFLO(IR,1,JG)
           ENDIF
   35     CONTINUE
          IF (FISOUR .LT. DZERO  .OR. SCASOU .LT. DZERO ) THEN
             WRITE(IOUT,'(1X,A25)')      'NEGATIVE FISOUR OR SCASOU'
             WRITE(IOUT,'(1X,A7,E14.6)') 'FISOUR=',FISOUR
             WRITE(IOUT,'(1X,A7,E14.6)') 'SCASOU=',SCASOU
          ENDIF
          IF (FISOUR .EQ. DZERO  .AND. SCASOU .EQ. DZERO ) THEN
             WRITE(IOUT,'(1X,A22)')       'ZERO FISOUR AND SCASOU'
             WRITE(IOUT,'(1X,A7,E14.6)') 'FISOUR=',FISOUR
             WRITE(IOUT,'(1X,A7,E14.6)') 'SCASOU=',SCASOU
          ENDIF
          FISOUR = FISOUR / RKEFF
          IF ((FISOUR + SCASOU).GT. 1.0D-06) THEN
             ALFA=((FACTOR*(FISOUR+SCASOU)-ADJSOU(IR,IG)) 
     >       /(FISOUR+SCASOU))
          ELSE
             ALFA= DONE 
          ENDIF
          IF (ALFA.GT.MAXALF) THEN
            MAXALF=ALFA
            FISO=FISOUR
            SCAS=SCASOU
            ADJS=ADJSOU(IR,IG)  
          ENDIF
200    CONTINUE
100   CONTINUE
      SCALE=MAXALF
      RETURN
      END