*DECK MOCSIG
      SUBROUTINE MOCSIG(NGSS,NGSS2,NALB,NMAT,NGRP,NANIS,NSOUT,MXGAUS,
     >                  ALBEDO,XSTOT,XSTRC,XSCAT,XSDIA,XTINV,XGSS,YGSS,
     >                  WGSS,XMTOT,XMTRC,XMCAT,XMDIA,LCACA,SIGANG,
     >                  BETASC,IFTRK2,LOWTRK)
* 
* ---------------------------  MOCSIG  ---------------------------------
* 
*  1- PROGRAMME STATISTICS:
* 
*      NAME     :  MOCSIG
*      USE      :  MODIFY CROSS SECTIONS ACCORDING TO LBETAS 
*                  AND LOWORDER TRACKING LINES
*      MODIFIED BY :  MONCHAI A. (24/02/05)   
* 
*  2- ROUTINE PARAMETERS:
*    INPUT
*      BETASC   : MODIFIED SCATTERING FACTOR             R
*      IFTRK2   : LOW ORDER TRACKING FILE                I
*      LOWTRK   : FLAG TO USE LOW ORDER TRACKING LINES   L
*      LCACA    : INDEX TO POLAR QUADRATURE TYPE         I
*      MXGAUS   : FIXED MAXIMUM NUMBER OF SEGMENTS(=64)  I
*      NGRP     : NUMBER OF GROUPS TREATED               I
*      NANIS    : NUMBER OF ANISOTROPY IN                I
*                 SCATTERING CROSS SECTIONS
*      NALB     : NUMBER OF ALBEDO                       I
*      NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX      I
*      NGSS     : NUMBER OF GAUSS INTEGRATION POINTS     I  
*      NGSS2    : NUMBER OF LOW ORDER GAUSS INTEGRATION  I  
*                 POINTS
*      NMAT     : NUMBER OF MIXTURES CONSIDERED          I
*      NSOUT    : NUMBER OF SURFACES CONSIDERED          I
*      WGSS     : POLAR WEIGHT                           R(MXGAUS)
*      YGSS     : COSINE OF POLAR ANGLE                  R(MXGAUS)
*      XMDIA    : REFERENCE WITHIN GROUP SCATTERING      R(NMAT+1)
*                 CROSS SECTION
*      XMCAT    : REFERENCE SCATTERING CROSS SECTION     R(NMAT+1)
*      XMTRC    : REFERENCE TRANSPORT-CORRECTED TOTAL    R(NMAT+1)
*                 CROSS SECTION    
*   WORK VECTOR
*      SIGANG   : SIGMATRCxSEGMENTLENGTH/COSINE(POLAR)   D(NGSS,-NALB:
*                                                          NMAT,NGRP)
*      XSDIA    : MODIFIED WITHIN GROUP SCATTERING       R(NMAT+1)
*                 CROSS SECTION
*      XSCAT    : MODIFIED SCATTERING CROSS SECTION      R(NMAT+1)
*      XSTRC    : MODIFIED RANSPORT-CORRECTED TOTAL      R(NMAT+1)
*                 CROSS SECTION   
*
* --------------------------   MOCSIG    ------------------------------
*
      IMPLICIT NONE
      INTEGER      NREG,NGRP,NMAT,NSOUT,NANIS,NGSS,NGSS2,NALB,MXGAUS,
     >             IG,JG,IR,IMAT,IANIS,IGSS,IOUT,LCACA,JGSS
      REAL    SIGANG(NGSS,-NALB:NMAT,NGRP),
     >        XSTOT(0:NMAT,NGRP),XSTRC(0:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >        XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >        XMTOT(0:NMAT,NGRP),XMTRC(0:NMAT,NGRP),
     >        XMCAT(NGRP,0:NMAT,0:NANIS,NGRP),XMDIA(0:NMAT,NGRP),
     >        YGSS(MXGAUS),WGSS(MXGAUS),XGSS(MXGAUS) 
      REAL    BETASC,ONE,ZERO,ALBEDO(NALB)
      INTEGER  IFTRK2,NGSSCL
      LOGICAL  LOWTRK
      IOUT = 6
      ONE  = 1.000000000E+00
      ZERO = 0.000000000E+00
      IF (LOWTRK .AND. IFTRK2.GT.0 ) THEN
        NGSSCL   = NGSS2 
      ELSE
        NGSSCL   = NGSS
      ENDIF
      IF( LCACA.EQ.0 )THEN
         CALL ALGPT ( NGSSCL, ZERO, ONE, XGSS, WGSS)
      ELSE
         CALL ALCACT( LCACA, NGSSCL, XGSS, WGSS)
      ENDIF
      DO 109 JGSS= 1, NGSSCL
         YGSS(JGSS)= ONE/SQRT(ONE - XGSS(JGSS)*XGSS(JGSS))
         WGSS(JGSS)= -WGSS(JGSS)*SQRT(ONE - XGSS(JGSS)*XGSS(JGSS))
  109 CONTINUE
      DO 1  IG = 1,NGRP
       DO 2 IMAT = 1,NMAT
          XSTRC(IMAT,IG)=XMTRC(IMAT,IG)-BETASC * XMDIA(IMAT,IG)
          XSDIA(IMAT,IG)=(1.0-BETASC) * XMDIA(IMAT,IG)
2      CONTINUE
1     CONTINUE
      DO 30 JG=1,NGRP
      DO 20 IG=1,NGSSCL
       DO 13 IR=-NALB,-1
            SIGANG(IG,IR,JG)=ONE-ALBEDO(-IR)
   13  CONTINUE
       SIGANG(IG,0,JG) = ZERO
       DO 10 IR=1,NMAT
            SIGANG(IG,IR,JG)=XSTRC(IR,JG)*YGSS(IG)
   10  CONTINUE
   20 CONTINUE
   30 CONTINUE 
      DO 120  IG = 1,NGRP
        XTINV(0,IG)= ONE
        DO 121 IMAT=1,NMAT
           IF( XSTRC(IMAT,IG).NE.ZERO )THEN
              XTINV(IMAT,IG)=ONE/XSTRC(IMAT,IG)
           ELSE
              XTINV(IMAT,IG)=ONE
           ENDIF
 121    CONTINUE
 120    CONTINUE
      RETURN
      END