*DECK MOCSWP
      SUBROUTINE MOCSWP(NREG  ,NGRP  ,NL,NMAT,NANIS ,NSOUT ,NIFIS,NGSS,
     >                  NALB  ,MXGAUS,IFTRK2 ,IFTRAK,NTRK2 ,NTRK,MXSEG,
     >                  MATALB,NRSEG ,SIGANG ,XSDIA ,XSCAT ,XTINV,
     >                  SEGLEN,ALBEDO,EXPT   ,EXP2  ,WGSS ,YGSS,NGSS2,
     >                  FLM   ,FLP   ,CYM    ,CYP   ,LXSCAT,LOWTRK,
     >                  VOLINV,LEXTSO,IGDEB  ,JGDEB ,MAXITR,LXSTYP,
     >                  X,Y,Z,S)
*
*---------------------------  MOCSWP  ---------------------------------
*
* 1- PROGRAMME STATISTICS:
*
*     NAME     :  MOCSWP
*     USE      :  MOCC SPATIAL SWEEPING
*     BY       :  MONCHAI A. (24/02/05)   
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     IGDEB    : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     LDAJCB   : FLAG TO TURN ON DIRECT AND ADJOINT FLUX L
*                CALCULATIONS
*     LMODE    : CONTROL FLAG FOR CALCULATION MODE       L(20)
*     MATALB   : MATERIAL CODE IN REGIONS.               I(-NSOUT:NREG)
*     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
*     MXSEG    : MAXIMUM NUMBER OF SEGMENTS ON EACH      I
*                TRACK (VARIABLE ACC. TO TRACKING LINE)
*     NALB     : NUMBER OF SURFACE ALBEDO                I 
*     NGRP     : NUMBER OF GROUPS TREATED                I
*     NGSS     : MAXIMUM NUMBER OF POLAR QUADRATURE      I
*     NE       : NUMBER OF POLAR QUADRATURE              I
*                (LOW OR HIGH ORDER TRACKING LINE)
*     NREG     : NUMBER OF REGIONS CONSIDERED            I
*     NRSEG    : KEY INDEX FOR REGIONAL NUMBER SEGMENT   I(MXSEG)
*     NSOUT    : NUMBER OF SURFACES CONSIDERED           I
*     SEGLEN   : LENGTH OF EACH SEGMENT                  R(MXSEG)  
*     TITRE    : EXECUTION TITLE                         C*72
*     VOL      : VOLUME OF REGION                        R(NREG)
*     VOLINV   : INVERSE OF VOLUMES                      R(NREG)
*     X        : FLUXES                                  D(NREG,NGRP)
*     Y        : FIXED EXTERNAL INPUT                    D(NREG,NGRP)
*
*   WORK VECTOR
*     FLM      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*     FLP      : DUMMY VARIABLE                          D(NGSS,MXSEG) 
*     CYM      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*     CYP      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*      S       : DUMMY VARIABLE                          D(NREG,NGRP)
*   OUTPUT
*      Z       : NEW FLUXES                              D(NREG,NGRP)
*--------------------------   MOCSWP    ------------------------------
*
      IMPLICIT NONE
      INTEGER      NREG,NGRP,NL,NMAT,NANIS,NSOUT,NIFIS,NGSS,MAXITR,
     >             NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,NTRK,MXSEG,
     >             MATALB(-NSOUT:NREG),NRSEG(MXSEG),IGDEB,JGDEB
      INTEGER      IG,JG,IR,IS,IOUT,IPRT,IFFIL,NTRACK,IGCAT,JGCAT,
     >             NGSSCL,NGSS2 
      REAL         SIGANG(NGSS,-NALB:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >             XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >             SEGLEN(MXSEG),ALBEDO(NALB),
     >             EXPT(NGSS,MXSEG),EXP2(NGSS,MXSEG),
     >             WGSS(MXGAUS),YGSS(MXGAUS),VOLINV(NREG)
      DOUBLE PRECISION  FLM(NGSS,MXSEG),FLP(NGSS,MXSEG),
     >                  CYM(NGSS,MXSEG),CYP(NGSS,MXSEG)
      DOUBLE PRECISION X(NREG,NGRP),Y(NREG,NGRP),Z(NREG,NGRP),
     >                 S(NREG,NGRP),DZERO  
      LOGICAL  LXSCAT,LOWTRK,LEXTSO,LXSTYP
      IOUT  = 6
      DZERO = 0.00000000000D+0
      CALL MOCRWD( IFTRAK )
      IF (IFTRK2 .GT.0)  CALL MOCRWD( IFTRK2 )
      DO 45 IG= NGRP-JGDEB , IGDEB , -1 
           DO 65 IR= 1,NREG
            IF (LEXTSO ) THEN
              S(IR,IG)= Y(IR,IG)
            ELSE
              S(IR,IG)= DZERO
            ENDIF
            IF (LXSCAT ) THEN
              DO 35 JG= 1,NGRP
                 IF( IG.EQ.JG )THEN
                    S(IR,IG)= S(IR,IG)+XSDIA(MATALB(IR),JG)* X(IR,JG)
                 ELSE
                  IF (LXSTYP) THEN
                   IGCAT = JG
                   JGCAT = IG
                  ELSE
                   IGCAT = IG
                   JGCAT = JG
                  ENDIF
                   S(IR,IG)= S(IR,IG)+
     >             XSCAT(IGCAT,MATALB(IR),0,JGCAT)*X(IR,JG)
                 ENDIF
   35         CONTINUE
            ENDIF
             S(IR,IG)= S(IR,IG)* XTINV(MATALB(IR),IG)
   65      CONTINUE
          IF (LOWTRK .AND. IFTRK2.GT.0 ) THEN
           IFFIL   = IFTRK2
           NTRACK  = NTRK2
           NGSSCL  = NGSS2
          ELSE
           IFFIL   = IFTRAK
           NTRACK  = NTRK
           NGSSCL  = NGSS
          ENDIF
            IF( NL.EQ.0 )THEN
               CALL MOC3A0(IFFIL,NGRP,NMAT,NREG,NGSS,NSOUT,MXSEG,
     >                NTRACK,MATALB,WGSS,YGSS,SEGLEN,NRSEG,EXPT,
     >                EXP2,FLM,FLP,CYM,CYP,NALB,SIGANG,S,Z,IG,
     >                (NGRP-IG),NGSSCL)
            ELSE
               CALL XABORT('MOCSWP: ANISOTROPIC OPTION *N/A* YET')
            ENDIF
            DO 165 IR= 1,NREG
                  Z(IR,IG)=Z(IR,IG)*XTINV(MATALB(IR),IG)*VOLINV(IR)
  165       CONTINUE
         CALL MOCRWD( IFTRAK )
         IF (IFTRK2 .GT.0)  CALL MOCRWD( IFTRK2 )
         MAXITR= MAXITR+1 
   45 CONTINUE
      RETURN
      END