*DECK MOCSWX
      SUBROUTINE MOCSWX(IPFLUX,IPRT,NREG,NGRP,NL,NMAT,NANIS,NSOUT,NIFIS,
     >                  NGSS,NPHI,NALB,MXGAUS,IFTRK2 ,IFTRAK,NTRK2,NTRK,
     >                  MXSEG,IPTRK,IPTRK2,NANGL,NANGL2,MATALB,NRSEG ,
     >                  SIGANG ,XSDIA ,XSCAT ,XTINV,SEGLEN,ALBEDO,EXPT ,
     >                  EXP2,WGSS ,YGSS,NGSS2,FLM,FLP,CYM,CYP,LXSCAT,
     >                  LOWTRK,VOLINV,LEXTSO,IGDEB,JGDEB,MAXITR,LXSTYP,
     >                  X,Y,Z,S,ANGADJ,LMODE)
*
*---------------------------  MOCSWX  ---------------------------------
*
* 1- PROGRAMME STATISTICS:
*
*     NAME     :  MOCSWX
*     USE      :  MOCC SPATIAL SWEEPING FOR ANGULAR FORWARD/ADJOINT 
*                 FUNCTIONS
*     BY       :  MONCHAI A. (24/02/05)   
*
* 2- ROUTINE PARAMETERS:
*   INPUT
*     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
*     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
*     IGDEB    : MULTIGROUP REDUCTION IN FORWARD         I
*                DIRECTION
*     JGDEB    : MULTIGROUP REDUCTION IN BACKWARD        I
*                DIRECTION
*     LDAJCB   : FLAG TO TURN ON DIRECT AND ADJOINT FLUX L
*                CALCULATIONS
*     LMODE    : CONTROL FLAG FOR CALCULATION MODE       L(20)
*     MATALB   : MATERIAL CODE IN REGIONS.               I(-NSOUT:NREG)
*     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
*     MXSEG    : MAXIMUM NUMBER OF SEGMENTS ON EACH      I
*                TRACK (VARIABLE ACC. TO TRACKING LINE)
*     NALB     : NUMBER OF SURFACE ALBEDO                I 
*     NGRP     : NUMBER OF GROUPS TREATED                I
*     NGSS     : MAXIMUM NUMBER OF POLAR QUADRATURE      I
*     NE       : NUMBER OF POLAR QUADRATURE              I
*                (LOW OR HIGH ORDER TRACKING LINE)
*     NREG     : NUMBER OF REGIONS CONSIDERED            I
*     NRSEG    : KEY INDEX FOR REGIONAL NUMBER SEGMENT   I(MXSEG)
*     NSOUT    : NUMBER OF SURFACES CONSIDERED           I
*     SEGLEN   : LENGTH OF EACH SEGMENT                  R(MXSEG)  
*     TITRE    : EXECUTION TITLE                         C*72
*     VOL      : VOLUME OF REGION                        R(NREG)
*     VOLINV   : INVERSE OF VOLUMES                      R(NREG)
*     X        : FLUXES                                  D(NREG,NGRP)
*     Y        : FIXED EXTERNAL INPUT                    D(NREG,NGRP)
*
*   WORK VECTOR
*     FLM      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*     FLP      : DUMMY VARIABLE                          D(NGSS,MXSEG) 
*     CYM      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*     CYP      : DUMMY VARIABLE                          D(NGSS,MXSEG)
*      S       : DUMMY VARIABLE                          D(NREG,NGRP)
*   OUTPUT
*      Z       : NEW FLUXES                              D(NREG,NGRP)
*--------------------------   MOCSWX    ------------------------------
*
      IMPLICIT NONE
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      INTEGER      NREG,NGRP,NL,NMAT,NANIS,NSOUT,NIFIS,NGSS,MAXITR,
     >             NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,NTRK,MXSEG,IPFLUX,
     >             MATALB(-NSOUT:NREG),NRSEG(MXSEG),IGDEB,JGDEB,AFLXDU,
     >             IPTRK,IPTRK2,NANGL,NANGL2,IPTRCL,AZIWGT
      INTEGER      IG,JG,IR,IS,IOUT,IPRT,IFFIL,NTRACK,IGCAT,JGCAT,IDUM,
     >             NANGCL,NGSSCL,NGSS2,NPHI,IANG,IE,ILCMUP,ILCMDN 
      REAL         SIGANG(NGSS,-NALB:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >             XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >             SEGLEN(MXSEG),ALBEDO(NALB),
     >             EXPT(NGSS,MXSEG),EXP2(NGSS,MXSEG),
     >             WGSS(MXGAUS),YGSS(MXGAUS),VOLINV(NREG),WPOLAR(MXGAUS)
      REAL         ZERO
      DOUBLE PRECISION  FLM(NGSS,MXSEG),FLP(NGSS,MXSEG),
     >                  CYM(NGSS,MXSEG),CYP(NGSS,MXSEG)
      DOUBLE PRECISION X(NREG,NGRP),Y(NREG,NGRP),Z(NREG,NGRP),
     >                 S(NREG,NGRP),DZERO,DDUM,ERROR
      DOUBLE PRECISION ANGADJ(NREG,-NPHI:NPHI,NGSS,NGRP)
      CHARACTER   CAZIMU*12,CPOLAR*12,NAMFLX*12,CSAV*12 
      LOGICAL  LXSCAT,LOWTRK,LEXTSO,LXSTYP,LMODE(20),LANGUL
      PARAMETER(IOUT=6,ILCMUP=1,ILCMDN=2,DZERO = 0.00000000000D+0,
     >          ZERO=0.00000000000E+0)
      ERROR =DZERO
      CAZIMU='AZIMUTHALWGT'
      CPOLAR='POLAR WEIGHT'
      LANGUL=LMODE(6)
      AFLXDU=0
      CALL SETARA(A,2*NPHI*NREG*NGSS,AFLXDU )
      IF (LOWTRK.AND.IFTRK2.GT.0) THEN
           IPTRCL=IPTRK2
           IFFIL =IFTRK2
           NTRACK=NTRK2
           NGSSCL=NGSS2
           NANGCL=NANGL2
      ELSE
           IPTRCL=IPTRK
           IFFIL =IFTRAK
           NTRACK=NTRK
           NGSSCL=NGSS
           NANGCL=NPHI
      ENDIF
      CALL LCMGET(IPFLUX,CPOLAR,WPOLAR )
      CALL SETARA(A,NANGCL,AZIWGT )
      CALL XDRSET(A(AZIWGT),NANGCL,0.0)
      CALL LCMGET(IPFLUX,CAZIMU,A(AZIWGT) )
      CALL MOCRWD(IFTRAK)
       IF (IFTRK2.GT.0)CALL MOCRWD(IFTRK2)
       DO 45 IG= NGRP-JGDEB,IGDEB,-1 
           DO 65 IR= 1,NREG
            IF (LEXTSO ) THEN
              S(IR,IG)= Y(IR,IG)
            ELSE
              S(IR,IG)= DZERO
            ENDIF
            IF (LXSCAT ) THEN
              DO 35 JG= 1,NGRP
                 IF( IG.EQ.JG )THEN
                    S(IR,IG)=S(IR,IG)+XSDIA(MATALB(IR),JG)*X(IR,JG)
                 ELSE
                  IF (LXSTYP) THEN
                   IGCAT=JG
                   JGCAT=IG
                  ELSE
                   IGCAT=IG
                   JGCAT=JG
                  ENDIF
                   S(IR,IG)=S(IR,IG)+XSCAT(IGCAT,MATALB(IR),0,JGCAT)
     >                      *X(IR,JG)
                 ENDIF
   35         CONTINUE
            ENDIF
             S(IR,IG)= S(IR,IG)*XTINV(MATALB(IR),IG)
   65      CONTINUE
           IF( NL.EQ.0 )THEN
               CALL MOC3AX(LMODE,IFFIL,NPHI,NGRP,NMAT,NREG,NGSS,NSOUT,             
     >                     MXSEG,NTRACK,MATALB,WGSS,YGSS,SEGLEN,NRSEG,
     >                     EXPT,EXP2,FLM,FLP,CYM,CYP,NALB,SIGANG,S,Z ,
     >                     IG,(NGRP-IG),NGSSCL,ANGADJ)
           ELSE
               CALL XABORT('MOCSWX: ANISOTROPIC OPTION *N/A* YET')
           ENDIF
           DO 165 IR= 1,NREG
             Z(IR,IG) = Z(IR,IG)*XTINV(MATALB(IR),IG)*VOLINV(IR)
             IF (LANGUL) THEN
              DO 166 IANG= -NANGCL,NANGCL
              DO 167 IE= 1,NGSSCL
               IF (ABS(IANG).GT. 0 ) THEN
                ANGADJ(IR,IANG,IE,IG)=ANGADJ(IR,IANG,IE,IG)/YGSS(IE)
     >          /A(AZIWGT+ABS(IANG)-1)*XTINV(MATALB(IR),IG)*VOLINV(IR) 
                IF(YGSS(IE).LE.ZERO .OR. A(AZIWGT+ABS(IANG)-1).LE.ZERO)
     >          THEN
                  CALL XABORT('MOCSWX: YGSS OR AZIMUTHAL WGT < 0.0')
                 ENDIF
               ENDIF
  167         CONTINUE
  166         CONTINUE
              IF (.NOT.LXSTYP)  THEN
               DO 168 IANG= 1,NANGCL
               DO 169 IE= 1,NGSSCL
                DDUM = ANGADJ(IR,IANG,IE,IG)
                ANGADJ(IR,IANG,IE,IG)=ANGADJ(IR,-IANG,IE,IG)
                ANGADJ(IR,-IANG,IE,IG)=DDUM
  169          CONTINUE
  168          CONTINUE
              ENDIF 
             ENDIF 
  165      CONTINUE
           CALL MOCRWD(IFTRAK)
           IF (IFTRK2 .GT.0) CALL MOCRWD( IFTRK2 )
           MAXITR= MAXITR+1 
   45  CONTINUE
       IF (LANGUL) THEN
        ERROR=DZERO  
        DO 1164 IR = 1,NREG
        DO 1165 IG = IGDEB,NGRP-JGDEB
        ERROR=ERROR+Z(IR,IG)
        Z(IR,IG)= DZERO
         DO 1166 IANG= -NANGCL,NANGCL
         DO 1167 IE= 1,NGSS
            IF( ABS(IANG) .GT. 0) THEN
               ERROR=ERROR-A(AZIWGT+ABS(IANG)-1)*WPOLAR(IE)*
     >                     ANGADJ(IR,IANG,IE,IG) 
               Z(IR,IG)=Z(IR,IG)+A(AZIWGT+ABS(IANG)-1)*WPOLAR(IE)*
     >                  ANGADJ(IR,IANG,IE,IG) 
              ENDIF
 1167    CONTINUE
 1166    CONTINUE
 1165   CONTINUE
 1164   CONTINUE
       ENDIF
       CALL RLSARA(A(AZIWGT))
       CALL RLSARA(A(AFLXDU))
      RETURN
      END
