*DECK NDIST
      SUBROUTINE NDIST(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To extract library parameters with normal distribution 
* around the average.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*
*Parameters: input
* NENTRY  number of data structures transfered to this module.
* HENTRY  name of the data structures.
* IENTRY  data structure type where:
*         \begin{itemize} 
*         \item IENTRY=1 for LCM memory object; 
*         \item IENTRY=2 for XSM file; 
*         \item IENTRY=3 for sequential binary file;
*         \item IENTRY=4 for sequential ASCII file.  
*         \end{itemize}
*JENTRY   access permission for the data structure where:
*         \begin{itemize} 
*         \item JENTRY=0 for a data structure in creation mode;
*         \item JENTRY=1 for a data structure in modifications mode;
*         \item JENTRY=2 for a data structure in read-only mode.
*         \end{itemize} 
* KENTRY  data structure pointer.
* 
*Comments:
* Instructions for the use of the NDIST: module:
*   Param := NDIST: [ Param ] [[ ParamDist ]]  :: (NDIpara) (NDIget) ;
* where
*     Param     : parameter data structure.
*     ParamDist : sequential binary/ASCII parameter distributions
*     (NDIpara) : PARA keyword processing options (routine NDIPAR)
*     (NDIget)  : GET keyword processing options (routine NDIGET)
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          NENTRY                  	
      CHARACTER        HENTRY(NENTRY)*12
      INTEGER          IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NDIST ')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
      INTEGER          NSTATE,NTC
      PARAMETER       (NSTATE=40,NTC=3)
*----
*  Input and output parameters
*----
      INTEGER          ITYPLU,INTLIR
      CHARACTER        CARLIR*12,CARRET*12
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
*----
*  Memory allocation
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          NBREN,NAMNEW,NRENEW,NAMOLD,NREOLD,NXTOLD,
     >                 REFOLD,RECD
*----
*  Local variables
*----
      INTEGER          IKNDI,IPNDI
      INTEGER          IEN,ITC,KCHAR(NTC)
      CHARACTER        HSIGN*12
      INTEGER          ISTATE(NSTATE)
      INTEGER          NOREC,NLREC,NTREC,NFREC,MAXD
      INTEGER          IUNIT,IPRINT,IFORM
*----
*  Validate entry parameters
*----
      IF(NENTRY .LT. 1) CALL XABORT(NAMSBR//
     >  ': At least one data structures required for this module.')
      IKNDI=0
      IPNDI=0
*----
*  Output structure
*----
      IEN=1
      CALL XDISET(ISTATE,NSTATE,0)
      IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
        IF(JENTRY(IEN) .EQ. 2) CALL XABORT(NAMSBR//
     >  ': Data structure in read-only mode.')
        IPNDI=KENTRY(IEN)
        IF(JENTRY(IEN) .EQ. 1) THEN
          IKNDI=1
          CALL LCMGET(IPNDI,'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          IF(HSIGN .NE. 'L_NDIST') CALL XABORT(NAMSBR//
     >      ': Invalid signature for '//HENTRY(IEN)//'.')
          CALL LCMGET(IPNDI,'STATE-VECTOR',ISTATE)
        ELSE
          IKNDI=-1
          HSIGN='L_NDIST     '
          READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          CALL LCMPUT(IPNDI,'SIGNATURE',NTC,3,KCHAR)
          CALL LCMPUT(IPNDI,'STATE-VECTOR',NSTATE,1,ISTATE)
          HSIGN='            '
          READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          CALL LCMPUT(IPNDI,'ParametreNom',3,3,KCHAR)
          CALL LCMPUT(IPNDI,'ParametreNbr',1,1,0)
          CALL LCMPUT(IPNDI,'ParametreNxt',1,1,0)
          CALL LCMPUT(IPNDI,'ParametreRef',1,2,0.0)
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//
     >  ': Invalid data structure format for '//HENTRY(IEN)//'.')
      ENDIF
*----
*  Input structure
*----
      CALL SETARA(IBASE,NENTRY,NBREN)
      NOREC=ISTATE(1)
      MAXD=ISTATE(2)
      IBASE(NBREN)=NOREC
      NTREC=0
      DO IEN=2,NENTRY
        IUNIT=KENTRY(IEN)
*        write(6,'(A12,5X,I10)') HENTRY(IEN),KENTRY(IEN)
        IF(IENTRY(IEN) .EQ. 4) THEN
          READ(IUNIT,*) NLREC
*          write(6,'(I10)') NLREC
        ELSE
          CALL XABORT(NAMSBR//
     >    ': Invalid data structure format for '//HENTRY(IEN)//'.')
        ENDIF
        IBASE(NBREN+IEN-1)=NLREC
        NTREC=NTREC+NLREC
      ENDDO
*----
*  Recover parameter names from output and input structures      
*----
      NFREC=NTREC+NOREC
      CALL SETARA(IBASE,3*(NFREC+1),NAMOLD)
      CALL SETARA(IBASE,(NFREC+1),NREOLD)
      CALL SETARA(RBASE,(NFREC+1),REFOLD)
      CALL SETARA(IBASE,(NFREC+1),NXTOLD)
      CALL SETARA(IBASE,3*(NTREC+1),NAMNEW)
      CALL SETARA(IBASE,(NTREC+1),NRENEW)
      CALL NDICRE(NENTRY,KENTRY,NOREC,NTREC,NFREC,MAXD,
     >            IBASE(NBREN),IBASE(NAMNEW),IBASE(NRENEW),
     >            IBASE(NAMOLD),IBASE(NREOLD),
     >            RBASE(REFOLD),IBASE(NXTOLD)) 
      ISTATE(1)=NFREC
      ISTATE(2)=MAXD
      CALL LCMPUT(IPNDI,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL RLSARA(IBASE(NRENEW))
      CALL RLSARA(IBASE(NAMNEW))
      CALL RLSARA(IBASE(NBREN))
*----
*  INPUT/OUTPUT VARIABLES 
*  Input data is of the form
*    [ EDIT iprint ]
*    [ SET (dataSET) ]
*    [ GET (dataGET) ]
*  where
*  EDIT               = keyword for print level 
*  iprint             = integer print level 
*  SET                = keyword to set reference value 
*                       and extract number of parameters
*  GET                = keyword to set next value  
*                       and to extract parameter
*  (dataSET)          = SET data processed by NDISET
*  (dataGET)          = GET data processed by NDIGET
*----
      CALL SETARA(RBASE,MAXD,RECD)
      IPRINT=1    
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  CONTINUE
      IF(ITYPLU .EQ. 10) GO TO 105
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': Read error -- Character variable expected')
      IF(CARLIR .EQ. ';') THEN
        GO TO 105 
      ELSE IF(CARLIR .EQ. 'EDIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': Read error -- integer value for EDIT expected.')
        IPRINT=INTLIR
      ELSE IF(CARLIR .EQ. 'SET') THEN
        CARRET=CARLIR
        CALL NDISET(IPNDI,IPRINT,CARRET,NFREC,
     >              IBASE(NAMOLD),IBASE(NREOLD),
     >              RBASE(REFOLD),IBASE(NXTOLD))
        CARLIR=CARRET
        GO TO 101
      ELSE IF(CARLIR .EQ. 'GET') THEN
        CARRET=CARLIR
        CALL NDIGET(IPNDI,IPRINT,CARRET,NFREC,MAXD,
     >              IBASE(NAMOLD),IBASE(NREOLD),
     >              RBASE(REFOLD),IBASE(NXTOLD),
     >              RBASE(RECD))
        CARLIR=CARRET
        GO TO 101
      ELSE
        CALL XABORT(NAMSBR//
     >  ': '//CARLIR//' is an invalid keyword.')
      ENDIF
      GO TO 100
 105  CONTINUE
*----
*  Release memory and return
*----
      CALL RLSARA(RBASE(RECD))
      CALL RLSARA(RBASE(REFOLD))
      CALL RLSARA(IBASE(NXTOLD))
      CALL RLSARA(IBASE(NREOLD))
      CALL RLSARA(IBASE(NAMOLD))
      RETURN
*----
*  Warning formats
*----
      END
