*DECK PIJNRM
      SUBROUTINE PIJNRM(NREGIO,NBMIX ,ITYPE ,MATCOD,VOLUME,
     >                  XSSIGT,XSSIGW,PIJSYM,PIS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* normalisation of the CP matrix (standard or scattering-reduced) 
* to force neutron conservation (no leakage).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NREGIO  number of regions considered.
* NBMIX   number of mixtures.
* ITYPE   type of CP matrix where :
*         scattering-reduced CP matrices for ITYPE = 1;
*         standard CP matrices otherwise.
* MATCOD  mixture code in each region.
* VOLUME  volume of each region.
* XSSIGT  total macroscopic cross sections.
* XSSIGW  P0 within-group scattering cross sections.
*
*Parameters: input/output
* PIJSYM  group condensed reduce/symmetric scattering-reduced pij
*         matrix.
*
*Parameters: scratch
* PIS
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  Subroutine arguments
*----
      INTEGER    NREGIO,NBMIX,ITYPE
      INTEGER    MATCOD(NBMIX)
      REAL       VOLUME(NREGIO),XSSIGT(0:NBMIX),XSSIGW(0:NBMIX),
     >           PIJSYM(NREGIO*(NREGIO+1)/2),PIS(NREGIO)
*----
*  Local variables
*----
      INTEGER    I,J,MATNUM,INDPIJ
      DOUBLE PRECISION SUM,DD
*----
*  Intrinsic function
*----
      INTEGER    INDPOS
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
*----
*  Find cross sectons to use depending on ITYPE  
*----
      IF(ITYPE .EQ. 1) THEN
        DO MATNUM=1,NBMIX
          XSSIGT(MATNUM)=XSSIGT(MATNUM)-XSSIGW(MATNUM)
        ENDDO
      ENDIF
      DD=0.0D0
      DO I=1,NREGIO
        SUM=0.0D0
        DO J=1,NREGIO
          MATNUM=MATCOD(J)
          IF(MATNUM.GT.0) THEN
           SUM=SUM+XSSIGT(MATNUM)*PIJSYM(INDPOS(I,J))/VOLUME(I)
          ENDIF
        ENDDO
        PIS(I)=1.0D0-SUM
        MATNUM=MATCOD(I)
        IF(MATNUM.GT.0) THEN
          DD=DD+XSSIGT(MATNUM)*VOLUME(I)*PIS(I)
        ENDIF
      ENDDO
      IF(DD.NE.0.0D0) THEN
        DO I=1,NREGIO
          DO J=1,I
            INDPIJ=INDPOS(I,J)
            PIJSYM(INDPIJ)=PIJSYM(INDPIJ)
     >                    +PIS(I)*PIS(J)*VOLUME(I)*VOLUME(J)/DD
          ENDDO
        ENDDO
      ENDIF
      IF(ITYPE .EQ. 1) THEN
        DO MATNUM=1,NBMIX
          XSSIGT(MATNUM)=XSSIGT(MATNUM)+XSSIGW(MATNUM)
        ENDDO
      ENDIF
      RETURN
      END
