*DECK PIJSBC
      SUBROUTINE PIJSBC(IPSYS ,NREG  ,NSOUT ,NPRB  ,ITYPE,
     >                  SIGTAL,PROB  ,PSVT  ,PSVF )
C
C-------------------------    PIJSBC    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : PIJSBC
C     LEVEL    : 2 (CALLED BY 'EXCELP')
C     USE      : THIS ROUTINE RECONSTRUCT EXCAPE PROBABILITIES (Psv)
C                FOR  ALL ZONES ELIMINATING INCLUDING BOUNDARY
C                CONDITIONS
C     CREATED  : 2003/02/17
C     AUTHOR   : P. Dufour
C
C 2- PARAMETERS:
C  INPUT
C     IPSYS   : POINTER TO THE PIJ data structure    I
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSOUT   : # OF SURFACES FOR GEOMETRY.          I
C     NPRB    : NUMBER OF PROBABILITIES IN PROB      I
C     ITYPE   : BC type. If ITYPE=1, use BC
C               if ITYPE = 0 do not use BC           I
C     SIGTAL  : ALBEDO-SIGT VECTOR                   R(-NSOUT:NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NPRB)
C     PSVT    : WORKING AREA FOR PSVT                D(NSOUT,NREG)
C               PSVT=PSST*PSV                   
C  OUTPUT    
C     PSVF    : Complete PSV matrix for surface flux R(NSOUT,NREG)
C               calculation
C
C  3-EXTERNAL ROUTINE CALLED
C     XABORT  : DRAGON ABORT ROUTINE
C
C----------------------------------------------------------------------
      IMPLICIT         NONE
C----
C  ROUTINE PARAMETERS
C-----
      INTEGER          IPSYS,NREG,NSOUT,NPRB,ITYPE
      REAL             SIGTAL(-NSOUT:NREG),PROB(NPRB)
      DOUBLE PRECISION PSVT(NSOUT,NREG),PSVF(NSOUT,NREG)
C---- 
C  VARIABLES LOCALES
C----
      INTEGER          NSP1,IPSS,ISUR,ISX,IS,JSX,JS,IER,
     >                 IV,JV,IVSI,IVVI,ISV,IVS,IVV,ISS,ISSI
C----
C    [PSVF]= [PSS][PSVT]+[PSV]
C----
      NSP1=NSOUT+1
      IVSI=(NSP1*(NSP1+1))/2
      IVVI=IVSI+NSP1
      IF(ITYPE .EQ. 1) THEN
        DO 110 IV=1,NREG
          ISSI=1
           DO 112 IS=1,NSOUT
              PSVF(NSP1-IS,IV)=0.0D0
              ISSI=ISSI+IS-1
              ISS=ISSI
              DO 120 JS=1,NSOUT
                 PSVF(NSP1-IS,IV)=PSVF(NSP1-IS,IV)+(PROB(ISS)*
     >                            PSVT(JS,IV))
                 IF(JS.LT.IS) THEN
                    ISS=ISS+1
                 ELSE
                    ISS=ISS+JS
                 ENDIF
 120          CONTINUE
              ISV=IVSI+IS
              PSVF(NSP1-IS,IV)=PSVF(NSP1-IS,IV)+PROB(ISV)
 112       CONTINUE
           IVSI=IVSI+NSP1+IV
 110    CONTINUE
      ELSE
        DO 100 IV=1,NREG
          ISSI=1
           DO 102 IS=1,NSOUT
              ISV=IVSI+IS
              PSVF(NSP1-IS,IV)=PROB(ISV)
 102       CONTINUE
           IVSI=IVSI+NSP1+IV
 100    CONTINUE
      ENDIF
C----
C    [PSVF]=[S^(-1)][PSVF][SIGTOT^(-1)]
C----
      ISSI=(NSOUT*NSP1)/2
      DO 130 IS=1,NSOUT
         ISS=ISSI+IS
         DO 131 IV=1,NREG
            IF(SIGTAL(IV).EQ.0.0) THEN
               PSVF(NSP1-IS,IV)=(1/PROB(ISS))*PSVF(NSP1-IS,IV)
            ELSE
               PSVF(NSP1-IS,IV)=(1/PROB(ISS))*(1/SIGTAL(IV))*
     >                          PSVF(NSP1-IS,IV)
            ENDIF
 131     CONTINUE
 130  CONTINUE
C----
C    TRANSCRIPTION DE PSVF
C----
      CALL LCMPUT(IPSYS,'DRAGON-PCFUI',2*NREG*NSOUT,4,PSVF)
      RETURN
      END
