*DECK PIJSMD
      SUBROUTINE PIJSMD(IMPX,NBMIX,NREGIO,MATCOD,VOLUME,XSSIGW,XSSIGT,
     >                  ILK,PIJSYM,PIJSCT,IOP)
C
C---------------------------  PIJSMD  ---------------------------------
C
C   1-  PROGRAMME STATISTICS:
C
C          NAME       :  PIJSMD
C          FUNCTION   :  EVALUATE SCATTERING MODIFIED CP MATRIX
C          DATE       :  03-07-1988
C          MODIFIED   :  94-07-21  (I. PETROVIC, G. MARLEAU)
C          AUTHOR     :  G.MARLEAU
C
C   2-  PARAMETRES D'ENTREE ET DE RETOUR:
C
C         IMPX    : PRINT/CHECK FLAG.                    I            I
C                   EQUAL TO 0 FOR NO PRINT.
C         NBMIX   : NUMBER OF MIXTURES CONSIDERED        I            I
C         NREGIO  : NUMBER OF REGIONS CONSIDERED         I            I
C         MATCOD  : MATERIAL CODE IN REGION              I(NREGIO)    I
C         VOLUME  : VOLUME OF REGION                     R(NREGIO)    I
C         XSSIGW  : WITHIN GROUP SCATTERING 0 OR 1 HARM. R(NBMIX+1)   D
C         XSSIGT  : TOTAL MACROSCOPIC X.S.               R(NBMIX+1)   D
C         ILK     : ILK=.TRUE. IF LEAKAKE EXISTS         L            I
C         PIJSYM  : GROUP CONDENSED REDUCE/SYMMETRIC     R(NREGIO*
C                   PIJ OR PIJK MATRIX                   (NREGIO+1)/2 D
C         ITYP    : 1 OR 4: PIJ OR PIJK COL. PROB.       I
C         PIJSCT  : SIGW MODIFIED CP MATRIX              R(2*NREGIO**2)D
C                   (PIJ OR PIJK)
C         IOP     : PIJ OR PIJK OPTION                   I
C                   = 1 FOR PIJ
C                   = 4 FOR PIJK
C
C---------------------------   PIJSMD  --------------------------------
C
      IMPLICIT  NONE
C----
C PARAMETERS
C----
      INTEGER     IUNOUT
      REAL        EPS1
      PARAMETER  (IUNOUT=6,EPS1=1.0E-4)
C----
C INTERNAL FUNCTIONS (USER-DEFINED)
C----
      INTEGER     INDPOS
C----
C VARIABLES
C----
      INTEGER     IERROR,I,J,INDPIJ
      INTEGER     IMPX,NBMIX,NREGIO,MATCOD(*),IOP
      REAL        VOLTOT
      REAL        VOLUME(*),XSSIGW(0:NBMIX),XSSIGT(0:NBMIX),
     >            PIJSYM(*)
      LOGICAL     ILK
      DOUBLE PRECISION PIJSCT(NREGIO,*),WRK,F1
C----
C  INTRINSIC FUNCTION FOR POSITION IN CONDENSE PIJ MATRIX
C----
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
C----
C  COMPUTE SCATTERING MODIFIED PIJ
C----
      DO 10 I=1,NREGIO
        DO 11 J=1,NREGIO
          INDPIJ=INDPOS(I,J)
          PIJSCT(I,J)=-XSSIGW(MATCOD(J))*PIJSYM(INDPIJ)
          PIJSCT(I,NREGIO+J)=PIJSYM(INDPIJ)
 11     CONTINUE
        PIJSCT(I,I)=VOLUME(I)+PIJSCT(I,I)
 10   CONTINUE
      CALL ALSBD(NREGIO,NREGIO,PIJSCT,IERROR,NREGIO)
      IF(IERROR.NE.0) CALL XABORT('PIJSMD: SINGULAR MATRIX.')
      DO 20 I=1,NREGIO
        DO 21 J=1,NREGIO
          PIJSCT(I,J)=PIJSCT(I,NREGIO+J)
 21     CONTINUE
 20   CONTINUE
      IF (IMPX.GE.8) THEN
        IF(IOP.EQ.4)THEN
          WRITE(IUNOUT,330) (J,J=1,NREGIO)
        ELSE
          WRITE(IUNOUT,130) (J,J=1,NREGIO)
        ENDIF
        DO 30 I=1,NREGIO
          WRITE(IUNOUT,140) I,(PIJSCT(I,J),J=1,NREGIO)
 30     CONTINUE
        WRITE(IUNOUT,'(//)')
      ENDIF
      IF((IMPX.GE.10).OR.(IMPX.LT.0).AND.(IOP.EQ.1)) THEN
C----
C  CHECK THE RECIPROCITY CONDITIONS.
C----
        VOLTOT=0.0
        DO 50 I=1,NREGIO
          VOLTOT=VOLTOT+VOLUME(I)
 50     CONTINUE
        VOLTOT=VOLTOT/REAL(NREGIO)
        WRK=0.0D0
        DO 60 I=1,NREGIO
          DO 61 J=1,NREGIO
            WRK=MAX(WRK,ABS(PIJSCT(I,J)*VOLUME(I)
     >         -PIJSCT(J,I)*VOLUME(J))/VOLTOT)
 61       CONTINUE
 60     CONTINUE
        IF (WRK.GE.EPS1) WRITE(IUNOUT,150) WRK
C----
C  CHECK THE CONSERVATION CONDITIONS.
C----
        IF (.NOT.ILK) THEN
          WRK=0.0D0
          DO 70 I=1,NREGIO
            F1=1.0D0
            DO 71 J=1,NREGIO
              F1=F1-PIJSCT(I,J)*(XSSIGT(MATCOD(J))-XSSIGW(MATCOD(J)))
 71         CONTINUE
            WRK=MAX(WRK,ABS(F1))
 70       CONTINUE
          IF (WRK.GE.EPS1) WRITE(IUNOUT,160) WRK
        ENDIF
      ENDIF
      RETURN
  130 FORMAT (//53H SCATTERING MODIFIED COLLISION PROBABILITIES MATRIX :
     1 //(11X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,
     2 I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,
     3 2HJ=,I4,:,5X,2HJ=,I4))
  330 FORMAT (//53H SCATTERING MODIFIED DIRECTIONAL COL. PROB.  MATRIX :
     1 //(11X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,
     2 I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,
     3 2HJ=,I4,:,5X,2HJ=,I4))
  140 FORMAT (3H I=,I4,2H: ,1P,11E11.3/(9X,11E11.3))
  150 FORMAT (/50H PIJSMD: THE SCATTERING MODIFIED CP MATRIX DO NOT ,
     1 40HMEET THE RECIPROCITY CONDITIONS. RECIP =,1P,E10.3/)
  160 FORMAT (/50H PIJSMD: THE SCATTERING MODIFIED CP MATRIX DO NOT ,
     1 40HMEET THE CONSERVATION CONDITIONS. LEAK =,1P,E10.3/)
      END
