*DECK QIJSMD
      SUBROUTINE QIJSMD(NREG,NSNEG,NPRB,NGRP,SIGINV,SIGSS,
     >                  PROB,VOLUME,PIJSCT)
C
C-------------------------    QIJSMD    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJSMD
C     LEVEL    : 4 (CALLED BY 'XL3TRK')
C     USE      : THIS ROUTINE RECONSTRUCT COLLISION PROBABILITIES (CP)
C                WITH SELF-SCATTERING REDUCTION.
C     AUTHOR   : R. ROY (96-09-19)
C                G. MARLEAU (2001/10/02) 
C                -> ROUTINE CHANGED TO REFLECT PIJSMD
C                   BECAUSE USE OF SYMMETRIC INVERSION
C                   CAUSES PROBLEM WHEN SCATTERING CROSS SECTION
C                   VOID.
C
C 2- PARAMETERS:
C  INPUT
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSOUT   : # OF SURFACES FOR GEOMETRY.          I
C     NPRB    : NUMBER OF PROBABILITIES IN PROB I
C     SIGINV  : SIGT INVERSE VECTOR                  R(NREG)
C     SIGSS   : SELF-SCATTERING VECTOR               R(NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,*)
C     VOLUME  : VOLUME VECTOR.                       R(NREG)
C  OUTPUT
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,*)
C  WORK
C     PIJSCT  : WORKING AREA                         R(NREG,2*NREG)
C
C  3-EXTERNAL ROUTINE CALLED
C     ALLDLF  : DECOMPOSE A SYMMETRIC SYSTEM USING CHOLESKY SCHEME.
C     ALLDLS  : SOLVE A CHOLESKY DECOMPOSED SYSTEM.
C
C-----------------------------------------------------------------
      IMPLICIT NONE
      INTEGER NREG,NSNEG,NGRP,NPRB
      INTEGER NSOUT,IPRB,INEW,ISS,IVOL,IPAR,IG,
     >        IV,JV,KV,NDEBV,I,J,IERROR
      REAL    SIGINV(NGRP,NREG),SIGSS(NGRP,NREG),VOLUME(NREG)
      DOUBLE PRECISION PROB(NGRP,*),PIJSCT(NREG,2*NREG),TMPPIJ
C----
C  STORE SYMMETRIC -PVV AT BEGINNING OF WORK ARRAY
C----
      NSOUT=-NSNEG
      NDEBV=(NSOUT+1)*(NSOUT+2)/2+1
      DO 100 IG=1,NGRP
        IPAR= NDEBV
        DO 110 IV=1,NREG
          IPRB= IPAR+NSOUT+1
          DO 111 JV=1,IV-1
            TMPPIJ=PROB(IG,IPRB)*SIGINV(IG,IV)*SIGINV(IG,JV)
            PIJSCT(IV,JV)=TMPPIJ
            PIJSCT(JV,IV)=TMPPIJ
            IPRB= IPRB+1
 111      CONTINUE
          PIJSCT(IV,IV)=PROB(IG,IPRB)
     >                   *SIGINV(IG,IV)*SIGINV(IG,IV)
          IPRB= IPRB+1
          IPAR= IPRB
 110    CONTINUE
C----
C  STORE I-PVV*SIGS AT BEGINNING OF WORK 
C  ARRAY PIJSCT AND PVV AT END
C----
        DO 120 IV=1,NREG
          DO 121 JV=1,NREG
            PIJSCT(IV,JV+NREG)=PIJSCT(IV,JV)
            PIJSCT(IV,JV)=-SIGSS(IG,JV)*PIJSCT(IV,JV)
 121      CONTINUE
          PIJSCT(IV,IV)=VOLUME(IV)+PIJSCT(IV,IV)
 120    CONTINUE
        CALL ALSBD(NREG,NREG,PIJSCT,IERROR,NREG)
        INEW=(NREG*(NREG+1))/2
        DO 130 JV=1,NREG
          DO 131 IV=1,NREG
            INEW=INEW+1
            PROB(IG,INEW)=PIJSCT(IV,JV+NREG)
 131      CONTINUE
 130    CONTINUE
 100  CONTINUE
      RETURN
      END
