*DECK READEU
      SUBROUTINE READEU (MAXPTS,IPGEOM,IR,MAT,ILK,NMCEL,NMERGE,NGEN,
     1 INUM,IGEN,NMBLK,LX,LY,XX,YY,RAYRE,NMC,IORI,NCODE,ZCODE,IHEX,
     2 IMPX,MILIE,NBREG,MILIEU,RAYON,CELL,XXX,YYY)
*
*-----------------------------------------------------------------------
*
* RECOVER INPUT DATA FOR THE DESCRIPTION OF A 2-D ASSEMBLY (EURYDICE-2).
*
* GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMCEL, NMERGE OR
*            NGEN.
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   IR     : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NMBLK)
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   NMCEL  : TOTAL NUMBER OF CELLS IN THE DOMAIN.
*   NMERGE : TOTAL NUMBER OF MERGED CELLS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY CELLS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING CELL. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMCEL
*   NGEN   : TOTAL NUMBER OF GENERATING CELLS. A GENERATING CELL IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   INUM   : INDEX-NUMBER OF THE MERGED CELL ASSOCIATED TO EACH CELL.
*            DIMENSION INUM(NMCEL)
*   IGEN   : INDEX-NUMBER OF THE GENERATING CELL ASSOCIATED WITH EACH
*            MERGED CELL. DIMENSION IGEN(NMERGE)
*   NMBLK  : TOTAL NUMBER OF VOLUMES IN ALL THE MERGED CELLS.
*   LX     : NUMBER OF CELLS ALONG THE X-AXIS.
*   LY     : ... ALONG THE Y-AXIS.
*   XX,YY  : X AND Y-THICKNESS OF EACH CELL / SIDE OF THE HEXAGONS.
*            DIMENSION XX(NGEN),YY(NGEN)
*   RAYRE  : RADIUS OF THE TUBES IN THE GENERATING CELLS.
*            DIMENSION RAYRE(NMC(NGEN+1))
*   NMC    : OFFSETS OF THE FIRST ZONE IN EACH GENERATING CELL.
*            DIMENSION NMC(NGEN+1)
*   IORI   : ORIENTATION OF THE CELLS. DIMENSION IORI(NMCEL)
*   NCODE  : BOUNDARY CONDITION RELATIVE TO EACH SIDE OF THE DOMAIN.
*            =0 : NOT USED   =1 : VIDE     =2 : REFL
*            =3 : DIAG       =4 : TRAN     =5 : SYME
*   ZCODE  : ALBEDO RELATIVE TO EACH SIDE OF THE DOMAIN.
*            DIMENSION NCODE(6),ZCODE(6)
*   IHEX   : TYPE OF SYMMETRY FOR HEXAGONAL GEOMETRY.
*            =0 : CARTESIAN GEOMETRY.
*            =1 : S30   =2 : SA60   =3 : SB60   =4 : S90   =5 : R120
*            =6 : R180  =7 : SA180  =8 : SB180  =9 : COMPLETE
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
* SCRATCH STORAGE:
*   MILIE  : DIMENSION MILIE(MAXPTS)
*   NBREG  : DIMENSION NBREG(MAXPTS)
*   MILIEU : DIMENSION MILIEU(MAXPTS)
*   RAYON  : DIMENSION RAYON(MAXPTS)
*   CELL   : DIMENSION CELL(3*MAXPTS)
*   XXX    : DIMENSION XXX(MAXPTS+1)
*   YYY    : DIMENSION YYY(MAXPTS+1)
*
*---------------------------------------- AUTHOR: A. HEBERT 15/05/87 ---
*
      PARAMETER (NSTATE=40)
      LOGICAL ILK,LL1,LL2,LCM
      CHARACTER GEONAM*12,TEXT12*12,NAMT*12,HSMG*131
      DIMENSION MAT(*),INUM(*),IGEN(*),XX(*),YY(*),RAYRE(*),NMC(*),
     1 IORI(*),MILIE(*),NBREG(*),MILIEU(*),RAYON(*),CELL(*),XXX(*),
     2 YYY(*),NCODE(6),ZCODE(6),ICODE(6),
     3 ISTATE(NSTATE),ISTAT2(NSTATE)
      COMMON BASE(1)
*
      NAMT=' '
      CALL LCMNXT(IPGEOM,NAMT,GEONAM,TEXT12,LCM)
      IHEX=0
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      IF((ISTATE(1).EQ.8).OR.(ISTATE(1).EQ.24)) THEN
         CALL LCMGET(IPGEOM,'IHEX',IHEX)
      ENDIF
      IF((ISTATE(5).NE.0).OR.((ISTATE(11).NE.0).AND.(ISTATE(1).EQ.5))
     1 .OR.((ISTATE(11).NE.0).AND.(ISTATE(1).EQ.8)).OR.(ISTATE(13).NE.0)
     2 ) CALL XABORT('READEU: UNABLE TO PROCESS THE GEOMETRY.')
      LX=ISTATE(3)
      LY=ISTATE(4)
      IF(LX.GT.MAXPTS) CALL XABORT('READEU: INSUFFICIENT STORAGE(1).')
      IF(LY.GT.MAXPTS) CALL XABORT('READEU: INSUFFICIENT STORAGE(2).')
      IF(ISTATE(6).GT.MAXPTS) CALL XABORT('READEU: INSUFFICIENT STORA'
     1 //'GE(3).')
*
* RECOVER THE BOUNDARY CONDITIONS.
      CALL LCMGET(IPGEOM,'NCODE',NCODE)
      CALL LCMGET(IPGEOM,'ZCODE',ZCODE)
      CALL LCMGET(IPGEOM,'ICODE',ICODE)
      I2=0
      DO 10 IC=1,4
      IF(ICODE(IC).NE.0) THEN
         CALL XABORT('READEU: MACROLIB DEFINED ALBEDOS ARE NOT IMPLEMEN'
     1   //'TED.')
      ENDIF
      IF(NCODE(IC).EQ.2) ZCODE(IC)=1.0
      IF(NCODE(IC).EQ.6) NCODE(IC)=1
      IF(NCODE(IC).GE.7) CALL XABORT('READEU: INVALID TYPE OF B.C.')
      IF(NCODE(IC).EQ.3) I2=I2+1
   10 CONTINUE
      IF(NCODE(1).EQ.0) GO TO 550
      LL1=.FALSE.
      LL2=.FALSE.
      IF(IHEX.EQ.0) THEN
         IF((NCODE(2).EQ.0).OR.(NCODE(3).EQ.0).OR.(NCODE(4).EQ.0))
     1   GO TO 550
         NSUPCE=LX*LY
         IF(I2.GT.0) THEN
            IF(I2.NE.2) GO TO 560
            IF(LX.NE.LY) CALL XABORT('READEU: LX=LY WITH A DIAGONAL S'
     1      //'YMMETRY.')
            NSUPCE=(LX+1)*LX/2
            LL1=((NCODE(2).EQ.3).AND.(NCODE(3).EQ.3))
            LL2=((NCODE(1).EQ.3).AND.(NCODE(4).EQ.3))
            IF((.NOT.LL1).AND.(.NOT.LL2)) GO TO 560
         ENDIF
         DO 20 IC=1,4,2
         IF((NCODE(IC).EQ.4).AND.(NCODE(IC+1).NE.4))
     1      CALL XABORT('READEU: THE TRANSLATION CONDITIONS X- TRAN X+'
     2      //' TRAN AND Y- TRAN Y+ TRAN ARE THE ONLY PERMITTED.')
   20    CONTINUE
      ELSE
         NSUPCE=LX
         IF((NCODE(2).NE.0).OR.(NCODE(3).NE.0).OR.(NCODE(4).NE.0))
     1   CALL XABORT('READEU: INVALID TYPE OF HEXAGONAL B.C.')
         IF(NCODE(1).EQ.5) THEN
            IF(IHEX.EQ.1) THEN
               IHEX=10
            ELSE IF(IHEX.EQ.2) THEN
               IHEX=11
            ELSE
               CALL XABORT('READEU: BOUNDARY CONDITION HBC WITH OPTION'
     1         //' SYME CAN ONLY BE USED WITH OPTION S30 OR SA60.')
            ENDIF
         ELSE IF(NCODE(1).GT.2) THEN
            CALL XABORT('READEU: BOUNDARY CONDITION HBC CAN ONLY BE US'
     1      //'ED WITH OPTIONS VOID, REFL, SYME OR ALBE.')
         ENDIF
      ENDIF
*
      NMC(1)=0
      IG=0
      IR=0
      NMERGE=1
      NGEN=1
      IF((ISTATE(1).EQ.5).OR.(ISTATE(1).EQ.8)) THEN
         NMCEL=ISTATE(6)
         IF(NMCEL.GT.MAXPTS) THEN
            WRITE(HSMG,'(36HREADEU: INSUFFICIENT STORAGE. NMCEL=,I8,
     1      8H MAXPTS=,I8)') NMCEL,MAXPTS
            CALL XABORT(HSMG)
         ENDIF
         DO 30 IKK=1,NMCEL
         INUM(IKK)=IKK
   30    IORI(IKK)=1
         IF(ISTATE(8).EQ.1) THEN
*           MIXED GEOMETRY.
            CALL LCMLEN(IPGEOM,'MERGE',ILEN,ITYLCM)
            IF(ILEN.GT.0) CALL LCMGET(IPGEOM,'MERGE',INUM)
            CALL LCMLEN(IPGEOM,'TURN',ILEN,ITYLCM)
            IF(ILEN.GT.0) CALL LCMGET(IPGEOM,'TURN',IORI)
            CALL LCMGET(IPGEOM,'CELL',CELL)
            CALL LCMGET(IPGEOM,'GENERATING',NBREG)
            DO 35 IKK=1,NMCEL
            IGEN(INUM(IKK))=NBREG(IKK)
            NGEN=MAX(NGEN,NBREG(IKK))
            NMERGE=MAX(NMERGE,INUM(IKK))
            DO 35 JKK=1,NMCEL
            IF(INUM(IKK).EQ.INUM(JKK)) THEN
               IF(NBREG(IKK).NE.NBREG(JKK)) CALL XABORT('READEU: TWO C'
     1         //'ELLS WITH THE SAME MERGED NUMBER DO NOT HAVE THE SAM'
     2         //'E GENERATING CELL.')
            ENDIF
   35       CONTINUE
            IF(NGEN.NE.ISTATE(9)) CALL XABORT('READEU: INVALID NUMBER'
     1      //' OF SUB GEOMETRIES.')
            DO 70 IKG=1,NGEN
            WRITE (TEXT12(:4),'(A4)') CELL(3*(IKG-1)+1)
            WRITE (TEXT12(5:8),'(A4)') CELL(3*(IKG-1)+2)
            WRITE (TEXT12(9:),'(A4)') CELL(3*(IKG-1)+3)
            CALL LCMLEN(IPGEOM,TEXT12,ILEN,ITYLCM)
            IF((ILEN.EQ.0).OR.(ITYLCM.NE.0)) CALL XABORT('READEU: SUB'
     1      //' GEOMETRY '//TEXT12//' IS MISSING FROM L_GEOM.')
            CALL LCMSIX(IPGEOM,TEXT12,1)
               CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTAT2)
               IF(ISTAT2(6).GT.MAXPTS) CALL XABORT('READEU: INSUFFICI'
     1         //'ENT STORAGE(4).')
               IF((ISTAT2(1).EQ.20).OR.(ISTAT2(1).EQ.24)) THEN
                  NZONE=ISTAT2(2)+1
                  CALL LCMLEN(IPGEOM,'SPLITR',ILENN,ITYLCM)
                  IF(ILENN.GT.0) THEN
                     CALL LCMGET(IPGEOM,'SPLITR',NBREG)
                  ELSE
                     DO 40 I=1,NZONE
   40                NBREG(I)=1
                  ENDIF
                  IF(NZONE.GT.1) CALL LCMGET(IPGEOM,'RADIUS',RAYON)
               ELSE IF(((ISTAT2(1).EQ.5).OR.(ISTAT2(1).EQ.8)).AND.
     1         (ISTAT2(6).EQ.1)) THEN
                  NZONE=1
                  NBREG(1)=1
               ELSE
                  CALL XABORT('READEU: INVALID SUB GEOMETRY.')
               ENDIF
               CALL LCMGET(IPGEOM,'MIX',MILIEU)
               IF(IHEX.EQ.0) THEN
                  CALL LCMGET(IPGEOM,'MESHX',XXX)
                  CALL LCMGET(IPGEOM,'MESHY',YYY)
               ELSE
                  CALL LCMGET(IPGEOM,'SIDE',SIDE)
               ENDIF
            CALL LCMSIX(IPGEOM,' ',2)
*
            RJ=0.0
            RAYRE(IG+1)=0.0
            DO 60 I=1,NZONE-1
            IF(RAYON(I+1).LE.RJ) GO TO 520
            PAS=(RAYON(I+1)-RJ)/REAL(ABS(NBREG(I)))
            IF(NBREG(I).LT.0) PAS=PAS*(RAYON(I+1)+RJ)
            DO 50 J=1,ABS(NBREG(I))
            IG=IG+1
            MILIE(IG)=MILIEU(I)
            IR=MAX(IR,MILIEU(I))
            IF(NBREG(I).GT.0) THEN
               RJ=RJ+PAS
            ELSE
               RJ=SQRT(RJ*RJ+PAS)
            ENDIF
   50       RAYRE(IG+1)=RJ
   60       RJ=RAYON(I+1)
            IG=IG+1
            IF(IG.GT.MAXPTS) THEN
               WRITE(HSMG,'(28HREADEU: INCREASE MAXPTS FROM,I7,3H TO,
     1         I7,1H.)') MAXPTS,IG
               CALL XABORT(HSMG)
            ENDIF
            MILIE(IG)=MILIEU(NZONE)
            IR=MAX(IR,MILIE(IG))
            IF(IHEX.EQ.0) THEN
               XX(IKG)=XXX(2)-XXX(1)
               YY(IKG)=YYY(2)-YYY(1)
               TEST=2.0*RAYRE(IG)
               IF(SQRT(XX(IKG)**2+YY(IKG)**2).LE.TEST) GO TO 520
            ELSE
               XX(IKG)=SIDE
               YY(IKG)=0.0
               IF(SIDE.LE.RAYRE(IG)) GO TO 520
            ENDIF
            NMC(IKG+1)=IG
   70       CONTINUE
*           COMPUTE THE MIXTURE NUMBERS IN THE MERGED CELLS.
            NMBLK=0
            DO 90 IKK=1,NMERGE
            IKG=IGEN(IKK)
            I1=NMC(IKG)
            I2=NMC(IKG+1)-I1
            DO 80 I=1,I2
   80       MAT(NMBLK+I)=MILIE(I1+I)
   90       NMBLK=NMBLK+I2
         ELSE IF(IHEX.EQ.0) THEN
*           PURE CARTESIAN GEOMETRY.
            CALL LCMGET(IPGEOM,'MIX',MILIEU)
            DO 100 I=1,NMCEL
  100       MAT(INUM(I))=MILIEU(I)
            CALL LCMGET(IPGEOM,'MESHX',XXX)
            IF(LL1.OR.LL2) THEN
               CALL LCMGET(IPGEOM,'MESHX',YYY)
            ELSE
               CALL LCMGET(IPGEOM,'MESHY',YYY)
            ENDIF
            NGEN=0
            IBLK=0
            DO 120 K1=1,LY
            LXM=1
            LXP=LX
            IF(LL1) LXP=K1
            IF(LL2) LXM=K1
            DO 120 K2=LXM,LXP
            IBLK=IBLK+1
            IF(MAT(INUM(IBLK)).EQ.0) GO TO 120
            IKK=INUM(IBLK)
            NMERGE=MAX(NMERGE,IKK)
            A=XXX(K2+1)-XXX(K2)
            B=YYY(K1+1)-YYY(K1)
            DO 110 JBLK=1,IBLK-1
            JKG=IGEN(INUM(JBLK))
            IF(MAT(INUM(IBLK)).EQ.MAT(INUM(JBLK))) THEN
               IF((A.EQ.XX(JKG)).AND.(B.EQ.YY(JKG))) THEN
                  IGEN(IKK)=JKG
                  GO TO 120
               ELSE IF((B.EQ.XX(JKG)).AND.(A.EQ.YY(JKG))) THEN
                  IGEN(IKK)=JKG
                  IORI(IBLK)=2
                  GO TO 120
               ENDIF
            ENDIF
  110       CONTINUE
            NGEN=NGEN+1
            IGEN(IKK)=NGEN
            XX(NGEN)=A
            YY(NGEN)=B
            NMC(NGEN+1)=NMC(NGEN)+1
            RAYRE(NGEN)=0.0
  120       CONTINUE
            NMBLK=NMERGE
         ELSE IF(IHEX.GT.0) THEN
*           PURE HEXAGONAL GEOMETRY.
            CALL LCMGET(IPGEOM,'MIX',MILIEU)
            DO 125 I=1,NMCEL
  125       MAT(INUM(I))=MILIEU(I)
            CALL LCMGET(IPGEOM,'SIDE',SIDE)
            NGEN=0
            DO 140 IBLK=1,LX
            IF(MAT(INUM(IBLK)).EQ.0) GO TO 140
            IKK=INUM(IBLK)
            NMERGE=MAX(NMERGE,IKK)
            DO 130 JBLK=1,IBLK-1
            IF(MAT(INUM(IBLK)).EQ.MAT(INUM(JBLK))) THEN
               IGEN(IKK)=IGEN(INUM(JBLK))
               GO TO 140
            ENDIF
  130       CONTINUE
            NGEN=NGEN+1
            IGEN(IKK)=NGEN
            XX(NGEN)=SIDE
            YY(NGEN)=0.0
            NMC(NGEN+1)=NMC(NGEN)+1
            RAYRE(NGEN)=0.0
  140       CONTINUE
            NMBLK=NMERGE
         ENDIF
      ELSE IF((ISTATE(1).EQ.20).OR.(ISTATE(1).EQ.24)) THEN
         NZONE=ISTATE(2)+1
         NMCEL=1
         IGEN(1)=1
         INUM(1)=1
         IORI(1)=1
         CALL LCMLEN(IPGEOM,'SPLITR',ILENN,ITYLCM)
         IF(ILENN.GT.0) THEN
            CALL LCMGET(IPGEOM,'SPLITR',NBREG)
         ELSE
            DO 150 I=1,NZONE
  150       NBREG(I)=1
         ENDIF
         IF(NZONE.GT.1) CALL LCMGET(IPGEOM,'RADIUS',RAYON)
         CALL LCMGET(IPGEOM,'MIX',MILIEU)
         IF(IHEX.EQ.0) THEN
            CALL LCMGET(IPGEOM,'MESHX',XXX)
            CALL LCMGET(IPGEOM,'MESHY',YYY)
         ELSE
            CALL LCMGET(IPGEOM,'SIDE',SIDE)
         ENDIF
*
         RJ=0.0
         RAYRE(1)=0.0
         DO 170 I=1,NZONE-1
         IF(RAYON(I+1).LE.RJ) GO TO 520
         PAS=(RAYON(I+1)-RJ)/REAL(ABS(NBREG(I)))
         IF(NBREG(I).LT.0) PAS=PAS*(RAYON(I+1)+RJ)
         DO 160 J=1,ABS(NBREG(I))
         IG=IG+1
         MAT(IG)=MILIEU(I)
         IR=MAX(IR,MAT(IG))
         IF(NBREG(I).GT.0) THEN
            RJ=RJ+PAS
         ELSE
            RJ=SQRT(RJ*RJ+PAS)
         ENDIF
  160    RAYRE(IG+1)=RJ
  170    RJ=RAYON(I+1)
         IG=IG+1
         MAT(IG)=MILIEU(NZONE)
         IR=MAX(IR,MAT(IG))
         IF(IHEX.EQ.0) THEN
            XX(1)=XXX(2)-XXX(1)
            YY(1)=YYY(2)-YYY(1)
            TEST=2.0*RAYRE(IG)
            IF(SQRT(XX(1)**2+YY(1)**2).LE.TEST) GO TO 520
         ELSE
            XX(1)=SIDE
            YY(1)=0.0
            IF(SIDE.LE.RAYRE(IG)) GO TO 520
         ENDIF
         NMC(2)=IG
         NMBLK=IG
      ELSE
         CALL XABORT('READEU: INVALID PRIMARY GEOMETRY.')
      ENDIF
      IF(NSUPCE.NE.NMCEL) CALL XABORT('READEU: THE CALCULATED NUMBER O'
     1 //'F CELLS IS INCONSISTENT.')
*
      ILK=((NCODE(1).EQ.1).AND.(ZCODE(1).NE.1.0)).OR.
     1    ((NCODE(2).EQ.1).AND.(ZCODE(2).NE.1.0)).OR.
     2    ((NCODE(3).EQ.1).AND.(ZCODE(3).NE.1.0)).OR.
     3    ((NCODE(4).EQ.1).AND.(ZCODE(4).NE.1.0))
      IF(IMPX.GT.0) THEN
         IF(IHEX.EQ.0) THEN
            WRITE (6,'(/43H CARTESIAN MULTICELL OPTION (EURYDICE-2) BA,
     1      28HSED ON GEOMETRY LOCATED IN '',A12,2H''./)') GEONAM
         ELSE
            WRITE (6,'(/43H HEXAGONAL MULTICELL OPTION (EURYDICE-2) BA,
     1      28HSED ON GEOMETRY LOCATED IN '',A12,2H''./)') GEONAM
         ENDIF
         IF(IHEX.EQ.0) THEN
            WRITE (6,670) LX,LY,MAXPTS,NMBLK,IR
         ELSE
            WRITE (6,680) LX,MAXPTS,NMBLK,IR
         ENDIF
         WRITE (6,630)
         DO 180 IKG=1,NGEN
         IF((ISTATE(1).EQ.20).OR.(ISTATE(1).EQ.24)) THEN
            TEXT12=GEONAM
         ELSE IF(ISTATE(8).EQ.1) THEN
            WRITE (TEXT12(:4),'(A4)') CELL(3*(IKG-1)+1)
            WRITE (TEXT12(5:8),'(A4)') CELL(3*(IKG-1)+2)
            WRITE (TEXT12(9:),'(A4)') CELL(3*(IKG-1)+3)
         ELSE
            WRITE (TEXT12,'(4HCELL,I5)') IKG
         ENDIF
         I1=NMC(IKG)+1
         I2=NMC(IKG+1)
         IF(I1.EQ.I2) THEN
            IF(IHEX.EQ.0) THEN
               WRITE (6,660) IKG,TEXT12,XX(IKG),YY(IKG)
            ELSE
               WRITE (6,665) IKG,TEXT12,XX(IKG)
            ENDIF
         ELSE
            WRITE (6,640) IKG,TEXT12,(RAYRE(I),I=I1,I2)
            IF(IHEX.EQ.0) THEN
               WRITE (6,650) XX(IKG),YY(IKG)
            ELSE
               WRITE (6,655) XX(IKG)
            ENDIF
         ENDIF
  180    CONTINUE
         WRITE (6,'(/)')
         IF(.NOT.ILK) WRITE (6,'(17H INFINITE DOMAIN./)')
      ENDIF
      RETURN
*
  520 CALL XABORT('READEU: RADIUS ARE INCONSISTENTS.')
  550 CALL XABORT('READEU: A BOUNDARY CONDITION IS MISSING.')
  560 CALL XABORT('READEU: THE DIAGONAL CONDITIONS X+ DIAG Y- DIAG AND'
     1 //' X- DIAG Y+ DIAG ARE THE ONLY PERMITTED.')
*
  630 FORMAT (/5X,24HGENERATING CELL   RADIUS)
  640 FORMAT (/1X,I4,2H ',A12,3H'  ,1P,9E12.5/(22X,9E12.5))
  650 FORMAT (23X,3HA =,1P,E12.5,6H   B =,E12.5)
  655 FORMAT (23X,6HSIDE =,1P,E12.5)
  660 FORMAT (/1X,I4,2H ',A12,1H',3X,3HA =,1P,E12.5,6H   B =,E12.5)
  665 FORMAT (/1X,I4,2H ',A12,1H',3X,6HSIDE =,1P,E12.5)
  670 FORMAT (/35H NUMBER OF CELLS ALONG THE X-AXIS =,I4/17X,
     1 18HALONG THE Y-AXIS =,I4,5X,26HAVAILABLE STORAGE MAXPTS =,I4/
     2 27H NUMBER OF MERGED VOLUMES =,I5/
     3 39H NUMBER OF DISTINCT PHYSICAL MIXTURES =,I5/)
  680 FORMAT (/34H NUMBER OF HEXAGONS IN ONE PLANE =,I4,5X,9HAVAILABLE,
     1 17H STORAGE MAXPTS =,I4/27H NUMBER OF MERGED VOLUMES =,I5/
     2 39H NUMBER OF DISTINCT PHYSICAL MIXTURES =,I5/)
      END
