*DECK SADEDR
      SUBROUTINE SADEDR(IPDEPL,IPLIB ,IPRINT,INDREC,NBISO ,NGROUP,
     >                  NBMIX ,NREGIO,NTIME ,NPROC ,NDEPL ,NCOMB ,
     >                  NREGCE,ITYPE ,INR   ,ISAVE ,ITIXS ,RPAR  ,
     >                  XT    ,
     >                  ISONAM,ISONRF,AWRISO,ISONDP,TIMES ,MIX   ,
     >                  DEN   ,MAT   ,VOL   ,FLUXES,MASK  ,MASKL ,
     >                  JM    ,MILVO ,YDPL  ,FLX   ,VX    ,ISOCMB,
     >                  ADJSOU)
C
C-----------------------------------------------------------------------
C NOTES:       1) modified from EVODRV.f
C-------------- Original Version AUTHOR: A. HEBERT 22/05/90 ----
C-------------- Original Version MODIF : G.M.      97-02-13 ----
C
C
C
C  ISOTOPIC DEPLETION CALCULATION.
C
C  INPUT PARAMETERS:
C   IPDEPL  : POINTER TO THE DEPLETION HISTORY (L_BURNUP SIGNATURE).
C   IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C             (L_LIBRARY SIGNATURE).
C   IPRINT  : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C   INDREC  : =1: BEGINNING OF DEPLETION; =2: ELSE.
C   NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C   NGROUP  : NUMBER OF ENERGY GROUPS.
C   NBMIX   : NUMBER OF MIXTURES.
C   NREGIO  : NUMBER OF VOLUMES.
C   NTIME   : NUMBER OF TIME STEP SAVED + 1 .
C   NPROC   : NUMBER OF MICROSCOPIC XS TO PROCESS .
C   NDEPL   : NUMBER OF DEPLETING ISOTOPES.
C   NCOMB   : NUMBER OF DEPLETING MIXTURES.
C   NREGCE  : NUMBER OF DEPLETING + ENERGY PRODUCTION  MIXTURES.
C   ITYPE   : =1 FIFTH-ORDER RUNGE-KUTTA METHOD.
C             =2 FOURTH-ORDER KAPS-RENTROP METHOD.
C   INR     : =0: OUT-OF-CORE DEPLETION.
C             =1: CONSTANT FLUX DEPLETION.
C             =2: CONSTANT POWER DEPLETION.
C   ISAVE   : =-1: DO NOT SAVE LAST FLUX CALCULATION IN THE DEPLETION
C                  TABLE.
C             .GE.0: SAVE LAST FLUX CALCULATION IN THE DEPLETION
C                    TABLE AT TIME XTI.
C   ITIXS   : FLAG FOR TIME DEPENDENT XS (=0 ON/=1 OFF)
C   RPAR    : BURNUP PARAMETERS                           R(5)
C             RPAR(1) = EPS1   ACCURACY FOR ODE SOLVER.
C             RPAR(2) = EPS2   ACCURACY FOR CONSTANT
C                              POWER  ITERATIONS.
C             RPAR(3) = EXPMAX ISOTOPE SATURATING LIMIT.
C             RPAR(4) = H1     GUESSED FIRST STEPSIZE.
C             RPAR(5) = FIT    FLUX (N/CMC C 2/S) OR
C                              POWER (kW/kG INITIAL HEAVY)
C                              NORMALIZATIO FACTOR.
C   XT      : TIME TABLE                                  R(5)
C             XT(1)   = INITIAL TIME FOR DEPLETION
C             XT(2)   = FINAL TIME FOR DEPLETION
C             XT(3)   = TIME FOR SAVE
C             XT(4)   = LAST SET TIME
C             XT(5)   = TIME FOR CURRENT SET
C   ISONAM  : ALIAS NAMES OF ISOTOPES. ISONAM(3,NBISO)
C   ISONRF  : REFERENCE NAMES OF ISOTOPES. ISONRF(3,NBISO))
C   AWRISO  : ATOMIC WEIGHT ISOTOPES. AWRISO(NBISO)
C   ISONDP  : NAMES OF DEPLETING ISOTOPES. ISONDP(2,NDEPL)
C   TIMES   : TIME STEP SAVED. TIMES(NTIME+2)
C   MIX     : MIX NUMBER OF EACH ISOTOPE.  MIX(NBISO)
C   DEN     : DENSITY OF EACH ISOTOPE. DEN(NBISO)
C   MAT     : MIXTURE TYPE ASSIGNED TO EACH REGION. MAT(NREGIO)
C   VOL     : VOLUMES. VOL(NREGIO)
C   FLUXES  : RESULTING AVERAGE FLUXES OBTAINED USING THE INPUT INTERNAL
C             LIBRARY. FLUXES(NREGIO,NGROUP)
C   MASK    : DIMENSION MASK(NBMIX)
C   MASKL   : DIMENSION MASKL(NGROUP)
C   JM      : DIMENSION JM((NDEPL,NREGCE)
C   MILVO   : DIMENSION MILVO(NREGCE)
C   YDPL    : DIMENSION YDPL(NDEPL+1,2,NREGCE)
C   FLX     : DIMENSION FLX(NGROUP,2,NREGCE)
C   VX      : DIMENSION VX(NREGCE)
C   ISOCMB  : DIMENSION ISOCMB(NDEPLC NREGCE)
C
C
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE,ILCMUP,ILCMDN,MAXR,MXTIMS
      CHARACTER   NAMSBR*6
      REAL        CNVDAY,DELTM
      PARAMETER  (IOUT=6,NSTATE=40,ILCMUP=1,ILCMDN=2,
     >            MAXR=5,MXTIMS=2,CNVDAY=1.0/8.64E-4,
     >            DELTM=1.0E-4,NAMSBR='SADEDR')
C----
C FUNCTIONS
C----
      DOUBLE PRECISION XDRCST
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPDEPL,IPLIB,IPRINT,INDREC,NBISO,NGROUP,
     >            NBMIX,NREGIO,NTIME,NPROC,NDEPL,NCOMB,
     >            NREGCE,ITYPE,INR,ISAVE,ITIXS
      INTEGER     ISONAM(3,NBISO),ISONRF(3,NBISO),ISONDP(2,NBISO),
     >            MIX(NBISO),MAT(NREGIO),JM(NDEPL,NREGCE),
     >            MILVO(NREGCE),ISOCMB(NDEPL*NREGCE)
      LOGICAL     MASK(NBMIX),MASKL(NGROUP)
      REAL        RPAR(5),XT(5),AWRISO(NBISO),TIMES(NTIME+2),
     >            DEN(NBISO),VOL(NREGIO),FLUXES(NREGIO,NGROUP),
     >            YDPL(NDEPL+1,2,NREGCE),FLX(NGROUP,2,NREGCE),
     >            VX(NREGCE)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IDR,IRR,IKPAR,IBPAR,IYIELD,
     >            IMU1,IIMA,ISIG,ISFIS,ILP,IQF
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   TEXT12*12
      INTEGER     ISTATE(NSTATE),ITIM(MXTIMS),ILCMLN,ILCMTY,
     >            NDEPLD,NDFI,NDFP,NHEAVY,NLIGHT,NOTHER,NBXR,
     >            ICOMB,ISO,IBM,IDP,ICM,JCOMB,IT,IP,ICMB,
     >            IREG,IGR,IMIXI,IMIXC,ITC,NISOCC,ITSTMP
      REAL        EPS1,EPS2,EXPMAX,H1,FIT,XTI,VIOVT,
     >            XTF,FNORM(MXTIMS),DELTA(2),
     >            BRNWIR(2),TMPDAY(3) 
      INTEGER           ILNLCM,ITYLCM,IH
      REAL              VTOT,FUELDN(2)
      INTEGER           NBHEA
      DOUBLE PRECISION  FUELDD
      INTEGER           JDP
      DOUBLE PRECISION  AVCON
      DOUBLE PRECISION  ADJSOU(NDEPL,NGROUP,NCOMB)
C----
C  RECOVER DEPLETION CHAIN INFO FROM LCM.
C----
      AVCON=XDRCST('Neutron mass','amu')/
     >      (1.0D-24*XDRCST('Avogadro','N/moles'))
      EPS1=RPAR(1)
      EPS2=RPAR(2)
      EXPMAX=RPAR(3)
      H1=RPAR(4)
      FIT=RPAR(5)
      CALL LCMLEN(IPLIB,'DEPL-CHAIN',ILCMLN,ILCMTY)
      IF(ILCMLN.EQ.0) CALL XABORT(NAMSBR//
     >': DEPLETION CHAIN MISSING.')
      CALL LCMSIX(IPLIB,'DEPL-CHAIN',ILCMUP)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NDEPLD=ISTATE(1)
      NDFI=ISTATE(2)
      NDFP=ISTATE(3)
      NHEAVY=ISTATE(4)
      NLIGHT=ISTATE(5)
      NOTHER=ISTATE(6)
      NBXR=ISTATE(7)
      IF(NDEPL .NE. NDEPLD) CALL XABORT(NAMSBR//
     >': INCONSISTENT NDEPL. ')
      IF(NDEPLD .EQ. 0) CALL XABORT(NAMSBR//
     >': NO DEPLETING ISOTOPES')
      CALL SETARA(IBASE,MAXR*NDEPL,IDR)
      CALL SETARA(RBASE,MAXR*NDEPL,IRR)
      CALL SETARA(IBASE,NBXR*NDEPL,IKPAR)
      CALL SETARA(RBASE,NBXR*NDEPL,IBPAR)
      CALL SETARA(RBASE,NDFI*NDFP,IYIELD)
      CALL LCMGET(IPLIB,'ISOTOPESUSED',ISONDP)
      CALL LCMGET(IPLIB,'DEPLETE-REAC',IBASE(IDR))
      CALL LCMGET(IPLIB,'DEPLETE-RATE',RBASE(IRR))
      CALL LCMGET(IPLIB,'PRODUCE-REAC',IBASE(IKPAR))
      CALL LCMGET(IPLIB,'PRODUCE-RATE',RBASE(IBPAR))
      IF(NDFI .GT. 0 .AND. NDFP .GT. 0) THEN
        CALL LCMGET(IPLIB,'FISSIONYIELD',RBASE(IYIELD))
      ENDIF
      CALL LCMSIX(IPLIB,'DEPL-CHAIN',ILCMDN)
C----
C  LOCATE BURNUP REGIONS
C----
      CALL XDISET(JM,NDEPL*NREGCE,0)
      ICOMB=0
      DO 100 ISO=1,NBISO
        IBM=MIX(ISO)
        IF(IBM .GT. 0) THEN
          DO 110 IDP=1,NDEPL
            IF(ISONRF(1,ISO) .EQ. ISONDP(1,IDP).AND.
     >         ISONRF(2,ISO) .EQ. ISONDP(2,IDP)) THEN
              DO 111 ICM=1,ICOMB
                IF(IBM .EQ. MILVO(ICM)) THEN
                  JCOMB=ICM
                  GO TO 115
                ENDIF
 111          CONTINUE
              ICOMB=ICOMB+1
              MILVO(ICOMB)=IBM
              JCOMB=ICOMB
 115          CONTINUE
              JM(IDP,JCOMB)=ISO
            ENDIF
 110      CONTINUE
        ENDIF
 105    CONTINUE
 100  CONTINUE
      IF(ICOMB .NE. NCOMB) CALL XABORT(NAMSBR//
     >': INVALID VALUE OF NCOMB.') 
      CALL XDRSET(VX,NREGCE,0.0)
      DO 120 IREG=1,NREGIO
        DO 121 ICOMB=1,NCOMB 
          IF(MAT(IREG) .EQ. MILVO(ICOMB)) VX(ICOMB)=VX(ICOMB)+VOL(IREG)
 121    CONTINUE
 120  CONTINUE
C----
C  SAVE THE LAST FLUX CALCULATION IN THE DEPLETION TABLE.
C----
      XTI=XT(3)
      XTF=XT(5)
      IF(ISAVE .GE. 0) THEN
        ITIM(1)=0
        IF(NTIME .GT. 0) THEN
          DO 130 IT=1,NTIME
            IF(ABS(TIMES(IT)-XTI) .LE. DELTM*XTI) THEN
              ITIM(1)=IT
              GO TO 135
            ENDIF
 130      CONTINUE
          IF(XT(1) .LT. TIMES(NTIME)) CALL XABORT(NAMSBR//
     >    ': INVALID INITIAL TIME')
        ENDIF
        ITIM(1)=NTIME+1
        TIMES(NTIME+1)=XTI
 135    CONTINUE
        WRITE(TEXT12,'(8HDEPL-DAT,I4)') ITIM(1)
        CALL LCMSIX(IPDEPL,TEXT12,ILCMUP)
        CALL LCMPUT(IPDEPL,'ISOTOPESDENS',NBISO,2,DEN)
        CALL LCMPUT(IPDEPL,'FLXDIR',NREGIO*NGROUP,2,FLUXES)
        IF(INDREC .EQ. 1) THEN
          BRNWIR(1)=0.0
          BRNWIR(2)=0.0
          CALL LCMPUT(IPDEPL,'BURNUP-IRRAD',2,2,BRNWIR)
        ENDIF
        CALL LCMSIX(IPDEPL,TEXT12,ILCMDN)
      ENDIF
      IF(ITIM(1) .EQ. NTIME+1) NTIME=ITIM(1)
C----
C  PERFORM A DEPLETION CALCULATION BETWEEN TIMES XT(1) AND XT(2).
C----
        CALL XDRSET(FLX,2*NREGCE*NGROUP,0.0)
        CALL XDRSET(YDPL,2*NREGCE*(NDEPL+1),0.0)
        DO 140 IP=1,2
          FNORM(IP)=1.0
          ITIM(IP)=0
          DO 141 IT=1,NTIME
            IF(ABS(TIMES(IT)-XT(IP)) .LE. DELTM*XT(IP)) THEN
              ITIM(IP)=IT
              WRITE(TEXT12,'(8HDEPL-DAT,I4)') ITIM(IP)
              IF(IPRINT .GE. 5)
     >          WRITE(IOUT,5010) NAMSBR,XT(IP),XT(IP)*CNVDAY,TEXT12
              CALL LCMSIX(IPDEPL,TEXT12,ILCMUP)
              CALL LCMGET(IPDEPL,'ISOTOPESDENS',DEN)
              IF(IP .EQ. 1) THEN
                CALL LCMGET(IPDEPL,'BURNUP-IRRAD',BRNWIR)
              ENDIF
              CALL LCMLEN(IPDEPL,'FLXDIR',ILCMLN,ILCMTY)
              IF((INR .NE. 0) .AND.
     >           (ILCMLN .GT. 0 .OR. IP.EQ.1)) THEN
                CALL LCMGET(IPDEPL,'FLXDIR',FLUXES)
              ENDIF
              CALL LCMSIX(IPDEPL,TEXT12,ILCMDN)
              GO TO 145
            ENDIF
 141      CONTINUE
          IF(IP .EQ. 1)
     >      CALL XABORT(NAMSBR//': NO DEPLETION DATA STORED.')
          IF(IPRINT .GE. 5) WRITE(IOUT,6003) NAMSBR
 145      CONTINUE
          DO 150 ICMB=1,NREGCE
            IF(VX(ICMB) .NE. 0.0) THEN
              DO 151 IREG=1,NREGIO
                IF(MAT(IREG) .EQ. MILVO(ICMB)) THEN
                  VIOVT=VOL(IREG)/VX(ICMB)
                  DO 152 IGR=1,NGROUP
                    FLX(IGR,IP,ICMB)=FLX(IGR,IP,ICMB)+VIOVT*
     >                              FLUXES(IREG,IGR)
 152              CONTINUE
                ENDIF
 151          CONTINUE
            ENDIF
            DO 153 IDP=1,NDEPL
              IF(JM(IDP,ICMB) .GT. 0) THEN
                YDPL(IDP,IP,ICMB)=DEN(JM(IDP,ICMB))
              ENDIF
 153        CONTINUE
 150      CONTINUE
 140    CONTINUE
        CALL SETARA(IBASE,NDEPL+1,IMU1)
        CALL SETARA(IBASE,NDEPL+1,IIMA)
        CALL SETARA(RBASE,NDEPL*NREGCE*5*MXTIMS*NGROUP,ISIG)
        CALL SETARA(RBASE,NHEAVY*2,ISFIS)
        CALL SETARA(IBASE,NDEPL,ILP)
        CALL SETARA(RBASE,NHEAVY,IQF)
        CALL SADBLD(IPDEPL,IPLIB,IPRINT,INDREC,NBISO,NGROUP,NPROC,
     >              ISONAM,AWRISO,NCOMB,NDEPL,NDFI,NDFP,NHEAVY,
     >              NLIGHT,NOTHER,NBXR,EPS1,EPS2,EXPMAX,H1,ITYPE,
     >              INR,FIT,XT,ITIXS,DELTA,FNORM,JM,YDPL,FLX,VX,
     >              IBASE(IDR),RBASE(IRR),IBASE(IKPAR),RBASE(IBPAR),
     >              RBASE(IYIELD),IBASE(IMU1),IBASE(IIMA),
     >              RBASE(ISIG),RBASE(ISFIS),IBASE(ILP),RBASE(IQF),
     >              ADJSOU )
        CALL RLSARA(RBASE(IQF))
        CALL RLSARA(IBASE(ILP))
        CALL RLSARA(RBASE(ISFIS))
        CALL RLSARA(RBASE(ISIG))
        CALL RLSARA(IBASE(IIMA))
        CALL RLSARA(IBASE(IMU1))
C----
C  RELEASE THE ALLOCATED MEMORY.
C----
      CALL RLSARA(RBASE(IYIELD))
      CALL RLSARA(RBASE(IBPAR))
      CALL RLSARA(IBASE(IKPAR))
      CALL RLSARA(RBASE(IRR))
      CALL RLSARA(IBASE(IDR))
      RETURN
*
 5000 FORMAT(/1X,A6,': SAVE INFORMATION RELATED TO TIME',1P,E12.4,
     1 ' E+8 S (',E12.4,' DAY) ON LCM DIRECTORY NAMED ',A12/)
 5010 FORMAT(/1X,A6,': RECOVER INFORMATION RELATED TO TIME',1P,E12.4,
     1 ' E+8 S (',E12.4,' DAY) FROM LCM DIRECTORY NAMED ',A12/)
 6000 FORMAT(/1X,A6,': NUMBER DENSITIES PER ISOTOPE AT TIME',1P,
     1 E12.4,' E+8 S (',E12.4,' DAY)')
 6001 FORMAT(/' ISOTOPIC DENSITIES AFTER BURNUP FOR MIXTURE = ',I3)
 6002 FORMAT(1P,5(4X,2A4,':',E12.4))
 6003 FORMAT(/1X,A6,': USE BEGINNING-OF-STAGE FLUX DISTRIBUTION',
     >        ' AT END-OF-STAGE.')
 6004 FORMAT(/' ISOTOPIC DENSITIES BEFORE BURNUP FOR MIXTURE = ',I3)
 6005 FORMAT(/' Isotopes in fuel = ',I3)
 6006 FORMAT(4X,2I10,4X,2A4,1P,E12.4)
 6007 FORMAT(4X,I10,'No isotope provided')
 6100 FORMAT(1X,A6,': NO BURNUP CALCULATION PERFORMED'/1P,
     >       '    RECOVER TIME =',E14.6,' DAYS')
 6101 FORMAT(1X,A6,': NO BURNUP CALCULATION PERFORMED'/1P,
     >       '    SAVE TIME    =',E14.6,' DAYS'/
     >       '    RECOVER TIME =',E14.6,' DAYS')
 6102 FORMAT(1X,A6,': BURNUP CALCULATION WILL BE PERFORMED'/1P,
     >       '    RECOVER TIME =',E14.6,' DAYS')
 6103 FORMAT(1X,A6,': BURNUP CALCULATION WILL BE PERFORMED'/1P,
     >       '    SAVE TIME    =',E14.6,' DAYS'/
     >       '    RECOVER TIME =',E14.6,' DAYS')
 6200 FORMAT(' -> INITIAL AND FINAL TIME STEP FOR BURNUP',1P/
     >       '    INITIAL TIME =',E14.6,' DAYS'/
     >       '    FINAL TIME   =',E14.6,' DAYS')
 6201 FORMAT(' -> BURNUP FLUX',1P/
     >       '    FLUX         =',E14.6,' N/CM**2/S ')
 6202 FORMAT(' -> BURNUP POWER',1P/
     >       '    POWER        =',E14.6,' MWatts/Tonne (kw/kg)')
 6203 FORMAT(' -> SOLUTION METHOD IS FIFTH-ORDER CASH-KARK'/
     >       '    NUMBER OF DEPLETING MIXTURES = ',I14/1P,
     >       '    ACCURACY FOR ODE SOLVER      = ',E14.6/
     >       '    ACCURACY FOR POWER ITERATIONS= ',E14.6/
     >       '    ISOTOPE SATURATING LIMIT     = ',E14.6)
 6204 FORMAT(' -> SOLUTION METHOD IS FOURTH-ORDER KAPS-RENTROP'/
     >       '    NUMBER OF DEPLETING MIXTURES = ',I14/1P,
     >       '    ACCURACY FOR ODE SOLVER      = ',E14.6/
     >       '    ACCURACY FOR POWER ITERATIONS= ',E14.6/
     >       '    ISOTOPE SATURATING LIMIT     = ',E14.6)
 6300 FORMAT(' ******    WARNING ',A6,' ******'/
     >       ' MIXTURE = ',I3,'  ISOTOPE = ',2A4,5X,
     >       ' NEGATIVE DENSITIES AFTER BURNUP = ',1P,E12.4,
     >       ' RESET TO 0.0')
 6400 FORMAT(' -> FINAL BURNUP AT TIME  = ',1P,E14.6,' DAYS'/
     >       '    FUEL BURNUP           = ',E14.6,' MW*D/TON'/
     >       '    NEUTRON EXPOSURE      = ',E14.6,' N/KB')
      END
