*DECK SADFLU
      SUBROUTINE SADFLU(IPFLUX,IPSYS ,IPTMP ,IPMACR,IPTRK ,IPBTF ,
     >                  ITPIJ ,CXDOOR,COPTIO,IPRINT,LEAKSW,ITYPEC,
     >                  ISADJ ,ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,
     >                  NREGIO,NMERGE,NGCOND,NUNKNO,NIFISS,MXNL  ,
     >                  INITFL,B2    ,REFKEF,TITLE ,
     >                  NAMGPA,NAMGAS,
     >                  NNAM  ,INAMEA,IXSFL ,
     >                  FUNKNO,SUNKNO,MATCOD,VOLUME,
     >                  KEYFLX,IMERGE,IGCOND,
     >                  LMOC  ,NGSS  ,ITRANC,ITRB  ,NL    ,NBN    ,
     >                  NANIS ,LCACA ,LEXIN ,ILK   ,NALB  ,FLUXO  ,
     >                  ADJFLO,DIFFB1,VOLINV,XSOLD ,XSSIGT,XTINV  ,
     >                  XSSIGW,XSCHI ,XSNUF ,XSTK  ,XSCAT ,VSCAT  ,
     >                  ALBEDO,NJJ   ,IJJ   ,IPOS  ,RKEFF ,IKLIB  ,
     >                  LOADFX,LMODE ,FLUX)
*
*---------------------------  SADFLU  ---------------------------------
*
*   1- PROGRAMME STATISTICS:
*       NAME     : SADFLU
*       USE      : COMPUTE THE GENERALIZED ADJOINT FLUXES ASSOCIATED
*                  WITH THE CONDENSATION & HOMOGENIZATION SOURCES
*       AUTHOR   : T. Courau 2000/08/01
*                  M. Assawaroongruengchot 2007/01/01
*
*   2- ROUTINE PARAMETERS:
*    INPUT/OUTPUT:
*      B2      : INITIAL ESTIMATE OF THE BUCKLING FOR      R(4)
*                THE B2 SEARCH OR IMPOSED BUCKLING.
*      CXDOOR  : NAME OF DOOR FOR FLUX SOLUTION            C*12
*      COPTIO  : CHARACTER FOR OPTION TYPE                 C*4
*      EPSPAR  : ITERATION PRECISION                       R(NBEPS)
*                EPSINR=EPSPAR(1)
*                EPSOUT=EPSPAR(2)
*                EPSUNK=EPSPAR(3)
*                EPSGPA=EPSPAR(4)
*                CONGPA=EPSPAR(5)
*      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,
*                                                            NGROUP)
*      IPFLUX  : POINTER TO THE SOLUTION                   I
*      IPSYS   : POINTER TO THE PIJ                        I
*      IPTMP    : TEMPORARY POINTER TO THE GENERALIZED     I
*                 ADJOINT SOURCES
*      IPMACR  : POINTER TO THE MACROLIB                   I
*      IPTRK   : POINTER TO THE TRACKING                   I
*      IPBTF   : POINTER TO THE BINARY TRACKING FILE       I
*      ITPIJ   : TYPE OF CP AVAILABLE                      I
*                = 1 SCATT MOD PIJ (WIJ)
*                = 2 STAND. PIJ
*                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
*                = 4 STAND. PIJ+PIJK
*      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
*                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
*      ITYPEC  : TYPE OF FLUX EVALUATION                   I
*                = 0  SKIP THE FLUX CALCULATION
*                = 1  DIRECT PROBLEM FISSION SOURCES/
*                     K EFFECTIVE CONVERGENCE
*                = 2  DIRECT PROBLEM FISSION SOURCES/
*                     K EFFECTIVE CONVERGENCE/
*                     FIXED DB2 BUCKLING
*                = 3  DIRECT PROBLEM FISSION SOURCES/
*                     K EFFECTIVE CONVERGENCE/
*                     DB2 BUCKLING EVALUATION
*                = 4  DIRECT PROBLEM FISSION SOURCES/
*                     DB2 BUCKLING CONVERGENCE
*                = 5  DIRECT PROBLEM DB2 SOURCES
*                = 6  DIRECT PROBLEM FIXED SOURCES
*      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION       I
*                = 0 ONLY DIRECT FLUX
*                = 1 DIRECT AND PSEUDO ADJOINT FLUX
*                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
*                = 3 DIRECT, PSEUDO ADJOINT AND
*                    GENERALIZED PSEUDO-ADJOINT
*                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
*                    GENERALIZED PSEUDO-ADJOINT AND
*                    GENERALIZED ADJOINT
*      ILEAK   : METHOD USED TO INCLUDE LEAKAGE EFFECT     I
*                = 0 NO DB2 EFFECT
*                = 1 HOMOGENEOUS WITH SCATTERING MODIFIED
*                    CP MATRIX MULTIPLIED BY A PNL FACTOR
*                = 2 HOMOGENEOUS REDUCED CP MATRIX
*                    MULTIPLIED BY PNL FACTOR
*                    (DEFAULT FOR TYPE B AND L)
*                = 3 HOMOGENEOUS SIGS0-DB2 APPROXIMATION
*                = 4 HOMOGENEOUS ALBEDO APPROXIMATION
*                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
*                  AND FIXED B->5 OR
*                  SEARCH X->6 Y->7 Z->8 R->9 ALL->10
*      ICOPAR  : ITERATION PARAMETERS                      I(6)
*                IFRITR=ICOPAR(1)
*                IACITR=ICOPAR(2)
*                IREBAL=ICOPAR(3)
*                MAXINR=ICOPAR(4)
*                MAXOUT=ICOPAR(5)
*                ICONTM=ICOPAR(6)
*      INITFL  : FLUX INITIALIZATION OPTION                I
*                = 0 DEFAULT INITIALIZATION
*                = 1 TAKEN FROM FLXUNK
*                = 2 TAKEN FROM DATA STRUCTURE
*      ILK     : REBALANCE FLAG FOR MOC                    L
*      IGCOND  : LIMIT CONDENSED GROUPS                    I(NGCOND)
*      IMERGE  : MERGED REGIONS POSITION                   I(NREGIO)
*      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
*      LINIFX  : FLAG TO INITIALIZE FLUXES                 L
*      LEAKSW  : LEAKAGE SWITCH.                           L
*      LMOC    : FLAG TO USE MOC CALCULATION               L
*      LMODE   : CONTROL FLAG FOR CALCULATION MODE         L(20)
*      MATCOD  : MATERIAL CODE IN REGIONS.                 I(NREGIO)
*      MXNL    : MAXIMUM ANISOTROPY
*      NAMGPA  : NAME OF GENERALIZED PSEUDO-ADJOINT        C(2)*12
*      NAMGAS  : NAME OF GENERALIZED ADJOINT SOURCE        C*12
*      NNAM    : NUMBER OF STANDARD XS TREATED             I
*      IXSFL   : FLAG FOR THE XS TO BE TREATED             I(NNAM)
*      INAMEA  : NAME OF THE XS TO BE TREATED              I(2,NNAM)
*      NBMIX   : NUMBER OF MIXTURES.                       I
*      NGCOND  : NUMBER OF GROUPS CONDENSED                I
*      NGROUP  : NUMBER OF GROUPS.                         I
*      NIFISS  : NUMBER OF FISSILE ISOTOPES.               I
*      NMERGE  : NUMBER OF REGIONS MERGED                  I
*      NREGIO  : NUMBER OF REGIONS.                        I
*      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
*      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L        D
*      RKEFF   : REFERENCE KEFF VALUE FOR EIGENVALUE       R 
*                PROBLEM (ONLY FOR MOC)
*      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.             R(NUNKNO,
*                                                            NGROUP)
*      TITLE   : TITLE OF THE DRAGON CASE.                 C*72
*      VOLUME  : VOLUME OF REGIONS. DIMENSION              R(NREGIO)
*
*---------------------------   SADFLU  --------------------------------
*
      IMPLICIT    NONE
      INTEGER     IOUT,NBEPS,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NBEPS=5,ILCMUP=1,ILCMDN=2,NAMSBR='SADFLU')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPSYS,IPTMP,IPMACR,IPTRK,IPBTF,ITPIJ,IPRINT,
     >            ITYPEC,ISADJ,ILEAK,ICOPAR(6),NGROUP,NBMIX,NREGIO,
     >            NMERGE,NGCOND,NUNKNO,NIFISS,MXNL,INITFL,
     >            MATCOD(NREGIO),KEYFLX(NREGIO),IMERGE(NREGIO),
     >            IGCOND(NGCOND),IKLIB,NNAM
      INTEGER     INAMEA(2,NNAM),IXSFL(NNAM)
      CHARACTER   CXDOOR*12,COPTIO*4,TITLE*72,NAMGPA(2)*12,NAMGAS*12,
     >            NAMTMP*12
      LOGICAL     LEAKSW,LOADFX
      REAL        EPSPAR(NBEPS),B2(4),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),VOLUME(NREGIO)
      DOUBLE PRECISION REFKEF,FLUX(8*NUNKNO*NGROUP) 
      INTEGER     IFSUR
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ADJS
      INTEGER     ILOC
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12
      INTEGER     IPMAD,ITYCCT,INAM,IMRG,IGRC,ITYCC,IGRFIN,
     >            IGRDEB,IGROUP,IREG,IMIX,IL,JGROUP,JGRC,INITAD
      LOGICAL     LNILL,LMOD1O,LMOD4O
      INTEGER     NJJ(NBMIX),IJJ(NBMIX),IPOS(NBMIX)
      REAL        FLUXO(NUNKNO,NGROUP),ADJFLO(NUNKNO,NGROUP),
     >            RKEFF
      REAL        ZERO, ONE
      PARAMETER ( ZERO=0., ONE=1. )
      REAL      AIEPS,AEEPS  
      LOGICAL   LMOC,LDRFLX,LDAJCB,LEXIN,ILK,LMODE(20),LINIFX
      CHARACTER CDIRFL*12,CADJFL(2)*12,CFIXSR*12,CSTDAJ*12
      INTEGER   NGSS,ITRANC,ITRB,NL,NBN,NANIS,LCACA,NALB
      INTEGER   ITYLCM,ILNLCM,IFRITR,IACITR,NINTMX,NEXTMX,IMADJ,
     >          ITYMOC,IPTRK2,IFTRK2,NGSS2
      CHARACTER*12 NAMFLX
      INTEGER   IR,IG,ILCMLN,ILCMTY
      REAL        VOLINV(NREGIO),DIFFB1(NGROUP),XSOLD(0:NBMIX,NGROUP),
     >            XSSIGT(0:NBMIX,NGROUP),XTINV(0:NBMIX,NGROUP),
     >            XSSIGW(0:NBMIX,NGROUP),XSCHI(0:NBMIX,NIFISS,NGROUP),
     >            XSNUF(0:NBMIX,NIFISS,NGROUP),XSTK(NBMIX,NIFISS),
     >            XSCAT(NGROUP,0:NBMIX,0:NANIS,NGROUP),
     >            VSCAT(NREGIO*NGROUP*NGROUP),
     >            ALBEDO(NALB)
      CHARACTER   NAMREA*8
      INTEGER     IREAC
C     ---  INITIALIZE ---
      CFIXSR   ='FIXA        '    
      CDIRFL   ='FLUXDIRECT  '
      CSTDAJ   ='FLUXADJOINT '
      CADJFL(1)='            '
      CADJFL(2)='            '
      IFRITR = ICOPAR(1)
      IACITR = ICOPAR(2)
      NINTMX = ICOPAR(4)
      NEXTMX = ICOPAR(5)
      AIEPS  = EPSPAR(1)
      AEEPS  = EPSPAR(2)
      LINIFX = .TRUE.
      LMOD1O   = LMODE(1)
      LMOD4O   = LMODE(4)
      LMODE(4) = .FALSE.
      IPTRK2 = 0
      IFTRK2 = 0
      NGSS2  = 0
      RKEFF  = RKEFF
C-----
C GENERATE A TEMPORARY MACROLIB
C-----
      IFSUR=0
*  ORIGINAL COMMAND  CHANGE XSM TYPE TO LINKLIST TYPE
*      CALL LCMOP(IPMAD,'MACRO+SADJ',0,2,0)
      IF (IKLIB .GT. 0) THEN
C       --- IF L_LIBRARY SIGNATURE, USE LINKLIST FILE--
        CALL LCMOP(IPMAD,'MACRO+SADJ',0,1,0)      
      ELSE
C       --- IF L_MACROLIB SIGNATURE, USE XSM FILE --
        CALL LCMOP(IPMAD,'MACRO+SADJ',0,2,0)
      ENDIF
      CALL LCMEQU(IPMACR,IPMAD)
C-----
C GENERALIZED ADJOINT FLUX COMPUTATION
C----
      NAMGAS='TMPFIXG'
C     ---IF MOC MODE, USE STANDARD ADJOINT SOURCE DIRECTORY ---
      IF (LMOC)   NAMGAS = CFIXSR
C----
C SET FLUX PARAMETERS
C----
      INITAD=2
      COPTIO='B0  '
      B2(1)=0.0
      B2(2)=0.0
      B2(3)=0.0
      B2(4)=0.0
C----
C IF LEAKAGE IMPOSE FIXED DB2 COMPUTATION
C----
      IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5)THEN
        ITYCCT=2
      ELSE
        ITYCCT=ITYPEC
      ENDIF
C---
C LOOP OVER STANDARD CROSS SECTIONS
C---
      CALL SETARA(RBASE,(NBMIX+1)*NGROUP,ADJS)   
      DO 1 INAM=1,NNAM
       IF (INAM .EQ. 5) GO TO 1
       IF (IXSFL(INAM) .LT. 2) GO TO 1
       WRITE(NAMREA,'(2A4)') INAMEA(1,INAM),INAMEA(2,INAM)
       DO 10 IMRG=1,NMERGE
        IGRFIN=0
        DO 20 IGRC=1,NGCOND
         ITYCC=ITYCCT
         CALL XDRSET(RBASE(ADJS),(NBMIX+1)*NGROUP,0.0)   
         IGRDEB=IGRFIN+1
         IGRFIN=IGCOND(IGRC)
         DO IGROUP=IGRDEB,IGRFIN
C--
C SOURCE READ
C--
          ILOC=ADJS+(IGROUP-1)*(NBMIX+1)
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
          CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
          CALL LCMGET(IPTMP,NAMREA,RBASE(ILOC))
          CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
         ENDDO
C---
C CHECK IF THE SOURCE VANISHES (LNILL=.TRUE.)
C---
         LNILL=.TRUE.
         DO IGROUP=1,NGROUP
          ILOC=ADJS+(IGROUP-1)*(NBMIX+1)
          DO IMIX=0,NBMIX
           LNILL=LNILL.AND.RBASE(ILOC+IMIX).EQ.0.0
          ENDDO
         ENDDO
         IF (LNILL) THEN
           ITYCC=0
           GO TO 47
         ENDIF
C---
C SOURCE PRINTED ON THE TEMPORARY MACROLIB
C---
         DO IGROUP=1,NGROUP
          ILOC=ADJS+(IGROUP-1)*(NBMIX+1)
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
          CALL LCMSIX(IPMAD,CGRPNM,ILCMUP)
          CALL LCMPUT(IPMAD,NAMGAS,NBMIX+1,2,RBASE(ILOC))
          CALL LCMSIX(IPMAD,CGRPNM,ILCMDN)
         ENDDO
 47      CONTINUE
C---
C GENERALIZED ADJOINT FLUX COMPUTATION
C---
         WRITE(NAMGPA(1),'(A6,2I2)') NAMREA(1:6),IGRC,IMRG
         NAMTMP=NAMGPA(1)
         NAMGPA(2)='F'//NAMTMP(2:12)
         IF (IPRINT.GE.1) THEN
           WRITE(IOUT,6010) '  SOURCE'
           WRITE(IOUT,6014)
           WRITE(IOUT,6016) NAMGPA(1)
           IF (LNILL) WRITE(IOUT,6017)
         ENDIF
         IF (.NOT.LMOC) THEN
           CALL FLUDRV(IPFLUX,IPSYS ,IPMAD ,IPTRK ,IPBTF ,ITPIJ ,
     >                 CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >                 ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >                 NUNKNO,NIFISS,MXNL  ,INITAD,B2    ,REFKEF,
     >                 TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >                 FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
         ELSE
           CADJFL(1)=NAMGPA(1)
           CADJFL(2)=NAMGPA(2) 
           IMADJ  = 3
           ITYMOC = 1 
C---
C      IF GENERALIZED ADJOINT SOURCES VANISH
C---
           IF (LNILL) THEN
             ITYMOC = 0 
           ENDIF
C---
C      INITIALIZE GEN. ADJOINT FLUXES
C---
           IF(LOADFX) THEN 
             CALL LCMLEN(IPFLUX,CADJFL(1),ILCMLN,ILCMTY)
             IF (ILCMLN .NE. -1) THEN
               LINIFX = .TRUE.
             ELSE
               LINIFX = .FALSE.
             ENDIF
           ENDIF
           CALL XDRSET(ADJFLO,NGROUP*NREGIO,0.0)
           IF ((.NOT.LINIFX) .AND.IMADJ .EQ.3 .AND. 
     >          ITYMOC .EQ. 1) THEN
             CALL LCMSIX(IPFLUX,CSTDAJ,ILCMUP)
             DO 22 IG= 1, NGROUP
               WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
               CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
               IF(ILCMLN.EQ.NREGIO) THEN
                 CALL LCMGET(IPFLUX,NAMFLX,ADJFLO(1,IG))
               ELSE
                 CALL XABORT(NAMSBR//
     >           ': INVALID DATA LENGTH OF ADJFLUX IN IPFLUX')
               ENDIF
22           CONTINUE  
             CALL LCMSIX(IPFLUX,CSTDAJ,ILCMDN)
           ELSE
             CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
             DO 23 IG= 1, NGROUP
               WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
               CALL LCMPUT(IPFLUX,NAMFLX,NUNKNO,2,ADJFLO(1,IG) )
23           CONTINUE  
             CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN)
           ENDIF
           LDRFLX = .FALSE. 
           LDAJCB = .FALSE. 
           CALL MOCDRV(IPFLUX,IPTRK,IPMAD,IPBTF ,NGROUP,NBMIX,NIFISS,
     1     NANIS ,NL    ,NBN   ,NUNKNO,FLUXO ,DIFFB1,
     2     NINTMX,AIEPS ,NEXTMX,AEEPS ,IFRITR,IACITR,ITYMOC,ILEAK,
     3     NREGIO,MATCOD,VOLUME,VOLINV,XSOLD ,XSSIGT,XTINV,XSSIGW,
     4     XSCHI ,XSNUF ,XSTK  ,NJJ,IJJ,IPOS ,XSCAT ,VSCAT,
     5     NALB,ALBEDO,LEAKSW,ILK,ITRANC,ITRB,TITLE,LCACA ,NGSS,LEXIN,
     6     REFKEF,IPRINT,CDIRFL,ADJFLO,CADJFL,IMADJ,LDRFLX,LDAJCB,
     7     RKEFF ,LMODE ,IPTRK2,IFTRK2,NGSS2,FLUX)
         ENDIF
 20    CONTINUE
 10    CONTINUE
 1    CONTINUE
C---
C LOOP OVER SCATTERING CROSS SECTIONS
C---
      IF (IXSFL(5) .EQ. 2) THEN
       IL=1
      DO 105 JGRC=1,NGCOND
      DO 110 IMRG=1,NMERGE
          IGRFIN=0
      DO 120 IGRC=1,NGCOND
           ITYCC=ITYCCT
           CALL XDRSET(RBASE(ADJS),(NBMIX+1)*NGROUP,ADJS)   
           IGRDEB=IGRFIN+1
           IGRFIN=IGCOND(IGRC)
C--
C SOURCE READ
C--
           DO IGROUP=IGRDEB,IGRFIN
            ILOC=ADJS+(IGROUP-1)*(NBMIX+1)
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
            CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
            WRITE(NAMGPA(2),'(A4,2I2)')  'SCAD',IL-1,JGRC
            CALL LCMGET(IPTMP,NAMGPA(2),RBASE(ILOC))
            CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
           ENDDO
C---
C CHECK IF THE SOURCE VANISHES (LNILL=.TRUE.)
C---
           LNILL=.TRUE.
           DO IGROUP=1,NGROUP
            ILOC=ADJS+(IGROUP-1)*(NBMIX+1)
            DO IMIX=1,NBMIX
             LNILL=LNILL.AND.RBASE(ILOC+IMIX).EQ.0.0
            ENDDO
           ENDDO
           IF (LNILL) THEN
            ITYCC=0
            GO TO 147
           ENDIF
C---
C SOURCE PRINTED ON THE TEMPORARY MACROLIB
C---
           DO IGROUP=1,NGROUP
            ILOC=ADJS+(IGROUP-1)*(NBMIX+1)
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
            CALL LCMSIX(IPMAD,CGRPNM,ILCMUP)
            CALL LCMPUT(IPMAD,NAMGAS,NBMIX+1,2,RBASE(ILOC))
            CALL LCMSIX(IPMAD,CGRPNM,ILCMDN)
           ENDDO
 147       CONTINUE
C---
C GENERALIZED ADJOINT FLUX COMPUTATION
C---
           WRITE(NAMGPA(1),'(A4,4I2)')'SCAD',IL-1,JGRC,IGRC,IMRG
           NAMTMP=NAMGPA(1)
           NAMGPA(2)='F'//NAMTMP(2:12)
           IF (IPRINT.GE.1) THEN
            WRITE(IOUT,6010) '  SOURCE'
            WRITE(IOUT,6014)
            WRITE(IOUT,6016) NAMGPA(1)
            IF (LNILL) WRITE(IOUT,6017)
           ENDIF
           IF (.NOT.LMOC) THEN
             CALL FLUDRV(IPFLUX,IPSYS ,IPMAD ,IPTRK ,IPBTF ,ITPIJ ,
     >                   CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >                   ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >                   NUNKNO,NIFISS,MXNL  ,INITAD,B2    ,REFKEF,
     >                   TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >                   FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)

           ELSE
             CADJFL(1)=NAMGPA(1)
             CADJFL(2)=NAMGPA(2)
C---
C  SET TYPE OF MOC ADJOINT CALCULATION
C  SOLVING GEN. ADJOINT PROBLEM
C  IF GENERALIZED ADJOINT SOURCES VANISH
C---
             IMADJ  = 3
             ITYMOC = 1 
             IF (LNILL) THEN
               ITYMOC = 0 
             ENDIF
C---
C       INITIALIZE GEN. ADJOINT FLUXES
C       AND VERIFY THE CADJFL(1) DIRECTORY
C---
             IF(LOADFX) THEN 
               CALL LCMLEN(IPFLUX,CADJFL(1),ILCMLN,ILCMTY)
               IF (ILCMLN .NE. -1) THEN
                 LINIFX = .TRUE.
               ELSE
                 LINIFX = .FALSE.
               ENDIF
             ENDIF
             CALL XDRSET(ADJFLO,NGROUP*NREGIO,0.0)
             IF ( (.NOT.LINIFX) .AND.IMADJ .EQ.3 .AND. 
     >              ITYMOC .EQ. 1) THEN
               CALL LCMSIX(IPFLUX,CSTDAJ,ILCMUP)
               DO 24 IG= 1, NGROUP
                 WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
                 CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
                 IF(ILCMLN.NE.NREGIO) CALL XABORT(NAMSBR//
     >             ': INVALID DATA LENGTH OF ADJFLUX IN IPFLUX')
                 CALL LCMGET(IPFLUX,NAMFLX,ADJFLO(1,IG))
24             CONTINUE  
               CALL LCMSIX(IPFLUX,CSTDAJ,ILCMDN)
             ELSE
               CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
               DO 25 IG= 1, NGROUP
                 WRITE(NAMFLX,'(4HFLUX,I3,1X)') IG
                 CALL LCMPUT(IPFLUX,NAMFLX,NUNKNO,2,ADJFLO(1,IG) )
25             CONTINUE  
               CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN)
             ENDIF
             LDRFLX = .FALSE. 
             LDAJCB = .FALSE. 
             CALL MOCDRV(IPFLUX,IPTRK,IPMAD,IPBTF ,NGROUP,NBMIX,NIFISS,
     1       NANIS,NL,NBN ,NUNKNO,FLUXO ,DIFFB1,
     2       NINTMX,AIEPS ,NEXTMX,AEEPS ,IFRITR,IACITR,ITYMOC,ILEAK,
     3       NREGIO,MATCOD,VOLUME,VOLINV,XSOLD,XSSIGT,XTINV,XSSIGW,
     4       XSCHI ,XSNUF ,XSTK,NJJ,IJJ ,IPOS,XSCAT,VSCAT,
     5       NALB  ,ALBEDO,LEAKSW,ILK,ITRANC,ITRB,TITLE,LCACA,NGSS,
     6       LEXIN ,REFKEF,IPRINT,CDIRFL,ADJFLO,CADJFL,IMADJ,LDRFLX,
     7       LDAJCB,RKEFF ,LMODE ,IPTRK2,IFTRK2,NGSS2,FLUX)
           ENDIF
 120  CONTINUE
 110  CONTINUE
 105  CONTINUE
      ENDIF
      CALL RLSARA(RBASE(ADJS))   
      CALL LCMCL(IPMAD,2)
      LMODE(1) = LMOD1O  
      LMODE(4) = LMOD4O
      RETURN
 6010 FORMAT(////1X,'SOLUTION TO TRANSPORT EQUATION ',/
     > ' CALCULATION TYPE            =',2X,A8)
 6014 FORMAT(
     > ' PROBLEM TYPE                =   GEN-ADJ')
 6016 FORMAT(
     > ' SOURCE TYPE                 =',A12)
 6017 FORMAT(1X,'*** WARNING *** GENERALIZED ADJOINT SOURCE',
     >  ' VANISHES -> NO FLUX COMPUTATION')
      END