*DECK SADINI
      SUBROUTINE SADINI(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >                  CXDOOR,COPTIO,IPRINT,INITFL,LEAKSW,ITYPEC,
     >                  ISADJ ,ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,
     >                  NREGIO,NUNKNO,NIFISS,MXNL  ,B2    ,REFKEF,
     >                  TITLE ,NAMGPA,NAMGAS,
     >                  FUNKNO,SUNKNO,MATCOD,
     >                  VOLUME,KEYFLX,
     >                  LMOC  ,NGSS  ,ITRANC,ITRB  ,NL    ,NBN    ,
     >                  NANIS ,LCACA ,LEXIN ,ILK   ,NALB  ,FLUXO  ,
     >                  ADJFLO,DIFFB1,VOLINV,XSOLD ,XSSIGT,XTINV  ,
     >                  XSSIGW,XSCHI ,XSNUF ,XSTK  ,XSCAT ,VSCAT  ,
     >                  ALBEDO,NJJ   ,IJJ   ,IPOS  ,RKEFF ,LOADFX ,
     >                  LMODE ,FLUX)
*
*---------------------------  SADINI  ---------------------------------
*
*   1- PROGRAMME STATISTICS:
*       NAME     : SADINI
*       USE      : COMPUTE DIRECT AND ADJOINT FLUX
*       AUTHOR   : T. Courau 2000/08/01
*       MODIFIED : M. Assawaroongruengchot 2007/01/01
*
*   2- ROUTINE PARAMETERS:
*    INPUT:
*      B2      : INITIAL ESTIMATE OF THE BUCKLING FOR      R(4)
*                THE B2 SEARCH OR IMPOSED BUCKLING.
*      COPTIO  : LEAKAGE TYPE                              C*4
*                'LKRD' (IMPOSED LEAKAGE ), 'B0' (B-0),
*                'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
*                'B0TR' (B-0 WITH TRANSPORT CORRECTION).
*                COPTIO='B0' IS DEFAULT FOR TYPE B AND L
*      CXDOOR  : NAME OF DOOR FOR FLUX SOLUTION            C*12
*      EPSPAR  : ITERATION PRECISION                       R(NBEPS)
*                EPSINR=EPSPAR(1)
*                EPSOUT=EPSPAR(2)
*                EPSUNK=EPSPAR(3)
*                EPSGPA=EPSPAR(4)
*                CONGPA=EPSPAR(5)
*      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,
*                                                            NGROUP)
*      IPFLUX  : POINTER TO THE SOLUTION                   I
*      IPSYS   : POINTER TO THE PIJ                        I
*      IPMACR  : POINTER TO THE MACROLIB                   I
*      IPTRK   : POINTER TO THE TRACKING                   I
*      IPBTF   : POINTER TO THE BINARY TRACKING FILE       I
*      ITPIJ   : TYPE OF CP AVAILABLE                      I
*                = 1 SCATT MOD PIJ (WIJ)
*                = 2 STAND. PIJ
*                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
*                = 4 STAND. PIJ+PIJK
*      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
*                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
*      INITFL  : FLUX INITIALIZATION OPTION                I
*                = 0 DEFAULT INITIALIZATION
*      ITYPEC  : TYPE OF FLUX EVALUATION                   I
*                = 0  SKIP THE FLUX CALCULATION
*                = 1  DIRECT PROBLEM FISSION SOURCES/
*                     K EFFECTIVE CONVERGENCE
*                = 2  DIRECT PROBLEM FISSION SOURCES/
*                     K EFFECTIVE CONVERGENCE/
*                     FIXED DB2 BUCKLING
*                = 3  DIRECT PROBLEM FISSION SOURCES/
*                     K EFFECTIVE CONVERGENCE/
*                     DB2 BUCKLING EVALUATION
*                = 4  DIRECT PROBLEM FISSION SOURCES/
*                     DB2 BUCKLING CONVERGENCE
*                = 5  DIRECT PROBLEM DB2 SOURCES
*                = 6  DIRECT PROBLEM FIXED SOURCES
*      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION       I
*                = 0 ONLY DIRECT FLUX
*                = 1 DIRECT AND PSEUDO ADJOINT FLUX
*                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
*                = 3 DIRECT, PSEUDO ADJOINT AND
*                    GENERALIZED PSEUDO-ADJOINT
*                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
*                    GENERALIZED PSEUDO-ADJOINT AND
*                    GENERALIZED ADJOINT
*      ILEAK   : METHOD USED TO INCLUDE LEAKAGE EFFECT     I
*                = 0 NO DB2 EFFECT
*                = 1 HOMOGENEOUS WITH SCATTERING MODIFIED
*                    CP MATRIX MULTIPLIED BY A PNL FACTOR
*                = 2 HOMOGENEOUS REDUCED CP MATRIX
*                    MULTIPLIED BY PNL FACTOR
*                    (DEFAULT FOR TYPE B AND L)
*                = 3 HOMOGENEOUS SIGS0-DB2 APPROXIMATION
*                = 4 HOMOGENEOUS ALBEDO APPROXIMATION
*                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
*                  AND FIXED B->5 OR
*                  SEARCH X->6 Y->7 Z->8 R->9 ALL->10
*      ICOPAR  : ITERATION PARAMETERS                      I(6)
*                IFRITR=ICOPAR(1)
*                IACITR=ICOPAR(2)
*                IREBAL=ICOPAR(3)
*                MAXINR=ICOPAR(4)
*                MAXOUT=ICOPAR(5)
*                ICONTM=ICOPAR(6)
*      ILK     : REBALANCE FLAG FOR MOCC                   L
*      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
*      LEAKSW  : LEAKAGE SWITCH.                           L
*      LINIFX  : FLAG TO INITIALIZE FLUXES                 L
*      LMOC    : FLAG TO USE MOC CALCULATION               L
*      LMODE   : CONTROL FLAG FOR CALCULATION MODE         L(20)
*      MATCOD  : MATERIAL CODE IN REGIONS.                 I(NREGIO)
*      MXNL    : MAXIMUM ANISOTROPY                        I
*      NAMGPA  : NAME OF GENERALIZED PSEUDO-ADJOINT        C(2)*12
*      NAMGAS  : NAME OF GENERALIZED ADJOINT SOURCE        C*12
*      NBMIX   : NUMBER OF MIXTURES.                       I
*      NGROUP  : NUMBER OF GROUPS.                         I
*      NIFISS  : NUMBER OF FISSILE ISOTOPES.               I
*      NREGIO  : NUMBER OF REGIONS.                        I
*      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
*      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L        D
*      RKEFF   : REFERENCE KEFF VALUE FOR EIGENVALUE       R 
*                PROBLEM 
*      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.             R(NUNKNO,
*                                                            NGROUP)
*      TITLE   : TITLE OF THE DRAGON CASE.                 C*72
*      VOLUME  : VOLUME OF REGIONS. DIMENSION              R(NREGIO)
*
*---------------------------   SADINI  --------------------------------
*
      IMPLICIT    NONE
      INTEGER     IOUT,NBEPS,NSTATE
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=40,NBEPS=5,NAMSBR='SADINI')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPSYS,IPMACR,IPTRK,IPBTF,ITPIJ,IPRINT,ITYPEC,
     >            ITYCC,ISADJ,ILEAK,NGROUP,NBMIX,NREGIO,NUNKNO,NIFISS,
     >            MXNL,INITFL,ISTATE(NSTATE),ISADT
      CHARACTER   CXDOOR*12,COPTIO*4,TITLE*72,NAMGPA(2)*12,NAMGAS*12
      INTEGER     ICOPAR(6),MATCOD(NREGIO),KEYFLX(NREGIO)
      LOGICAL     LEAKSW
      REAL        EPSPAR(NBEPS),B2(4),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),VOLUME(NREGIO)
      DOUBLE PRECISION REFKEF,FLUX(8*NUNKNO*NGROUP) 
      INTEGER     IFSUR,ILCMLN,ILCMTY
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ILCMUP,ILCMDN
      PARAMETER   (ILCMUP=1,ILCMDN=2)
      INTEGER     NJJ(NBMIX),IJJ(NBMIX),IPOS(NBMIX)
      REAL        FLUXO(NUNKNO,NGROUP)
      REAL        ADJFLO(NUNKNO,NGROUP)
      REAL        RKEFF,KEFDIR,KEFADJ
      REAL        ZERO, ONE
      PARAMETER ( ZERO=0., ONE=1. )
      REAL        AIEPS,AEEPS  
      LOGICAL     LMOC,LDRFLX,LDAJCB,LEXIN,ILK,ILKO,LOADFX,LINIFX,
     >            LMODE(20),LMOD4O
      CHARACTER   CDIRFL*12,CADJFL(2)*12,CFIXSR*12
      INTEGER     NGSS,ITRANC,ITRB,NL,NBN,NANIS,LCACA,NALB,IMADJ
      INTEGER     ITYLCM,ILNLCM,IFRITR,IACITR,NINTMX,NEXTMX,
     >            IPTRK2,IFTRK2,NGSS2
      REAL        VOLINV(NREGIO),DIFFB1(NGROUP),XSOLD(0:NBMIX,NGROUP),
     >            XSSIGT(0:NBMIX,NGROUP),XTINV(0:NBMIX,NGROUP),
     >            XSSIGW(0:NBMIX,NGROUP),XSCHI(0:NBMIX,NIFISS,NGROUP),
     >            XSNUF(0:NBMIX,NIFISS,NGROUP),XSTK(NBMIX,NIFISS),
     >            XSCAT(NGROUP,0:NBMIX,0:NANIS,NGROUP),
     >            VSCAT(NREGIO*NGROUP*NGROUP),
     >            ALBEDO(NALB)
C-----
C DATA
C-----
      CHARACTER   HTYPE(6)*8
      DATA        HTYPE
     >           /'   K-INF','   K-EFF','   K-EFF',
     >            'BUCKLING',' LEAKAGE','  SOURCE'/
C     --- INITIALIZE ---
      CFIXSR   ='FIXA        '    
      CDIRFL   ='FLUXDIRECT  '
      CADJFL(1)='FLUXADJOINT '
      CADJFL(2)='FLUXADJ     '
      LINIFX = .TRUE.
      IFRITR = ICOPAR(1)
      IACITR = ICOPAR(2)
      NINTMX = ICOPAR(4)
      NEXTMX = ICOPAR(5)
      AIEPS  = EPSPAR(1)
      AEEPS  = EPSPAR(2)
      RKEFF  = ONE
      KEFDIR = ONE
      KEFADJ = ONE
      IPTRK2 = 0
      IFTRK2 = 0
      NGSS2 = 0
C---  ENABLE REBALANCE ACC. FOR DIRECT AND ADJOINT CALCULATION  ---
C     THE ORIGINAL REBAL STATUS IS RESTORED AT THE END OF SADINI---
C     *** ILK == LREBAL *** 
      ILKO = ILK
      ILK  = .TRUE.
      IF(IPRINT .GT. 1) THEN
        WRITE(IOUT,6000) NAMSBR
        WRITE(IOUT,6001) 
      ENDIF
C     ---BACKUP LMODE(4) AND DISABLE LMODE(4) DURING DIRECT/ADJ CAL.-
      LMOD4O   = LMODE(4)
      LMODE(4) = .FALSE.
C-----
C DIRECT FLUX COMPUTATION
C----
      IFSUR=0
      ITYCC=ITYPEC
      ISADT=ISADJ
      ISADJ=0
      IF (IPRINT .GE. 1) THEN
        WRITE(IOUT,6010) HTYPE(ITYCC)
        WRITE(IOUT,6011)
      ENDIF
      IF (.NOT.LMOC) THEN
        CALL FLUDRV(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >              CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >              ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,NIFISS,MXNL  ,INITFL,B2    ,REFKEF,
     >              TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >              FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
      ELSE
        LDRFLX = .TRUE. 
        LDAJCB = .TRUE.
         IF (ITYPEC .EQ. 1) THEN
           IMADJ = 1
         ELSE IF (ITYPEC .EQ. 6) THEN
           IMADJ = 2
         ELSE
           CALL XABORT('SADINI:INVALID TYPEC FOR ADJOINT CALCULATION')
         ENDIF
         IF(LOADFX) THEN 
           CALL LCMLEN(IPFLUX,CDIRFL,ILCMLN,ILCMTY)
           IF (ILCMLN .NE. -1) THEN
             LINIFX = .TRUE.
           ELSE
             LINIFX = .FALSE.
           ENDIF
         ENDIF
         IF (LINIFX) THEN
          CALL LCMSIX(IPFLUX,CDIRFL,1)
          CALL MOCGFL(IPFLUX,ITYCC,NGROUP,NUNKNO,FLUXO)
          CALL LCMSIX(IPFLUX,CDIRFL,2)
         ENDIF
         IF(LOADFX) THEN 
           CALL LCMLEN(IPFLUX,CADJFL(1),ILCMLN,ILCMTY)
           IF (ILCMLN .NE. -1) THEN
             LINIFX = .TRUE.
           ELSE
             LINIFX = .FALSE.
           ENDIF
         ENDIF
         IF(LINIFX) THEN
          CALL LCMSIX(IPFLUX,CADJFL(1),ILCMUP)
          CALL MOCAGF(IPFLUX,IMADJ,NGROUP,NUNKNO,ADJFLO)
          CALL LCMSIX(IPFLUX,CADJFL(1),ILCMDN)
         ENDIF
         CALL MOCDRV(IPFLUX,IPTRK,IPMACR,IPBTF ,NGROUP,NBMIX,NIFISS,
     1   NANIS,NL,NBN ,NUNKNO,FLUXO,DIFFB1,
     2   NINTMX,AIEPS ,NEXTMX,AEEPS,IFRITR,IACITR,ITYCC,ILEAK,
     3   NREGIO,MATCOD,VOLUME,VOLINV,XSOLD,XSSIGT,XTINV,XSSIGW,
     4   XSCHI ,XSNUF ,XSTK  ,NJJ,IJJ,IPOS,XSCAT ,VSCAT,
     5   NALB,ALBEDO,LEAKSW,ILK,ITRANC,ITRB,TITLE,LCACA,NGSS,LEXIN,
     6   REFKEF,IPRINT,CDIRFL,ADJFLO,CADJFL,IMADJ,LDRFLX,LDAJCB,RKEFF,
     7   LMODE,IPTRK2,IFTRK2,NGSS2,FLUX)
         KEFDIR = RKEFF
       ENDIF
C-----
C ADJOINT FLUX COMPUTATION
C----
      ISADJ=ISADT
      IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5) ITYCC=2
      IF (IPRINT .GE. 1) THEN
        WRITE(IOUT,6010) HTYPE(ITYCC)
        WRITE(IOUT,6014)
      ENDIF
      IF (.NOT.LMOC) THEN
        CALL FLUDRV(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >              CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >              ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,NIFISS,MXNL  ,INITFL,B2    ,REFKEF,
     >              TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >              FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
      ELSE
        RKEFF  = ONE     
        IF (ITYPEC .EQ. 1) THEN
          IMADJ = 1
        ELSE IF (ITYPEC .EQ. 6) THEN
          IMADJ = 2
        ELSE
          CALL XABORT('SADINI:INVALID TYPEC FOR ADJOINT CALCULATION')
        ENDIF
        LDRFLX = .FALSE. 
        LDAJCB = .FALSE. 
        CALL MOCDRV(IPFLUX,IPTRK,IPMACR,IPBTF ,NGROUP,NBMIX,NIFISS,
     1  NANIS,NL,NBN,NUNKNO,FLUXO,DIFFB1,
     2  NINTMX,AIEPS,NEXTMX,AEEPS,IFRITR,IACITR,ITYCC,ILEAK,
     3  NREGIO,MATCOD,VOLUME,VOLINV,
     4  XSOLD,XSSIGT,XTINV,XSSIGW,
     5  XSCHI,XSNUF,XSTK,
     6  NJJ,IJJ,IPOS,XSCAT,VSCAT,
     7  NALB,ALBEDO,LEAKSW,ILK,ITRANC,ITRB,TITLE,LCACA,NGSS,LEXIN,
     8  REFKEF,IPRINT,CDIRFL,
     9  ADJFLO,CADJFL,IMADJ,LDRFLX,LDAJCB,RKEFF,LMODE,
     9  IPTRK2,IFTRK2,NGSS2,FLUX)
        KEFADJ = RKEFF
      ENDIF
      RKEFF = KEFDIR
      ILK = ILKO
      LMODE(4) = LMOD4O
      CALL LCMPUT(IPFLUX,'EPS-CONVERGE',NBEPS,2,EPSPAR)
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=ITYPEC
      ISTATE(2)=ILEAK
      ISTATE(3)=ICOPAR(1)
      ISTATE(4)=ICOPAR(2)
      ISTATE(5)=ICOPAR(3)
      ISTATE(6)=ICOPAR(4)
      ISTATE(7)=ICOPAR(5)
      ISADJ=3
      ISTATE(8)=ISADJ
      ISTATE(9)=NGROUP
      ISTATE(10)=NUNKNO
      ISTATE(11)=ICOPAR(6)
      ISTATE(12)=IFSUR
      CALL LCMPUT(IPFLUX,'STATE-VECTOR',NSTATE,1,ISTATE)
      RETURN
C---
C FORMAT
C---
 6000 FORMAT(1X,'Output from ',A6)
 6001 FORMAT(10X,'IMPOSED REBALANCE FLAG FOR DIRECT',
     >           ' & ADJOINT CALCULATION')
 6010 FORMAT(////1X,'SOLUTION TO TRANSPORT EQUATION ',/
     > ' CALCULATION TYPE            =',2X,A8)
 6011 FORMAT(
     > ' PROBLEM TYPE                =   DIRECT')
 6014 FORMAT(
     > ' PROBLEM TYPE                =   ADJOINT')
      END
