*DECK SADRRE
      SUBROUTINE SADRRE(IPLIB,NBISO,NGROUP,ISONAM,NCOMB,NDEPL,NDFI,
     >                  NHEAVY,XT,ITIXS,JM,YDPL,LCOOL,FLX,VX,IDR,RER,
     >                  VTOT,SIG,SFIS,QF,IBLOC,NPROC,INDPRO,
     >                  ITYPRO,XSREC)
C
C-----------------------------------------------------------------------
C NOTES:       1) modified from EVORRE.f
C-------------- Original Version AUTHOR: G. MARLEAU 95/08/29 ----
C-------------- Original Version MODIF : G.M.       97-02-13 ----
C
C
C
C COMPUTE THE DEPLETION REACTION RATES, AND TOTAL FISSION PRODUCTION
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C  NBISO   : NUMBER OF ISOTOPES/MATERIALS INCLUDING NON-DEPLETING ONES.
C  NGROUP  : NUMBER OF ENERGY GROUPS.
C  ISONAM  : ALIAS NAMES OF ISOTOPES. ISONAM(3,NBISO)
C  NCOMB   : NUMBER OF DEPLETING MIXTURES.
C  NDEPL   : NUMBER OF DEPLETING NUCLIDES.
C  NDFI    : NUMBER OF DIRECT FISSILE ISOTOPES.
C  XT      : INITIAL AND FINAL TIME FOR THE DEPLETION. XT(2)
C  ITIXS   : FLAG FOR TIME DEPENDENT XS (=0 ON/=1 OFF)
C  JM      : POSITION IN HNAMIS LIST OF EACH DEPLETING NUCLIDE.
C            DIMENSION JM(NDEPL,NCOMB)
C  YDPL    : INITIAL/FINAL NUMBER DENSITY OF ISOTOPE IN THE DEPLETION
C            CHAIN. YDPL(NDEPL+1,2,ICMB) IS THE STAGE BURNUP INCREMENT
C            IN REGION ICMB. DIMENSION YDPL(NDEPL+1,2,NCOMB)
C  LCOOL   : ZERO FLUX BURNUP FLAG. TRUE IF OUT OF CORE BURNUP
C            OF ZERO FLUX
C            DIMENSION FLX(NGROUP,2,NCOMB)
C  FLX     : INITIAL/FINAL VOLUME AVERAGED NEUTRON FLUXES.
C            DIMENSION FLX(NGROUP,2,NCOMB)
C  VX      : VOLUMES OF THE DEPLITING MIXTURES. DIMENSION VX(NCOMB)
C  IDR     : REACTION IDENTIFIER. IDR(MAXR,NDEPL)
C  RER     : REACTION CONSTANT. RER(MAXR,NDEPL)
C  VTOT    : TOTAL VOLUME.
C  NPROC   : NUMBER OF CROSS SECTION TYPES
C OUTPUT PARAMETERS:
C  SIG     : REACTION RATES. SIG(NDEPL,5,2,NGROUP,NCOMB)
C  SFIS    : TOTAL FISSION PRODUCTION. SFIS(NHEAVY,2)
C  QF      : FISSION ENERGY QF(NHEAVY)
C TEMPORARY STORAGE:
C  IBLOC   : BLOC WHERE ISOTOPE IS LOCATED. IBLOC(2,NCOMB)
C  INDPRO  : VECTOR FOR CROSS SECTION TO PROCESS. INDPRO(NPROC)
C  ITYPRO  : VECTOR FOR CROSS SECTION PROCESSED. ITYPRO(NPROC)
C  XSREC   : CROSS SECTION RECORD. REAL XSREC(NGROUP,NPROC)
C
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER           MXTIMS,NBXSE,KFISSI,MAXR
      PARAMETER        (MXTIMS=2,NBXSE=5,KFISSI=6,MAXR=5)
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1))
C----
C VARIABLES (INTERFACE AND LOCAL)
C----
      INTEGER           NXSPER,NDFI,ITIXS,IPLIB,NBISO,NGROUP,
     >                  NPROC,NDEPL,NCOMB,NHEAVY,
     >                  LENGTH,ITYLCM,IP,ISO,NBLOC,IDECAY,ICMB,IDEP,ITC,
     >                  NF,IXSGET,IXSPER,IXSE,IS,KBLOC,IG
      INTEGER           ISONAM(3,NBISO),ITXSE(NBXSE),INDPRO(NPROC),
     >                  JM(NDEPL,NCOMB),IDR(MAXR,NDEPL),IBLOC(2,NCOMB),
     >                  ITYPRO(NPROC)
C
      REAL              TMPREF,XREF
      REAL              XT(2),YDPL(NDEPL+1,2,NCOMB),
     >                  FLX(NGROUP,2,NCOMB),VX(NCOMB),RER(MAXR,NDEPL),
     >                  VTOT,SIG(NDEPL,5,2,NGROUP,NCOMB),SFIS(NHEAVY,2),
     >                  QF(NHEAVY),XSREC(NGROUP,NPROC),DELTAT(2,MXTIMS),
     >                  TIMEP(2,3)
C
      CHARACTER         HNAMIS*12
      DOUBLE PRECISION  GAR
      LOGICAL           LCOOL
      INTEGER           IPRLOC
C----
C DATA
C----
      SAVE  ITXSE
      DATA  ITXSE      /0,4,7,9,10/
C----
C  CHECK IF PERTURBATION XS OR STANDARD XS
C----
      IPRLOC=0
      IF(.NOT.LCOOL) THEN
        NXSPER=1
        CALL LCMLEN(IPLIB,'TIMESPER',LENGTH,ITYLCM)
        IF(LENGTH.GE.2.AND.LENGTH.LE.6) THEN
          CALL LCMGET(IPLIB,'TIMESPER',TIMEP)
          DELTAT(1,1)=TIMEP(1,1)
          DELTAT(2,1)=TIMEP(2,1)
          TMPREF=DELTAT(1,1)
          NXSPER=2
          IF(ITIXS.EQ.0) THEN
            DO 100 IP=1,MXTIMS
              DELTAT(1,IP)=1.0
              DELTAT(2,IP)=XT(IP)/8.64E-4-TMPREF
 100        CONTINUE
          ELSE
            XREF=XT(1)/8.64E-4
            DO 101 IP=1,MXTIMS
              DELTAT(1,IP)=1.0
              DELTAT(2,IP)=XREF-TMPREF
 101        CONTINUE
          ENDIF
          CONTINUE
        ELSE
          DO 102 IP=1,MXTIMS
            DELTAT(1,IP)=1.0
            DELTAT(2,IP)=0.0
 102      CONTINUE
        ENDIF
      ENDIF
C----
C  SCAN ISOTOPES AND TEST IF THEY ARE REQUIRED
C----
      DO 110 ISO=1,NBISO
        NBLOC=0
        CALL XDISET(INDPRO,NPROC,0)
        IDECAY=0
        DO 120 ICMB=1,NCOMB
          DO 121 IDEP=1,NDEPL
            IF(ISO.EQ.JM(IDEP,ICMB)) THEN
C----
C  TEST FOR REACTIONS
C----
              IF(MOD(IDR(4,IDEP),10).GT.0) IDECAY=1
              IF(MOD(IDR(1,IDEP),10).GT.0.OR.
     >           MOD(IDR(2,IDEP),10).GT.0.OR.
     >           MOD(IDR(3,IDEP),10).GT.0) THEN
                 INDPRO(7)=1
                 INDPRO(9)=1
                 INDPRO(10)=1
              ENDIF
              IF(MOD(IDR(5,IDEP),10).EQ.KFISSI) INDPRO(4)=1
              NBLOC=NBLOC+1
              IBLOC(1,NBLOC)=IDEP
              IBLOC(2,NBLOC)=ICMB
              GO TO 125
            ENDIF
 121      CONTINUE
 125      CONTINUE
 120    CONTINUE
        IF(NBLOC.GT.0) THEN
          IF(.NOT.LCOOL) THEN
C----
C  FOR ISOTOPES REQUIRED READ MICROSCOPIC XS AND STORE IN SIG
C----
            WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
            CALL LCMSIX(IPLIB,HNAMIS,1)
            NF=1
            IXSGET=-1
            DO 130 IXSPER=1,NXSPER
              CALL XDRLGX(IPLIB,IXSGET,IPRLOC,IXSPER,NGROUP,
     >                    INDPRO,XSREC(1,1),ITYPRO)
              DO 140 IXSE=2,NBXSE
                IS=ITXSE(IXSE)
                IF(INDPRO(IS).EQ.1.AND.ITYPRO(IS).GT.0) THEN
                  DO 141 KBLOC=1,NBLOC
                    IDEP=IBLOC(1,KBLOC)
                    ICMB=IBLOC(2,KBLOC)
                    DO 142 IP=1,MXTIMS
                      GAR=0.0D0
                      DO 143 IG=1,NGROUP
                      SIG(IDEP,IXSE,IP,IG,ICMB)= XSREC(IG,IS)
     >                                           *YDPL(IDEP,IP,ICMB)
 143                  CONTINUE
 142                CONTINUE
 141              CONTINUE
                ENDIF
 140          CONTINUE
 130        CONTINUE
            CALL LCMSIX(IPLIB,' ',2)
          ENDIF
        ENDIF
 110  CONTINUE
      CALL XDRSET(SFIS,MXTIMS*NHEAVY,0.0)
      CALL XDRSET(QF,NHEAVY,0.0)
C----
C  RETURN
C----
      RETURN
      END
