*DECK SHELL1
      SUBROUTINE SHELL1 (NGPT,RAYRE,SIGT,PETIT,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
*   INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 1-D
*   SPHERE OR SPHERICAL SHELL (DP-0 SURFACE ANGULAR FLUX APPROXIMATION)
*
*   INPUT PARAMETERS.
*   NGPT    : NUMBER OF GAUSS-JACOBI INTEGRATION POINTS.
*   RAYRE   : RADIUS OF SHELL.DIMENSION RAYRE(2)
*   SIGT    : CROSS SECTION.
*   PETIT   : VOIDED BLOCK CRITERION.
*
*   OUTPUT PARAMETERS.
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES.
*             DIMENSION PVS(1) (SPHERE) OR PVS(2) (SHELL)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             1 TO 1, 2 TO 1, 1 TO 2, 2 TO 2 WHERE 1 IS THE INTERNAL
*             SURFACE. DIMENSION PSS(1) (SPHERE) OR PSS(4) (SHELL)
*
*------------------------------------------------------ G. MARLEAU -----
*
      PARAMETER (PI=3.1415926535,MGPT=64)
      LOGICAL FILLED
      DIMENSION RAYRE(2),PVS(*),PSS(*),SEGPT(2*MGPT),SEGWT(2*MGPT),
     >          ZJKSI(MGPT),WJKSI(MGPT)
*
      CALL ALGJP(NGPT,ZJKSI,WJKSI)
      RAI=RAYRE(1)
      RAA=RAYRE(2)
      SS1=RAI/RAA
      SS2=SS1*SS1
      SS3=SS2*SS1
      FILLED=(RAI.EQ.0.0)
*
      IF (FILLED) THEN
         IOS=1
         DO 110 J=1,NGPT
         SEGPT(IOS)=2.0*RAA*ZJKSI(J)
         SEGWT(IOS)=2.0*WJKSI(J)
  110    IOS=IOS+1
      ELSE
         FP2=1.0-SS2
         SP2=SQRT(FP2)
         IOS=1
         DO 120 J=1,NGPT
         XZ2=ZJKSI(J)*ZJKSI(J)
         SEGPT(IOS)=2.0*RAA*SP2*ZJKSI(J)
         SEGWT(IOS)=2.0*WJKSI(J)*FP2
         IOS=IOS+1
         SEGPT(IOS)=RAA*(SQRT(1.0-SS2*XZ2)-SS1*SQRT(1.0-XZ2))
         SEGWT(IOS)=2.0*WJKSI(J)
  120    IOS=IOS+1
      ENDIF
*
      PSESE=0.0
      PSESI=0.0
      SIGR=SIGT*RAA
      SIGCUT=4.0*SIGR*(1.0-SS3)/(3.0*(1.0+SS2))
*
      IF (SIGCUT.GE.PETIT) THEN
*        NON-VOID SPHERICAL REGION
         IF (FILLED) THEN
*           INTEGRATE SPHERE PSS
            IOS=1
            DO 210 IPT=1,NGPT
            XE=SEGPT(IOS)*SIGT
            PSESE=PSESE+SEGWT(IOS)*EXP(-XE)
  210       IOS=IOS+1
*
*------    COMPUTE PVS,PVV
*
            PSS(1)=PSESE
            PVS(1)=(1.0-PSESE)*3.0/(4.0*SIGR)
            PVV=(1.0-PVS(1))/SIGT
         ELSE
*           INTEGRATE SHELL PROBABILITIES
            PVOL=4.0*(1.0-SS3)*SIGR/3.0
            SV1=SS2/PVOL
            SV2=1.0/PVOL
            IOS=1
            DO 220 IPT=1,NGPT
            XE=SEGPT(IOS)*SIGT
            PSESE=PSESE+SEGWT(IOS)*EXP(-XE)
            IOS=IOS+1
            XI=SEGPT(IOS)*SIGT
            PSESI=PSESI+SEGWT(IOS)*EXP(-XI)
  220       IOS=IOS+1
*
*------    COMPUTE PVS,PVV
*
            PSS(4)=PSESE
            PSS(3)=PSESI
            PSS(2)=PSS(3)*SS2
            PSS(1)=0.0
            PVS(2)=(1.0-PSS(2)-PSS(4))*SV2
            PVS(1)=(1.0-PSS(3))*SV1
            PVV=(1.0-PVS(2)-PVS(1))/SIGT
         ENDIF
      ELSE
*        COMPUTE PVV FROM SERIES EXPANSION OF EXP
         IF (FILLED) THEN
*           TREAT SPHERE
            PSS(1)=1.0-4.0*SIGR/3.0+SIGR*SIGR
            PVS(1)=1.0-3.0*SIGR/4.0
            PVV=3.0*RAA/4.0
         ELSE
*           TREAT SHELL
            SGR2=SIGR*SIGR
            SS4=SS3*SS1
            ALB=1.0/(1.0-SS3)
            DE2=FP2**1.5
            ASV=(1.0-DE2)/SS2
            AVV=(1.0+SS2-ALOG((1.0+SS1)/SQRT(FP2))*FP2*FP2/SS1)
     >           /(8.0*SS1)
            PSS(4)=FP2-SIGR*DE2*4.0/3.0+SGR2*(FP2*FP2)
            PSS(3)=1.0+SIGR*(SS1-ASV)*2.0/3.0+SGR2*(0.5-2.0*AVV)
            PSS(2)=PSS(3)*SS2
            PSS(1)=0.0
            PVS(2)=(DE2-SS2*(SS1-ASV)/2.0+((2.0*AVV-0.5)*SS2
     >             -FP2*FP2)*0.75*SIGR)*ALB
            PVS(1)=((ASV-SS1)*2.0-SIGR*(0.5-2.0*AVV)*3.0)*SS2*ALB/4.0
            PVV=(1.0+SS4-SS2-4.0*SS2*AVV)*0.75*RAA*ALB
         ENDIF
      ENDIF
      RETURN
      END
