*DECK SPHTRA
      SUBROUTINE SPHTRA(IPTRK ,IFTRAK,NDOOR ,KSPH  ,IPRINT,NREG  ,
     >                  NBMIX ,MAT   ,VOL   ,ILK   ,CTITRE,COURIN,
     >                  SIGMA ,SIGMS ,SOURCE,FUNKNO,PIJSYM,FLXTOT,
     >                  WORK  )
C
C-----------------------------------------------------------------------
C
C  TRANSPORT CALCULATION OVER THE MACRO-GEOMETRY IN A SINGLE CONDENSED
C  GROUP.
C
C INPUT PARAMETERS:
C   IPTRK  : POINTER TO THE TRACKING OF THE MACRO-GEOMETRY (L_TRACK
C            SIGNATURE).
C   IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRAVKING FILE.
C   NDOOR  : TRACKING MODULE USED TO TRACK THE MACRO-GEOMETRY.
C   KSPH   : =1: AVERAGE FLUX NORMALIZATION;
C            =2 OR 3: SELENGUT NORMALIZATION. THE 'ALBS' OPTION SHOULD
C            BE USED INSIDE MODULE ASM:.
C   IPRINT : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
C   NREG   : NUMBER OF MACRO-REGIONS (IN THE MACRO CALCULATION).
C   NBMIX  : NUMBER OF MATERIAL MIXTURES.
C   MAT    : MIXTURE INDEX PER MACRO-REGION. DIMENSION MAT(NREG)
C   VOL    : VOLUME OF MACRO-REGIONS. DIMENSION VOL(NREG)
C   ILK    : LEAKAGE SWITCH.
C   CTITRE : TITLE.
C   COURIN : AVERAGED FLUX IF KSPH=1; 4 TIMES THE INCOMING CURRENT PER
C            UNIT SURFACE IF KSPH=2 OR 3.
C   SIGMA  : TOTAL MACROSCOPIC X-SECTIONS PER MACRO-MIXTURE.
C            DIMENSION SIGMA(0:NBMIX)
C   SIGMS  : DIMENSION SIGMS(MAXMAT)
C   SOURCE : FISSION AND OUT-OF-GROUP DIFFUSION SOURCES.
C            DIMENSION SOURCE(NREG)
C
C OUTPUT PARAMETERS.
C   FUNKNO : NEUTRON FLUX. DIMENSION FUNKNO(NREG)
C
C SCRATCH STORAGE:
C   PIJSYM : DIMENSION PIJSYM(NREG*(NREG+1)/2)
C   FLXTOT : INTEGRATED FLUX
C   WORK   : DOUBLE PRECISION WORK(NREG,NREG+1)
C
C------------------------------------- AUTHOR: A. HEBERT ; 15/04/91 ----
C
      IMPLICIT   NONE
      INTEGER    IOUT
      CHARACTER  NAMSBR*6
      PARAMETER (IOUT=6,NAMSBR='SPHTRA')
C----
C INTERNAL FUNCTION
C----
      INTEGER    INDPOS
C----
C INTERFACE AND LOCAL VARIABLES
C----
      INTEGER    IPTRK,IFTRAK,NBMIX,KNORM,IPRINT,IREG,JREG,
     >           I,J,IPIJK,NELPIJ,NREG,IPSYS,KSPH,IER
      INTEGER    MAT(*)
C
      REAL       COURIN,FLXTOT,WORK(NREG,NREG+1),CP,VOLT,FLXT
      REAL       VOL(*),SIGMA(0:NBMIX),SIGMS(0:NBMIX),SOURCE(*),
     >           FUNKNO(*),PIJSYM(*)
      CHARACTER  NDOOR*12,CTITRE*72,HSMG*131,CFILNA*12
      LOGICAL    ILK
*----
*  Variables to implement modifications in EXCELP
*----
      INTEGER     NSBG,NPST,JGL,NGROUP,IFUITE
      REAL        PROBKS
      INTEGER     NALBP
      REAL        ALBP
C
C INTRINSIC FUNCTION FOR POSITION IN CONDENSE PIJ MATRIX
C
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
C
C COMPUTE COLLISION PROBABILITIES.
C
      NSBG=1
      NPST=1
      PROBKS=1.0
      IPIJK=1
      NELPIJ=NREG*(NREG+1)/2
      SIGMA(0)=0.0
      CFILNA='SPH-SYS'
      IPSYS=1
      IF(NDOOR.EQ.'EXCELL'.OR.NDOOR.EQ.'NXT') THEN
        JGL=0
        IFUITE=-1
        NGROUP=NSBG
        KNORM=1
        NALBP=0
        CALL EXCELP(IPSYS ,IPTRK ,IFTRAK,IPRINT,NREG  ,NBMIX ,
     >              MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,ILK   ,
     >              CTITRE,SIGMA ,PIJSYM,NSBG  ,NPST  ,PROBKS,
     >              JGL   ,NGROUP,IFUITE,
     >              NALBP ,ALBP  )
      ELSE IF(NDOOR.EQ.'JPM') THEN
        CALL LCMOP(IPSYS,CFILNA,0,1,0)
        CALL JPMP  (IPSYS ,IPTRK ,IFTRAK,IPRINT,NREG  ,NBMIX ,
     >              MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,ILK   ,
     >              CTITRE,SIGMA ,PIJSYM       )
        CALL LCMCL(IPSYS,2)
      ELSE IF(NDOOR.EQ.'SYBIL') THEN
        CALL SYBILP(IPSYS ,IPTRK ,IFTRAK,IPRINT,NREG  ,NBMIX ,
     >              MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,ILK   ,
     >              CTITRE,SIGMA ,PIJSYM       )
      ELSE
        WRITE(HSMG,9000) NAMSBR,NDOOR
        CALL XABORT(HSMG)
      ENDIF
*
* COMPUTE THE NEUTRON FLUXES.
      IF(KSPH .GT. 1) THEN
        DO 20 IREG=1,NREG
          WORK(IREG,NREG+1)=COURIN
          DO 10 JREG=1,NREG
            CP=PIJSYM(INDPOS(IREG,JREG))/VOL(IREG)
            WORK(IREG,JREG)=-SIGMS(MAT(JREG))*CP
            WORK(IREG,NREG+1)=WORK(IREG,NREG+1)+(SOURCE(JREG)
     >                       -COURIN*SIGMA(MAT(JREG)))*CP
 10       CONTINUE
          WORK(IREG,IREG)=WORK(IREG,IREG)+1.0
 20     CONTINUE
      ELSE
        DO 40 IREG=1,NREG
          WORK(IREG,NREG+1)=0.0
          DO 30 JREG=1,NREG
            CP=PIJSYM(INDPOS(IREG,JREG))/VOL(IREG)
            WORK(IREG,JREG)=-SIGMS(MAT(JREG))*CP
            WORK(IREG,NREG+1)=WORK(IREG,NREG+1)+SOURCE(JREG)*CP
 30       CONTINUE
          WORK(IREG,IREG)=WORK(IREG,IREG)+1.0D0
 40     CONTINUE
      ENDIF
      CALL ALSB(NREG,1,WORK,IER,NREG)
      IF(IER .NE. 0) CALL XABORT(NAMSBR//': SINGULAR MATRIX.')
      VOLT=0.0
      FLXT=0.0
      DO 120 IREG=1,NREG
        VOLT=VOLT+VOL(IREG)
        FUNKNO(IREG)=WORK(IREG,NREG+1)
        FLXT=FLXT+WORK(IREG,NREG+1)*VOL(IREG)
 120  CONTINUE
      FLXT=FLXT/VOLT
*
* FLUX NORMALIZATION.
      IF(KSPH .EQ. 1) THEN
        IF(IPRINT .GT. 0) WRITE(IOUT,6000) NAMSBR,COURIN/FLXT
        DO 130 IREG=1,NREG
          FUNKNO(IREG)=FUNKNO(IREG)*COURIN/FLXT
 130    CONTINUE
        FLXT=COURIN
      ENDIF
      FLXTOT=FLXT
      RETURN
 6000 FORMAT(/1X,A6,': FLUX NORMALIZATION FACTOR=',1P,E13.5)
 9000 FORMAT(A6,': ',A12,' IS AN INVALID CP CALCULATION MODULE NAME')
      END
