*DECK SYBALP
      SUBROUTINE SYBALP (NPIJ,MAXPTS,IT,Y,SIGT,PIJ,ALBG,ALBD,AUXK,
     >                   X,IND,INDV)
*
*----------------------------------------------------------------------
*
* PIJ CALCULATION USING THE METHOD OF KAVENOKY IN SLAB GEOMETRY.
*
* INPUT PARAMETERS:
*    NPIJ   : NUMBER OF REGIONS.
*    MAXPTS : FIRST DIMENSION OF MATRIX PIJ.
*    IT     : Type of BC.
*    Y      : ABSCISSA ARRAY. DIMENSION Y(NPIJ+1)
*    SIGT   : TOTAL CROSS SECTION ARRAY. DIMENSION SIGT(NPIJ)
*    ALBG   : LEFT ALBEDO.
*    ALBD   : RIGHT ALBEDO.
*    AUXK   : DUMMY WORK ARRAY. DIMENSION AUXK(4*NPIJ)
*    X      : DUMMY VECTOR OF LENGTH NPIJ+1
*    IND    : DUMMY VECTOR OF LENGTH NPIJ+1
*    INDV   : DUMMY VECTOR OF LENGTH NPIJ+1
*
* OUTPUT PARAMETERS:
*    PIJ    : REDUCED COLLISION PROBABILITIES ARRAY.
*             DIMENSION  PIJ(MAXPTS,NPIJ)
*
* REFERENCE:
* A. KAVENOKY, 'CALCUL ET UTILISATION DES PROBABILITES DE PREMIERE
* COLLISION POUR LES MILIEUX HETEROGENES A UNE DIMENSION: LES PROGRAMMES
* ALCOLL ET CORTINA', NOTE CEA-N-1077, COMMISSARIAT A L'ENERGIE
* ATOMIQUE, MARS 1969.
*
*----------------------------------------------------------------------
*
      IMPLICIT   NONE
*----
* Routine parameters
*---- 
      INTEGER    NPIJ,MAXPTS,IT
      REAL       Y(NPIJ+1),SIGT(NPIJ),PIJ(MAXPTS,NPIJ)
      REAL       ALBG,ALBD
      REAL       AUXK(NPIJ),X(NPIJ+1)
      INTEGER    IND(NPIJ+1),INDV(NPIJ+1)
*----
*  Local parameters
*---- 
      INTEGER    IOUT
      CHARACTER  NAMSBR*6
      PARAMETER (IOUT=6,NAMSBR='SYBALP')
*----
*  Functions
*----
      REAL       SYBALZ,TABEN 
*----
*  Local variables
*----
      INTEGER    I,J,KMAX,N1,NA,KI,KJ,I1
      REAL       TAU,EPS1,EPS2,S,T,A,EXPIN,AUXI,AUXJ,E10
*----
*  Validate input to routine
*----
      DO 10 J=1,NPIJ
        IF((Y(J+1)-Y(J)) .LT. 0.0) CALL XABORT(NAMSBR//
     >  ': Slab position vector must be increasing')
        IF(SIGT(J) .LT. 0.0) CALL XABORT(NAMSBR//
     >  ': A negative cross section was found')
 10   CONTINUE
*----
*  Scan for void regions
*  AUXK(I)= 0.0 FOR A VOID ZONE
*            1.0 FOR A FILLED ZONE
*  Evaluate optical path vector
*----
      N1=NPIJ+1
      KMAX=0
      X(1)=0.0
      DO 20 I=1,NPIJ
        IF(SIGT(I) .LT. 1.E-6) THEN
          IND(I)=0
          KMAX=KMAX+1
          INDV(KMAX)=I
        ELSE
          IND(I)=1
        ENDIF
        X(I+1)=X(I)+SIGT(I)*(Y(I+1)-Y(I))
        AUXK(I)=IND(I) 
 20   CONTINUE
*----
*  TAU is the total optical path for the slab
*----
      TAU=X(NPIJ+1)
      E10=TABEN(1,0.0)
      IF(TAU .LT. 1.E-6 .OR. KMAX .EQ. NPIJ) THEN
*----
*  When TAU=0 or KMAX=NPIJ 
*  case with all void
*  write warning, store E1(0) in PIJ and return
*----
        WRITE(IOUT,9000) NAMSBR
        DO 30 I=1,NPIJ
          DO 31 J=1,NPIJ
            PIJ(I,J)=E10
 31       CONTINUE
 30     CONTINUE
        RETURN
      ELSE
*----
*  Initialize "PIJ" matrix to 0.0
*----
        DO 40 I=1,NPIJ
          DO 41 J=1,NPIJ
            PIJ(I,J)=0.0
 41       CONTINUE
 40     CONTINUE
      ENDIF
*---- 
*  Contribution from NON-VOID to NON-VOID zones
*  "E3" FUNCTIONS ARE USED. 
*----
      EPS1=1.0
      EPS2=1.0
      NA=3
      DO 100 I=1,N1
        DO 101 J=I,N1
          S=X(I)
          T=X(J)
          EXPIN=SYBALZ(IT,NA,S,T,TAU,ALBG,ALBD,EPS1,EPS2)
          IF(I .GT. 1) THEN
            IF(J .GT. 1) 
     >      PIJ(I-1,J-1)=PIJ(I-1,J-1)-EXPIN*AUXK(I-1)*AUXK(J-1)
            IF(J .LT. N1) 
     >      PIJ(I-1,J)=PIJ(I-1,J)+EXPIN*AUXK(I-1)*AUXK(J)
          ENDIF
          IF(I .LT. N1) THEN
            A=1.0
            IF(J .GT. I) THEN
              IF(J .EQ. I+1) A=2.0
              PIJ(I,J-1)=PIJ(I,J-1)+EXPIN*AUXK(I)*AUXK(J-1)*A
            ENDIF
            IF(J .LT. N1) 
     >      PIJ(I,J)=PIJ(I,J)-EXPIN*AUXK(I)*AUXK(J) 
          ENDIF
 101    CONTINUE
 100  CONTINUE
*-----
*  Contribution from NON-VOID to VOID zones
*  "E2" FUNCTIONS ARE USED.
*----
*      IF(KMAX .GT. 0) THEN 
        EPS2=-1.0
        NA=2
        DO 110 KI=1,KMAX
          I=INDV(KI)
          IF(I .NE. 1) THEN
            EPS1=-1.0
            DO 111 J=1,I
              S=X(J)
              T=X(I)
              EXPIN=SYBALZ(IT,NA,S,T,TAU,ALBG,ALBD,EPS1,EPS2)
              IF(J .GT. 1) PIJ(J-1,I)=PIJ(J-1,I)+EXPIN*AUXK(J-1)
              IF(J .NE. I) PIJ(J,I)=PIJ(J,I)-EXPIN*AUXK(J) 
 111        CONTINUE
          ENDIF
          I1=I+1
          EPS1=1.0
          DO 112 J=I1,N1
            S=X(I1)
            T=X(J)
            EXPIN=SYBALZ(IT,NA,S,T,TAU,ALBG,ALBD,EPS1,EPS2)
            IF(J .LT. N1) PIJ(I,J)=PIJ(I,J)+EXPIN*AUXK(J)
            IF(J .GT. I1) PIJ(I,J-1)=PIJ(I,J-1)-EXPIN*AUXK(J-1)
 112      CONTINUE
 110    CONTINUE
*      ENDIF
*-----
*  Contribution from VOID to VOID zones
*  "E1" FUNCTIONS ARE USED.
*----
*      IF(KMAX.GT.1) THEN
        EPS1=-1.0
        EPS2=1.0
        NA=1
        DO 120 KI=1,KMAX
         I= INDV(KI)
         PIJ(I,I)=E10
         DO 121 KJ=KI+1,KMAX
            J= INDV(KJ)
            S=X(I)
            T=X(J)
            EXPIN=SYBALZ(IT,NA,S,T,TAU,ALBG,ALBD,EPS1,EPS2)
            PIJ(I,J)=PIJ(I,J)+EXPIN
 121      CONTINUE
 120    CONTINUE
*      ENDIF
*----
* Complete reduced "PIJ" matrix.
*----
      DO 130 I=1,NPIJ
        IF(IND(I) .GT. 0) THEN
          PIJ(I,I)=(1.+PIJ(I,I)/2./(X(I+1)-X(I)))/SIGT(I)
          IF(I .EQ. NPIJ) RETURN
        ENDIF
        I1=I+1
        AUXI=SIGT(I)
        IF(IND(I) .EQ. 0) AUXI=1./(Y(I+1)-Y(I))
        DO 131 J=I1,NPIJ
          AUXJ=SIGT(J)
          IF(IND(J) .EQ. 0) AUXJ=1./(Y(J+1)-Y(J))
          PIJ(I,J)=PIJ(I,J)/(2.*AUXI*AUXJ*(Y(I+1)-Y(I)))
          PIJ(J,I)=PIJ(I,J)*(Y(I+1)-Y(I))/(Y(J+1)-Y(J))
  131   CONTINUE
  130 CONTINUE
      RETURN 
*----
*  Format
*----
 9000 FORMAT('***** Warning in ',A6,' *****'/
     >       'All regions are voided in slab      '/
     >       'CP matrix is infinite               ')
      END
