*DECK TUBE1
      SUBROUTINE TUBE1 (NGPT,RAYRE,SIGT,TRONC,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 1-D
* CYLINDER OR TUBE (DP-0 SURFACE ANGULAR FLUX APPROXIMATION).
*
* INPUT PARAMETERS.
*   NGPT    : NUMBER OF GAUSS-JACOBI INTEGRATION POINTS.
*   RAYRE   : RADIUS OF TUBE. DIMENSION RAYRE(2)
*   SIGT    : CROSS SECTION.
*   TRONC   : VOIDED BLOCK CRITERION.
*
* OUTPUT PARAMETERS.
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES.
*             DIMENSION PVS(1) (CYLINDER) OR PVS(2) (TUBE)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             1 TO 1, 2 TO 1, 1 TO 2, 2 TO 2 WHERE 1 IS THE INTERNAL
*             SURFACE. DIMENSION PSS(1) (CYLINDER) OR PSS(4) (TUBE).
*
*------------------------------------------------------ G. MARLEAU -----
*
      PARAMETER (MKI3=600)
      PARAMETER (PI=3.1415926535,MGPT=64)
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      LOGICAL FILLED
      REAL    RAYRE(2),PVS(*),PSS(*),SEGPT(2*MGPT),SEGWT(2*MGPT),
     >        ZJKSI(MGPT),WJKSI(MGPT)
*
      CALL ALGJP(NGPT,ZJKSI,WJKSI)
      RAI=RAYRE(1)
      RAA=RAYRE(2)
      FILLED=(RAI.EQ.0.0)
      DPI=2.0/PI
      QPI=2.0*DPI
      PI8=2.0*QPI
      PI2=0.5*PI
      PI4=0.5*PI2
*
      PVOL=PI*(RAA*RAA-RAI*RAI)
      IF (FILLED) THEN
         IOS=1
         DO 110 J=1,NGPT
         SEGPT(IOS)=2.0*RAA*ZJKSI(J)*SQRT(2.0-ZJKSI(J)*ZJKSI(J))
         SEGWT(IOS)=PI8*WJKSI(J)
  110    IOS=IOS+1
      ELSE
         SS1=RAI/RAA
         FP1=1.0-SS1
         FP2=SS1*SS1
         SP1=SQRT(FP1)
         WFAI=PI8*SS1
         WFAE=PI8*FP1
         IOS=1
         DO 120 J=1,NGPT
         XZ2=ZJKSI(J)*ZJKSI(J)
         SEGPT(IOS)=2.0*RAA*SP1*ZJKSI(J)*SQRT(2.0-FP1*XZ2)
         SEGWT(IOS)=WFAE*WJKSI(J)
         IOS=IOS+1
         SEGPT(IOS)=RAA*(SQRT(1.0-FP2*(1.0-XZ2)**2)
     >             -SS1*ZJKSI(J)*SQRT(2.0-XZ2))
         SEGWT(IOS)=WFAI*WJKSI(J)
  120    IOS=IOS+1
      ENDIF
*
      PSESE=0.0
      PSESI=0.0
      SIGR=SIGT*RAA
      SIGCUT=2.0*SIGT*(RAA-RAI)
*
      IF (SIGCUT.GE.TRONC) THEN
*        NON-VOID ANNULAR REGION
         IF (FILLED) THEN
*           INTEGRATE FILLED ANNULUS PSS
            IOS=1
            DO 210 IPT=1,NGPT
            XE=SEGPT(IOS)*SIGT
            ZI3=0.0
            IF (XE.LT.XLIM3) THEN
               K=NINT(XE*PAS3)
               ZI3=BI3(K,1)+XE*(BI3(K,2)+XE*BI3(K,3))
            ENDIF
            PSESE=PSESE+SEGWT(IOS)*ZI3
  210       IOS=IOS+1
*
*------    COMPUTE PVS,PVV
*
            PSS(1)=PSESE
            PVS(1)=(1.0-PSESE)/(2.0*SIGR)
            PVV=(1.0-PVS(1))/SIGT
         ELSE
*           INTEGRATE TUBE PROBABILITIES
            SV1=PI2*RAI/(PVOL*SIGT)
            SV2=SV1*RAA/RAI
            IOS=1
            DO 220 IPT=1,NGPT
            XE=SEGPT(IOS)*SIGT
            ZI3=0.0
            IF (XE.LT.XLIM3) THEN
               K=NINT(XE*PAS3)
               ZI3=BI3(K,1)+XE*(BI3(K,2)+XE*BI3(K,3))
            ENDIF
            PSESE=PSESE+SEGWT(IOS)*ZI3
            IOS=IOS+1
            XI=SEGPT(IOS)*SIGT
            ZI3=0.0
            IF (XI.LT.XLIM3) THEN
               K=NINT(XI*PAS3)
               ZI3=BI3(K,1)+XI*(BI3(K,2)+XI*BI3(K,3))
            ENDIF
            PSESI=PSESI+SEGWT(IOS)*ZI3
  220       IOS=IOS+1
*
*------    COMPUTE PVS,PVV
*
            PSS(4)=PSESE
            PSS(3)=PSESI*RAA/RAI
            PSS(2)=PSESI
            PSS(1)=0.0
            PVS(2)=(1.0-PSESE-PSESI)*SV2
            PVS(1)=(1.0-PSS(3))*SV1
            PVV=(1.0-PVS(2)-PVS(1))/SIGT
         ENDIF
      ELSE
*        COMPUTE PVV FROM SERIES EXPANSION OF KI3
         IF (FILLED) THEN
*           TREAT FILLED ANNULUS
            PSS(1)=1.0-2.0*SIGR+8.0*SIGR*SIGR/3.0
            PVS(1)=1.0-4.0*SIGR/3.0
            PVV=4.0*RAA/3.0
         ELSE
*           TREAT TUBES
            IOS=2
            DO 230 IPT=1,NGPT
            PSESI=PSESI+SEGWT(IOS)*SEGPT(IOS)*SEGPT(IOS)
  230       IOS=IOS+2
*
*------    COMPUTE PVS,PVV
*
            PSXSX=PI4*PSESI/(RAA*RAA)
            SGR2=SIGR*SIGR
            SS2=SS1*SS1
            RSV=1.0/(2.0*(1.0-SS2))
            ASV=SS1*SQRT(1.0-SS2)+ASIN(SS1)
            AVV=4.0*(1.0-SS1)*(1.0-SS1)*(2.0+SS1)/3.0
            PSS(4)=(1.0-SS1)-2.0*SIGR+QPI*SIGR*ASV+SGR2*AVV
            PSS(3)=1.0+SIGR*SS1+(-DPI*SIGR*ASV+SGR2*PSXSX)/SS1
            PSS(2)=PSS(3)*SS1
            PSS(1)=0.0
            PVS(2)=RSV*(2.0-SS2-DPI*ASV-SIGR*AVV-SIGR*PSXSX)
            PVS(1)=RSV*(DPI*ASV-SS2-SIGR*PSXSX)
            PVV=RAA*RSV*(2.0*PSXSX+AVV)
         ENDIF
      ENDIF
      RETURN
      END
