*DECK XDRLGS
      SUBROUTINE XDRLGS(IPLIB ,IGS   ,IPRINT,MINLEG,MAXLEG,IORD  ,
     >                  NGROUP,XSREC ,SCAT  ,ITYPRO,XSSCMP,NJJ   ,
     >                  IJJ   )
*
*-----------------------------------------------------------------------
*
*Purpose:
* get/save Legendre-dependent cross section data from/on IPLIB
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): G. Marleau
*
*Parameters: input
* IPLIB   pointer to the internal library
* IGS     get or save flag:
*         >0 save;
*         =0 do not process;
*         <0 get
* IPRINT  Print level (cross sections printed if IPRINT>9)
* MINLEG  mimimum Legendre order to process for scattering
* MAXLEG  maximum Legendre order to process for scattering
* IORD    cross section order:
*         =1 constant;
*         =2 linear;
*         =3 quadratic
* NGROUP  number of energy groups
*
*Parameters: input/output
* XSREC   cross section records (scattering cross section of
*         order MINLEG to MAXLEG for IRPROC=MINLEG+1,MAXLEG+1)
* SCAT    complete scattering matrix (SCAT(JG,IG) is from IG to JG
*         for order MINLEG to MAXLEG)
*
*Parameters: output
* ITYPRO  vector for cross section processed indices:
*         =0 -> absent  (not processed);
*         >0 -> present (processed)
*
*Parameters: scratch
* XSSCMP  compress scattering for transfer
* NJJ     bandwidth of the scattering group
* IJJ     maximum scattering group index
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  Local parameters
*  LIMLEG = maximum legendre orders permitted.
*----
      INTEGER          IOUT,LIMLEG
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,LIMLEG=100,NAMSBR='XDRLGS')      
*----
*  Subroutine arguments
*----
      INTEGER          IPLIB,IGS,IPRINT,MINLEG,MAXLEG,IORD,NGROUP
      REAL             XSREC(NGROUP,MAXLEG-MINLEG+1),
     >                 SCAT(NGROUP,NGROUP,MAXLEG-MINLEG+1)
      INTEGER          ITYPRO(MAXLEG-MINLEG+1)
      REAL             XSSCMP(NGROUP*NGROUP)
      INTEGER          NJJ(NGROUP),IJJ(NGROUP)
*----
*  Local variables and data
*----
      INTEGER          NPROC,IDLEG(LIMLEG),IODIV,LONG,ITYP,LONG2,ILEG,
     >                 IXSR,IXSTN,IG,JG,NXSCMP,IGTO,IGMIN,IGMAX,IGFROM
      CHARACTER*12     NAMXS
      CHARACTER*6      NORD(3)
      DATA    NORD    /'      ','   LIN','   QUA'/
*----
*  Test input options
*----
      IF(IORD .LT. 1 .OR. IORD .GT. 3) CALL XABORT(NAMSBR//
     >  ': Invalid XS order -> only 1, 2 or 3 permitted.')
      IF(MAXLEG+1 .GT. LIMLEG) CALL XABORT(NAMSBR//
     >  ': Maximum legendre order permitted is 99')
      IF(MINLEG .LT. 0) CALL XABORT(NAMSBR//
     >  ': Minimum legendre order permitted is 0')
      IF(MAXLEG .LT. MINLEG) CALL XABORT(NAMSBR//
     >  ': Invalid legendre orders -> MAXLEG.LT.MINLEG')
      IODIV=2**(IORD-1)
      NPROC=MAXLEG-MINLEG+1
*----
*  Read/initialize state vector
*----
      CALL XDISET(ITYPRO,NPROC,0)
      CALL LCMLEN(IPLIB,'SCAT-SAVED',LONG,ITYP)
      CALL XDISET(IDLEG,MAXLEG+1,0)
      IF(LONG .GT. 0) THEN
        CALL LCMGET(IPLIB,'SCAT-SAVED',IDLEG)
        DO ILEG=MINLEG+1,MIN(LONG,MAXLEG+1)
          ITYPRO(ILEG-MINLEG)=IDLEG(ILEG)
        ENDDO 
      ENDIF
      LONG=MAX(LONG,MAXLEG+1)
      IF(IGS.GT.0) THEN
*----
*  Saving scattering XS
*----
        IXSR=0
        DO ILEG=MINLEG+1,MAXLEG+1
          IXSR=IXSR+1
          IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
          IF((ILEG.EQ.1).AND.(IODIV.EQ.1)) THEN
*----
*  For legendre order 0, constant term write scattering XS
*  even if all 0.0
*----
            IF(IXSTN.EQ.0) THEN
              ITYPRO(IXSR)=ITYPRO(IXSR)+IODIV
              IDLEG(ILEG)=IDLEG(ILEG)+IODIV
              IXSTN=1
            ENDIF
          ELSE
*----
*  For all other case write write scattering XS
*  only if not all 0.0
*----
            DO IG=1,NGROUP
              DO JG=1,NGROUP
                IF(SCAT(IG,JG,IXSR).NE.0.0) THEN
                  IF(IXSTN.EQ.0) THEN
                    ITYPRO(IXSR)=ITYPRO(IXSR)+IODIV
                    IDLEG(ILEG)=IDLEG(ILEG)+IODIV
                    IXSTN=1
                  ENDIF
                  GO TO 135
                ENDIF
              ENDDO
            ENDDO
          ENDIF
 135      CONTINUE
          IF(IXSTN.NE.0) THEN
            WRITE(NAMXS,'(A4,I2.2,A6)') 'SIGS',ILEG-1,NORD(IORD)
            CALL LCMPUT(IPLIB,NAMXS,NGROUP,2,XSREC(1,IXSR))
*----
*  COMPRESS SCATTERING MATRIX
*  SCAT(IGTO,IGFROM) REPRESENT SCATTERING CROSS SECTION
*    FROM GROUP "IGFROM" TO GROUP "IGTO"
*  IJJ(IGTO) IS MAXIMUM GROUP NUMBER
*    WITH SCATTERING TO "IGTO" GROUP
*  NJJ(IGTO) IS NUMBER OF GROUPS
*    WITH SCATTERING TO "IGTO" GROUP
*  XSSCMP(IX) IS COMPRESSED SCATTERING MATRIX
*  IX CAN BE LOCALIZED IN SCAT(IGTO,IGFROM) USING
*    IF(IGTO=1) THEN
*      IPOSD=1
*    ELSE
*      IPOSD=1+SUM( NJJ(IGF) , IGF=1,IGTO-1)
*    ENDIF
*    IF(IGFROM.GT.IJJ(IGTO)) THEN
*      XSSCMP NOT STORED
*    ELSE IF(IGFROM.LT.IJJ(IGTO)-NJJ(IGTO)+1) THEN
*      XSSCMP NOT STORED
*    ELSE
*      IX=IPOSD+IJJ(IGTO)-IGFROM
*      XSSCMP(IX)=SCAT(IGTO,IGFROM)
*  ENDIF
*----
            NXSCMP=0
            DO IGTO=1,NGROUP
              IGMIN=IGTO
              IGMAX=IGTO
              DO IGFROM=1,NGROUP
                IF(SCAT(IGTO,IGFROM,IXSR).NE.0.0) THEN
                  IGMIN=MIN(IGMIN,IGFROM)
                  IGMAX=MAX(IGMAX,IGFROM)
                ENDIF
              ENDDO
              IJJ(IGTO)=IGMAX
              NJJ(IGTO)=IGMAX-IGMIN+1
              DO IGFROM=IGMAX,IGMIN,-1
                NXSCMP=NXSCMP+1
                XSSCMP(NXSCMP)=SCAT(IGTO,IGFROM,IXSR)
              ENDDO
            ENDDO
            WRITE(NAMXS,'(A4,I2.2,A6)') 'NJJS',ILEG-1,NORD(IORD)
            CALL LCMPUT(IPLIB,NAMXS,NGROUP,1,NJJ)
            WRITE(NAMXS,'(A4,I2.2,A6)') 'IJJS',ILEG-1,NORD(IORD)
            CALL LCMPUT(IPLIB,NAMXS,NGROUP,1,IJJ)
            WRITE(NAMXS,'(A4,I2.2,A6)') 'SCAT',ILEG-1,NORD(IORD)
            CALL LCMPUT(IPLIB,NAMXS,NXSCMP,2,XSSCMP)
          ENDIF
        ENDDO
        CALL LCMPUT(IPLIB,'SCAT-SAVED',LONG,1,IDLEG)
      ELSE IF(IGS .LT. 0) THEN
*----
*  Get scattering XS
*----
        IXSR=0
        DO ILEG=MINLEG+1,MAXLEG+1
          IXSR=IXSR+1
*----
*  Read if IXSTN = 1 (cross sections exists)
*  Initialize to 0.0 if IXSTN = 0 (cross sections absent)
*----
          CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
          CALL XDRSET(SCAT(1,1,IXSR),NGROUP*NGROUP,0.0)
          IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
          IF(IXSTN.EQ.1) THEN
            WRITE(NAMXS,'(A4,I2.2,A6)') 'SIGS',ILEG-1,NORD(IORD)
            CALL LCMGET(IPLIB,NAMXS,XSREC(1,IXSR))
            WRITE(NAMXS,'(A4,I2.2,A6)') 'NJJS',ILEG-1,NORD(IORD)
            CALL LCMGET(IPLIB,NAMXS,NJJ)
            WRITE(NAMXS,'(A4,I2.2,A6)') 'IJJS',ILEG-1,NORD(IORD)
            CALL LCMGET(IPLIB,NAMXS,IJJ)
            WRITE(NAMXS,'(A4,I2.2,A6)') 'SCAT',ILEG-1,NORD(IORD)
            CALL LCMGET(IPLIB,NAMXS,XSSCMP)
*----
*  DECOMPRESS SCATTERING MATRIX
*  SCAT(IGTO,IGFROM) REPRESENT SCATTERING CROSS SECTION
*    FROM GROUP "IGFROM" TO GROUP "IGTO"
*  IJJ(IGTO) IS MAXIMUM GROUP NUMBER
*    WITH SCATTERING TO "IGTO" GROUP
*  NJJ(IGTO) IS NUMBER OF GROUPS
*    WITH SCATTERING TO "IGTO" GROUP
*  XSSCMP(IX) IS COMPRESSED SCATTERING MATRIX
*  SCAT(IGTO,IGFROM) CAN BE LOCALIZED IN XSSCMP(IX) USING
*    IF(IGTO=1) THEN
*      IPOSD=1
*    ELSE
*      IPOSD=1+SUM( NJJ(IGF) , IGF=1,IGTO-1)
*    ENDIF
*    IF(IGFROM.GT.IJJ(IGTO)) THEN
*      SCAT(IGTO,IGFROM)=0.0
*    ELSE IF(IGFROM.LT.IJJ(IGTO)-NJJ(IGTO)+1) THEN
*      SCAT(IGTO,IGFROM)=0.0
*    ELSE
*      SCAT(IGTO,IGFROM)=XSSCMP(IPOSD+IJJ(IGTO)-IGFROM)
*    ENDIF
*----
            NXSCMP=0
            DO IGTO=1,NGROUP
              IGMAX=IJJ(IGTO)
              IGMIN=IGMAX-NJJ(IGTO)+1
              DO IGFROM=IGMAX,IGMIN,-1
                NXSCMP=NXSCMP+1
                SCAT(IGTO,IGFROM,IXSR)=XSSCMP(NXSCMP)
              ENDDO
            ENDDO
          ENDIF
        ENDDO
      ENDIF
      IF(IPRINT .GE. 10) THEN
*----
*  Print XS
*----
        IXSR=0
        DO ILEG=MINLEG+1,MAXLEG+1
          IXSR=IXSR+1
          IXSTN=MOD(ITYPRO(ILEG)/IODIV,2)
          IF(IXSTN.NE.0) THEN
            WRITE(NAMXS,'(A4,I2.2,A6)') 'SIGS',ILEG-1,NORD(IORD)
            WRITE(IOUT,6000) NAMXS
            WRITE(IOUT,6010) (XSREC(IG,IXSR),IG=1,NGROUP)
*----
*  SCAT(IGTO,IGFROM) REPRESENT SCATTERING CROSS SECTION
*  FROM GROUP "IGFROM" TO GROUP "IGTO"
*----
            WRITE(NAMXS,'(A4,I2.2,A6)') 'SCAT',ILEG-1,NORD(IORD)
            WRITE(IOUT,6000) NAMXS
            DO IGFROM=1,NGROUP
              WRITE(IOUT,6001) IGFROM
              WRITE(IOUT,6010) (SCAT(IGTO,IGFROM,IXSR),IGTO=1,NGROUP)
            ENDDO
          ENDIF
        ENDDO
      ENDIF
      RETURN
*----
*  FORMAT
*----
 6000 FORMAT(/' CROSS SECTION TYPE    = ',A12)
 6001 FORMAT(/' SCATTERING FROM GROUP = ',I10)
 6010 FORMAT(1P,5E16.7)
      END
