*DECK XDRLXS
      SUBROUTINE XDRLXS(IPLIB ,IGS   ,IPRINT,NPROC ,NAMDXS,IORD  ,
     >                  NGROUP,XSREC )
*
*-----------------------------------------------------------------------
*
*Purpose:
* get/save specific vector cross section from/on IPLIB
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): G. Marleau
*
*Parameters: input
* IPLIB   pointer to the internal library
* IGS     get or save flag:
*         >0 save;
*         <0 get
* IPRINT  Print level (cross sections printed if IPRINT>9)
* NPROC   number of vector cross sections to process
* NAMDXS  names of cross sections to process
* IORD    cross section order:
*         =1 constant;
*         =2 linear;
*         =3 quadratic
* NGROUP  number of energy groups
*
*Parameters: input/output
* XSREC   cross section records for IRPROC=1,NPROC
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='XDRLXS')      
*----
*  Subroutine arguments
*----
      INTEGER          IPLIB,IGS,IPRINT,NPROC
      CHARACTER*6      NAMDXS(NPROC)
      INTEGER          IORD,NGROUP
      REAL             XSREC(NGROUP,NPROC)
*----
*  Local variables and data
*----
      INTEGER          IXSR,IG,JG,ILENG,ITYLCM
      CHARACTER*12     NAMXS
      CHARACTER*6      NORD(3)
      DATA             NORD/'      ','   LIN','   QUA'/
*----
*  Test input options
*----
      IF(NPROC.LE.0) CALL XABORT(NAMSBR//
     >  ': Number of XS to process must be positive.')
      IF(IORD.LT. 1 .OR. IORD.GT.3) CALL XABORT(NAMSBR//
     >  ': Invalid XS order -> only 1, 2 or 3 permitted.')
      IF(IGS.GT.0) THEN
*----
*  Saving vector XS
*----
        DO IXSR=1,NPROC
*----
*  Save only if all XS not 0.0
*----
          NAMXS=NAMDXS(IXSR)//NORD(IORD)
          DO IG=1,NGROUP
            IF(XSREC(IG,IXSR).NE.0.0) THEN
              CALL LCMPUT(IPLIB,NAMXS,NGROUP,2,XSREC(1,IXSR))
              GO TO 100
            ENDIF
          ENDDO
 100      CONTINUE
        ENDDO
      ELSE IF(IGS .LT. 0) THEN
*----
*  Get vector XS
*----
        DO IXSR=1,NPROC
          CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
          NAMXS=NAMDXS(IXSR)//NORD(IORD)
          CALL LCMLEN(IPLIB,NAMXS,ILENG,ITYLCM)
          IF(ILENG.GT.0) THEN
            CALL LCMGET(IPLIB,NAMXS,XSREC(1,IXSR))
          ENDIF
        ENDDO
      ENDIF
      IF(IPRINT .GE. 10) THEN
*----
*  Print XS
*----
        DO IXSR=1,NPROC
          NAMXS=NAMDXS(IXSR)//NORD(IORD)
          DO IG=1,NGROUP
            IF(XSREC(IG,IXSR).NE.0.0) THEN
              WRITE(IOUT,6000) NAMXS
              WRITE(IOUT,6010) (XSREC(JG,IXSR),JG=1,NGROUP)
              GO TO 110
            ENDIF
          ENDDO
 110      CONTINUE
        ENDDO
      ENDIF
      RETURN
*----
*  Formats
*----
 6000 FORMAT(/' CROSS SECTION TYPE    = ',A12)
 6010 FORMAT(1P,5E16.7)
      END
