*DECK XELPA1
      SUBROUTINE XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        A,      B,      C,
     >                     PARR,  PERPB,  PERPA )
************************************************************************
*                                                                      *
*           NAME: XELPA1                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: (CALLED BY 'XELPA3')                                 *
*        VERSION: 0.8                                                  *
*       CREATION: 87/01                                                *
*       MODIFIED: 00/03 (R.R.) DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE WILL COMPUTE SURFACE-TO-SURFACE "CP"    *
*                 FOR A RECTANGULAR SINGLE VOLUME BLOCK, USING GAUSS   *
*                 INTEGRATION POINTS.                                  *
*                                                                      *
*      REFERENCE: 'ANALYTIC REDUCTIONS FOR TRANSMISSION AND LEAKAGE    *
*                  PROBABILITIES IN FINITE TUBES AND HEXAHEDRA',       *
*                  G.MARLEAU, R.ROY AND A.HEBERT,                      *
*                  NUCLEAR SCIENCE & ENGINEERING, V 104, P 204 (1990). *
*                                                                      *
*--------+------------- P A R A M E T E R S --------------+--+---------*
*  NAME  /                  DESCRIPTION                   /IO/  MODE   *
*--------+------------------------------------------------+--+---------*
* NGAUSS / # OF GAUSS INTEGRATION POINTS & WEIGHTS.       /I./INT      *
* XGAUSS / "ARRAY" WITH GAUSS POINTS.                     /I./REL      *
*        /                                                /  / (NGAUSS)*
* WGAUSS / "ARRAY" WITH GAUSS WEIGHTS.                    /I./REL      *
*        /                                                /  / (NGAUSS)*
*      A / SIDE OF A RECT. HOMOGENEOUS BLOCK ( A . B . C )/I./REL      *
*      B / SIDE OF A RECT. HOMOGENEOUS BLOCK ( A . B . C )/I./REL      *
*      C / SIDE OF A RECT. HOMOGENEOUS BLOCK ( A . B . C )/I./REL      *
*   PARR / "CP" FOR 2 RECT PARALLEL SURFACES.             /.O/REL      *
*  PERPB / "CP" FOR 2 RECT PERPENDICULAR SURFACES.        /.O/REL      *
*        /            ( COMMON AXIS: B )                  /  /         *
*  PERPA / "CP" FOR 2 RECT PERPENDICULAR SURFACES.        /.O/REL      *
*        /            ( COMMON AXIS: A )                  /  /         *
*--------+-------------- R O U T I N E S -----------------+--+---------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
*  TABEN / TABULAR EXPONENTIAL-INTEGRAL FUNCTIONS                      *
************************************************************************
C
      IMPLICIT NONE
C
C     DECLARE  DUMMY ARGUMENTS
      INTEGER  NGAUSS
      REAL     XGAUSS(NGAUSS), WGAUSS(NGAUSS)
      REAL     A, B, C, PARR, PERPB, PERPA
C
C     DECLARE  LOCAL VARIABLES
      REAL     BB, CC, A2, BB2A, U, UU, UUCC, UUBB, UUBBCC,
     >         E3U, E3UB, E3UC, E3UBC, UM2A, FC, FCA, FCB
      INTEGER  IGAU
C
C     DECLARE  FUNCTION
      REAL     TABEN
C
      CC=C*C
      BB=B*B
      A2=2.*A
      BB2A=A2*BB
      PARR=0.
      PERPB=0.
      PERPA=0.
C
      DO 10 IGAU=1,NGAUSS
          U=A*XGAUSS(IGAU)
          UU=U*U
          UUCC=UU+CC
          UUBB=UU+BB
          UUBBCC=UUBB+CC
          E3U=TABEN(3,U)
          E3UB=TABEN(3,SQRT(UUBB))
          E3UC=TABEN(3,SQRT(UUCC))
          E3UBC=TABEN(3,SQRT(UUBBCC))
          UM2A=U-A2
          FC=UM2A*E3UC/UUCC-(UM2A+BB2A/UUBB)*E3UBC/UUBBCC
          FCB=UM2A*(UU*(E3UC/UUCC+E3UB/UUBB-E3UBC/UUBBCC)-E3U)
          FCA=U*(E3UB/UUBB-E3UBC/UUBBCC)
          PARR=PARR+WGAUSS(IGAU)*FC
          PERPB=PERPB+WGAUSS(IGAU)*FCB
          PERPA=PERPA+WGAUSS(IGAU)*FCA
   10 CONTINUE
C
      PARR=CC*PARR*A
      PERPB=PERPB*A
      PERPA=BB*PERPA*A
C
      RETURN
      END
